/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/* ADVDLG.C
   27/8/97
   advanced dialog box for capture settings 
*/


#include "common.h"

#include <ver.h>

#include "resource.h"
#include "vfwdlg.h" 

#include "vvlcap.h"
#include "ppc.h"
#include "oem.h"
#include "dvm_msg.h"     
#include "czoom.h"       
#include "advdlg.h"
                          
/* dll instance handle. Declared in dvm_msg.c */                          
extern HINSTANCE ghModule ;
extern int gAdvCapture;

/*-----------------------------------------------------------------*/

BOOL AdvancedCaptureOptions (HWND hwnd)

{
 return DialogBox (ghModule, MAKEINTRESOURCE(gAdvCapture), 
                   hwnd, dlgAdvanced) ;
}

/*-----------------------------------------------------------------*/

BOOL CALLBACK _loadds dlgAdvanced (HWND hwnd, UINT msg, WPARAM wparam,
                                   LPARAM lparam)
                                   
{                       
 static FRAMERATE old_fps,
                  curr_fps ;    /* previous frame rate (changes each time a new one
                                  selected) */
 static BOOL oldflick,
             oldcweight,
			 old_auto_lighting, 
             setfocus,
             zoom_changed,
             flickchanged,
             hzchanged,       /* change in mains frequency */
             cweightchanged,
			 curr_auto_lighting,
             fpschanged,
			 roichanged,
             zoom_avail,  /* true if size <= qcif */
             vidconf ;
              
 static RECT pos,    /* position of dialog box on previous open */
             roi ;
 
 static WORD OldZoom,
             pan,
             tilt,
             frbutton,
             hzbutton,
             vconfval,
             vcapval,
			 manualval;

 static HBRUSH hctlcolour ;
 static MAINS_FREQ oldmf ; 
 static FRAMERATE oldfr ;
 static COMP_MODE comp ;
 
 static UINT autolighting_timer_id;
 
 BOOL res = FALSE,
      buts,	doing_auto_framerate,
      oldfw ;
 WORD wid = 0,
      fwver ;
 int i ;
 HCURSOR oldcur ;
 POINT pt ;
 RECT tmp,
      rect ;
 HWND hCtl ;              
 LONG tres ;
 MAINS_FREQ mf ;
 HBRUSH hbr ;
 
 switch (msg)
 {
 case WM_INITDIALOG :
        hctlcolour = CreateSolidBrush( GetSysColor(COLOR_BTNFACE) );

        ppcGetSensorFramerate( &old_fps, &old_auto_lighting, NULL );
		curr_fps = old_fps;
		curr_auto_lighting = old_auto_lighting;

        oldcweight = ppcGetCentreWeight();
        cweightchanged = FALSE ;

        switch( curr_fps ) 
        {
        case FPS_30 : wid = IDC_30FPS;	break;
        case FPS_15 : wid = IDC_15FPS; 	break;
        case FPS_75 : wid = IDC_7P5FPS;	 break;
        }
        frbutton = wid ;                /* current frame rate button */

        CheckRadioButton( hwnd, IDC_30FPS, IDC_7P5FPS, wid );
		/* either enable or grey out the lighting radio buttons */
       for (i = IDC_30FPS; i <= IDC_7P5FPS; i++)
                 EnableWindow( GetDlgItem(hwnd, i), !curr_auto_lighting );

		CheckDlgButton( hwnd, IDC_AUTOLIGHTING, curr_auto_lighting );
        CheckDlgButton( hwnd, IDC_CENTREWEIGHT, oldcweight );


        flickchanged = FALSE ;
		oldflick = ppcGetFlicker () ;
		fwver = ppcGetFwVersion () ;
        ppcGetFlickerParams (&mf) ;
        oldmf = mf ;
        CheckDlgButton (hwnd, IDC_FLICKER, oldflick) ;
        hzbutton = mf == MAINS_50HZ ? IDC_50HZ : IDC_60HZ ;
        CheckRadioButton (hwnd, IDC_50HZ, IDC_60HZ, hzbutton) ;
        
        /* flicker control isn't available at 7.5 fps on 1-02 or 1-06 firmware,
           so disable all the controls if that framerate is selected. 
           Mains frequency is disabled if flicker control isn't selected 
        */                                            
        oldfw = fwver <= FW_VER106 ;
        EnableWindow (GetDlgItem (hwnd, IDC_FLICKER), 
                      !(oldfw && curr_fps == FPS_75)) ;          
                      
        EnableWindow (GetDlgItem (hwnd, IDC_50HZ), 
                      oldflick && !(oldfw && curr_fps == FPS_75)) ;
                      
        EnableWindow (GetDlgItem (hwnd, IDC_60HZ), 
                      oldflick && !(oldfw && curr_fps == FPS_75)) ;
        
		roichanged = FALSE;
        if ((zoom_avail = ppcGetPanTiltInfo (&roi, &OldZoom)) == TRUE)
         {
          if (OldZoom)
           {
            pan = pt. x = roi. left ;
            tilt = pt. y = roi. top ;
           }
          else
           {
            pan = (CIF_COLS - (roi. right - roi. left)) / 2 ;
            tilt = (CIF_ROWS - (roi. bottom - roi. top)) / 2 ;
           }

          hCtl = GetDlgItem (hwnd, IDC_PANTILT) ;
             
          SendMessage (hCtl, CZ_SETSIZE, 0, (LPARAM) &roi) ;
          /* if zoom is on, inform the control of the roi size, and the top 
             left of the image rectangle */     
          if (OldZoom)
           {
            SendMessage (hCtl, CZ_SETPOSITION, 0, (LPARAM) &pt) ;
           }

          EnableWindow (hCtl, OldZoom) ;
          CheckDlgButton (hwnd, IDC_ZOOM, OldZoom) ;          
         }
        else
         {                                         
          /* camera not in qcif or smaller mode, zoom not available */
          SetWindowText (GetDlgItem (hwnd, IDC_PANTILTGRP),
                         "Pan/Tilt only at 176x144 or smaller") ;
          EnableWindow (GetDlgItem (hwnd, IDC_PANTILT), FALSE) ;
          EnableWindow (GetDlgItem (hwnd, IDC_PANTILTGRP), FALSE) ;
          EnableWindow (GetDlgItem (hwnd, IDC_ZOOM), FALSE) ;          
         }

        /* set everything to unchanged for cancel button */        
        zoom_changed = flickchanged = hzchanged = cweightchanged =
         fpschanged = FALSE ;

        /* restore the dialog box to its previous location */
        if (!IsRectEmpty (&pos))
         SetWindowPos (hwnd, HWND_TOP,
                       pos. left, pos. top, 
                       0, 0,
                       SWP_NOSIZE | SWP_NOZORDER) ;
        break ;
        
   case WM_COMMAND :
        switch (wparam)
         {      
          case IDC_AUTOLIGHTING:
               buts = IsDlgButtonChecked( hwnd, IDC_AUTOLIGHTING );
               if ( buts != curr_auto_lighting ) 
               {
					curr_auto_lighting = buts;
					if ( !curr_auto_lighting ) 
					{ /* 
						about to switch off auto_lighting
						- need to get the current fps setting first 
					  */
	               		ppcGetSensorFramerate( &curr_fps, NULL, NULL );
					}
			        switch( curr_fps ) 
			        {
			        case FPS_30 : wid = IDC_30FPS;	break;
			        case FPS_15 : wid = IDC_15FPS; 	break;
			        case FPS_75 : wid = IDC_7P5FPS;	 break;
			        }
			        CheckRadioButton( hwnd, IDC_30FPS, IDC_7P5FPS, wid );

               		ppcSetSensorFramerate( curr_fps, curr_auto_lighting );
	    			for (i = IDC_30FPS; i <= IDC_7P5FPS; i++)
		                  EnableWindow( GetDlgItem(hwnd, i), !curr_auto_lighting);
               }
               res = TRUE ;
               break ;
                  
               /* the buttons are disabled round the call. This isn't
                  strictly necessary, but the rate change takes over a second,
                  so it gives the user a cue that it's finished */
          case IDC_30FPS :
          case IDC_15FPS :
          case IDC_7P5FPS :
               if (wparam != frbutton && !setfocus)
                {
                 frbutton = wparam ;
                 for (i = IDC_30FPS; i <= IDC_7P5FPS; i++)
                  EnableWindow (GetDlgItem (hwnd, i), FALSE) ;

                 oldcur = SetCursor (LoadCursor (NULL, IDC_WAIT)) ;
                                                                                      
                 fwver = ppcGetFwVersion () ;
                 /* VERSION_NOTE - flicker is switched off and disabled 
                    at 7.5fps */
                 if (IsDlgButtonChecked (hwnd, IDC_FLICKER) && 
                     fwver <= FW_VER106)
                  {                            
                   if (wparam == IDC_7P5FPS)
                    {
                     ppcSetFlicker (FALSE) ;
                     /* EnableWindow (GetDlgItem (hwnd, IDC_FLICKER), FALSE) ;*/
                     EnableWindow (GetDlgItem (hwnd, IDC_50HZ), FALSE) ;
                     EnableWindow (GetDlgItem (hwnd, IDC_60HZ), FALSE) ;
                     /* CheckDlgButton (hwnd, IDC_FLICKER, FALSE) ;*/
                    }
                   else
                    if (curr_fps == IDC_7P5FPS)
                     {                           
                      ppcSetFlicker (TRUE) ;
                      /* EnableWindow (GetDlgItem (hwnd, IDC_FLICKER), TRUE) ;*/
                      EnableWindow (GetDlgItem (hwnd, IDC_50HZ), TRUE) ;
                      EnableWindow (GetDlgItem (hwnd, IDC_60HZ), TRUE) ;
                     }
                  }
                  
                 /* VERSION_NOTE */
                 /* flicker control doesn't work at 7.5 fps at <= 1-06 */
                 EnableWindow (GetDlgItem (hwnd, IDC_FLICKER), 
                               !(fwver <= FW_VER106 && wparam == IDC_7P5FPS)) ;

                 ppcSetSensorFramerate( wparam == IDC_30FPS ? FPS_30 :
                                  		wparam == IDC_15FPS ? FPS_15 : FPS_75,
                                  		FALSE );
                                  
                 for (i = IDC_30FPS; i <= IDC_7P5FPS; i++)
                  EnableWindow (GetDlgItem (hwnd, i), TRUE) ;

                 /* the re-entrancy flag here is because, when giving the
                    button the focus, it ends up back here, which gives it the
                    focus, etc --- stack fault ---
                 */                  
                 curr_fps = wparam ;   /* make a note of current framerate */
                 setfocus = TRUE ;
                 SetFocus (GetDlgItem (hwnd, wparam)) ;
                 setfocus = FALSE ;
                 SetCursor (oldcur) ;
                 fpschanged = TRUE ;
                }
               res = TRUE ;
               break ;
                      
               /* changing the mains frequency requires switching flicker off,
                  re-priming the flicker params, and restarting flicker */
          case IDC_50HZ :
          case IDC_60HZ :
               if (wparam != hzbutton)
                {
                 FRAMERATE fps;   
                 fps = ppcGetSensorFramerate( &fps, NULL, NULL );    
                 hzchanged = TRUE;               
                 hzbutton = wparam;
                 ppcSetFlicker( FALSE );
                 ppcSetFlickerParams( wparam == IDC_50HZ ? MAINS_50HZ : MAINS_60HZ );
                 ppcSetFlicker( TRUE );
                }
               res = TRUE ;
               break ;
               
          case IDC_FLICKER :
               buts = IsDlgButtonChecked (hwnd, IDC_FLICKER) ;
               ppcSetFlicker (buts) ;
               EnableWindow (GetDlgItem (hwnd, IDC_50HZ), buts) ;
               EnableWindow (GetDlgItem (hwnd, IDC_60HZ), buts) ;
               flickchanged = !flickchanged ;
               res = TRUE ;
               break ;
               
          case IDC_CENTREWEIGHT :
               ppcSetCentreWeight (IsDlgButtonChecked (hwnd, IDC_CENTREWEIGHT)) ;
               cweightchanged = !cweightchanged ;
               res = TRUE ;
               break ;

          case IDC_PANTILT :
               switch (HIWORD (lparam)) 
                {
                 case CZN_BUTTONDOWN :
                      /* movement starting. switch compression off. There were
                         problems panning in 32 bit decode with compression on,
                         where something wasn't working and frames weren't being
                         decoded properly and artefacts were visible. To solve
                         this compression is now switched off while the pan is
                         being done */
                      /*tracemsg (TRC_MSG, "switching compression off") ;*/
                      GetPPCOptions (&comp, &vidconf, &vconfval, &vcapval, 
                                     &manualval) ;
                      SetPPCOptions (CMP_OFF, vidconf, vconfval, vcapval,
                                     manualval) ;
                      res = TRUE ;
                      break ;
                      
                 case CZN_BUTTONUP :
                      /* restore compression values */
                      /*tracemsg (TRC_MSG, "restoring compression") ;*/
                      SetPPCOptions (comp, vidconf, vconfval, vcapval,
                                     manualval) ;
                      res = TRUE ;
                      break ;     
                      
                 case CZN_MOVED :
                      tres = SendDlgItemMessage (hwnd, IDC_PANTILT, 
                                                 CZ_GETPOSITION, 0, 0L) ;
                      pan = LOWORD (tres) ;
                      tilt = HIWORD (tres) ;
                      tmp. left = pan ;
                      tmp. top = tilt ;
                      tmp. right = pan + (roi. right - roi. left) ;
                      tmp. bottom = tilt + (roi. bottom - roi. top) ;
                      ppcSetPanTiltInfo (TRUE, &tmp) ;
                      roichanged = TRUE;
                      break ;
                }
               break ;

          case IDC_ZOOM :    
               buts = IsDlgButtonChecked (hwnd, IDC_ZOOM) ;        
               EnableWindow (GetDlgItem (hwnd, IDC_PANTILT), buts) ;
               /* EnableWindow (GetDlgItem (hwnd, IDC_PANTILTGRP), buts) ; */
               zoom_changed = !zoom_changed ;
               if (buts)
                {                                                             
                 /* need to set/restore pan/tilt info */
                 tmp. left = pan ;
                 tmp. top = tilt ;
                 tmp. right = pan + (roi. right - roi. left) ;
                 tmp. bottom = tilt + (roi. bottom - roi. top) ;
                 ppcSetPanTiltInfo (TRUE, &tmp) ;
                }
               else
                {
                 /* this restores previous roi */     
                 ppcSetPanTiltInfo (FALSE, &roi) ;
                }
               break ;

          case IDCANCEL :
               if (cweightchanged)
                ppcSetCentreWeight (oldcweight) ;
                
               if (hzchanged)
                ppcSetFlickerParams (oldmf ) ;

               if (flickchanged)
                ppcSetFlicker (oldflick) ;
                
               if (fpschanged)
                {
                 oldcur = SetCursor( LoadCursor(NULL, IDC_WAIT) );
                 ppcSetSensorFramerate( old_fps, old_auto_lighting );
                 SetCursor( oldcur );
                }

               /* if OldZoom, restore previous zoom settings, else
                  set back to qcif, whole image */
               if (zoom_changed || roichanged)
                {
                  ppcSetPanTiltInfo (OldZoom, &roi) ;
                }
  
          case IDOK :
               DeleteObject (hctlcolour) ;
               EndDialog (hwnd, wparam == IDOK) ;
               GetWindowRect (hwnd, &pos) ;
   
			   KillTimer( hwnd, autolighting_timer_id );
   
               res = TRUE ;
               break ;
  
         }
        break ;	/* WM_COMMAND */

   case WM_TIMER:
		if ( curr_auto_lighting )
		{
			FRAMERATE tmp_fps;
		    ppcGetSensorFramerate( &tmp_fps, NULL, NULL );
			if ( tmp_fps != curr_fps ) 
			{
				curr_fps = tmp_fps;
		        switch( curr_fps ) 
        		{
        		case FPS_30 : wid = IDC_30FPS;	break;
        		case FPS_15 : wid = IDC_15FPS; 	break;
        		case FPS_75 : wid = IDC_7P5FPS;	 break;
        		}
		        CheckRadioButton( hwnd, IDC_30FPS, IDC_7P5FPS, wid );
			}
		}		
		break;

   case WM_ERASEBKGND:
        GetClientRect (hwnd, &rect);  
        hbr = CreateSolidBrush(GetSysColor(COLOR_BTNFACE)) ;
        FillRect ((HDC)wparam, &rect, hbr);
        DeleteObject (hbr) ;
        res = TRUE ;
        break;                    

   case WM_CTLCOLOR:
        SetBkColor((HDC) wparam, GetSysColor(COLOR_BTNFACE));
        return (int) hctlcolour ;
  }

  if ( curr_auto_lighting ) 
  {
	autolighting_timer_id = SetTimer( hwnd, ITC_AUTOLIGHTING, 1000, NULL );
  }
 
 return res ;
}                                   
