/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/* CAPTURE.H
   31-07-97
   capture commands */

 
/* macros around the GetCameraStatus call */

#define GET_SYSTEM_STATE(p, x) CPIA_GetCameraStatus (p, x, NULL, NULL, NULL, NULL, NULL, NULL, NULL)

#define GET_CMD_STATE(p, x) CPIA_GetCameraStatus (p, NULL, NULL, NULL, NULL, x, NULL, NULL, NULL)

#define GET_STREAM_STATE(p, x) CPIA_GetCameraStatus (p, NULL, NULL, x, NULL, NULL, NULL, NULL, NULL)   


#define CAP_NULL        -1   /* if supplied to most of the cap functions as
                                a parameter, will not change the existing 
                                value */

#define CAP_FMT_QCIF    0  
#define CAP_FMT_CIF     1

#define CAP_FMT_YUV420  0
#define CAP_FMT_YUV422  1
#define CAP_FMT_YUYV    0
#define CAP_FMT_UYVY    1


#define CAP_COMP_OFF    0    /* no compression */
#define CAP_COMP_AUTO   1    /* automatice compression */
#define CAP_COMP_MANUAL 2    /* use supplied compression values */

#define CAP_COMP_NODEC  0    /* no decimation */
#define CAP_COMP_VDEC   1    /* vertical decimation */

/* used by SetCompressionTarget to determine whether to favour quality or
   framerate */
#define CAP_TARGET_QUALITY 		0   /* favour quality over frame rate */
#define CAP_TARGET_FRAMERATE 	1   /* favour speed over quality */ 

#define CAP_DFLT_YTHRESH  6  /* default Y threshold */
#define CAP_DFLT_UVTHRESH 6  /* default UV threshold */


/* bit flags for what camera settings to change when a frame is being 
   uploaded */
#define CPIA_SET_FORMAT             0x0001
#define CPIA_SET_ROI                0x0002
#define CPIA_SET_COMPRESSION        0x0004
#define CPIA_SET_YUV_THRESH         0x0008
#define CPIA_SET_COMPRESSION_TARGET 0x0010
#define CPIA_SET_CAPTURE_SIZE       0x0020
#define CPIA_SET_SAMPLE_YUV         0x0040


typedef struct {
	HLOCAL HandleToThisInstance;

    PLINK pLink ;

    BOOL CameraInstalledOK;
    BOOL CamError ;

    LONG chflags ;       /* bitmap indicating which settings have changed */

    /* Capture Settings */
    BOOL CaptureSettingsChanged;

    BYTE CaptureSize ;
    BYTE ColStart ;
    BYTE ColEnd ;
    BYTE RowStart ;
    BYTE RowEnd ;
    BYTE SubSample ;
    BYTE YUVOrder ;
    BYTE CompMode ;
    BYTE Decimation ;
    BYTE YThresh ;
    BYTE UVThresh ;                      

    BYTE FRTargetting ;    	/* CPIA_SetCompression params 				*/
    BYTE TargetFR ;
    BYTE TargetQ ;          
    
	BOOL VidConf;			/* 0 = Capture Mode , 1 = Video Conf Mode 	*/		
	WORD TargetFramerate;	/* don't think this is actually still used 	*/

} CAPTURE, *PCAPTURE;


PCAPTURE capCreateCAPTURE( void );
BOOL capDestroyCAPTURE( PCAPTURE pCap );

BOOL capInitialiseCAPTURE( PCAPTURE pCap, PLINK pLink );
BOOL capDeinitialiseCAPTURE( PCAPTURE pCap );


BOOL capSetYUVThresholds( PCAPTURE pCap,
                              int YDiffThreshold, int UVDiffThreshold );
BOOL capGetYUVThresholds( PCAPTURE pCap, int *YDiffThreshold, 
                              int *UVDiffThreshold );

BOOL capSetSize( PCAPTURE pCap, BYTE CapSize );
BYTE capGetSize( PCAPTURE pCap );

BOOL capSetSampleAndYUV( PCAPTURE pCap, BYTE subsample, BYTE yuvorder );


BOOL capSetROI( PCAPTURE pCap, WORD xstart, WORD xend, WORD ystart, 
                    WORD yend );
BOOL capGetROI( PCAPTURE pCap, WORD *xstart, WORD *xend, WORD *ystart, 
                    WORD *yend );

BOOL capSetCompression( PCAPTURE pCap, int compmode, int decimation, 
                        int ythresh, int uvthresh);
BOOL capGetCompression( PCAPTURE pCap, WORD *compmode, WORD *decimation,
                        WORD *ythresh, WORD *uvthresh);

BOOL capSetDecimation( PCAPTURE pCap, BOOL decim_on);
BOOL capGetDecimation( PCAPTURE pCap);

/* function below returns TRUE if it sent a command to the camera */
BOOL capDoFrameTimeProcessing( PCAPTURE pCap, BYTE *FrameHeader, 
								WORD AppFrameRequestTime, 
								WORD PreviousFrameProcessingTime );

BOOL capSendCaptureParams( PCAPTURE pCap );
BOOL capDisableCompression( PCAPTURE pCap );


/* Private definations below here */

void capSetDefaults( PCAPTURE pCap );

BOOL capSetCompressionTarget( PCAPTURE pCap, BOOL vidconf, int confval,
                              int capval, int frate);

static void SetCompressionTarget(PCAPTURE pCap, unsigned char confmode, 
                                  unsigned int frval, unsigned int qval );

void ImproveQuality( PCAPTURE pCap, BOOL Improve );

BOOL GetCameraStatus();

