/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */



/* ====================================================================	*/
/* Definitions for Vision Packet Protocol 								*/
/* - similar to 1284.3 CPP (Chain Packet Protocol) 						*/		
/* ====================================================================	*/
#define VPP_PREAMBLE0	0xAA
#define VPP_PREAMBLE1	0x55
#define VPP_PREAMBLE2	0x00
#define VPP_PREAMBLE3	0xFF
#define VPP_PREAMBLE4	0x69
#define VPP_PREAMBLE5	0x96
#define VPP_MANUAL_CMD	0x01
#define	VPP_TERMINATOR	0xFF






/* note these two defines MUST match the info in the packet.h file */
#define CMD_DATA_LEN		4
#define DATA_LEN			8
#define SETUP_LEN			8



/* below are definitions of module identifiers */

#define CPIA_MODULE			0
#define SYSTEM_MODULE		1
#define INT_CTRL_MODULE		2
#define PROCESS_MODULE		3
#define XXX_COM_MODULE		4
#define VP_CTRL_MODULE		5
#define CAPTURE_MODULE		6
#define DEBUG_MODULE		7


#define CPIA_FLAG		(1<<CPIA_MODULE)
#define SYSTEM_FLAG		(1<<SYSTEM_MODULE)
#define INT_CTRL_FLAG	(1<<INT_CTRL_MODULE)
#define PROCESS_FLAG	(1<<PROCESS_MODULE)
#define XXX_COM_FLAG	(1<<XXX_COM_MODULE)
#define VP_CTRL_FLAG	(1<<VP_CTRL_MODULE)
#define CAPTURE_FLAG	(1<<CAPTURE_MODULE)
#define DEBUG_FLAG  	(1<<DEBUG_MODULE)

typedef struct {
	BYTE SystemState;
	BYTE VPCtrlState;
	BYTE StreamState;

	BYTE FatalError;   	/* these three fields all use the XXXX_FLAG 		*/
	BYTE CmdError;		/* #defines to identify each module					*/
	BYTE DebugFlags;	/*													*/

	BYTE VPStatus;
	BYTE ErrorCode;		/* if a module generates an error condition it may 	*/
						/* set this field to a non-zero value to indicate 	*/
						/* what has gone wrong. If this value is already 	*/
						/* non-zero, it should not be altered i.e. the first */
						/* error that was found should stay in here.		*/	
} CAMERA_STATUS;



#define MODULE_ADDR_BITS	0xE0

#define CPIA_ADDR         (CPIA_MODULE<<5)
#define SYSTEM_ADDR       (SYSTEM_MODULE<<5)
#define DEBUG_ADDR        (DEBUG_MODULE<<5)
#define SELFTEST_ADDR     (SELFTEST_MODULE<<5)  
#define XXX_COM_ADDR      (XXX_COM_MODULE<<5)
#define VP_CTRL_ADDR      (VP_CTRL_MODULE<<5) 
#define CAPTURE_ADDR      (CAPTURE_MODULE<<5) 
#define DEBUG_ADDR  	  (DEBUG_MODULE<<5)
                             



#define PROC_ADDR_BITS		0x1F
#define DEFAULT_PROC		0x00


/* ====================================	*/
/* 		CPIA Module Procedures			*/
/*										*/
#define GET_CPIA_VERSION 		0x01
#define GET_PNP_ID				0x02
#define GET_CAMERA_STATUS		0x03

#define GOTO_HI_POWER			0x04
#define GOTO_LO_POWER			0x05
#define CPIA_UNUSED_0			0x06
#define GOTO_SUSPEND			0x07
#define GOTO_PASS_THROUGH		0x08
#define ENABLE_WATCHDOG			0x09	/* not yet implemented */

#define MODIFY_CAMERA_STATUS 	0x0A
/* ====================================	*/


/* ====================================	*/
/* 		SYSTEM Module Procedures 		*/
/*										*/
#define READ_VC_REGS			0x01
#define WRITE_VC_REG			0x02
#define READ_MC_PORTS			0x03
#define WRITE_MC_PORT			0x04
#define SET_BAUD_RATE			0x05
#define SET_ECP_TIMING			0x06
#define READ_IDATA				0x07
#define WRITE_IDATA				0x08
#define GENERIC_CALL			0x09
#define SERIAL_START			0x0A
#define SERIAL_STOP				0x0B
#define SERIAL_WRITE			0x0C
#define SERIAL_READ				0x0D

/* ====================================	*/

/* ====================================	*/
/* 		VP_CTRL Module Procedures 		*/
/*										*/
#define GET_VP_VERSION			0x01
#define RESET_FRAME_COUNTER		0x02
#define SET_COLOUR_PARAMS		0x03

#define SET_EXPOSURE			0x04
#define VP_SPARE_CMD  			0x05	/* used to be SET_GAMMA */	
#define SET_COLOUR_BALANCE		0x06
#define SET_SENSOR_FPS			0x07
#define SET_VP_DEFAULTS			0x08
#define SET_APCOR				0x09
#define FLICKER_CTRL			0x0A
#define SET_VLOFFSET 			0x0B
 
#define GET_COLOUR_PARAMS		0x10
#define GET_COLOUR_BALANCE		0x11
#define GET_EXPOSURE			0x12

#define SET_SENSOR_MATRIX		0x13

#define COLOUR_BARS				0x1D
#define READ_VP_REGS			0x1E
#define WRITE_VP_REG			0x1F
/* ====================================	*/


/* ====================================	*/
/* 		CAPTURE Module Procedures 		*/
/*										*/
#define GRAB_FRAME				0x01
#define UPLOAD_FRAME			0x02
#define SET_GRAB_MODE			0x03

#define INIT_STREAM_CAP			0x04
#define FINI_STREAM_CAP			0x05

#define START_STREAM_CAP		0x06
#define STOP_STREAM_CAP			0x07 

#define SET_FORMAT				0x08
#define SET_ROI					0x09
#define SET_COMPRESSION			0x0A
#define SET_COMPRESSION_TARGET	0x0B
#define SET_YUV_THRESH			0x0C
#define SET_COMPRESSION_PARAMS  0x0D

#define DISCARD_FRAME			0x0E
#define GRAB_RESET				0x0F
/* ====================================	*/


/* ====================================	*/
/* 		DEBUG Module Procedures  		*/
/*										*/
#define OUTPUT_RS232			0x01

#define ABORT_PROCESS			0x04
#define SET_DRAM_PAGE			0x05
#define START_DRAM_UPLOAD		0x06
#define START_DUMMY_STREAM		0x08
#define ABORT_STREAM			0x09
#define DOWNLOAD_DRAM			0x0A
#define NULL_CMD				0x0B

/* ====================================	*/


typedef union {
	BYTE Raw[CMD_DATA_LEN];

	/* ================================================================	*/
	/* 				CPIA Module Command Parameters	 		  		  	*/
	/*																	*/
	struct {
		BYTE StatusByte;												
		BYTE AndMask;	/* mask of bits NOT to clear 	*/
		BYTE OrMask;	/* mask of bits to set 			*/
	} ModifyCameraStatus;												
	/* ==================================================================== */


	/* ====================================================================	*/
	/* 				SYSTEM Module Command Parameters 		  		    	*/
	/*																		*/
	struct { 
		/* READ_VC_REGS */
		BYTE Addr0;
		BYTE Addr1;
		BYTE Addr2;
		BYTE Addr3;
	} ReadVCRegs;
	struct { 
		/* WRITE_VC_REG */
		BYTE Addr;
		BYTE AndMask;
		BYTE OrMask;
	} WriteVCReg;
	struct { 
		/* WRITE_MC_PORT */
		BYTE Port;
		BYTE AndMask;
		BYTE OrMask;
	} WriteMCPort;
	struct {
		/* SET_BAUD_RATE */
 		BYTE BaudRate;		/* divided by 1200 eg 8 = 9600, 16 = 19200		*/
	} SetBaudRate;
	struct {
		/* SET_ECP_TIMING */
 		BYTE SlowTiming;	/* set to TRUE to enable slow ECP timing		*/
	} SetECPTiming;
	struct { 
		/* READ_IDATA */
		BYTE Addr0;
		BYTE Addr1;
		BYTE Addr2;
		BYTE Addr3;
	} ReadIDATA;
	struct { 
		/* WRITE_IDATA */
		BYTE Addr;
		BYTE AndMask;
		BYTE OrMask;
	} WriteIDATA;
	struct {
		/* GENERIC_CALL */
		BYTE AddrLo;
		BYTE AddrHi;
	} GenericCall;
	struct {
		/* SERIAL_WRITE */
		BYTE WriteData;
	} SerialWrite;
	struct {
		/* SERIAL_READ */
		BYTE AckRead;
	} SerialRead;
	/* ====================================================================	*/


	/* ====================================================================	*/
	/* 				VP_CTRL Module Command Parameters	 		  			*/
	/*																		*/
	struct {
		/* SET_COLOUR_PARAMS */
		BYTE Brightness;		/* range 0 -> 128, 64 nominal				*/
		BYTE Contrast;			/* range 0 -> 128, 64 nominal				*/
		BYTE Saturation;		/* range 0 -> 128, 64 nominal				*/
	} SetColourParams;
	struct {
		/* SET_EXPOSURE */
		BYTE GainMode;			/* 0 = Auto, 1 = Manual	 					*/
		BYTE ExpMode;			/* 0 = Auto, 1 = Manual	 					*/
		BYTE CompMode;   		/* 0 = Auto, 1 = Manual			 			*/
		BYTE CentreWeight;		
		/* NOTE - the accompanying DATA_BUFFER struct contains the actual 	*/
		/* values to use if a manual mode is requested.					 	*/ 
	} SetExposure;
	struct {
		/* SET_COLOUR_BALANCE */				
		BYTE BalanceMode;		/* 0 = Auto, 1 = Manual	 					*/
		BYTE RedGain;
		BYTE GreenGain;
		BYTE BlueGain;
	} SetColourBalance;
	struct {
		/* SET_SENSOR_FPS */
		BYTE SensorClkDivisor;	/* legal range 0 -> 2	 					*/
		BYTE SensorBaseRate;	/* 1 = 30fps, 0 = 25 fps 					*/
		BYTE PhaseOverride;		/* if TRUE then apply NewPhase 				*/
		BYTE NewPhase;
	} SetSensorFPS;
 	struct {
 		/* SET_APCOR */
		BYTE gain1;				/* apcor for gain 1							*/
		BYTE gain2;				/* apcor for gain 2							*/
		BYTE gain4;				/* apcor for gain 4							*/
		BYTE gain8;				/* apcor for gain 8							*/
 	} SetApcor;
	struct {
		/* FLICKER_CTRL */
		BYTE FlickerMode;
		BYTE CoarseJump;
		BYTE AllowableOverExposure;
	} FlickerCtrl;
	struct {
		/* SET_VLOFFSET */
		BYTE gain1;				// offset for gain 1
		BYTE gain2;				// offset for gain 2
		BYTE gain4;				// offset for gain 4
		BYTE gain8;				// offset for gain 8
	} SetVLOffset;
	struct {
		/* SET_SENSOR_MATRIX */
		BYTE m00;
		BYTE m01;
		BYTE m02;
		BYTE m10;
	} SetSensorMatrix;
	struct {
		/* COLOUR_BARS */
		BYTE EnableColourBars;
	} ColourBars;
	struct { 
		/* READ_VP_REGS */
		BYTE VPAddr0;
		BYTE VPAddr1;
		BYTE VPAddr2;
		BYTE VPAddr3;
	} ReadVPRegs;
	struct { 
		/* WRITE_VP_REG */
		BYTE VPAddr;
		BYTE AndMask;
		BYTE OrMask;
	} WriteVPReg;
	/* ====================================================================	*/


	/* ====================================================================	*/
	/* 				CAPTURE Module Command Parameters	 		  			*/
	/*																		*/
	struct { 
		/* GRAB_FRAME */
		BYTE Unused;
		BYTE StreamStartLine;	/* line of grab to start the image stream	*/
	} GrabFrame;
	struct { 
		/* UPLOAD_FRAME */
		BYTE ForceUpload;	/* cause the COMP_CTRL.FrameToUpload bit to be 	*/
							/* set before sytream is started, thus causing  */
							/* an immediate start to the upload independant	*/
							/* of the state of the Grab process.			*/
	} UploadFrame;
	struct {
		/* SET_GRAB_MODE */
		BYTE ContinuousGrab;
	} SetGrabMode;
	struct {
		/* INIT_STREAM_CAP */
		BYTE SkipFrames;
		BYTE StreamStartLine;	/* line of grab to start the image stream	*/
	} InitStreamCap;
	struct { 
		/* SET_FORMAT */
		BYTE VideoSize;		/* 0 = QCIF, 1 = CIF 							*/
		BYTE SubSampling;	/* 0 = 4:2:0, 1 = 4:2:2 						*/
		BYTE YUVOrder;		/* 0 = YUYV, 1 = UYVY(only in 4:2:2)			*/
	} SetFormat;
	struct { 
		/* SET_ROI */
		BYTE ColStart;		/* these are the positions divided by 4			*/	
		BYTE ColEnd;		
		BYTE RowStart;
		BYTE RowEnd;
	} SetROI;
	struct { 				
		/* SET_COMPRESSION */
		BYTE CompMode;		/* 0 = Disabled, 1 = Manual, 2 = Auto			*/

		BYTE Decimation;	/* if CompMode = 1, this enables decimation 	*/
							/* 	  CompMode = 2, this allows decimation		*/
	} SetCompression;
	struct { 				
		/* SET_COMPRESSION_TARGET */
		BYTE FRTargetting;		/* 1 = Target FR, 0 = Target Q				*/

		BYTE TargetFR;	/* framerate aimed for, or minimum fr if targetting q */
		BYTE TargetQ;	/* quality aimed for, or minimum q if tagetting FR 	*/
							
	} SetCompressionTarget;
 	struct { 
		/* SET_YUV_THRESH */
		BYTE YThresh;		/* if CompMode above is manual, then set these	*/
		BYTE UVThresh;		/* values.										*/
							/* if CompMode is auto, then these are the max	*/
							/* allowable threshold values. 					*/
	} SetYUVThresh;
	/* ====================================================================	*/


	/* ====================================================================	*/
	/* 				DEBUG Module Command Parameters	 		  				*/
	/*																		*/
	struct { 
		/* SET_DRAM_PAGE */
		BYTE StartPageLo;
		BYTE StartPageHi;
		BYTE EndPageLo;
		BYTE EndPageHi;
	} SetDRAMPage;
	/* ====================================================================	*/

} CMD_DATA;




/*																			*/
/* The union below defines the possible data formats of the 8 BYTE 			*/
/* returned data buffer.											 		*/
/* 																			*/
typedef union {

 	BYTE Raw[DATA_LEN];
			 
	union {	/* Host -> Camera Data */

		/* ================================================================	*/
		/* 				VP_CTRL Module Additional Parameters 		  	   	*/
		/*																   	*/
		struct {
			BYTE Gain;
			BYTE FineExp;		/* msb's of the 9 bit fine value		   	*/
			BYTE CoarseExpLo;
			BYTE CoarseExpHi;
			BYTE RedComp;
			BYTE Green1Comp;
			BYTE Green2Comp;
			BYTE BlueComp;
		} SetExposure;
		struct {
			/* SET_SENSOR_MATRIX */
			BYTE m11;
			BYTE m12;
			BYTE m20;
			BYTE m21;
			BYTE m22;
		} SetSensorMatrix;
		/* ================================================================	*/

		/* ================================================================	*/
		/* 				CAPTURE Module Additional Parameters 		  	   	*/
		/*																   	*/
		struct {
			BYTE Hysteresis;
			BYTE ThreshMax;
			BYTE SmallStep;
			BYTE LargeStep;
			BYTE DecimationHysteresis;
			BYTE FRDiffStepThresh;
			BYTE QDiffStepThresh;
			BYTE DecimationThreshMod;
		} SetCompressionParams;
		/* ================================================================	*/

		/* ================================================================	*/
		/* 				DEBUG Module Additional Parameters		  		   	*/
		/*																   	*/
		struct {
			BYTE Data[DATA_LEN];
		} DownloadDRAM;
		/* ================================================================	*/

	} Out;

	union {	/* Camera -> Host Data */

		/* ================================================================	*/
		/* 				CPIA Module Return Data			 		  		  	*/
		/*																	*/
		CAMERA_STATUS GetCameraStatus;	
		struct {
			BYTE FW_Version;
			BYTE FW_Revision;
			BYTE VC_Version;
			BYTE VC_Revision;
			BYTE spare[4];
		} GetCPIAVersion;
		struct {
			BYTE VendorLSB;
			BYTE VendorMSB;
			BYTE ProductLSB;
			BYTE ProductMSB;
			BYTE DeviceLSB;
			BYTE DeviceMSB;
			BYTE spare[2];
		} GetPnPID;
		/* ================================================================	*/
	
		/* ================================================================	*/
		/* 				SYSTEM Module Return Data		 		  		  	*/
		/*																	*/
		struct {
			BYTE Reg0;
			BYTE Reg1;
			BYTE Reg2;
			BYTE Reg3;
			/* maybe add in another 4 useful registers here ! */
		} ReadVCRegs;
		struct {
			BYTE Port0;
			BYTE Port1;
			BYTE Port2;
			BYTE Port3;
		} ReadMCPorts;
		struct {
			BYTE Byte0;
			BYTE Byte1;
			BYTE Byte2;
			BYTE Byte3;
		} ReadIDATA;
		struct {
			BYTE DataRead;
		} SerialRead;
		/* ================================================================	*/

		/* ================================================================	*/
		/* 				VP_CTRL Module Return Data			 		  	   	*/
		/*																   	*/
		struct {
			BYTE VP_Version;
			BYTE VP_Revision;
			BYTE CamDeviceHi;
			BYTE CamDeviceLo;
		} GetVPVersion;
		struct {
			BYTE Brightness;
			BYTE Contrast;
			BYTE Saturation;
		} GetColourParams;
		struct {
			BYTE RedGain;
			BYTE GreenGain;
			BYTE BlueGain;
		} GetColourBalance;
		struct {
			BYTE Gain;
			BYTE FineExp;		/* msb's of the 9 bit fine value		   	*/
			BYTE CoarseExpLo;
			BYTE CoarseExpHi;
			BYTE RedComp;
			BYTE Green1Comp;
			BYTE Green2Comp;
			BYTE BlueComp;
		} GetExposure;
		struct {
			BYTE VPReg0;
			BYTE VPReg1;
			BYTE VPReg2;
			BYTE VPReg3;	
			/* maybe add in another 4 useful registers here ! */
		} ReadVPRegs;
		/* ================================================================	*/

	} In;

} DATA_BUFFER;



/* ================================================================	*/
/* 	Definition of return data from GetCameraStatus command 		  	*/
/*																	*/
/*																	*/
/* definition of SystemState values */
#define UNINITIALISED_STATE	0
#define PASS_THROUGH_STATE	1
#define LOW_POWER_STATE		2
#define HI_POWER_STATE		3
#define WARM_BOOT_STATE		4


/* definition of VPCtrlState values */
#define VP_STATE_OK						0x00

#define VP_STATE_FAILED_VIDEOINIT	  	0x01
#define VP_STATE_FAILED_AECACBINIT	  	0x02
#define VP_STATE_AEC_MAX				0x04
#define VP_STATE_ACB_BMAX				0x08

#define VP_STATE_ACB_RMIN				0x10
#define VP_STATE_ACB_GMIN				0x20
#define VP_STATE_ACB_RMAX				0x40
#define VP_STATE_ACB_GMAX				0x80



/* definition of GrabState values 	*/
#define GRAB_IDLE			0
#define GRAB_ACTIVE			1
#define GRAB_DONE			2

/* definition of StreamState values */
#define STREAM_NOT_READY	0
#define STREAM_READY		1
#define STREAM_OPEN			2
#define STREAM_PAUSED		3
#define STREAM_FINISHING	4


/* definition of ErrorCode values	*/
#define ERROR_FLICKER_BELOW_MIN_EXP     0x01 /*flicker exposure got below minimum exposure */

/*																	*/
/* ================================================================	*/












 







