/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
//
//	File		cpia_api.c
//	Purpose		functions to access vvlcpia.vxd PM api functions
//
//	Created by 	S. Gillies
//	Copyright	(c) 1998 VLSI Vision Ltd, all rights reserved.
//
//	Revision history	
//		0-01	15-01-98	SG	Created
//		0-06	09-02-98	SG	#pragma pack( 1 ) for API structures
//		0-07	26-02-98	SG	API expanded to support PCI device configuration
//		0-08	27-02-98	SG	PCI dev configuration when the host knows the vendor and device ID
//		0-09	18-03-98	SG	GetResources returns an lptNo of 0 in the resources structure if
//								the port is not found
//		


//
//	System Includes
//
#include <string.h>
#include <stdlib.h>


//
//	Local Includes
//
#include "cpia_api.h"


//
//	Local declarations
//
typedef void (far *PVOIDFN)(void);
#define vxdid	0


//
//	Local data
//
static PVOIDFN pfVxDApi;
static BYTE *vxdNamePtr = (BYTE *)"VVLCPIA ";
static WORD	vxdVer;
static DMA_BUFFER_INFO vxdInfo = {0, 0, 0};
static CPIA_RESOURCE_INFO resources = {0, 0, 0, 0, 0, ""}; 
static PCI_CONFIG_REG pciConfigReg = {0, 0, 0, 0, 0};
static PCI_CONFIG_INFO pciConfigInfo = {0, 0, 0, 0};


//
//	Local function prototypes
//
PVOIDFN GetVxdApiEntry(WORD VxdId);

//
//	Exported functions
//

BOOL VVLCPIA_VXD_API_Init( void )
{          
WORD retVal;

	vxdVer = 0;
	
	pfVxDApi = GetVxdApiEntry( vxdid );
	if( !pfVxDApi )
		goto Error;

	_asm
	{
		mov ax, VVLCPIA_VXD_API_GET_VERSION
		call DWORD PTR pfVxDApi
		mov vxdVer, dx
		mov retVal, ax
	}
	if( retVal == FALSE )
		goto Error;

	return TRUE;

Error:
	return FALSE;
}


BOOL VVLCPIA_VXD_API_AllocDmaBuffer( WORD size, BYTE **bufPtrPtr, DWORD *phyAddrPtr)
{

WORD retVal, selector, myds;

	if( vxdVer < VER_ALLOC_DMA_BUFFER )
	{
		goto Error;
	}
	
	vxdInfo.bufferSize = size;

	_asm
	{
		mov ax, VVLCPIA_VXD_API_ALLOC_BUFFER
		// should we pass ds as well in case vxdInfo is not in the default DS ??
		lea si, vxdInfo
		call DWORD PTR pfVxDApi
		mov retVal, ax
	}
	if ( retVal == FALSE )
		goto Error;


	// make PM pointer with linear address returned from VxD
	_asm mov myds, ds
	selector = AllocSelector(myds);
	SetSelectorBase( selector, vxdInfo.bufferLinearAddress);
	SetSelectorLimit( selector, vxdInfo.bufferSize);
	*bufPtrPtr = MAKELP(selector, 0);

	//	place physical address into parameter
	*phyAddrPtr = vxdInfo.bufferPhys;

	return TRUE;

Error:
	return FALSE;
}


BOOL VVLCPIA_VXD_API_FreeDmaBuffer( BYTE *bufPtr )
{
WORD retVal;

	if( vxdVer < VER_FREE_DMA_BUFFER )
	{
		goto Error;
	}

	_asm
	{
		mov ax, VVLCPIA_VXD_API_FREE_BUFFER
		call DWORD PTR pfVxDApi
		mov retVal, ax
	}
	if ( retVal == FALSE )
		goto Error;

	FreeSelector( SELECTOROF( bufPtr ) );
	return TRUE;

Error:
	return FALSE;
}


BOOL VVLCPIA_VXD_API_GetResources( BYTE lptnum, PCPIA_RESOURCE_INFO resourcePtr )
{
WORD retVal;               

	if( vxdVer < VER_GET_RESOURCES )
	{
		goto Error;
	}
	
	resources.lptNo = lptnum;
	_asm
	{
		mov ax, VVLCPIA_VXD_API_GET_RESOURCES
		// should we pass ds as well in case vxdInfo is not in the default DS ??
//		lea si, vxdInfo
		lea si, resources
		call DWORD PTR pfVxDApi
		mov retVal, ax
	}
	if ( retVal == FALSE )
	{
		resourcePtr->lptNo = 0;
		resourcePtr->IOaddr = 0;
		resourcePtr->irqNo = 0;
		resourcePtr->dmaCh = 0;
		resourcePtr->ECP = 0;
		strcpy( resourcePtr->deviceDesc, "" );
    }

	resourcePtr->lptNo = resources.lptNo;
	resourcePtr->IOaddr = resources.IOaddr;
	resourcePtr->irqNo = resources.irqNo;
	resourcePtr->dmaCh = resources.dmaCh;
	resourcePtr->ECP = resources.ECP;
	strcpy( resourcePtr->deviceDesc, resources.deviceDesc );

	
	return TRUE;

Error:
	return FALSE;
}


BOOL VVLCPIA_VXD_API_ReadPCIRegister( BYTE Bus, BYTE Device, BYTE Function,
										BYTE Register, BYTE *ReadData )
{
WORD retVal;
BYTE dwordNo, byteNo;
div_t divResult;
	
	divResult = div( Register, 4 );
	dwordNo = divResult.quot;
	byteNo = divResult.rem;
	pciConfigReg.bus		= Bus;
	pciConfigReg.device		= Device;
	pciConfigReg.function	= Function;
	pciConfigReg.reg		= dwordNo;


	if( vxdVer < VER_PCI_CONFIG )
	{
		goto Error;
	}

	_asm
	{
		mov ax, VVLCPIA_VXD_API_READ_PCI_REG
		lea si, pciConfigReg
		call DWORD PTR pfVxDApi
		mov retVal, ax
	}
	if ( retVal == FALSE )
		goto Error;

	*ReadData = (BYTE)( pciConfigReg.value >> (byteNo*8) );

	return TRUE;

Error:
	return FALSE;
}


BOOL VVLCPIA_VXD_API_ConditionalWritePCIRegister( BYTE Bus, BYTE Device, BYTE Function,
										BYTE Register, BYTE ExpectedReadValue, BYTE WriteData )
{
WORD retVal;
BYTE dataByte;
BYTE dwordNo, byteNo;
div_t divResult;

	if( vxdVer < VER_PCI_CONFIG )
	{
		goto Error;
	}

	divResult = div( Register, 4 );
	dwordNo = divResult.quot;
	byteNo = divResult.rem;
	pciConfigReg.bus		= Bus;
	pciConfigReg.device		= Device;
	pciConfigReg.function	= Function;
	pciConfigReg.reg		= dwordNo;

// Critical section starts here
	_asm
	{
		mov ax, VVLCPIA_VXD_API_READ_PCI_REG
		lea si, pciConfigReg
		call DWORD PTR pfVxDApi
		mov retVal, ax
	}

	if ( retVal == FALSE )
		goto Error;

	dataByte = (BYTE)( pciConfigReg.value >> (byteNo*8) );
	
	if ( dataByte != ExpectedReadValue )
	{
		goto WrongData;
	}

	pciConfigReg.value &= ( 0xFFFFFFFF ^ ( (DWORD)0xFF << (byteNo*8) ) );
	pciConfigReg.value |= ( (DWORD)WriteData << (byteNo*8) );

	_asm
	{
		mov ax, VVLCPIA_VXD_API_WRITE_PCI_REG
		lea si, pciConfigReg
		call DWORD PTR pfVxDApi
		mov retVal, ax
	}
	if ( retVal == FALSE )
		goto Error;

// Critical section ends here

WrongData:
	return TRUE;

Error:
	return FALSE;
}


BOOL VVLCPIA_VXD_API_GetPCIcfgByte(WORD VendorId, WORD DeviceId, BYTE Register, BYTE *PreadData )
{
WORD retVal;
	
	pciConfigInfo.vendorId = VendorId;
	pciConfigInfo.deviceId = DeviceId;
	pciConfigInfo.reg = Register;


	if( vxdVer < VER_PCI_CONFIG_PNP )
	{
		goto Error;
	}

	_asm
	{
		mov ax, VVLCPIA_VXD_API_GETPCICFG
		lea si, pciConfigInfo
		call DWORD PTR pfVxDApi
		mov retVal, ax
	}
	if ( retVal == FALSE )
		goto Error;

	*PreadData = (BYTE)( pciConfigInfo.value );

	return TRUE;

Error:
	return FALSE;
}


BOOL VVLCPIA_VXD_API_SetPCIcfgByte(WORD VendorId, WORD DeviceId, BYTE Register, BYTE WriteData )
{
WORD retVal;
BYTE dataByte;

	if( vxdVer < VER_PCI_CONFIG_PNP )
	{
		goto Error;
	}

	pciConfigInfo.vendorId = VendorId;
	pciConfigInfo.deviceId = DeviceId;
	pciConfigInfo.reg = Register;

	_asm
	{
		mov ax, VVLCPIA_VXD_API_GETPCICFG
		lea si, pciConfigInfo
		call DWORD PTR pfVxDApi
		mov retVal, ax
	}

	if ( retVal == FALSE )
		goto Error;

//	dataByte = (BYTE)( pciConfigReg.value >> (byteNo*8) );
	
//	if ( dataByte != ExpectedReadValue )
//	{
//		goto WrongData;
//	}

	pciConfigInfo.value = WriteData;

	_asm
	{
		mov ax, VVLCPIA_VXD_API_SETPCICFG
		lea si, pciConfigInfo
		call DWORD PTR pfVxDApi
		mov retVal, ax
	}
	if ( retVal == FALSE )
		goto Error;

// Critical section ends here

WrongData:
	return TRUE;

Error:
	return FALSE;
}




//
//	Local functions
//
static PVOIDFN GetVxdApiEntry( WORD VxdId )
{
  PVOIDFN pfApi;

	// access vxd by name
	_asm
	{
		push di
		push es
    	mov es, WORD PTR vxdNamePtr + 2;
    	mov di, WORD PTR vxdNamePtr;
    	mov ax, 1684h
    	xor bx, bx
    	int 2fh
    	mov WORD PTR pfApi+2, es
    	mov WORD PTR pfApi, di
		pop es
		pop di
	}

/*
	// access vxd by ID
	 _asm
	 {
		push di
		push es
    	xor di, di
    	mov es, di
    	mov bx, VxdId
    	mov ax, 1684h
    	int 2fh
    	mov WORD PTR pfApi+2, es
    	mov WORD PTR pfApi, di
		pop es
		pop di
 	}
  */

  return ( pfApi );
}


