/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
//
//	File		cpia_api.h
//	Purpose		functions to access vvlcpia.vxd PM api functions
//
//	Created by 	S. Gillies
//	Copyright	(c) 1998 VLSI Vision Ltd, all rights reserved.
//
//	Revision history	
//		0-01	15-01-98	SG	Created
//		0-06	09-02-98	SG	#pragma pack( 1 ) for API structures
//		0-07	26-02-98	SG	API expanded to support PCI device configuration
//		0-08	27-02-98	SG	PCI device configuration when the host knows the vendor and device IDs
//		0-09	18-02-98	SG	Comments added to VVLCPIA_VXD_API_SetPCIcfgByte, etc. functions
//		
#ifndef _CPIA_API_H_
#define _CPIA_API_H_

//
//	System Includes
//
#include <windows.h>


//
//	Local Includes
//
#include "vvl_defs.h"
#include "api_pub.h"


//
//	Public definitions
//

#ifdef __cplusplus
extern "C" {
#endif 

//
//	Public function declarations
//

BOOL VVLCPIA_VXD_API_Init( void );
//
//	Initialises API interface, obtaining an api pointer from vvlcpia.vxd
//
//	Returns
//		TRUE if interface initialised successfully
//		FALSE if interface failed to initialise
//

BOOL VVLCPIA_VXD_API_AllocDmaBuffer( WORD size, BYTE **bufPtrPtr, DWORD *phyAddrPtr);
//
//	Allocates a fixed pagelocked physically contiguous buffer in the VxD
//	Finds both the physical address for programming the dma controller and a
//	protected mode address for accessing the data from an application
//
//	Parameters
//		size		number of bytes to allocate - 1
//		bufPtrPtr	points to a pointer to the dma buffer which can be used by an application
//		phyAddrPrt	points to the physical address of the buffer allocated
//	
//	Returns
//		TRUE	a buffer was allocated
//		FALSE	no buffer was allocated for any reason - bufPtrPtr and phyAddrPtr data is invalid
//


BOOL VVLCPIA_VXD_API_FreeDmaBuffer( BYTE *bufPtr );
//
//	Returns
//		TRUE if a buffer was freed
//		FALSE if no buffer was freed for any reason
//

BOOL VVLCPIA_VXD_API_GetResources( BYTE lptnum, CPIA_RESOURCE_INFO *resourcePtr );
//
//	Retrieves plug and play resource allocation for a specified lpt port
//
//	Parameters
//		lptnum		lpt port to retrieve resource allocation for
//		resourcePtr	points to a structure which the function will fill with
//					resource details
//	Returns
//		TRUE	resources were found
//		FALSE	no resources were found	- resourcePtr data is invalid
//


//BOOL VVLCPIA_VXD_API_FindPCIDevice( WORD Bus, WORD Device, WORD Function,
//										PPCI_RESOURCE_INFO *resourcePtr );


BOOL VVLCPIA_VXD_API_ReadPCIRegister( BYTE Bus, BYTE Device, BYTE Function,
										BYTE Register, BYTE *ReadData );
//
//	Reads a PCI register value
//
//	Parameters
//		
//		
//	Returns
//		TRUE	register was read successfully
//		FALSE	could not find the address of the requested register
//


BOOL VVLCPIA_VXD_API_ConditionalWritePCIRegister( BYTE Bus, BYTE Device, BYTE Function,
										BYTE Register, BYTE ExpectedReadValue, BYTE WriteData );
//
//	Does a conditional write to a PCI register. The write is conditional on the current
//  value in the register matches that is passed in, via the ExpectedReadValue 
//  parameter. A critical section is placed around the read/condition evaluation/write 
//  sequence to ensure we do not conflict with any other driver trying to use the
//  resource.
//
//	Parameters
//		Register 			- register offset
// 		ExpectedReadValue 	- the required value in the register that must be present 
//							  for the write operation to proceed.
//		WriteData			- data to be written to the register.
//		
//	Returns
//		TRUE	register was accessed. NOTE - does not guarantee that the write
//											  operation was actually performed.
//		FALSE	could not find the address of the requested register
//


BOOL VVLCPIA_VXD_API_GetPCIcfgByte(WORD VendorId, WORD DeviceId, BYTE Register, BYTE *ReadData );
//
//	Reads a PCI configuration register, using the VendorID and deviceID to find the correct PCI
//	configuration memory space.
//
//	Parameters
//		VendorID	vendor ID of PCI device
//		DeviceID	device ID of PCI device
//		Register	register index of data to retrieve
//		ReadData	pointer to data read from PCI configuration memory space
//
//	Returns
//		TRUE		Got data
//		FALSE		Could not get data ( wrong VxD version OR device not found )
//


BOOL VVLCPIA_VXD_API_SetPCIcfgByte(WORD VendorId, WORD DeviceId, BYTE Register, BYTE WriteData );
//
//	Writes to a PCI configuration register, using the VendorID and deviceID to find the correct
//	PCI configuration memory space.
//
//	Parameters
//		VendorID	vendor ID of PCI device
//		DeviceID	device ID of PCI device
//		Register	register index to write to
//		WriteData	data to write to PCI configuration memory
//
//	Returns
//		TRUE		Set data successfully
//		FALSE		Could not set data ( wrong VxD version OR device not found )
//


#ifdef __cplusplus
}
#endif 


#endif 
