/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/* VVLCPIA.C
   16-07-97
   camera information and control functions
*/

#include "warning.h"
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <toolhelp.h>
#include <mmsystem.h>

#pragma hdrstop

#ifndef PPLINK
#define PPLINK
#endif

#include "vvl_type.h"     
#include "camtypes.h"

#include "cmd.h"  
#include "cpia_vid.h"
   
#include "pport.h"
#include "pplink.h"
#include "cpialink.h"
#include "cpia_cmd.h"

#include "capture.h"   

#include "debuglvl.h"
#include "tracemsg.h"

#include "cpia_cam.h"


static int cmd_check( PCPIA_CAM pcam, LPSTR func);

static POWER_MODE PowerStateToMode( BYTE PowerState );
static BYTE PowerModeToState( POWER_MODE PowerMode );
static BYTE mCalculateOverExp( BYTE Brightness, BYTE FlickerExp, BYTE FlickerMax );


#define MB(x) MessageBox (NULL, x, "CPIA_CAM", MB_OK) 


#define COMMS_TEST_BUFFER_SIZE		((DWORD)1024*16)


/*-----------------------------------------------------------------*/

PCPIA_CAM camCreateCPIA_CAM( int ID, BYTE SerialAddr, PLINK plink )
{
	HLOCAL hcam;
 	PCPIA_CAM pcam = NULL;

	hcam = LocalAlloc( LPTR, sizeof( CPIA_CAM ) );
 	if ( hcam )
	{
 		pcam = (PCPIA_CAM)LocalLock( hcam );
		pcam->HandleToThisInstance = hcam;
 		if ( pcam == NULL )
    	{
     		/* unable to allocate/lock the cpia struct */
			if ( hcam )
      		{
       			LocalUnlock( hcam );
       			LocalFree( hcam );
      		}                  
     		pcam = NULL;
    	}
	}

	if ( pcam ) {
		/* do initialisation of member ariables here */
	}
	                      
 	return pcam;
}

BOOL camDestroyCPIA_CAM( PCPIA_CAM pCam )
{
	HLOCAL hcam;

	hcam = pCam->HandleToThisInstance;
	if ( hcam )
	{
		LocalUnlock( hcam );
		LocalFree( hcam );
	}                  

  	return TRUE;
}

/*-----------------------------------------------------------------*/
/* set default values for the camera settings.
   Hopefully, most of these can be read from the camera, but the functions
   aren't there yet */

BOOL camInitialiseCPIA_CAM( PCPIA_CAM pCam, PLINK plink )
{    
	BOOL rtn;              
	BYTE sys_state,
    	fw_version,
      	fw_rev,
      	vc_version,
      	vc_rev;

 	pCam->pLink = plink;
   
  	/* 
  		read the version number of the firmware.This is used by some routines
    	to modify their behaviour - mainly flicker control, which is monitored
    	by the driver at 1-02, but auto in firmware from 1-06 
    */
 	if ( !CPIA_GetVersion( pCam->pLink, &fw_version, &fw_rev, &vc_version, &vc_rev ) )
 		goto camInitFailed;

 	pCam->FwVersion = FW_VER( fw_version, fw_rev );

	/* check the version number is something sensible			*/


 	/* 
 		read the system state off the camera.Will be one of UNINITIALISED,
    	LO power or HI power 
    */ 
 	if ( !GET_SYSTEM_STATE( pCam->pLink, &sys_state ) )
		goto camInitFailed;

	if ( PowerStateToMode( sys_state ) != PWR_LOW ) 
		if ( !CPIA_GotoLoPower( pCam->pLink ) )
			goto camInitFailed;

 	if ( !GET_SYSTEM_STATE( pCam->pLink, &sys_state ) )
		goto camInitFailed;

	pCam->PowerMode = PowerStateToMode( sys_state );


	if ( pCam->FwVersion == FW_VER102 ) 
	{  /* 	
 			On the PPC2 Ver 1-02, parallel port only, the 48Mhz crystal is not
    		populated on the PCB. On power-up, or after a watchdog reset, the 
    		firmware detects this and sets two flag bits in the FatalError field of 
    		the camera status structure.This command resets these two bits. 
    	*/
		CPIA_ModifyCameraStatus( pCam->pLink, 3, (BYTE) ~(XXX_COM_FLAG | CPIA_FLAG), 0 );

		/*
			Also in Ver 1-02, the StreamState is not correctly reset by a
			Hi->Lo->Hi Power sequence. So if the driver crashed part way
			through a transfer it could leave the camera in a bad state.
			This command clears down the StreamState variable.
		*/
		CPIA_ModifyCameraStatus( pCam->pLink, 2, 0, 0 );
	}

 	pCam->bPassthrough = FALSE;
 	pCam->error = FALSE;
	pCam->EnableButtonChecking = FALSE;
	pCam->ButtonState = FALSE;

	pCam->cp.Brightness = 50;
	pCam->cp.Contrast = 50;
	pCam->cp.Saturation = 50;

    return TRUE;

camInitFailed:

	return FALSE;
}    


/*-----------------------------------------------------------------*/
/* put the camera into low power mode, then passthrough */

BOOL camDeinitialiseCPIA_CAM( PCPIA_CAM pCam )
{
 	CPIA_GotoLoPower( pCam->pLink );
                  
#ifdef NEVER
	/* code to put dongle into passthough mode - removed 28-10-97 */ 
	if ( pCam->FwVersion < FW_VER106 ) /* do we need to disable watchdog ? */
 		CPIA_WriteVCReg( pCam->pLink, 0, 0x7F, 0x80 );
  	CPIA_GotoPassThrough( pCam->pLink); 
#endif
    
 	return TRUE;
}


/* --------------------------------------------------------------------	*/
/* check that we can stream data correctly in the current transfer mode */
/* --------------------------------------------------------------------	*/
BOOL camDoHiPowerCommsTest( PCPIA_CAM pCam ) {

	BOOL rtn, upload_ok = TRUE;
	BYTE val;
	int count;
	DWORD pos;
	HGLOBAL	hbuffer;
	BYTE *pbuffer;

	hbuffer = GlobalAlloc( LPTR, COMMS_TEST_BUFFER_SIZE );
	if ( !hbuffer ) 
	{
		return( FALSE );
	}

	pbuffer = (BYTE *)GlobalLock( hbuffer );
	if ( !pbuffer )
	{
		GlobalFree( hbuffer );
		return( FALSE );
	}

	memset( pbuffer, COMMS_TEST_BUFFER_SIZE, 0xFE );
	 
	if ( !CPIA_MACRO_TestStream( pCam->pLink, 0, COMMS_TEST_BUFFER_SIZE/256, pbuffer ) )
	{
//	   	if ( IMSET() ) /* INITPPC Message */
	   		tracemsg( TRC_MSG, "CPIA_MACRO_TestStream failed\n" );
		upload_ok = FALSE;
	}
	else 
	{
		val = 0; count = 0;
		for( pos = 0; pos < COMMS_TEST_BUFFER_SIZE; pos++ ) {
			if ( pbuffer[ pos ] != val ) {
//				printf( "UploadTestData: Buffer at %lu   %d != %d\n", pos, Buffer[ pos ], val );
//			   	if ( IMSET() ) /* INITPPC Message */
	   			{
					char buf[128];
					wsprintf( buf, "UploadTestData: Buffer at %lu   %d != %d\n", pos, pbuffer[ pos ], val );
	   				tracemsg( TRC_MSG, buf );
				}
				
				count++;
				if ( count > 4 ) break;
				upload_ok = FALSE;
			}
			val++;
		}
	}
	rtn = CPIA_ModifyCameraStatus( pCam->pLink, 2, 0, 0 );

    cmd_check( pCam, "TestStream failed\n" );
    
 	if ( pbuffer ) GlobalUnlock( hbuffer );
	if ( hbuffer ) GlobalFree( hbuffer );

	/* need to do something here to reset the stream state on the camera	*/

 	return( upload_ok );
}




/* --------------------------------------------------------------------	*/
/* do all the initialisation tasks for the VP_CTRL mmodule				*/
/* --------------------------------------------------------------------	*/
BOOL camInitVP( PCPIA_CAM pCam ) 
{
	int res;

	if ( camGetPower( pCam ) != PWR_HI ) return( FALSE );
	
	/* set sensor framerate and mode */ 
 	pCam->Framerate = FPS_30;   /* camera starts in 30fps mode */
	pCam->DoAutoFramerate = FALSE;

 	/* start in auto exposure mode */
 	pCam->exp.ExpMode = EXP_AUTOMATIC;         
 	pCam->exp.CentreWeight = TRUE;
 	pCam->flick.FlickerCtrl = FALSE;

 	camSetFlickerThreshold( pCam, 250, 146 );
 	camSetFlickerParams( pCam, MAINS_50HZ );

   	camGetGain( pCam);
	pCam->cb.BalanceMode = CB_AUTOMATIC;
   	camGetColBal( pCam, NULL, NULL, NULL, NULL);
   	camGetColPar( pCam, NULL, NULL, NULL);


   	/* default colour compensation values */
   	pCam->exp.RedComp = COMP_RED;
   	pCam->exp.Green1Comp = COMP_GREEN1;
   	pCam->exp.Green2Comp = COMP_GREEN2;
   	pCam->exp.BlueComp = COMP_BLUE;
              
   	CPIA_SetVLOffset( pCam->pLink, 20, 24, 26, 26);
 
   	/* set gain ceiling to 2 (doesn't have the desired effect with 1-02
      firmware due to f/ware bug.This is handled in camSetGain) */
   	camSetGainCeiling( pCam, DEFAULT_GAIN_CEILING );

   	/* tweak the aperture and sharpness parameters */
   	CPIA_SetApcor( pCam->pLink, 0x18, 0x16, 0x24, 0x34 );              /* 0-13 */

   	res = cmd_check( pCam, "camera initialisation");

 	return res == OK;
}


BOOL camGetSensorID( PCPIA_CAM pCam, WORD *SensorID, BYTE *Revision ) 
{
	CPIA_GetVPVersion( pCam->pLink, NULL, NULL,
							&(pCam->SensorID), &(pCam->SensorRevision) );
	if ( SensorID ) *SensorID = (pCam->SensorID<<4) + (pCam->SensorRevision>>4);
	if ( Revision ) *Revision = pCam->SensorRevision & 0x0F;
	return( TRUE );
}


/* --------------------------------------------------------------------	*/
/* 	camDoFrameTimeProcessing											*/
/* --------------------------------------------------------------------	*/
/* function below returns TRUE if it sent a command to the 		*/
/* camera, it is called each frame by the GETFRAME module.		*/
BOOL camDoFrameTimeProcessing( PCPIA_CAM pCam, BYTE *pFrameHeader )
{
	BOOL cmd_sent = FALSE;
  	FLK_STATE flkstate = FLK_ON;  /* default to on, so that exposure stored */
 	WORD newexposure;
    static BOOL OldButtonChecking = FALSE;

    if ( pCam->EnableButtonChecking )
    {
        BOOL rtn;
        BYTE p1;
        rtn = CPIA_ReadMCPorts( pCam->pLink, NULL, &p1, NULL, NULL );
        if ( rtn ) 
        {
            if ( !(p1 & BUTTON_INPUT_MASK) ) /* button press pulls the input pin LO */
            {
                /* the first time that we check the button after button checking has been   */
                /* enabled we should disregard the button state, in case the latch had been */
                /* set before button checking was enabled.                                  */
                if ( pCam->EnableButtonChecking && OldButtonChecking ) 
                {
                    pCam->ButtonState = TRUE; 
                }

                /* now blip the BUTTON_RESET line LO to reset the button flip-flop  */
                rtn |= CPIA_WriteMCPort( pCam->pLink, BUTTON_RESET_PORT, (BYTE)~BUTTON_RESET_MASK, (BYTE)~BUTTON_RESET_MASK );
                rtn |= CPIA_WriteMCPort( pCam->pLink, BUTTON_RESET_PORT, 0xFF, 0xFF );
            }
        }
        cmd_sent = TRUE;
        if ( !rtn ) cmd_check( pCam, "CPIA_WriteMC_Port" ); /* clear down any error */
    }
    OldButtonChecking = pCam->EnableButtonChecking; 


 	if ( pCam->FwVersion <= FW_VER102 ) 
	{
		/* NOTE - in FW_VER102 if antiflicker is running then the current		*/
		/* exposure is placed in the CV_ERROR_CODE field of the frame header	*/
		
		if ( pCam->flick.FlickerCtrl && pFrameHeader[CV_ERROR_CODE] )
		{ /* 
			this handles flicker control, switching it back on
            if it gets switched off
            This also needs to be done in getreferenceframe, and anywhere
            else that uploads an image.
            Exposure is supplied divided by 2, so multiply back up 
		  */
            camRestartFlicker( pCam, pFrameHeader[CV_ERROR_CODE]<<1, 
                                pCam->LastExposure, &flkstate,
                                &newexposure );

//			if ( flkstate == FLK_RESTART )
//	 			tracemsg( TRC_MSG, "Anti Flicker restarted" ); 

			cmd_sent |= TRUE;
  		}
 		/* 
 			store last exposure setting read. If flicker has just been
           	restarted, the exposure has been changed, so store the value
           	that has been written to the camera 
        */
     	if ( flkstate == FLK_RESTART )
        	pCam->LastExposure = newexposure;
        else
        	pCam->LastExposure = pFrameHeader[CV_ERROR_CODE]<<1;
	}

	if ( pCam->DoAutoFramerate ) 
	{
		WORD newExp = 0;
 		FRAMERATE newrate = FPS_UNKNOWN;

		cmd_sent |= camImageSourceSupervisor( pCam, (WORD)(pFrameHeader[CV_ERROR_CODE]<<1), &newExp, &newrate );
 	}

	return( cmd_sent );
}



/*-----------------------------------------------------------------*/
/* enable/disable the auto exposure on the camera.
   cpia seems to run with auto exposure enabled...*/

int camSetAutoExposure( PCPIA_CAM pCam, BOOL autoexp )
{
 	int res = ERR;
 
 	if ( !pCam->error )
  	{
   		if ( CPIA_SetExposure( pCam->pLink, 0, 
        		                 autoexp ? EXP_AUTOMATIC : EXP_AUTODISABLE,
                		         0, 0, 0, 0, 0, 0, 0, 0, 0, 0) )
    	{     
     		if ( (res = cmd_check( pCam, "camSetAutoExposure()") ) == OK )
      			pCam->exp.ExpMode = autoexp ? EXP_AUTOMATIC : EXP_AUTODISABLE;
    	}
  	}  
    
 return res;
}

/*-----------------------------------------------------------------*/
/* return TRUE if camera running auto exposure */

BOOL camGetAutoExposure( PCPIA_CAM pCam )
{
 	return pCam->exp.ExpMode == EXP_AUTOMATIC;
}


/*-----------------------------------------------------------------*/
/* pCam runs auto gain...
   This calls set exposure, so needs to set all the other values as well */

int camSetAutoGain( PCPIA_CAM pCam, int Enable )
{
 	return camSetAutoExposure( pCam, Enable );
}


/*-----------------------------------------------------------------*/
/* set the gain ceiling on the sensor to a specific value

   VERSION_NOTE - 1.02 firmware sets the gain ceiling from the first parameter, 
                  rather than the fifth.As the first must be 0 or 1, the gain 
                  ceiling is effectively limited to 1.
                  The call will fail if the first parameter isn't 0 or 1
                  
                  1.06 firmware clears this bug.
*/
int camSetGainCeiling( PCPIA_CAM pCam, int Gain )
{
 	int res = ERR;

 	if ( !pCam->error )
  	{                                                 
   		if ( pCam->FwVersion > FW_VER102 )
		{
			/* actual value to send is = required_gain + 1 */
			Gain++;	
		}
		else
		{
	   		/* if running on 1.02 firmware then only send 		*/
	   		/* gain ceiling if x1 or x2 was requested 			*/
   			if ( Gain != 0 || Gain != 1 ) 
   				return( OK );
   			else 
   				Gain = 1;	 
   		}

   		if ( CPIA_SetExposure( pCam->pLink, Gain,
                         EXP_IGNORE, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ) )
    	{     
	     	if ( (res = cmd_check(pCam, "camSetGain()")) == OK )
    	  	{
       			pCam->exp.GainMode = GAIN_CEILING;
       			pCam->GainCeiling = Gain;
	      	}
    	}
  	}  
 	return res;
}

/*-----------------------------------------------------------------*/
/* read the gain setting from the camera */
   
int camGetGain( PCPIA_CAM pCam )
{
 	int res = ERR;
 
 	if ( CPIA_GetExposure(	pCam->pLink, 
         		            &pCam->exp.Gain, 
                		    &pCam->exp.FineExp,
                       		&pCam->exp.CoarseExpLo,
                       		&pCam->exp.CoarseExpHi,
                       		&pCam->exp.RedComp,
                       		&pCam->exp.Green1Comp,
                       		&pCam->exp.Green2Comp,
                       		&pCam->exp.BlueComp) )
  	{
  		res = (cmd_check (pCam, "camGetGain()") == OK);
 	}

 	return ( ( res == OK ) ? pCam->exp.Gain : ERR );
}



/*-----------------------------------------------------------------*/
/* change the power mode of the camera.

   state information needs to be saved and restored across a power
   transition.Currently, this will be the responsibility of the top layers,
   but it could equally be done here */

int camSetPower( PCPIA_CAM pCam, POWER_MODE power )
{
 	int res = FALSE;
 	BYTE cmd, sys, ftl;

 	if ( !pCam->error && power != pCam->PowerMode )
  	{
   		switch (power)
    	{
     	case PWR_SUSPEND :   
          		if ((res = CPIA_GotoSuspend (pCam->pLink)) == TRUE)
           			if (cmd_check (pCam, "GotoSuspend") == OK)
            		{
             			res = TRUE;
             			pCam->PowerMode = power;
            		}
          		break;
        
     	case PWR_LOW :
                Delay( 20 );
          		if ((res = CPIA_GotoLoPower( pCam->pLink) ) == TRUE)
           			if (cmd_check (pCam, "GotoLoPower") == OK)
            		{
             			res = TRUE;
             			pCam->PowerMode = power;
            		}
          		break;
        
     	case PWR_HI :
          		if ((res = CPIA_GotoHiPower( pCam->pLink) ) == TRUE)
           		{
                    Delay( 40 );
                    if (cmd_check (pCam, "GotoHiPower") == OK)
             		{                                      
              			res = TRUE;
              			pCam->PowerMode = power;                        
						pCam->StartTime = timeGetTime();
            
			            if (!pCam->error)
			            {
			                /* colour balance */ 
			                res = camSetColBal( pCam, CB_AUTOMATIC, 0, 0, 0);    /* switch on auto colour balance */
			            }
			             
			            /* read the settings and check the camera is ok */
						if ((res = CPIA_GetCameraStatus( pCam->pLink, &sys, NULL, 
			                                               NULL, &ftl, &cmd,
			                                               NULL, NULL, NULL) ) == TRUE)
			            {
			                res = (sys == PWR_HI) && (ftl == 0) && (cmd == 0);
			            }
			            res = TRUE;
             		}
           		}
          		break;
        
     	default :
          		res = FALSE;
          		break;
    	}
  	}
 
 	return res ? OK : ERR; 
}

/*-----------------------------------------------------------------*/
/* return the current power state of the camera */

POWER_MODE camGetPower( PCPIA_CAM pCam )
{
  	return pCam->PowerMode; 
}

/*-----------------------------------------------------------------*/
/* set the camera into passthrough mode or normal mode */

int camSetPass( PCPIA_CAM pCam, BOOL pthrough )
{
 	int res = OK;
  
 	if (!pCam->error && pthrough != pCam->bPassthrough)   /* want mode change */
  	{
   		if (pthrough)
    	{
/*     		if ((res = CPIA_GotoPassThrough (pCam->pLink)) == TRUE)*/
      		pCam->bPassthrough = TRUE;
    	}
   		else
    	{
     		/* send a Vision Packet Protocol packet to the camera to switch it
        		back */
     		res = OK;   /* there will be a function call here */
     		pCam->bPassthrough = FALSE;
    	}
  	}
  
 return res;
}

/*-----------------------------------------------------------------*/
/* return whether or not the camera is currently in passthrough mode */

int camGetPass( PCPIA_CAM pCam )
{
 return pCam->bPassthrough;
}

/*-----------------------------------------------------------------*/
/* implements the SetColourBalance command

   BYTE balmode   - colour balance mode
                      1 - uses RedGain etc to set the colour balance manually
                      2 - switches auto colour balance on
                      3 - disables auto colour balance
   BYTE RedGain   - red gain in range 0...100  (only used in mode 1)
   BYTE BlueGain  - blue gain in range 0...100 ("")
   BYTE GreenGain - green gain in range 0..100 ("")
   
   return
    OK  - change accepted
    ERR - error making change
*/   
   
int camSetColBal( PCPIA_CAM pCam, COL_BALANCE balmode, BYTE RedGain,
                  BYTE GreenGain, BYTE BlueGain )
{
 	int res = ERR;
 
 	/* these values are stored whether or not the command succeeds, so that,
    	if there is an error, sensible values will be returned on camera exit */
 	pCam->cb.BalanceMode = balmode;
 	if (balmode == CB_MANUAL)
  	{
   		pCam->cb.RedGain = RedGain;
   		pCam->cb.GreenGain = GreenGain;
   		pCam->cb.BlueGain = BlueGain;
  	}
  
 	if ( !pCam->error )
	{
 //		tracemsg(TRC_MSG,"SetColourBalance:%d %d %d %d", (BYTE) balmode, RedGain, GreenGain, BlueGain );
  		if ( CPIA_SetColourBalance (pCam->pLink, (BYTE) balmode, RedGain, 
                             GreenGain, BlueGain) )
		{
			res = cmd_check (pCam, "camSetColBal() : cmd fail");
  		}
	}
 	return res;
}

/*-----------------------------------------------------------------*/
/* read the colour balance details from the camera.
   If there has been a camera error the previous values are returned,
   although no new ones are retrieved.
   The function returns ERR if the camera call fails
*/
   
int camGetColBal( PCPIA_CAM pCam, COL_BALANCE *pBalanceMode, BYTE *pRedGain, BYTE *pGreenGain,
                  BYTE *pBlueGain )
{    
 	int res = ERR;
 
 	if (!pCam->error)
  		if (CPIA_GetColourBalance( pCam->pLink, 
        		                     &pCam->cb.RedGain, 
                		             &pCam->cb.GreenGain,
                        		     &pCam->cb.BlueGain ) )
   		{
   			res = cmd_check (pCam, "camGetColBal() : cmd fail");
   		}


 	if (pRedGain)
  		*pRedGain = pCam->cb.RedGain;
  
 	if (pGreenGain)
  		*pGreenGain = pCam->cb.GreenGain;
  
 	if (pBlueGain)
  		*pBlueGain = pCam->cb.BlueGain;
  
 	if (pBalanceMode)
  		*pBalanceMode = pCam->cb.BalanceMode;

	return res;
}

/*-----------------------------------------------------------------*/
/* Set the colour parameters on the camera.
   when the getXXX function is available, these will no longer be stored,
   but will be retrieved from the camera each call */

int camSetColPar( PCPIA_CAM pCam, BYTE Brightness, BYTE Contrast,
                 BYTE Saturation)
{
    int res = ERR,
        ContrastConst1,
        ContrastConst2;
    BYTE cmd;
	BOOL newBrightnessReqd;
	short m00, m01, m02;
	BYTE vpStatus;
	COL_BALANCE oldCBMode;
	BOOL aecEnabled;
	DWORD currentTime;
	BYTE camStatus_reg56;
	BYTE count;
	DWORD appliedBrightness;
    
	if ( Brightness != pCam->cp.Brightness )
	{
		newBrightnessReqd = TRUE;
	}
	else
	{
		newBrightnessReqd = FALSE;
	}

    /* values stored whether or not written to the camera */
    pCam->cp.Brightness = Brightness;
    pCam->cp.Contrast = Contrast;
    pCam->cp.Saturation = Saturation;     /* colour */
    
    if (pCam->FwVersion == FW_VER102)
    {
        ContrastConst1 = 28;
        ContrastConst2 = 14;
    }
    else
    {
        ContrastConst1 = 12;
        ContrastConst2 = 12;
    }
    
	if ( Brightness > 50 )
	{
		appliedBrightness = Brightness;
	}
	else
	{
		appliedBrightness = (Brightness>>1) + 25;
	}
    if (!pCam->error)
    {
		if ( newBrightnessReqd )
		{
			m00 = (66 * appliedBrightness);
			m01 = (129 * appliedBrightness);
			m02 = (25 * appliedBrightness);

			m00 = (m00 ? m00 / 50 : 0 );
			m01 = (m01 ? m01 / 50 : 0 );
			m02 = (m02 ? m02 / 50 : 0 );

			m00 = m00 - 64;
			m01 = m01 - 64;
			m02 = m02 - 64;

			m00 = ( m00 > 127 ? 127 : m00 );
			m01 = ( m01 > 127 ? 127 : m01 );
			m02 = ( m02 > 127 ? 127 : m02 );

			camGetColBal( pCam, &oldCBMode, 0, 0, 0 );
			aecEnabled = camGetAutoExposure( pCam );
			camSetColBal( pCam, CB_DISABLE, 0, 0, 0 );
			camSetAutoExposure( pCam, EXP_AUTODISABLE );
/*
			{
				// For debug only
				char buf[40];
				wsprintf( buf, "m00=%d, m01=%d, m02=%d, gain=%d", m00, m01, m02 );
				MessageBox(NULL, buf, "CPIA_CAM", MB_OK); 
			}
*/

			// Ensure fw has had time to write all bunches since booting
			// No need to wait for 6 frames since framerate changing since 
			// framerate changing function does 6 frames delay before letting go
			if ( ( currentTime = timeGetTime() ) <  ( pCam->StartTime + 350 ) )
			{
				Delay( pCam->StartTime + 350 - currentTime );
			}

			// using 1 frames delay
			
			count = 0;
			do {
				switch ( pCam->Framerate )
				{
				case FPS_30 :   Delay( 1 * 33 ); break;
				case FPS_25 :   Delay( 1 * 40 ); break;                 
				case FPS_15 :	Delay( 1 * 66 ); break;
				case FPS_125 :	Delay( 1 * 80 ); break;
				case FPS_75 :	Delay( 1 * 132 ); break;
				default :
					res = ERR;
					break;
				}			
				if ( !CPIA_ReadVPRegs( pCam->pLink, 56, 0, 0, 0, &camStatus_reg56, NULL, NULL, NULL ) )
				{
					camStatus_reg56 = 0x40;	// assume VP is busy
				}
			} while( (camStatus_reg56 & 0x40) && (count++ < 4) ); 

			// only write if vpbusy is low, ignore otherwise
			if ( !(camStatus_reg56 & 0x40) )
			{
				CPIA_WriteVPReg( pCam->pLink, 40, 0xFD, 0 );
				CPIA_WriteVPReg( pCam->pLink, 10, 0, (BYTE)m00 );
				CPIA_WriteVPReg( pCam->pLink, 11, 0, (BYTE)m01 );
				CPIA_WriteVPReg( pCam->pLink, 12, 0, (BYTE)m02 );
				CPIA_WriteVPReg( pCam->pLink, 63, 0, 0xFF );
			}

			// using 2 frames delay and may get away with less!
			// don't need at all because AEC and ACB are frame synchronised
			// and check VPBUSY before doing anything
/*
			switch ( pCam->Framerate )
    		{
     		case FPS_30 :   Delay( 2 * 33 ); break;
			case FPS_25 :   Delay( 2 * 40 ); break;                 
			case FPS_15 :	Delay( 2 * 66 ); break;
			case FPS_125 :	Delay( 2 * 80 ); break;
			case FPS_75 :	Delay( 2 * 132 ); break;
			default :
	    		res = ERR;
				break;
			}
*/
			// Set AEC and ACB modes back to previous states
			if ( oldCBMode != CB_DISABLE )
			{
				camSetColBal( pCam, oldCBMode, 0, 0, 0 );
			}
			if ( aecEnabled )
			{
				camSetAutoExposure( pCam, EXP_AUTOMATIC );
			}

		}

		if ( appliedBrightness < 50 )
		{
			Saturation = (Saturation * appliedBrightness) / 50;
		}

        if (Contrast > ContrastConst1)
        {
            if (CPIA_SetColourParams (pCam->pLink, 50, 
                Contrast - ContrastConst1,
                Saturation))
                res = cmd_check (pCam, "camSetColPar");
        }
        else
        {
            BYTE Gamma;
            
            Gamma = ContrastConst2 - (Contrast + 4) / 8;
            if (CPIA_SetColourParams (pCam->pLink, 50, 0, Saturation))
                res = cmd_check (pCam, "camSetColPar");
            CPIA_WriteVPReg (pCam->pLink, 20, 0, Gamma);
        }

    }

/* No longer needed now target exposure is fixed
    pCam->flick.OverExposure = mCalculateOverExp( Brightness, pCam->flick.FlickExp, pCam->flick.FlickMax );

    if ( pCam->flick.FlickerCtrl == FLK_ON )
    {
        // Call SetFlicker here if Flick is on to force down new values.....
        //
        res = camSetFlicker( pCam, pCam->flick.FlickerCtrl, 0 ); // 0== read back current exp from camera
    }
*/
    return res;
}


/*-----------------------------------------------------------------*/
/* Get the current colour parameters on the camera.
   Contrast is stored on the camera as a multiple of 8, so setting 50,
   then reading back, gives 48.to avoid confusion, the camera setting
   isn't used
*/

int camGetColPar( PCPIA_CAM pCam, BYTE *pBrightness, BYTE *pContrast,
                  BYTE *pSaturation)
{
 int res = ERR;
 BYTE tmp, dummyBrightness;
  
 /* if there has been a camera error, the current settings are returned,
    if there hasn't been an error, the values are retrieved from the camera
 */
 
 /* use driver values

 if (!pCam->error)
  if (CPIA_GetColourParams (pCam->pLink, 
//                            &pCam->cp.Brightness, 
							&dummyBrightness,		// do not return exposure brightness
                            &tmp, 
                            &pCam->cp.Saturation))
   res = cmd_check (pCam, "camGetColPar()"); 
  */
 if (pBrightness)
  *pBrightness = pCam->cp.Brightness;

 if (pContrast) 
  *pContrast = pCam->cp.Contrast;
  
 if (pSaturation)
  *pSaturation = pCam->cp.Saturation;

 return res;
}



/*-----------------------------------------------------------------*/
/* set sensor fps.This sets one of 4 predefined framerates.
   the 600ms delay is 4 frames at 7.5 fps  
*/
int camSetSensorFramerate( PCPIA_CAM pCam, FRAMERATE Framerate, BOOL AutoFramerate )
{
 	int res = OK;
 	BYTE clk_div, baserate;
 	BOOL autoexp, congrab, autoacb;
 	FLK_STATE fstate;
	BYTE status;
	
	if ( pCam->DoAutoFramerate != AutoFramerate )
	{
		pCam->DoAutoFramerate = AutoFramerate;
		return OK;
	}

 	if ( Framerate != pCam->Framerate )
  	{      
		/* framerate change bit flakey, so switch off auto exposure, wait a while,
		   make the rate change, then switch auto exposure back on */

		autoexp = camGetAutoExposure( pCam );
		camSetAutoExposure( pCam, FALSE );
		 
		autoacb = ( pCam->cb.BalanceMode != CB_DISABLE );
		camSetColBal( pCam, CB_DISABLE, 0, 0, 0 );

		/* 
			do a delay based on the current framerate to ensure 
            that all of the comms to the sensor will have finished
		*/
   		switch ( pCam->Framerate )
    	{										   /* + 10 just for safety ! */
     	case FPS_30 :   Delay( 4 * 33 + 10 ); break;
	    case FPS_25 :   Delay( 4 * 40 + 10 ); break;                 
	    case FPS_15 :	Delay( 4 * 66 + 10 ); break;
	    case FPS_125 :	Delay( 4 * 80 + 10 ); break;
	    case FPS_75 :	Delay( 4 * 132 + 10 ); break;
	    default :
	    	res = ERR;
	        break;
	    }                

		/* 
			work out new clk_div and baserate values
        */
        switch ( Framerate )
    	{												   
     	case FPS_30 : 	clk_div = 0;	baserate = 1; break;
	    case FPS_25 :	clk_div = 0;	baserate = 0; break;                 
	    case FPS_15 :	clk_div = 1;	baserate = 1; break;
	    case FPS_125 :	clk_div = 1;	baserate = 0; break;
	    case FPS_75 :	clk_div = 2;	baserate = 1; break;
	    default :
    	    clk_div = 0;	baserate = 1; 
	    	res = ERR;
	        break;
	    }                

		if ( (res != ERR) && !CPIA_SetSensorFPS( pCam->pLink, clk_div, baserate ) )
		{
			res = ERR;
		}
		else
		{
			/* wait 4 frames at target framerate + 1 frame to ensure we have a whole */
			/* good frame in memory before going on .. +10 just for safety !		 */
			/* + 1 frame to ensure AEC/ACB/Colour space are all settled before leaving */
			Delay( 6 * ((baserate ? 33 : 40)*(1<<clk_div)) + 10 );

			pCam->Framerate = Framerate;
//			res = cmd_check( pCam, "camSetSensorFramerate() : SetSensorFPS" );

		//////////////	
			CPIA_ReadVPRegs( pCam->pLink, 52, 52, 52, 52, &status, &status, &status, &status );
//			tracemsg(TRC_MSG, "status: 0x%x", status );
			if ( ( status & 0x0f ) )
			{
				// clear the data_corrupt bit
				CPIA_WriteVPReg( pCam->pLink, 52, 0xf0, 0x0 );
//				tracemsg(TRC_MSG, "error bits cleared" );
			}

		//////////////////////
		}

		/* write to the camera, to clear the vpBusy bit (via the bunch cache) */
		CPIA_WriteVPReg( pCam->pLink, 63, 0, 255 );

		if ( autoexp )                     
		 	camSetAutoExposure( pCam, TRUE );
		 
		if ( autoacb )
		 	camSetColBal( pCam, CB_AUTOMATIC, 0, 0, 0 );

	}
			   
 	return res; 
}

/*-----------------------------------------------------------------*/
/* return the current camera frame rate */

int camGetSensorFramerate( PCPIA_CAM pCam, FRAMERATE *Framerate,
								 BOOL *AutoFramerate )
{
 	if ( Framerate ) *Framerate = pCam->Framerate;
	if ( AutoFramerate ) *AutoFramerate = pCam->DoAutoFramerate;
 	return OK;
}

/*-----------------------------------------------------------------*/
/* switch flicker control on/off.
   If flicker is being switched on, startexposure can be used to override the
   default start exposure of the camera
*/

int camSetFlicker( PCPIA_CAM pCam, BOOL flickon, WORD currentexp)
{       
	BYTE red, green1, green2, blue;
	WORD startexp, cj;
	WORD coarse, fine;
	BYTE gain;
	int res = ERR;
	int	tmp;

//
// Retrieve current parameters
//

	if ( (camGetColComp( pCam, &red, &green1, &green2, &blue) != OK) ||
		 (camGetExposure( pCam, &coarse, &fine, &gain) != OK) )
	{
		goto Error;
	}
	if ( currentexp == 0 )
	{
		currentexp = coarse;
	}
	
	if (CPIA_FlickerCtrl( pCam->pLink, flickon, 
		pCam->flick.CoarseJump,  
		pCam->flick.OverExposure))
	{
		if ((res = cmd_check (pCam, "camSetFlicker()")) == OK)
		{
			pCam->flick.FlickerCtrl = flickon ? FLK_ON : FLK_DISABLED;
		}
	}

	
//
// Calculate new parameters
//

	if (res == OK)      
	{
		
	/* new colour compensation values are calculated only if the previous 
	exposure value was known, otherwise they're just written back to 
	the camera.
	if flicker is being switched off, the default comp values are restored
		*/
		if ( flickon )
		{
			cj = pCam->flick.CoarseJump;

			currentexp = currentexp << gain;
			gain = 0;
			
			/* round down current exposure to nearest value */
			startexp = (max (1, ( currentexp + ROUND_UP_EXP_FOR_FLICKER) / cj) * cj) - 1;
			
			/* note for 1-02fw only */
			if (pCam->FwVersion == FW_VER102)
			{
				while (startexp > 255)
				{
					startexp -= cj;
				}
			}
			else
			{
				while (startexp > 302)
				{
					startexp -= cj;
				}
			}
			
			if ( currentexp > startexp )
			{
				if ( currentexp > (2 * startexp) )
				{
					currentexp = 2 * startexp;
				}
				red = COMPGAIN (COMP_RED, currentexp, startexp);
				green1 = COMPGAIN (COMP_GREEN1, currentexp, startexp);
				green2 = COMPGAIN (COMP_GREEN2, currentexp, startexp);
				blue = COMPGAIN (COMP_BLUE, currentexp, startexp);

			}
			else
			{                 
				red = COMP_RED;
				green1 = COMP_GREEN1;
				green2 = COMP_GREEN2;
				blue = COMP_BLUE;
			}
/*
			{
				// For Debug only
				char buf[40];
				wsprintf(buf, "red=%d, currentexp=%d, startexp=%d, gain=%d",red, currentexp, startexp, gain);
				MessageBox(NULL, buf, "CPIA_CAM", MB_OK); 
			}
*/

		}
		else
		{
			// Coarse = average of equivalent coarse for each comp channel
			startexp = EXP_FROM_COMP(COMP_RED, red, currentexp);
			startexp += EXP_FROM_COMP(COMP_GREEN1, green1, currentexp);
			startexp += EXP_FROM_COMP(COMP_GREEN2, green2, currentexp);
			startexp += EXP_FROM_COMP(COMP_BLUE, blue, currentexp);
			startexp = startexp >> 2;
			
			while ( startexp > 302 && gain < pCam->GainCeiling )
			{
				startexp = startexp >> 1;
				gain++;
			}

/*
			{
				// For debug only
				char buf[40];
				wsprintf(buf, "red=%d, currentexp=%d, startexp=%d, gain=%d",red, currentexp, startexp, gain);
				MessageBox(NULL, buf, "CPIA_CAM", MB_OK); 
			}
*/

			red = COMP_RED;
			green1 = COMP_GREEN1;
			green2 = COMP_GREEN2;
			blue = COMP_BLUE;
			
			/* note for 1-02fw only */
			if (pCam->FwVersion == FW_VER102)
			{
				if (startexp > 255)
				{
					startexp = 255;
				}
			}
			
		}

//
// Set new parameters
//
		
		/* VERSION_NOTE
		compensation gains are only set when enabling flicker control
		for firmware above 1-02, or for all versions when flicker is being switched
		off  
		29.10.97 compensation only written above 1-02.Writing gains when 
		switching flicker off (at 1-02), then quickly restarting 
		flicker control, caused the camera to cycle through red, purple
		and green frames.Fun to watch, but little practical use
		*/
		
		if (pCam->FwVersion == FW_VER102)
		{         
			if (flickon)
			{
				CPIA_WriteIDATA( pCam->pLink, 0x1B, 0x00, (BYTE) (pCam->flick.CoarseJump << 7));
				CPIA_WriteIDATA( pCam->pLink, 0x1A, 0x00, pCam->flick.CoarseJump >> 1);
			}
			else
			{
				CPIA_WriteIDATA( pCam->pLink, 0xA6, 0x00, green1);
				CPIA_WriteIDATA( pCam->pLink, 0xA7, 0x00, blue);
			}
		}
		
		if (CPIA_SetExposure (pCam->pLink, 0,
//			flickon,  /* exposure only if enabling flicker */
			1,
			!flickon || pCam->FwVersion > FW_VER102,  /* comp gains */
			0,  
			gain,  
			fine,  
			LOBYTE (startexp),   /* exp lo */
			HIBYTE (startexp),   /* exp hi */
			red, green1, green2, blue))
		{               
			if ((res = cmd_check (pCam, "camSetFlicker() : SetExposure")) == OK)
			{
				pCam->exp.Gain = gain;
				pCam->exp.FineExp = fine;
				pCam->exp.CoarseExpLo = LOBYTE (startexp);
				pCam->exp.CoarseExpHi = HIBYTE (startexp);
				pCam->exp.RedComp = red;
				pCam->exp.Green1Comp = green1;
				pCam->exp.Green2Comp = green2;
				pCam->exp.BlueComp = blue;
			}
		}
		else
		{
			res = ERR;
		}
		
		if (res == OK)
		{               
			if (flickon)
			{
				CPIA_SetVLOffset (pCam->pLink, 20, 24, 26, 26);
				CPIA_SetApcor( pCam->pLink, 0x18, 0x18, 0x16, 0x14 );
			}
			else
			{
				CPIA_SetVLOffset (pCam->pLink, 20, 24, 26, 26);
				// CPIA_SetVLOffset (pCam->pLink, 24, 28, 30, 30);  - changed at 1-01drv
				CPIA_SetApcor( pCam->pLink, 0x18, 0x16, 0x24, 0x34 ); 
			}
			
			res = cmd_check (pCam, "camSetFlicker() : SetVLOffset");
		}
	}
	
// OK:
Error:

	return res;
}


int camGetExpFromFlicker( PCPIA_CAM pCam, WORD *Coarse, WORD *Fine, BYTE *Gain )
{
	int res = ERR;
	BYTE red, green1, green2, blue;
	WORD coarse;

	if ( (camGetColComp( pCam, &red, &green1, &green2, &blue) == OK) &&
		 (camGetExposure( pCam, &coarse, Fine, Gain) == OK) )
	{
		// Coarse = average of equivalent coarse for each comp channel
		*Coarse = EXP_FROM_COMP(COMP_RED, red, coarse);
		*Coarse += EXP_FROM_COMP(COMP_GREEN1, green1, coarse);
		*Coarse += EXP_FROM_COMP(COMP_GREEN2, green2, coarse);
		*Coarse += EXP_FROM_COMP(COMP_BLUE, blue, coarse);
		*Coarse = (*Coarse) >> 2;

		while ( *Coarse > 302 && *Gain < pCam->GainCeiling )
		{
			*Coarse = *Coarse >> 1;
			(*Gain)++;
		}

		res = OK;
	}

	return res;
}

/*-----------------------------------------------------------------*/
/* return the state of flicker control */

FLK_STATE camGetFlicker( PCPIA_CAM pCam)
{
 return pCam->flick.FlickerCtrl;
}

/*-----------------------------------------------------------------*/
/* set the exposure on the camera to a specific value */

int camSetExposure( PCPIA_CAM pCam, WORD coarse, WORD fine, BYTE gain )
{
	int res = ERR;
	
	/*
	function should be altered to take a WORD rather 
	than a BYTE	before being used in anger.
	NOTE - for Ver 1-02 firmware MSB of coarse must be zero.
	*/
	
	if (!pCam->error)
	{
		if (CPIA_SetExposure (
			pCam->pLink, 0,
			1,   //EXP_MANUAL,
			1,
			0,   // CW_IGNORE,
			gain,
			fine,
			((pCam->FwVersion <= FW_VER102) && (coarse >> 8)) ? 255 : coarse & 0xFF,
			(pCam->FwVersion <= FW_VER102) ? 0 : coarse >> 8,
			COMP_RED, COMP_GREEN1, COMP_GREEN2, COMP_BLUE
			))
		{     
			if ((res = cmd_check (pCam, "camSetExposure()")) == OK)
			{
				pCam->exp.Gain			= gain;
				pCam->exp.FineExp		= fine;
				pCam->exp.CoarseExpLo	= ((pCam->FwVersion <= FW_VER102) && (coarse >> 8)) ? 255 : coarse & 0xFF;
				pCam->exp.CoarseExpHi	= (pCam->FwVersion <= FW_VER102) ? 0 : coarse >> 8;	
			}
		}  
	}
	return res;
}
	
	
/*-----------------------------------------------------------------*/
/* return the current exposure setting on the camera */

int camGetExposure( PCPIA_CAM pCam, WORD *coarse, WORD *fine, BYTE *gain)

{
 	int res = ERR;
                                            
 	if ( CPIA_GetExposure( pCam->pLink, 
    	                   	&pCam->exp.Gain, 
        	               	&pCam->exp.FineExp,
            	           	&pCam->exp.CoarseExpLo,
                	       	&pCam->exp.CoarseExpHi,
                    	   	&pCam->exp.RedComp,
                       		&pCam->exp.Green1Comp,
                       		&pCam->exp.Green2Comp,
                       		&pCam->exp.BlueComp) )
	{
  		res = cmd_check (pCam, "camGetExposure");
	}

	if ( res == OK )
	{
		if ( coarse ) 
		{
			*coarse = pCam->exp.CoarseExpLo + 256 * pCam->exp.CoarseExpHi;
		}
		if ( fine ) 
		{
			*fine = pCam->exp.FineExp;
		}
		if ( gain ) 
		{
			*gain = pCam->exp.Gain;
		}	
	}
	else
	{
		if ( coarse ) 
		{
			*coarse = 0;
		}
		if ( fine ) 
		{
			*fine = 0;
		}
		if ( gain ) 
		{
			*gain = 0;
		}
	}

	return (res);
}


/*-----------------------------------------------------------------*/
/* set the flicker threshold parameters required
*/
int camSetFlickerThreshold( PCPIA_CAM pCam, BYTE FlickMax, BYTE FlickExp )
{
    pCam->flick.FlickMax = FlickMax;
    pCam->flick.FlickExp = FlickExp;
    pCam->flick.OverExposure = mCalculateOverExp( pCam->cp.Brightness, pCam->flick.FlickExp, pCam->flick.FlickMax );
    return OK;
}

/*-----------------------------------------------------------------*/
/* Get the flicker threshold parameters required
*/
int camGetFlickerThreshold( PCPIA_CAM pCam, BYTE *FlickMax, BYTE *FlickExp )
{
    if ( FlickMax ) *FlickMax = pCam->flick.FlickMax;
    if ( FlickExp ) *FlickExp = pCam->flick.FlickExp;
    return OK;
}


/*-----------------------------------------------------------------*/
/* set the flicker parameters required for a particular frame rate at a 
   particular mains frequency speed
*/
   
int camSetFlickerParams( PCPIA_CAM pCam, MAINS_FREQ mf )
{
 int res = OK;
 
 switch( mf )
  {          
   case MAINS_50HZ :
        switch( pCam->Framerate )
         {
          case FPS_30 :
               pCam->flick.CoarseJump = 92;
               pCam->flick.OverExposure = 40;
               pCam->flick.RestartLimit = 15;
               break;
               
          case FPS_25 :
               pCam->flick.CoarseJump = 76;
               pCam->flick.OverExposure = 40;
               pCam->flick.RestartLimit = 15;
               break;  
               
          case FPS_15 :
               pCam->flick.CoarseJump = 46;
               pCam->flick.OverExposure = 40;
               pCam->flick.RestartLimit = 15;
               break;
               
          case FPS_125 :
               pCam->flick.CoarseJump = 38;
               pCam->flick.OverExposure = 40;
               pCam->flick.RestartLimit = 15; 
               break;
               
          case FPS_75 :
               pCam->flick.CoarseJump = 23;
               pCam->flick.OverExposure = 40;
               pCam->flick.RestartLimit = 15;
               break;
               
          default :
               res = ERR;
               break;
         }
        break;
        
   case MAINS_60HZ :
        switch( pCam->Framerate )
         {
          case FPS_30 :
               pCam->flick.CoarseJump = 76;
               pCam->flick.OverExposure = 40;
               pCam->flick.RestartLimit = 15;
               break;
               
          case FPS_25 :
               pCam->flick.CoarseJump = 64;
               pCam->flick.OverExposure = 40;
               pCam->flick.RestartLimit = 15;
               break;
               
          case FPS_15 :
               pCam->flick.CoarseJump = 38;
               pCam->flick.OverExposure = 40;
               pCam->flick.RestartLimit = 15;
               break;
               
          case FPS_125 :
               pCam->flick.CoarseJump = 32;
               pCam->flick.OverExposure = 40;
               pCam->flick.RestartLimit = 15;
               break;
               
          case FPS_75 :
               pCam->flick.CoarseJump = 19;
               pCam->flick.OverExposure = 40;
               pCam->flick.RestartLimit = 15;
               break;
               
          default :
               res = ERR;
               break;
         }
        break;
        
   default :
        res = ERR;
        break;
  }

 if (res == OK)
  {  
   pCam->flick.MainsFreq = mf;
  }
  
 return res;
}

/*-----------------------------------------------------------------*/
/* return the current framerate and mains frequency values stored for
   the camera.The framerate here is independent of any actual framerate,
   and is the rate assumed for flicker correction purposes
*/
   
int camGetFlickerParams( PCPIA_CAM pCam, MAINS_FREQ *mf )
{           
 if (mf)
  *mf = pCam->flick.MainsFreq;
 
 return OK; 
}

/*-----------------------------------------------------------------*/
/* if flicker is switched off, this function switches it back on.It checks,
   however, that conditions are suitable before restarting it.

   It also adjust the colour balance when an exposure step is detected - as
   long as flicker is running
   
   if curexposure == 0, the current exposure is read from the camera (although
   this command doesn't return the correct values - exphi is wrong)
*/
int camRestartFlicker( PCPIA_CAM pCam, WORD curexposure, WORD prevexposure,
                       FLK_STATE *restarted, WORD *newexposure )
{
 int res = OK,
     cam_exposure,
     old_exp;
             
 	if ( restarted ) *restarted = pCam->flick.FlickerCtrl;   /* assume state won't change */

 	/* 
 		if an exposure value is supplied, use it, else get the 
 		current one from the camera 
    */              
 	old_exp = cam_exposure = curexposure;
	/* if exposure 0, read it from the camera */        
	if (old_exp == 0)
	{
		camGetExposure( pCam, &old_exp, NULL, NULL );
	}

 
 	/* 
 		see how far away camera exposure is from a valid 
 		flicker exposure value
 	*/
 	cam_exposure %= pCam->flick.CoarseJump;
 
 	if ( pCam->FwVersion <= FW_VER102 &&
    	 pCam->flick.FlickerCtrl == FLK_ON &&
     	 cam_exposure <= pCam->flick.CoarseJump - 3 )
  	{
   		pCam->flick.FlickerCtrl = FLK_OFF;
   		if ( restarted ) *restarted = FLK_OFF;
  	}

 	/* 
 		if the flicker control is off, but exposure is now high
 		enough to switch it back on, then switch it back on 
 	*/
 	if ( pCam->FwVersion <= FW_VER102 &&
     	 pCam->flick.FlickerCtrl == FLK_OFF &&
     	 old_exp > pCam->flick.CoarseJump + pCam->flick.RestartLimit)
  	{  
   		if ((res = camSetFlicker (pCam, TRUE, old_exp)) == OK)
    	{
     		if ( restarted ) *restarted = FLK_RESTART;
    	}
  	}
  
 	if ( newexposure )
  	{
    	WORD cj = pCam->flick.CoarseJump;

   		/* round down current exposure to nearest value */
   		*newexposure = (max (1, curexposure / cj) * cj) - 1;
  	}
 
 	return res;
}

/*-----------------------------------------------------------------*/
/* if weightcentre is TRUE, the centre of the scene is given precedence in the
   exposure calculation.If it's false, the whole scene is evenly weighted */

int camSetCentreWeight( PCPIA_CAM pCam, BOOL weightcentre)
{
 	int res = ERR;
 
 	if (!pCam->error)
  	{
   		if ( CPIA_SetExposure( pCam->pLink, 0, 0, 0,
        	                 weightcentre ? CW_WEIGHT : CW_FLAT,
            	             0, 0, 0, 0, 0, 0, 0, 0 ) )
    	{     
 			/* the firmware can pause for up to 2 frametimes after changing 	*/
			/* the state of the CentreWeight parameter.							*/
			/* wait here before checking status to prevent the cmd_check() 		*/
			/* status timing out.												*/
   			switch (  pCam->Framerate )
			{						    			  /* + 10 just for safety ! */
	     	case FPS_30 : 	Delay( 2 * 33 + 10 ); break;
		    case FPS_25 :	Delay( 2 * 40 + 10 ); break;                 
	    	case FPS_15 :	Delay( 2 * 66 + 10 ); break;
		    case FPS_125 :	Delay( 2 * 80 + 10 ); break;
		    default :
	    	case FPS_75 :	Delay( 2 * 132 + 10 ); break;
		    }                

    	 	if ((res = cmd_check (pCam, "camSetCentreWeight()")) == OK)
      		pCam->exp.CentreWeight = weightcentre;
    	}
  	}  
    
 	return res;
}

/*-----------------------------------------------------------------*/
/* return the current value of centreweight */

BOOL camGetCentreWeight( PCPIA_CAM pCam)
{
 return pCam->exp.CentreWeight;
}


/*-----------------------------------------------------------------*/
/* read the colour compensation values from the camera */

int camGetColComp( PCPIA_CAM pCam, BYTE *red, BYTE *green1, BYTE *green2, 
                   BYTE *blue)
{
 int res = ERR;
             
 if (CPIA_GetExposure (pCam->pLink, 
                       &pCam->exp.Gain, 
                       &pCam->exp.FineExp,
                       &pCam->exp.CoarseExpLo,
                       &pCam->exp.CoarseExpHi,
                       &pCam->exp.RedComp,
                       &pCam->exp.Green1Comp,
                       &pCam->exp.Green2Comp,
                       &pCam->exp.BlueComp))
  res = cmd_check (pCam, "camGetColComp()");
  
 if (red)
  *red = pCam->exp.RedComp;
  
 if (green1)
  *green1 = pCam->exp.Green1Comp;
  
 if (green2)
  *green2 = pCam->exp.Green2Comp;
  
 if (blue)
  *blue = pCam->exp.BlueComp;
             
 return res;
}
 
/*
//	cISS
//		periodically checks that the AecAcb is obtaining an acceptable image
//		quality and intervenes if any problems are found.
//
//	25/11/97	SG	lowers sensor framerate if the image is continously
//					underexposed
//	26/11/97	SG	raises sensor framerate if exposure very low
//	27/11/97	SG	raises sensor framerate if exposure accumulator very high
//	27/11/97	SG	increases gain when reducing framerate and vice-versa
//	10/03/98	REM	modified to return true if it did communications with the
//					camera, else FALSE.
//					sets newFPS to FPS_UNKNOWN if it does not change the framerate
*/ 

#define MAX_EXP				302
#define LOW_EXP				140
#define VERY_LOW_EXP		70
#define TC					94
#define	EXP_ACC_DARK		50
#define	EXP_ACC_LIGHT		90
#define	DARK_TIME			3
#define	LIGHT_TIME			3
#define HIGH_COMP_102		160		
#define MAX_COMP			239		


#define G_CEILING			DEFAULT_GAIN_CEILING	/* NOTE - this should really use the pCam->GainCeiling value */
													/* but it is not set correctly by camSetGainCeiling() yet	 */


BOOL camImageSourceSupervisor( CPIA_CAM *pCam, WORD curexposure, WORD *new_exp, FRAMERATE *newFPS )
{
WORD new_exposure, old_exposure;
BYTE exp_acc,
	 bcomp,
	 gain,
	coarseL ,
	 max_comp;

static int dark_count = 0,
           light_count = 0,
           very_dark_count = 0,
           very_light_count = 0,
		   light_exp = 255,
		   dark_exp = 0,
		   very_dark_exp = 0;
static DWORD current_time,
             timeout = 0;
             
	if ( newFPS ) *newFPS = FPS_UNKNOWN;

	//	check stats after 1 second has passed
	if( (current_time = timeGetTime()) < timeout )
		return FALSE;

	//	set new timeout for 2 second's time
	timeout = current_time + 2000;		

	//	if an exposure value is supplied, use it, else get the current one from
	//	the camera
 	old_exposure = curexposure;
	/* if exposure 0, read it from the camera */        
	if (old_exposure == 0)
	{
		camGetExposure( pCam, &old_exposure, NULL, NULL );
	}

	//	get necessary stats and register settings from camera
	CPIA_ReadVPRegs( pCam->pLink, 30, 4, 9, 8, &exp_acc, &bcomp, &gain, &coarseL );
	
//	if ( TMSET() ) 
//		tracemsg( TRC_MSG, "VP Regs -- exp_acc=%d  bcomp=%d  gain=%d  coarseL=%d old_exp=%d\n", exp_acc, bcomp, gain, coarseL, old_exposure );   

	light_exp = pCam->cp.Brightness + TC - 50 + EXP_ACC_LIGHT;
	light_exp = light_exp > 255 ? 255 : light_exp;
	dark_exp = pCam->cp.Brightness + TC - 50 - EXP_ACC_DARK;
	dark_exp = dark_exp < 0 ? 0 : dark_exp;
	very_dark_exp = dark_exp >> 1;

	// banding filter enabled
	if(pCam->flick.FlickerCtrl == FLK_ON) {
		max_comp = pCam->FwVersion > FW_VER102 ? MAX_COMP : HIGH_COMP_102 ; 
		bcomp += 128;	//	decode
		//	dark ??
		if(bcomp >= max_comp && exp_acc < dark_exp) {
			light_count = 0;
			very_light_count = 0;
			// very dark ??
			if(exp_acc < very_dark_exp) {
				very_dark_count++;
				dark_count = 0;
			}
			// just dark
			else {
				dark_count++;
				very_dark_count=0;
			}
		}
		else {
			// not dark
			dark_count = 0;
			very_dark_count = 0;
			// light ??
			if(old_exposure <= LOW_EXP || exp_acc > light_exp) {
				// very light ??
				if(old_exposure <= VERY_LOW_EXP) {
					very_light_count++;
					light_count = 0;
				}
				// just light
				else {
					very_light_count = 0;
					light_count++;
				}
			}
			// not light
			else {
				very_light_count = 0;
				light_count = 0;
			}
		}
	}
	// no banding filter
	else {
		//	dark ??
		if(old_exposure >= MAX_EXP && exp_acc < dark_exp) {
			light_count = 0;
			very_light_count = 0;
			// very dark ??
			if(exp_acc < very_dark_exp) {
				very_dark_count++;
				dark_count = 0;
			}
			// just dark
			else {
				dark_count++;
				very_dark_count=0;
			}
		}
		else {
			// not dark
			dark_count = 0;
			very_dark_count = 0;
			// light ??
			if(old_exposure <= LOW_EXP || exp_acc > EXP_ACC_LIGHT) {
				// very light ??
				if(old_exposure <= VERY_LOW_EXP) {
					very_light_count++;
					light_count = 0;
				}
				// just light
				else {
					very_light_count = 0;
					light_count++;
				}
			}
			// not light
			else {
				very_light_count = 0;
				light_count = 0;
			}
		}
	}



	// banding filter enabled ??
	if(pCam->flick.FlickerCtrl == FLK_ON) {
		// dark or very dark for DARK_TIME seconds ??
		if(dark_count >= DARK_TIME || very_dark_count >= DARK_TIME) {
			switch( pCam->Framerate ) {
		     case FPS_30 :
					*newFPS = FPS_15;
					if ( camSetSensorFramerate( pCam, FPS_15, TRUE ) == OK )
					{
						camSetFlickerParams(pCam, pCam->flick.MainsFreq ) ;
 						CPIA_FlickerCtrl( pCam->pLink, FLK_ON, pCam->flick.CoarseJump,  pCam->flick.OverExposure);
						new_exposure = pCam->flick.CoarseJump - 1;
						while ( new_exposure < old_exposure>>1 )
						{
							new_exposure += pCam->flick.CoarseJump;
						}
						new_exposure += pCam->flick.CoarseJump;
			   			CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, 0, 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8), 0, 0, 0, 0);
					}
					break;
		        
		     case FPS_25 :
					*newFPS = FPS_125;
					if ( camSetSensorFramerate( pCam, FPS_125, TRUE ) == OK )
					{
						camSetFlickerParams(pCam, pCam->flick.MainsFreq ) ;
 						CPIA_FlickerCtrl( pCam->pLink, FLK_ON, pCam->flick.CoarseJump,  pCam->flick.OverExposure);
						new_exposure = pCam->flick.CoarseJump - 1;
						while ( new_exposure < old_exposure>>1 )
						{
							new_exposure += pCam->flick.CoarseJump;
						}
						new_exposure += pCam->flick.CoarseJump;
			   			CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, 0, 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8), 0, 0, 0, 0);
					}
					break;
		          
		     case FPS_15 :
		     case FPS_125 :
		     case FPS_75 :
		     default :
		          break ;
		    }                
		}
		// light or very light for LIGHT_TIME seconds ??
		else if(light_count >= LIGHT_TIME || very_light_count >= LIGHT_TIME) {
			WORD max_exp = pCam->FwVersion <= FW_VER102 ? 255 : MAX_EXP ; 
			switch(pCam->Framerate) {
		     case FPS_15 :
					*newFPS = FPS_30;
					if ( camSetSensorFramerate( pCam, FPS_30, TRUE ) == OK )
					{
						camSetFlickerParams(pCam, pCam->flick.MainsFreq ) ;
 						CPIA_FlickerCtrl( pCam->pLink, FLK_ON, pCam->flick.CoarseJump,  pCam->flick.OverExposure);
						new_exposure = pCam->flick.CoarseJump - 1;
						while(new_exposure < 2*old_exposure && new_exposure+pCam->flick.CoarseJump < max_exp)
							new_exposure+=pCam->flick.CoarseJump ;
			   			CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, 0, 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8), 0, 0, 0, 0);
					}
					break;
		        
		     case FPS_125 :
					*newFPS = FPS_25;
					if ( camSetSensorFramerate( pCam, FPS_25, TRUE ) == OK )
					{
						camSetFlickerParams(pCam, pCam->flick.MainsFreq );
 						CPIA_FlickerCtrl( pCam->pLink, FLK_ON, pCam->flick.CoarseJump,  pCam->flick.OverExposure);
						new_exposure = pCam->flick.CoarseJump - 1;
						while(new_exposure < 2*old_exposure && new_exposure+pCam->flick.CoarseJump < max_exp)
							new_exposure+=pCam->flick.CoarseJump ;
			   			CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, 0, 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8), 0, 0, 0, 0);
					}
					break;
		        
		     case FPS_75 :
					if(very_light_count >= LIGHT_TIME) {
						*newFPS = FPS_30;
						if ( camSetSensorFramerate( pCam, FPS_30, TRUE ) == OK )
						{
							camSetFlickerParams(pCam, pCam->flick.MainsFreq );
 							CPIA_FlickerCtrl( pCam->pLink, FLK_ON, pCam->flick.CoarseJump,  pCam->flick.OverExposure);
							new_exposure = pCam->flick.CoarseJump - 1;
							while(new_exposure < 4*old_exposure && new_exposure+pCam->flick.CoarseJump < max_exp)
								new_exposure+=pCam->flick.CoarseJump ;
			   				CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, 0, 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8), 0, 0, 0, 0);
						}
					}
					else {
						*newFPS = FPS_15;
						if ( camSetSensorFramerate( pCam, FPS_15, TRUE ) == OK )
						{
							camSetFlickerParams(pCam, pCam->flick.MainsFreq );
 							CPIA_FlickerCtrl( pCam->pLink, FLK_ON, pCam->flick.CoarseJump,  pCam->flick.OverExposure);
							new_exposure = pCam->flick.CoarseJump - 1;
							while(new_exposure < 2*old_exposure && new_exposure+pCam->flick.CoarseJump < max_exp)
								new_exposure+=pCam->flick.CoarseJump ;
			   				CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, 0, 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8), 0, 0, 0, 0);
						}
					}
					break;
		     case FPS_30 :
		     case FPS_25 :
		     default :
		          break ;
		    }                
		}
		else
			return TRUE;
	}
	else {
		//	a 1-02 bug does not allow exposure settings of > 255
		new_exposure = (old_exposure > 255 && pCam->FwVersion <= FW_VER102) ? 255 : old_exposure ; 
		// dark or very dark for DARK_TIME seconds ??
		if(dark_count >= DARK_TIME || very_dark_count >= DARK_TIME) {
			switch(pCam->Framerate) {
		     case FPS_30 :
					if(very_dark_count >= DARK_TIME) {
						*newFPS = FPS_75;
						if ( camSetSensorFramerate( pCam, FPS_75, TRUE ) == OK )
						{
	               			camSetColBal( pCam, CB_MANUAL, 30, 30, 200) ;    /* switch on auto colour balance */
							CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, (gain>1?gain-2:0), 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8),  0, 0, 0, 0);
						}
 					}
					else {
						*newFPS = FPS_15;
						if ( camSetSensorFramerate( pCam, FPS_15, TRUE ) == OK )
						{
				   			CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, (gain>0?gain-1:0), 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8), 0, 0, 0, 0);
						}
					}
					break;
		        
		     case FPS_25 :
					*newFPS = FPS_125;
					if ( camSetSensorFramerate( pCam, FPS_125, TRUE ) == OK )
					{
				   		CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, (gain>0?gain-1:0), 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8), 0, 0, 0, 0);
					}
					break;
		          
		     case FPS_15 :
					*newFPS = FPS_75;
					if ( camSetSensorFramerate( pCam, FPS_75, TRUE ) == OK )
					{
               			camSetColBal( pCam, CB_MANUAL, 30, 30, 200) ;    /* switch on auto colour balance */
			   			CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, (gain>0?gain-1:0), 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8), 0, 0, 0, 0);
					}
					break;
		        
		     case FPS_125 :
		     case FPS_75 :
		     default :
		          break ;
		    }                
		}
		// light or very light for LIGHT_TIME seconds ??
		else if(light_count >= LIGHT_TIME || very_light_count >= LIGHT_TIME) {
			switch(pCam->Framerate) {
		     case FPS_15 :
					*newFPS = FPS_30;
					if ( camSetSensorFramerate( pCam, FPS_30, TRUE ) == OK )
					{
			   			CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, (gain<G_CEILING?gain+1:G_CEILING), 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8), 0, 0, 0, 0);
					}
					break;
		        
		     case FPS_125 :
					*newFPS = FPS_25;
					if ( camSetSensorFramerate( pCam, FPS_25, TRUE ) == OK )
					{
			   			CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, (gain<G_CEILING?gain+1:G_CEILING), 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8), 0, 0, 0, 0);
					}
					break;
		        
		     case FPS_75 :
					if(very_light_count >= LIGHT_TIME) {
						*newFPS = FPS_30;
						if ( camSetSensorFramerate( pCam, FPS_30, TRUE ) == OK )
						{
			   				CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, (gain+1<G_CEILING?gain+2:G_CEILING), 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8), 0, 0, 0, 0);
						}
					}
					else {
						*newFPS = FPS_15;
						if ( camSetSensorFramerate( pCam, FPS_15, TRUE ) == OK )
						{
			   				CPIA_SetExposure( pCam->pLink, 0, 1, 0, 0, (gain<G_CEILING?gain+1:G_CEILING), 1, (BYTE)(new_exposure & 0xff),  (BYTE)((new_exposure & 0x100) >> 8), 0, 0, 0, 0);
						}
					}
					break;
		     case FPS_30 :
		     case FPS_25 :
		     default :
		          break ;
		    }                
		}
		else
			return TRUE;
	}
  
	//	set new framerate if new framerate differs from current framerate
	//res = ( (res == ERR) ? ERR : camSetFrameRate (pCam, newFramerate));
 
	//	reset counters so nothing can happen for at least CHANGE_FPS_TIME seconds
	dark_count = 0;
	light_count = 0;
	very_light_count = 0;
	very_dark_count = 0;

	return TRUE;
}

                    
/*-----------------------------------------------------------------*/
/* return the version of the firmware on the camera */

WORD camGetFwVersion( PCPIA_CAM pCam)
                    
{
 return pCam->FwVersion;
}

/*-----------------------------------------------------------------*/

BOOL camNullCmd( PCPIA_CAM pCam)
{
 	if ( pCam ) return CPIA_NullCmd (pCam->pLink);
	else return( FALSE );
}


/*-----------------------------------------------------------------*/

BOOL camPushButtonSet( PCPIA_CAM pCam, BOOL Check, BOOL Pushed )
{
    if ( pCam ) 
    {
        pCam->EnableButtonChecking = Check;
        pCam->ButtonState = Pushed;
        return( TRUE );
    }
    else
        return( FALSE );
}


/*-----------------------------------------------------------------*/

BOOL camPushButtonGet( PCPIA_CAM pCam, BOOL *pCheck, BOOL *pPushed )
{
    if ( pCam ) 
    {
        if ( pCheck ) *pCheck = pCam->EnableButtonChecking;
        if ( pPushed ) *pPushed = pCam->ButtonState;
        pCam->ButtonState = FALSE;
        return( TRUE );
    }
    else
        return( FALSE );
}


BOOL camSerialStart( PCPIA_CAM pCam )
{
	return CPIA_SerialStart( pCam->pLink );
}


BOOL camSerialStop( PCPIA_CAM pCam )
{
	return CPIA_SerialStop( pCam->pLink );
}


BOOL camSerialWrite( PCPIA_CAM pCam, BYTE WriteData )
{
	return CPIA_SerialWrite( pCam->pLink, WriteData );
}


BOOL camSerialRead( PCPIA_CAM pCam, BOOL AckRead, BYTE *ReadData )
{
	return CPIA_SerialRead( pCam->pLink, AckRead, ReadData );
}


BOOL camSetSensorMatrix( PCPIA_CAM pCam, BYTE Matrix[9] )
{
	if ( pCam )
	{
		return CPIA_SetSensorMatrix( pCam->pLink, Matrix );
	}
}


BOOL camGetSensorMatrix( PCPIA_CAM pCam, BYTE Matrix[9] )
{
	if ( pCam )
	{
		return FALSE;
	}
}


/*-----------------------------------------------------------------*/
/* check that the previous camera command was accepted.
   This doesn't check power state, as it's called when powering up...
*/
static int cmd_check ( PCPIA_CAM pCam, LPSTR func )
{
 	BYTE stream, cmd, pwr, ftl;
 	int res = OK;
 	char buf [255];
 	int tmp = 0;
                
 	if (pCam && pCam->error)
  		res = ERR;
 	else
  	{                                                   
		tmp = CPIA_GetCameraStatus( pCam->pLink, &pwr, NULL, &stream, 
		                                 &ftl, &cmd, NULL, NULL, NULL );
		if ( !tmp )
        {
		  	if ( TMSET() )
            {
                wsprintf( buf, "CPIA_GetCameraStatus() failed !   Previous command : %s", func );
		   	    tracemsg( TRC_MSG, buf );
            }
            return( ERR );
        }
        
        if ( ftl != 0 || cmd != 0)
		{
		  	if ( TMSET() )
			{
				 wsprintf( buf, "Error in %s, FatalError = 0x%02x, CmdError = 0x%02x%s",
		            			func, ftl, cmd, 
		            			pwr != HI_POWER_STATE ? (LPSTR) " ** NOT IN HI POWER **" : 
		                                    (LPSTR) "" );
		                                    
			   	tracemsg( TRC_MSG, buf );
			}
			res = ERR;

		 }

		/* if cmd has generated an error, mask out any bits that are set */                 
		if ( cmd )
		 	CPIA_ModifyCameraStatus( pCam->pLink, 4, ~cmd, 0 );

		if ( ftl )
		 	CPIA_ModifyCameraStatus( pCam->pLink, 3, ~ftl, 0 );
	}

 	return res;
}


static POWER_MODE PowerStateToMode( BYTE PowerState ) {
	switch( PowerState )
	{
	case UNINITIALISED_STATE: 	return( PWR_UNINITIALISED );

	case PASS_THROUGH_STATE:	return( PWR_PASSTHROUGH	);

	case LOW_POWER_STATE:		return( PWR_LOW );

	case HI_POWER_STATE	:		return( PWR_HI );
	
	default:					return( PWR_UNINITIALISED );
	}
}


static BYTE PowerModeToState( POWER_MODE PowerMode ) {
	switch( PowerMode )
	{
	case PWR_UNINITIALISED: 	return( UNINITIALISED_STATE );

	case PWR_PASSTHROUGH:		return( PASS_THROUGH_STATE );

	case PWR_LOW:				return( LOW_POWER_STATE );

	case PWR_HI	:				return( HI_POWER_STATE );
	
	default:					return( UNINITIALISED_STATE );
	}
}


BYTE mCalculateOverExp( BYTE Brightness , BYTE FlickerExp , BYTE FlickerMax )
{       
    WORD MaxExp;
    WORD OverExp;

	MaxExp = FlickerMax - Brightness - 59;

	if ( FlickerExp > MaxExp )
	{
		OverExp = MaxExp;
	}
	else
	{
		OverExp = FlickerExp;
	}

	return (BYTE)OverExp;
}

