/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/***************************************************************************
*  HEADER VVL_402      Definitions for VVL_402                             *
*           Note: NOT COMPLIANT WITH VVL CODING CONVENTIONS                *
*                                                                          *
*    Date     Author            Comments                                   *
*  03-08-94   WHB   I2C codes                                              *
*  11-08-94   WHB   Added definitions from image.h                         *
*  16-08-94   NGA   Modified from VVL1042 for 1043                         *
*  29-08-95   NGA   Modified for 1043C                                     *
*  25-03-96   SA  Modified for VVL_401                                     *
*  14-04-96   SA  Modified for vvl_402                                     *
***************************************************************************/

#pragma warning( disable : 4214 )      

#ifndef _CPIA_CAM_H_
#define _CPIA_CAM_H_


#define OK      0
#define ERR     -1


#define MAKEWORD(l, h) ((WORD) (l) | (((WORD) (h)) << 8))

/* store a version number in a word. version is stored in the high byte, 
   revision in the lobyte */
#define FW_VER(vers, rev)   MAKEWORD ((rev), (vers))

/* extract the version and revision numbers from a firmware version */
#define FW_GETVERSION(fw)   HIBYTE ((fw))
#define FW_GETREVISION(fw)  LOBYTE ((fw))

/* firmware version numbers */
#define FW_VER102   FW_VER (1, 2)
#define FW_VER106   FW_VER (1, 6)
#define FW_VER109   FW_VER (1, 9)
#define FW_VER120   FW_VER (1, 20) 

/* gain and exposure settings for the cpia_setexposure func */
#define GAIN_IGNORE     0   /* don't change gain setting */
#define GAIN_CEILING    1   /* set gain ceiling to value in param 5
                               - see 1.02 firware note against function */
                                         
#define COMP_AUTO       0   /* automatic compression */
#define COMP_MANUAL     1   /* manual compression */

/* default (minimum) compensation values */
#define COMP_RED        220
#define COMP_GREEN1     214
#define COMP_GREEN2     COMP_GREEN1
#define COMP_BLUE       230

#define MAKESHORT(low,high) (((WORD)(high)<<8)|(low))

#define DEFAULT_I2C_ADDR    32


#define DEFAULT_GAIN_CEILING  1		/* ie a sensor gain of x2 */

/* Sensor pixel info */

#define EIA_ROWS            243
#define EIA_COLS            320

#define CCIR_ROWS           287
#define CCIR_COLS           384



#define BUTTON_INPUT_MASK   0x02
#define BUTTON_RESET_PORT   3      
#define BUTTON_RESET_MASK   0x20



/* define for compgain calculation */
#define COMPGAIN(base, curexp, newexp) \
    (BYTE) ((((float) base - 128.0) * ((float) curexp / (float) newexp)) + 128.5)
#define EXP_FROM_COMP(basecomp, curcomp, curexp) \
    (WORD)((float)curexp * (float)(BYTE)(curcomp + 128) / (float)(BYTE)(basecomp - 128))
#define ROUND_UP_EXP_FOR_FLICKER 15

#pragma warning( default : 4214 )

                                                                 
typedef struct {
	HLOCAL HandleToThisInstance;
    PLINK pLink;      /* transfer layer link object */

    WORD FwVersion;   /* firmware verision - hibyte = release, lobyte = revision */
	WORD SensorID;
	BYTE SensorRevision;
	POWER_MODE PowerMode;
    BOOL bPassthrough;
    BOOL EnableButtonChecking;
    BOOL ButtonState;

    BOOL error;             /* TRUE if a command has failed */

	BYTE GainCeiling;
    struct {      
            COL_BALANCE BalanceMode;
            BYTE RedGain;
            BYTE BlueGain;
            BYTE GreenGain;
           } cb;    /* colour balance */
           
    struct {
            BYTE Brightness;
            BYTE Contrast;
            BYTE Saturation;
           } cp;    /* colour params */
           
    struct {      
            BYTE GainMode;
            BYTE ExpMode;
            BYTE CompMode;
            BOOL CentreWeight;
            BYTE Gain;
            BYTE FineExp;
            BYTE CoarseExpLo;
            BYTE CoarseExpHi;
            BYTE RedComp;
            BYTE Green1Comp;
            BYTE Green2Comp;
            BYTE BlueComp;
           } exp;               /* exposure control settings */
           
    FRAMERATE Framerate;       /* camera frame rate */
    struct {        
            FLK_STATE FlickerCtrl;  /* current flicker state */  
            MAINS_FREQ MainsFreq;   /* mains frequency for flicker  */
            BYTE CoarseJump;        /* min/start coarsejump setting */
            BYTE OverExposure;      /* AllowableOverExposure        */
            BYTE RestartLimit;      /* restart flicker when current_exposure >
                                        CoarseJump + RestartLimit */
            BYTE FlickExp;
            BYTE FlickMax;
           } flick;
	WORD LastExposure;      /* exposure value on last frame */

	BOOL DoAutoFramerate;

	DWORD StartTime;

} CPIA_CAM, *PCPIA_CAM;


#ifndef LPT1
#define LPT1    1
#define LPT2    2
#define LPT3    3
#endif

/* function prototypes */
PCPIA_CAM camCreateCPIA_CAM( int ID, BYTE SerialAddr, PLINK plink );
BOOL camDestroyCPIA_CAM( PCPIA_CAM pCam );
BOOL camInitialiseCPIA_CAM( PCPIA_CAM pCam, PLINK plink );
BOOL camDeinitialiseCPIA_CAM( PCPIA_CAM pCam );

BOOL camDoHiPowerCommsTest( PCPIA_CAM pCam );

BOOL camInitVP( PCPIA_CAM pCam );
BOOL camGetSensorID( PCPIA_CAM pCam, WORD *SensorID, BYTE *Revision );

/* function below returns TRUE if it sent a command to the 		*/
/* camera, it is called each frame by the GETFRAME module.		*/
BOOL camDoFrameTimeProcessing( PCPIA_CAM pCamera, BYTE *FrameHeader );


int camSetAutoExposure( PCPIA_CAM pCam, BOOL autoexp );
BOOL camGetAutoExposure( PCPIA_CAM pCam );

int camSetAutoGain ( PCPIA_CAM pCam, int Enable  );
int camSetGainCeiling( PCPIA_CAM pCam, int Gain  );
int camGetGain( PCPIA_CAM pCam );

int camSetPower( PCPIA_CAM pCam, POWER_MODE power );
POWER_MODE camGetPower( PCPIA_CAM pCam );      

int camSetPass( PCPIA_CAM pCam, BOOL pthrough );
int camGetPass( PCPIA_CAM pCam );

int camSetColBal( PCPIA_CAM pCam, COL_BALANCE balmode, BYTE RedGain,
                  BYTE GreenGain, BYTE BlueGain );
int camGetColBal( PCPIA_CAM pCam, COL_BALANCE *pBalanceMode,
				  BYTE *pRedGain, BYTE *pGreenGain, BYTE *pBlueGain );

int camSetColPar( PCPIA_CAM pCam, BYTE Brightness, BYTE Contrast,
                  BYTE Saturation );
int camGetColPar( PCPIA_CAM pCam, BYTE *pBrightness, BYTE *pContrast,
                  BYTE *pSaturation );

int camSetSensorFramerate( PCPIA_CAM pCam, FRAMERATE Framerate,
								 			BOOL AutoFramerate );

int camGetSensorFramerate( PCPIA_CAM pCam, FRAMERATE *Framerate,
								 			BOOL *AutoFramerate );


int camSetFlicker( PCPIA_CAM pCam, BOOL flickon, WORD startexp );
int camGetFlicker( PCPIA_CAM pCam );

int camSetExposure( PCPIA_CAM pCam, WORD coarse, WORD fine, BYTE gain );
int camGetExposure( PCPIA_CAM pCam, WORD *coarse, WORD *fine, BYTE *gain);

int camSetFlickerThreshold( PCPIA_CAM pCam, BYTE FlickMax, BYTE FlickExp );
int camGetFlickerThreshold( PCPIA_CAM pCam, BYTE *FlickMax, BYTE *FlickExp );

int camSetFlickerParams( PCPIA_CAM pCam, MAINS_FREQ mf );
int camGetFlickerParams( PCPIA_CAM pCam, MAINS_FREQ *mf );
int camRestartFlicker( PCPIA_CAM pCam, WORD curexposure, WORD prevexposure, 
                       FLK_STATE *restarted, WORD *newexposure );
int camGetExpFromFlicker( PCPIA_CAM pCam, WORD *Coarse, WORD *Fine, BYTE *Gain );

int camSetCentreWeight( PCPIA_CAM pCam, BOOL weightcentre );
BOOL camGetCentreWeight( PCPIA_CAM pCam );

int camGetColComp( PCPIA_CAM pCam, BYTE *red, BYTE *green1, BYTE *green2, 
                   BYTE *blue );


BOOL camImageSourceSupervisor( CPIA_CAM *pCam, WORD curexposure,
									 WORD *new_exp, FRAMERATE *newFPS );

WORD camGetFwVersion( PCPIA_CAM pCam );

BOOL camNullCmd( PCPIA_CAM pCam );


/* 
  following two functions added to support push button hardware
*/
BOOL camPushButtonSet( PCPIA_CAM pCam, BOOL Check, BOOL Pushed );
BOOL camPushButtonGet( PCPIA_CAM pCam, BOOL *pCheck, BOOL *pPushed );

// Sets colour restoration matrix for different sensors
BOOL camSetSensorMatrix( PCPIA_CAM pCam, BYTE Matrix[9] );

// Not operational yet (and may never work anyway )
BOOL camGetSensorMatrix( PCPIA_CAM pCam, BYTE Matrix[9] );


// Serial Interface functions
BOOL camSerialStart( PCPIA_CAM pCam );
BOOL camSerialStop( PCPIA_CAM pCam );
BOOL camSerialWrite( PCPIA_CAM pCam, BYTE WriteData );
BOOL camSerialRead( PCPIA_CAM pCam, BOOL AckRead, BYTE *ReadData );


#endif   /* _CPIA_CAM_H_ */
