/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */


#include <stdio.h>
#include <string.h>



#ifdef _WINDOWS
#include <windows.h>
#endif


#include "vvl_defs.h"


#include "cmd.h"

#ifdef USBLINK
#include "usblink.h"
#endif

#ifdef PPLINK
#include "pport.h"
#include "pplink.h"
#include "cpialink.h"
#endif

#include "cpia_cmd.h"

#include "debuglvl.h"
#include "tracemsg.h"


BOOL CPIA_CmdInit( void ) {

 	return( TRUE );
}

void CPIA_CmdFini( void ){

	return;
}


/* ====================================================================	*/
/* 				CPIA Module Procedures	 		  						*/
/*																		*/
BOOL CPIA_GetVersion( PLINK pLink, BYTE *FW_Version, BYTE *FW_Revision,
				 BYTE *VC_Version, BYTE *VC_Revision ) {
	BOOL rtn;
	DATA_BUFFER Data;
	rtn = CLINK_TransferMsg( pLink, CPIA_ADDR, GET_CPIA_VERSION, NULL, TRUE, DATA_LEN, &Data );
	if ( rtn ) {
		*FW_Version = Data.In.GetCPIAVersion.FW_Version;
		*FW_Revision = Data.In.GetCPIAVersion.FW_Revision;
		*VC_Version = Data.In.GetCPIAVersion.VC_Version;
		*VC_Revision = Data.In.GetCPIAVersion.VC_Revision;
	}
	return( rtn );
}

BOOL CPIA_GetPnPID( PLINK pLink, WORD *VendorID, WORD *ProductID, WORD *DeviceID ){
	BOOL rtn;
	DATA_BUFFER Data;
	rtn = CLINK_TransferMsg( pLink, CPIA_ADDR, GET_PNP_ID, NULL, TRUE, DATA_LEN, &Data );
	if ( rtn ) {
		*VendorID = ((WORD)Data.In.GetPnPID.VendorMSB<<8) + Data.In.GetPnPID.VendorLSB;
		*ProductID = ((WORD)Data.In.GetPnPID.ProductMSB<<8) + Data.In.GetPnPID.ProductLSB;
		*DeviceID = ((WORD)Data.In.GetPnPID.DeviceMSB<<8) + Data.In.GetPnPID.DeviceLSB;
	}
	return( rtn );
}

BOOL CPIA_GetCameraStatus( PLINK pLink, BYTE *SystemState, BYTE *VPCtrlState,
						   BYTE *StreamState, BYTE *FatalError,
						   BYTE *CmdError,    BYTE *DebugFlags,
						   BYTE *VPStatus,    BYTE *ErrorCode ) {
	BOOL rtn;
	DATA_BUFFER Data;

	rtn = CLINK_TransferMsg( pLink, CPIA_ADDR, GET_CAMERA_STATUS, NULL, TRUE, DATA_LEN, &Data );

	if ( rtn ) {
		if ( SystemState ) *SystemState = Data.In.GetCameraStatus.SystemState;
		if ( VPCtrlState ) *VPCtrlState = Data.In.GetCameraStatus.VPCtrlState;
		if ( StreamState ) *StreamState = Data.In.GetCameraStatus.StreamState;
		if ( FatalError) *FatalError = Data.In.GetCameraStatus.FatalError;
		if ( CmdError ) *CmdError = Data.In.GetCameraStatus.CmdError;
		if ( DebugFlags ) *DebugFlags = Data.In.GetCameraStatus.DebugFlags;
		if ( VPStatus ) *VPStatus = Data.In.GetCameraStatus.VPStatus;
		if ( ErrorCode ) *ErrorCode = Data.In.GetCameraStatus.ErrorCode;
 	}

 	return( rtn );
}

BOOL CPIA_GotoHiPower( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, CPIA_ADDR, GOTO_HI_POWER, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_GotoLoPower( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, CPIA_ADDR, GOTO_LO_POWER, NULL, FALSE, 0, NULL );
	return( rtn );
} 

BOOL CPIA_GotoSuspend( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, CPIA_ADDR, GOTO_SUSPEND, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_GotoPassThrough( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, CPIA_ADDR, GOTO_PASS_THROUGH, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_ModifyCameraStatus( PLINK pLink, BYTE StatusByte, BYTE AndMask, BYTE OrMask ) {
	BOOL rtn;
	CMD_DATA CmdData;
	
	CmdData.ModifyCameraStatus.StatusByte = StatusByte;
	CmdData.ModifyCameraStatus.AndMask = AndMask;
	CmdData.ModifyCameraStatus.OrMask = OrMask;

	rtn = CLINK_TransferMsg( pLink, CPIA_ADDR, MODIFY_CAMERA_STATUS, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

/* ====================================================================	*/


/* ====================================================================	*/
/* 				SYSTEM Module Procedures 		  						*/
/*																		*/
BOOL CPIA_ReadVCRegs( PLINK pLink, BYTE Addr0, BYTE Addr1, BYTE Addr2, BYTE Addr3,
						BYTE *Reg0, BYTE *Reg1, BYTE *Reg2, BYTE *Reg3 ) {
	BOOL rtn;
	CMD_DATA CmdData;
	DATA_BUFFER Data;
	
	CmdData.ReadVCRegs.Addr0 = Addr0;
	CmdData.ReadVCRegs.Addr1 = Addr1;
	CmdData.ReadVCRegs.Addr2 = Addr2;
	CmdData.ReadVCRegs.Addr3 = Addr3;

	rtn = CLINK_TransferMsg( pLink, SYSTEM_ADDR, READ_VC_REGS, &CmdData, TRUE, DATA_LEN, &Data );
	if ( rtn ) {
		if ( Reg0 ) *Reg0 = Data.In.ReadVCRegs.Reg0;
	   	if ( Reg1 ) *Reg1 = Data.In.ReadVCRegs.Reg1;
	   	if ( Reg2 ) *Reg2 = Data.In.ReadVCRegs.Reg2;
	   	if ( Reg3 ) *Reg3 = Data.In.ReadVCRegs.Reg3;
	}
	return( rtn );
}

BOOL CPIA_WriteVCReg( PLINK pLink, BYTE Addr, BYTE AndMask, BYTE OrMask ) {
	BOOL rtn;
	CMD_DATA CmdData;
	
	CmdData.WriteVCReg.Addr = Addr;
	CmdData.WriteVCReg.AndMask = AndMask;
	CmdData.WriteVCReg.OrMask = OrMask;

	rtn = CLINK_TransferMsg( pLink, SYSTEM_ADDR, WRITE_VC_REG, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_ReadMCPorts( PLINK pLink, BYTE *Port0, BYTE *Port1, BYTE *Port2, BYTE *Port3 ) {
	BOOL rtn;
	DATA_BUFFER Data;
	
	rtn = CLINK_TransferMsg( pLink, SYSTEM_ADDR, READ_MC_PORTS, NULL, TRUE, DATA_LEN, &Data );
	if ( rtn ) {
		if ( Port0 ) *Port0 = Data.In.ReadMCPorts.Port0;
		if ( Port1 ) *Port1 = Data.In.ReadMCPorts.Port1;
		if ( Port2 ) *Port2 = Data.In.ReadMCPorts.Port2;
		if ( Port3 ) *Port3 = Data.In.ReadMCPorts.Port3;
	}
	return( rtn );
}

BOOL CPIA_WriteMCPort( PLINK pLink, BYTE Port, BYTE AndMask, BYTE OrMask ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.WriteMCPort.Port = Port;
	CmdData.WriteMCPort.AndMask = AndMask;
	CmdData.WriteMCPort.OrMask = OrMask;
	rtn = CLINK_TransferMsg( pLink, SYSTEM_ADDR, WRITE_MC_PORT, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetBaudRate( PLINK pLink, DWORD BaudRate ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetBaudRate.BaudRate = (BYTE)(BaudRate / (DWORD)1200);
	rtn = CLINK_TransferMsg( pLink, SYSTEM_ADDR, SET_BAUD_RATE, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetECPTiming( PLINK pLink, BOOL SlowTiming ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetECPTiming.SlowTiming = SlowTiming;
   	rtn = CLINK_TransferMsg( pLink, SYSTEM_ADDR, SET_ECP_TIMING, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_ReadIDATA( PLINK pLink, BYTE Addr0, BYTE Addr1, BYTE Addr2, BYTE Addr3,
						BYTE *Byte0, BYTE *Byte1, BYTE *Byte2, BYTE *Byte3 ) {
	BOOL rtn;
	CMD_DATA CmdData;
	DATA_BUFFER Data;
	
	CmdData.ReadIDATA.Addr0 = Addr0;
	CmdData.ReadIDATA.Addr1 = Addr1;
	CmdData.ReadIDATA.Addr2 = Addr2;
	CmdData.ReadIDATA.Addr3 = Addr3;

	rtn = CLINK_TransferMsg( pLink, SYSTEM_ADDR, READ_IDATA, &CmdData, TRUE, DATA_LEN, &Data );
	if ( rtn ) {
		if ( Byte0 ) *Byte0 = Data.In.ReadIDATA.Byte0;
	   	if ( Byte1 ) *Byte1 = Data.In.ReadIDATA.Byte1;
	   	if ( Byte2 ) *Byte2 = Data.In.ReadIDATA.Byte2;
	   	if ( Byte3 ) *Byte3 = Data.In.ReadIDATA.Byte3;
	}
	return( rtn );
}


BOOL CPIA_WriteIDATA( PLINK pLink, BYTE Addr, BYTE AndMask, BYTE OrMask ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.WriteIDATA.Addr = Addr;
	CmdData.WriteIDATA.AndMask = AndMask;
	CmdData.WriteIDATA.OrMask = OrMask;
	rtn = CLINK_TransferMsg( pLink, SYSTEM_ADDR, WRITE_IDATA, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_GenericCall( PLINK pLink, WORD CallAddr ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.GenericCall.AddrLo = LOBYTE(CallAddr);
	CmdData.GenericCall.AddrHi = HIBYTE(CallAddr);
	rtn = CLINK_TransferMsg( pLink, SYSTEM_ADDR, GENERIC_CALL, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SerialStart( PLINK pLink ) {
  	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, SYSTEM_ADDR, SERIAL_START, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SerialStop( PLINK pLink ) {
  	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, SYSTEM_ADDR, SERIAL_STOP, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SerialWrite( PLINK pLink, BYTE WriteData ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SerialWrite.WriteData = WriteData;
	rtn = CLINK_TransferMsg( pLink, SYSTEM_ADDR, SERIAL_WRITE, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SerialRead( PLINK pLink, BOOL AckRead, BYTE *ReadData ) {
	BOOL rtn;
	CMD_DATA CmdData;
	DATA_BUFFER Data;
	CmdData.SerialRead.AckRead = AckRead;
	rtn = CLINK_TransferMsg( pLink, SYSTEM_ADDR, SERIAL_READ, &CmdData, TRUE, DATA_LEN, &Data );
	if ( rtn && ReadData ) *ReadData = Data.In.SerialRead.DataRead;
	return( rtn );
}


/* ====================================================================	*/
/* 				VP_CTRL Module Procedures 		  						*/
/*																		*/
BOOL CPIA_GetVPVersion( PLINK pLink, BYTE *VP_Version, BYTE *VP_Revision,
	 				  BYTE *Sensor_Version, BYTE *Sensor_Revision ) {
	BOOL rtn;
	DATA_BUFFER Data;
	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, GET_VP_VERSION, NULL, TRUE, DATA_LEN, &Data );
	if ( rtn ) {
		if ( VP_Version ) *VP_Version = Data.In.GetVPVersion.VP_Version;
		if ( VP_Revision ) *VP_Revision = Data.In.GetVPVersion.VP_Revision;

		/* 
			Stuff below to fix a bug in CPIA 
		*/
//		if ( Sensor_Version ) *Sensor_Version = Data.In.GetVPVersion.CamDeviceLo;
		if ( Sensor_Version ) *Sensor_Version = 0x19;
		if ( Sensor_Revision ) *Sensor_Revision = Data.In.GetVPVersion.CamDeviceHi;
	}
	return( rtn );
}

BOOL CPIA_SetColourParams( PLINK pLink, BYTE Brightness, BYTE Contrast, BYTE Saturation ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetColourParams.Brightness = Brightness;
	CmdData.SetColourParams.Contrast = Contrast;
	CmdData.SetColourParams.Saturation = Saturation;
	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, SET_COLOUR_PARAMS, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetExposure( PLINK pLink, BYTE GainMode, BYTE ExpMode, BYTE CompMode, BYTE CentreWeight,
					   BYTE Gain, BYTE FineExp, BYTE CoarseExpLo, BYTE CoarseExpHi, 
					   BYTE RedComp, BYTE Green1Comp, BYTE Green2Comp, BYTE BlueComp ) {
	BOOL rtn;
	CMD_DATA CmdData;
	DATA_BUFFER Data;
	
	CmdData.SetExposure.GainMode = GainMode;
	CmdData.SetExposure.ExpMode = ExpMode;
	CmdData.SetExposure.CompMode = CompMode;
	CmdData.SetExposure.CentreWeight = CentreWeight;

	Data.Out.SetExposure.Gain = Gain;
	Data.Out.SetExposure.FineExp = FineExp;
	Data.Out.SetExposure.CoarseExpLo = CoarseExpLo;
	Data.Out.SetExposure.CoarseExpHi = CoarseExpHi;
	Data.Out.SetExposure.RedComp = RedComp;
	Data.Out.SetExposure.Green1Comp = Green1Comp;
	Data.Out.SetExposure.Green2Comp = Green2Comp;
	Data.Out.SetExposure.BlueComp = BlueComp;

	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, SET_EXPOSURE, &CmdData, FALSE, DATA_LEN, &Data );
	return( rtn );
}

BOOL CPIA_SetColourBalance( PLINK pLink, BYTE BalanceMode, BYTE RedGain,
							BYTE GreenGain, BYTE BlueGain ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetColourBalance.BalanceMode = BalanceMode;
	CmdData.SetColourBalance.RedGain = RedGain;
	CmdData.SetColourBalance.GreenGain = GreenGain;
	CmdData.SetColourBalance.BlueGain = BlueGain;
	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, SET_COLOUR_BALANCE, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetSensorFPS( PLINK pLink, BYTE SensorClkDivisor, BYTE BaseRate ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetSensorFPS.SensorClkDivisor = SensorClkDivisor;
	CmdData.SetSensorFPS.SensorBaseRate = BaseRate;
	CmdData.SetSensorFPS.PhaseOverride = 0;
	CmdData.SetSensorFPS.NewPhase = 0;
	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, SET_SENSOR_FPS, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetVPDefaults( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, SET_VP_DEFAULTS, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetApcor( PLINK pLink, BYTE ApCorGain1, BYTE ApCorGain2, BYTE ApCorGain4, BYTE ApCorGain8 ){
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetApcor.gain1 = ApCorGain1;
	CmdData.SetApcor.gain2 = ApCorGain2;
	CmdData.SetApcor.gain4 = ApCorGain4;
	CmdData.SetApcor.gain8 = ApCorGain8;
	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, SET_APCOR, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_FlickerCtrl( PLINK pLink, BYTE FlickerMode, BYTE CoarseJump, BYTE AllowableOverExposure ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.FlickerCtrl.FlickerMode = FlickerMode;
	CmdData.FlickerCtrl.CoarseJump = CoarseJump;
	CmdData.FlickerCtrl.AllowableOverExposure = AllowableOverExposure;
	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, FLICKER_CTRL, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetVLOffset( PLINK pLink, BYTE VLOffsetGain1, BYTE VLOffsetGain2, BYTE VLOffsetGain4, BYTE VLOffsetGain8 ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetVLOffset.gain1 = VLOffsetGain1;
	CmdData.SetVLOffset.gain2 = VLOffsetGain2;
	CmdData.SetVLOffset.gain4 = VLOffsetGain4;
	CmdData.SetVLOffset.gain8 = VLOffsetGain8;
	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, SET_VLOFFSET, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_GetColourParams( PLINK pLink, BYTE *Brightness, BYTE *Contrast, BYTE *Saturation ) {
	BOOL rtn;
	DATA_BUFFER Data;
	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, GET_COLOUR_PARAMS, NULL, TRUE, DATA_LEN, &Data );
	if ( rtn ) {
		*Brightness = Data.In.GetColourParams.Brightness;
		*Contrast = Data.In.GetColourParams.Contrast;
		*Saturation = Data.In.GetColourParams.Saturation;
	}
	return( rtn );
}

BOOL CPIA_GetColourBalance( PLINK pLink, BYTE *RedGain, BYTE *GreenGain, BYTE *BlueGain ) {
	BOOL rtn;
	DATA_BUFFER Data;
	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, GET_COLOUR_BALANCE, NULL, TRUE, DATA_LEN, &Data );
	if ( rtn ) {
		*RedGain = Data.In.GetColourBalance.RedGain;
		*GreenGain = Data.In.GetColourBalance.GreenGain;
		*BlueGain = Data.In.GetColourBalance.BlueGain;
	}
	return( rtn );
}


BOOL CPIA_GetExposure( PLINK pLink, BYTE *Gain, BYTE *FineExp, BYTE *CoarseExpLo, BYTE *CoarseExpHi, 
					   		BYTE *RedComp, BYTE *Green1Comp, BYTE *Green2Comp, BYTE *BlueComp ) {
	BOOL rtn;
	DATA_BUFFER Data;
	
	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, GET_EXPOSURE, NULL, TRUE, DATA_LEN, &Data );

	if ( rtn ) {
		if ( Gain ) *Gain = Data.In.GetExposure.Gain;
		if ( FineExp ) *FineExp = Data.In.GetExposure.FineExp;
		if ( CoarseExpLo ) *CoarseExpLo = Data.In.GetExposure.CoarseExpLo;
		if ( CoarseExpHi ) *CoarseExpHi = Data.In.GetExposure.CoarseExpHi;
		if ( RedComp ) *RedComp = Data.In.GetExposure.RedComp;
		if ( Green1Comp ) *Green1Comp = Data.In.GetExposure.Green1Comp;
		if ( Green2Comp ) *Green2Comp = Data.In.GetExposure.Green2Comp;
		if ( BlueComp ) *BlueComp = Data.In.GetExposure.BlueComp;
	}
	return( rtn );
}

BOOL CPIA_SetSensorMatrix( PLINK pLink, BYTE Matrix[9] ) {
	BOOL rtn;
	CMD_DATA CmdData;
	DATA_BUFFER Data;

	CmdData.SetSensorMatrix.m00 = Matrix[0];
	CmdData.SetSensorMatrix.m01 = Matrix[1];
 	CmdData.SetSensorMatrix.m02 = Matrix[2];
	CmdData.SetSensorMatrix.m10 = Matrix[3];

	Data.Out.SetSensorMatrix.m11 = Matrix[4];
	Data.Out.SetSensorMatrix.m12 = Matrix[5];
	Data.Out.SetSensorMatrix.m20 = Matrix[6];
	Data.Out.SetSensorMatrix.m21 = Matrix[7];
	Data.Out.SetSensorMatrix.m22 = Matrix[8];

	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, SET_SENSOR_MATRIX, &CmdData, FALSE, DATA_LEN, &Data );
	return( rtn );
}

BOOL CPIA_ColourBars( PLINK pLink, BOOL EnableColourBars ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.ColourBars.EnableColourBars = EnableColourBars;
	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, COLOUR_BARS, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_ReadVPRegs( PLINK pLink, BYTE Addr0, BYTE Addr1, BYTE Addr2, BYTE Addr3,
						BYTE *Reg0, BYTE *Reg1, BYTE *Reg2, BYTE *Reg3 ) {
	BOOL rtn;
	CMD_DATA CmdData;
	DATA_BUFFER Data;
	
	CmdData.ReadVPRegs.VPAddr0 = Addr0;
	CmdData.ReadVPRegs.VPAddr1 = Addr1;
	CmdData.ReadVPRegs.VPAddr2 = Addr2;
	CmdData.ReadVPRegs.VPAddr3 = Addr3;

	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, READ_VP_REGS, &CmdData, TRUE, DATA_LEN, &Data );
	if ( rtn ) {
		if ( Reg0 ) *Reg0 = Data.In.ReadVPRegs.VPReg0;
		if ( Reg1 ) *Reg1 = Data.In.ReadVPRegs.VPReg1;
		if ( Reg2 ) *Reg2 = Data.In.ReadVPRegs.VPReg2;
		if ( Reg3 ) *Reg3 = Data.In.ReadVPRegs.VPReg3;
	}
	return( rtn );
}

BOOL CPIA_WriteVPReg( PLINK pLink, BYTE Addr, BYTE AndMask, BYTE OrMask ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.WriteVPReg.VPAddr = Addr;
	CmdData.WriteVPReg.AndMask = AndMask;
	CmdData.WriteVPReg.OrMask = OrMask;
	rtn = CLINK_TransferMsg( pLink, VP_CTRL_ADDR, WRITE_VP_REG, &CmdData, FALSE, 0, NULL );
	return( rtn );
}
/* ====================================================================	*/



/* ====================================================================	*/
/* 				CAPTURE Module Procedures 		  						*/
/*																		*/
BOOL CPIA_GrabFrame( PLINK pLink, BYTE StreamStartLine ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.GrabFrame.StreamStartLine = StreamStartLine;
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, GRAB_FRAME, &CmdData, FALSE, 0, NULL );
	return( rtn );
}
      
BOOL CPIA_UploadFrame( PLINK pLink, BYTE ForceUpload ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.UploadFrame.ForceUpload = ForceUpload;
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, UPLOAD_FRAME, &CmdData, FALSE, 0, NULL );
	return( rtn );
}
      
BOOL CPIA_SetGrabMode( PLINK pLink, BYTE ContinuousGrab ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetGrabMode.ContinuousGrab = ContinuousGrab;
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, SET_GRAB_MODE, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_InitStreamCap( PLINK pLink, BYTE SkipFrames, BYTE StreamStartLine ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.InitStreamCap.SkipFrames = SkipFrames; 
	CmdData.InitStreamCap.StreamStartLine = StreamStartLine; 
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, INIT_STREAM_CAP, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_FiniStreamCap( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, FINI_STREAM_CAP, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_StartStreamCap( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, START_STREAM_CAP, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_StopStreamCap( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, STOP_STREAM_CAP, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetFormat( PLINK pLink, BYTE VideoSize, BYTE SubSampling, BYTE YUVOrder ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetFormat.VideoSize = VideoSize;
	CmdData.SetFormat.SubSampling = SubSampling;
	CmdData.SetFormat.YUVOrder = YUVOrder;
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, SET_FORMAT, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetROI( PLINK pLink, BYTE ColStart, BYTE ColEnd, BYTE RowStart, BYTE RowEnd ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetROI.ColStart = ColStart;
	CmdData.SetROI.ColEnd = ColEnd;
	CmdData.SetROI.RowStart = RowStart;
	CmdData.SetROI.RowEnd = RowEnd;
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, SET_ROI, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetCompression( PLINK pLink, BYTE CompMode, BYTE Decimation ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetCompression.CompMode = CompMode;
	CmdData.SetCompression.Decimation = Decimation;
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, SET_COMPRESSION, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetCompressionTarget( PLINK pLink, BYTE FRTargetting, BYTE TargetFR, BYTE TargetQ ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetCompressionTarget.FRTargetting = FRTargetting;
	CmdData.SetCompressionTarget.TargetFR = TargetFR;
	CmdData.SetCompressionTarget.TargetQ = TargetQ;
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, SET_COMPRESSION_TARGET, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetYUVThresh( PLINK pLink, BYTE YThresh, BYTE UVThresh ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetYUVThresh.YThresh = YThresh;
	CmdData.SetYUVThresh.UVThresh = UVThresh;
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, SET_YUV_THRESH, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetCompressionParams( PLINK pLink, BYTE Hysteresis, BYTE ThreshMax, BYTE SmallStep, BYTE LargeStep, 
								BYTE DecimationHysteresis, BYTE FRDiffStepThresh, BYTE QDiffStepThresh,
								BYTE DecimationThreshMod ) {
	BOOL rtn;
	DATA_BUFFER Data;

	/* more params could be passed in the CmdData structure here */
		
	Data.Out.SetCompressionParams.Hysteresis = Hysteresis;
	Data.Out.SetCompressionParams.ThreshMax = ThreshMax;
	Data.Out.SetCompressionParams.SmallStep = SmallStep;
	Data.Out.SetCompressionParams.LargeStep = LargeStep;
	Data.Out.SetCompressionParams.DecimationHysteresis = DecimationHysteresis;
	Data.Out.SetCompressionParams.FRDiffStepThresh = FRDiffStepThresh;
	Data.Out.SetCompressionParams.QDiffStepThresh = QDiffStepThresh;
	Data.Out.SetCompressionParams.DecimationThreshMod = DecimationThreshMod;

	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, SET_COMPRESSION_PARAMS, NULL, FALSE, DATA_LEN, &Data );
	return( rtn );
}

BOOL CPIA_DiscardFrame( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, DISCARD_FRAME, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_GrabReset( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, CAPTURE_ADDR, GRAB_RESET, NULL, FALSE, 0, NULL );
	return( rtn );
}
/* ====================================================================	*/


/* ====================================================================	*/
/* 				DEBUG Module Procedures 		  						*/
/*																		*/
BOOL CPIA_OutputRS232( PLINK pLink, char String[8] ) {
	BOOL rtn;
	DATA_BUFFER Data;
	strncpy( (char *)&Data.Raw[0], &String[0], 8 );
	rtn = CLINK_TransferMsg( pLink, DEBUG_ADDR, OUTPUT_RS232, NULL, FALSE, DATA_LEN, &Data );
	return( FALSE );
}

BOOL CPIA_AbortProcess( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, DEBUG_ADDR, ABORT_PROCESS, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_NullCmd( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, DEBUG_ADDR, NULL_CMD, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_SetDRAMPage( PLINK pLink, WORD StartPage, WORD EndPage ) {
	BOOL rtn;
	CMD_DATA CmdData;
	CmdData.SetDRAMPage.StartPageLo = LOBYTE(StartPage);
	CmdData.SetDRAMPage.StartPageHi = HIBYTE(StartPage);
	CmdData.SetDRAMPage.EndPageLo = LOBYTE(EndPage);
	CmdData.SetDRAMPage.EndPageHi = HIBYTE(EndPage);
	rtn = CLINK_TransferMsg( pLink, DEBUG_ADDR, SET_DRAM_PAGE, &CmdData, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_StartDRAMUpload( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, DEBUG_ADDR, START_DRAM_UPLOAD, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_StartDummyStream( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, DEBUG_ADDR, START_DUMMY_STREAM, NULL, FALSE, 0, NULL );
	return( rtn );
}

BOOL CPIA_AbortStream( PLINK pLink ) {
	BOOL rtn;
	rtn = CLINK_TransferMsg( pLink, DEBUG_ADDR, ABORT_STREAM, NULL, FALSE, 0, NULL );
	return( rtn );
}
/* ====================================================================	*/






/* ====================================================================	*/
/* 				MACRO Procedures 				  						*/
/*																		*/

#ifdef PPLINK

#define PAGES	16
BOOL CPIA_MACRO_TestStream( PLINK pLink, WORD StartPage, WORD EndPage, BYTE __huge *Buffer ) {
	BOOL rtn;
	DWORD actual_bytes;
	BYTE stream_state, status;

	rtn = CPIA_StartDummyStream( pLink );
	if ( !rtn ) return( rtn );

	rtn = CPIA_GetCameraStatus( pLink, NULL, NULL, &stream_state, NULL, NULL, NULL, NULL, NULL );
	if ( rtn != TRUE ) {
		printf( "CPIA_TestStream: GetCameraStatus failed\n" );
		return( FALSE );
	}
	if ( stream_state != STREAM_READY ) printf( "CPIA_TestStream: StreamState = %d\n", stream_state );

   	rtn = CLINK_StartUpload( pLink, FALSE, FALSE, (DWORD)(EndPage-StartPage)*256, Buffer, (DWORD)4*1024, FALSE );
	do {
		rtn = CLINK_GetUploadStatus( pLink, &status, &actual_bytes );

		switch( status ) {
		case UPLOAD_IDLE: printf( "\nIDLE" ); break;
		case UPLOAD_STARTING: printf( " S" ); break;
		case UPLOAD_CHUNK_START: printf( " CS" ); break;
		case UPLOAD_CHUNK_END: printf( " CE" ); break;
		case UPLOAD_FINISHING: printf( "\nFINISHING" ); break;
		case UPLOAD_FAILED: printf( "\nFAILED" ); break;
		}

		if ( status == UPLOAD_FAILED ) break;
	} while( status != UPLOAD_IDLE );

	if ( rtn ) {
		if ( status == UPLOAD_FAILED ) {
			CLINK_AbortUpload( pLink );
	 		printf("CPIA_TestStream: GetUploadStatus reported UPLOAD_FAILED\n" );
			rtn = FALSE;
		}
		if ( status == UPLOAD_IDLE ) {		
		 	if ( actual_bytes != (DWORD)(EndPage-StartPage)*256 ) {
		 		printf("CPIA_TestStream: UploadStreamData uploaded the wrong number of bytes\n" );
				rtn = FALSE;
	 		}			
		}
	}

	return( rtn );
}

#define STREAM_CHUNK_SIZE	4096
BOOL CPIA_MACRO_UploadDRAM( PLINK pLink, WORD StartPage, WORD EndPage, BYTE __huge *Buffer ) {
	BOOL rtn;
	DWORD dummy;

	rtn = CPIA_SetDRAMPage( pLink, StartPage, EndPage );
	if ( !rtn ) return( rtn );

	rtn = CPIA_StartDRAMUpload( pLink );
	if ( !rtn ) return( rtn );

    rtn = CLINK_UploadStreamData( pLink, FALSE, ((DWORD)EndPage-StartPage)*256, Buffer, &dummy, STREAM_CHUNK_SIZE );

	return( rtn );
}


BOOL CPIA_MACRO_DownloadDRAM( PLINK pLink, WORD StartPage, WORD EndPage, BYTE __huge *Buffer ) {
	BOOL rtn;
	WORD chunks, remainder;

	/* NOTE - SetDRAMPage.EndPageLo/Hi not used by DOWNLOAD_DRAM command */
	rtn = CPIA_SetDRAMPage( pLink, StartPage, 0 );
	if ( !rtn ) return( rtn );


	 /* NOTE - as the maximum size of data transfer associated with a 	*/
	/* is 64k-1, we here choose to send data down in 32k chunks.		*/

   	chunks = (EndPage-StartPage)/128;	/* ie ((End-Start)*256)/(32*1024) */
	remainder = (EndPage-StartPage)%128;

	while( chunks > 0 ) {
		rtn = CLINK_TransferMsg( pLink, DEBUG_ADDR, DOWNLOAD_DRAM, NULL, FALSE, 
									(WORD)32*1024, (DATA_BUFFER *) Buffer );
		if ( !rtn ) goto DownloadError;
		Buffer += (WORD)32*1024;
		chunks--;
	}
	if ( remainder ) {
		rtn = CLINK_TransferMsg( pLink, DEBUG_ADDR, DOWNLOAD_DRAM, NULL, FALSE, 
										remainder*256, (DATA_BUFFER *)Buffer );
		if ( !rtn ) goto DownloadError;
	}

	return( TRUE );

DownloadError:
	return( rtn );

}
#endif
/* ====================================================================	*/
/* ====================================================================	*/


