/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */





#ifdef __cplusplus
extern "C"
{
#endif

/* Initialisation Functions */
BOOL CPIA_CmdInit( void );
void CPIA_CmdFini( void );

/* CPIA Functions */
BOOL CPIA_GetVersion( PLINK pLink, BYTE *FW_Version, BYTE *FW_Revision,
	 				  BYTE *VC_Version, BYTE *VC_Revision );

BOOL CPIA_GetPnPID( PLINK pLink, WORD *VendorID, WORD *ProductID, WORD *DeviceID );

BOOL CPIA_GetCameraStatus( PLINK pLink, BYTE *SystemState, BYTE *GrabState,
						   				BYTE *StreamState, BYTE *FatalError,
						   				BYTE *CmdError,    BYTE *DebugFlags,
						   				BYTE *VPStatus,    BYTE *ErrorCode );

BOOL CPIA_GotoHiPower( PLINK pLink );
BOOL CPIA_GotoLoPower( PLINK pLink );
BOOL CPIA_GotoSuspend( PLINK pLink );
BOOL CPIA_GotoPassThrough( PLINK pLink );

BOOL CPIA_ModifyCameraStatus( PLINK pLink, BYTE StatusByte, BYTE AndMask, BYTE OrMask );


/* SYSTEM Functions */
BOOL CPIA_ReadVCRegs( PLINK pLink, BYTE Addr0, BYTE Addr1, BYTE Addr2, BYTE Addr3,
								   BYTE *Reg0, BYTE *Reg1, BYTE *Reg2, BYTE *Reg3 );
BOOL CPIA_WriteVCReg( PLINK pLink, BYTE Addr, BYTE AndMask, BYTE OrMask );

BOOL CPIA_ReadMCPorts( PLINK pLink, BYTE *Port0, BYTE *Port1, BYTE *Port2, BYTE *Port3 );
BOOL CPIA_WriteMCPort( PLINK pLink, BYTE Port, BYTE AndMask, BYTE OrMask );

BOOL CPIA_SetBaudRate( PLINK pLink, DWORD BaudRate );
BOOL CPIA_SetECPTiming( PLINK pLink, BOOL SlowTiming );

BOOL CPIA_ReadIDATA( PLINK pLink, BYTE Addr0, BYTE Addr1, BYTE Addr2, BYTE Addr3,
								  BYTE *Byte0, BYTE *Byte1, BYTE *Byte2, BYTE *Byte3 );

BOOL CPIA_WriteIDATA( PLINK pLink, BYTE Addr, BYTE AndMask, BYTE OrMask );

BOOL CPIA_GenericCall( PLINK pLink, WORD CallAddr );

BOOL CPIA_SerialStart( PLINK pLink );
BOOL CPIA_SerialStop( PLINK pLink );
BOOL CPIA_SerialWrite( PLINK pLink, BYTE WriteData );
BOOL CPIA_SerialRead( PLINK pLink, BOOL AckRead, BYTE *ReadData );


/* VP_CTRL Functions */
BOOL CPIA_GetVPVersion( PLINK pLink, BYTE *VP_Version, BYTE *VP_Revision,
  						BYTE *Sensor_Version, BYTE *Sensor_Revision );
BOOL CPIA_ResetFrameCounter( PLINK pLink );

BOOL CPIA_SetColourParams( PLINK pLink, BYTE Brightness, BYTE Contrast, BYTE Saturation );
BOOL CPIA_SetExposure( PLINK pLink, BYTE GainMode, BYTE ExpMode, BYTE CompMode, BYTE CentreWeight,
					   				BYTE Gain, BYTE FineExp, BYTE CoarseExpLo, BYTE CoarseExpHi, 
					   				BYTE RedComp, BYTE Green1Comp, BYTE Green2Comp, BYTE BlueComp );
BOOL CPIA_SetColourBalance( PLINK pLink, BYTE BalanceMode, BYTE RedGain,
										 BYTE GreenGain, BYTE BlueGain );
BOOL CPIA_SetSensorFPS( PLINK pLink, BYTE SensorClkDivisor, BYTE BaseRate );

BOOL CPIA_SetVPDefaults( PLINK pLink );
BOOL CPIA_SetApcor( PLINK pLink, BYTE ApCorGain1, BYTE ApCorGain2, BYTE ApCorGain4, BYTE ApCorGain8 );
BOOL CPIA_FlickerCtrl( PLINK pLink, BYTE FlickerMode, BYTE CoarseJump, BYTE AllowableOverExposure);
BOOL CPIA_SetVLOffset( PLINK pLink, BYTE VLOffsetGain1, BYTE VLOffsetGain2, BYTE VLOffsetGain4, BYTE VLOffsetGain8 );

BOOL CPIA_GetColourParams( PLINK pLink, BYTE *Brightness, BYTE *Contrast, BYTE *Saturation );
BOOL CPIA_GetColourBalance( PLINK pLink, BYTE *RedGain, BYTE *GreenGain, BYTE *BlueGain );

BOOL CPIA_GetExposure( PLINK pLink, BYTE *Gain, BYTE *FineExp, BYTE *CoarseExpLo, BYTE *CoarseExpHi, 
					   		BYTE *RedComp, BYTE *Green1Comp, BYTE *Green2Comp, BYTE *BlueComp );
BOOL CPIA_SetSensorMatrix( PLINK pLink, BYTE Matrix[9] );
BOOL CPIA_ColourBars( PLINK pLink, BOOL EnableColourBars );

BOOL CPIA_ReadVPRegs( PLINK pLink, BYTE Addr0, BYTE Addr1, BYTE Addr2, BYTE Addr3,
								   BYTE *Reg0, BYTE *Reg1, BYTE *Reg2, BYTE *Reg3 );
BOOL CPIA_WriteVPReg( PLINK pLink, BYTE Addr, BYTE AndMask, BYTE OrMask );


/* CAPTURE Functions */
BOOL CPIA_GrabFrame( PLINK pLink, BYTE InterruptLine );
BOOL CPIA_UploadFrame( PLINK pLink, BYTE ForceUpload );
BOOL CPIA_SetGrabMode( PLINK pLink, BYTE ContinuousGrab );

BOOL CPIA_InitStreamCap( PLINK pLink, BYTE SkipFrames, BYTE InterruptLine );
BOOL CPIA_FiniStreamCap( PLINK pLink );

BOOL CPIA_StartStreamCap( PLINK pLink );
BOOL CPIA_StopStreamCap( PLINK pLink );

BOOL CPIA_SetFormat( PLINK pLink,  BYTE VideoSize, BYTE SubSampling, BYTE YUVOrder );
BOOL CPIA_SetROI( PLINK pLink, BYTE ColStart, BYTE ColEnd, BYTE RowStart, BYTE RowEnd );
BOOL CPIA_SetCompression( PLINK pLink, BYTE CompMode, BYTE Decimation );
BOOL CPIA_SetCompressionTarget( PLINK pLink, BYTE FRTargetting, BYTE TargetFR, BYTE TargetQ );
BOOL CPIA_SetYUVThresh( PLINK pLink, BYTE YThresh, BYTE UVThresh );
BOOL CPIA_SetTargetFPS( PLINK pLink, BYTE TargetFPS );
BOOL CPIA_SetCompressionParams( PLINK pLink, BYTE Hysteresis, BYTE ThreshMax, BYTE SmallStep, BYTE LargeStep, 
								BYTE DecimationHysteresis, BYTE FRDiffStepThresh, BYTE QDiffStepThresh,
								BYTE DecimationThreshMod );
BOOL CPIA_DiscardFrame( PLINK pLink );
BOOL CPIA_GrabReset( PLINK pLink );


/* DEBUG Functions */
BOOL CPIA_OutputRS232( PLINK pLink, char String[8] );
BOOL CPIA_AbortProcess( PLINK pLink );
BOOL CPIA_SetDRAMPage( PLINK pLink, WORD StartPage, WORD EndPage );
BOOL CPIA_StartDRAMUpload( PLINK pLink );
BOOL CPIA_StartDummyStream( PLINK pLink );
BOOL CPIA_AbortStream( PLINK pLink );
BOOL CPIA_DownloadDRAM( PLINK pLink, WORD Packets, BYTE __huge *Buffer ); /* Packets is in terms of 8 byte packets */
BOOL CPIA_NullCmd( PLINK pLink );

#ifdef PPLINK
/* MACRO functions to access stream data */
BOOL CPIA_MACRO_TestStream( PLINK pLink, WORD StartPage, WORD EndPage, BYTE __huge *Buffer );
BOOL CPIA_MACRO_UploadDRAM( PLINK pLink, WORD StartPage, WORD EndPage, BYTE __huge *Buffer );
BOOL CPIA_MACRO_DownloadDRAM( PLINK pLink, WORD StartPage, WORD EndPage, BYTE __huge *Buffer );
#endif 

#ifdef __cplusplus
}
#endif

