/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */



#include <dos.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* next few bits needed for definition of GetTime() */
#ifdef _WINDOWS
#include <windows.h>
#include <mmsystem.h>	

#include "devload.h"
#include "cpia_api.h"
#endif

#ifdef DOS_APP
#include <time.h>
#endif

#include "vvl_defs.h"
#include "pport.h"

#include "packet.h"
#include "cmd.h"

#include "pplink.h"
#include "cpialink.h"



#define RETURN_ERROR_IF_NULL_LINK  if ( !pLink ) return( FALSE )

#define RETURN_ERROR_IF_DOING_UPLOAD  if ( pLink->UploadState != UPLOAD_IDLE ) return( FALSE )



/*--------------------------------------------------------------------------*/
/*	CLINK_Init																*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI CLINK_Init( void ) {
	int res;
	while ( !UnloadVxD("VVLCPIA") );
	
	res = LoadVxD("vvlcpia.vxd");

	if ( !res ) return( TRUE );
	else {
//		tracemsg( TRC_MSG, "Error loading vvlcpia.vxd : %s", GetVXDLDRStatusString( res ) );
		return( FALSE );
	}
}



/*--------------------------------------------------------------------------*/
/*	CLINK_Fini		 														*/
/*--------------------------------------------------------------------------*/
void  __export WINAPI CLINK_Fini( void ) {

	while ( !UnloadVxD("VVLCPIA") );

	return;
}


/*--------------------------------------------------------------------------*/
/*	PPLINK_CreateLink 														*/
/*--------------------------------------------------------------------------*/
PLINK  __export WINAPI PPLINK_CreateLink( BYTE PortNumber ) {
	PLINK plink;

#ifdef _WINDOWS
	HLOCAL hlink;
  	hlink = LocalAlloc( LPTR, sizeof( LINK ) );
	if ( hlink == 0 ) return( FALSE );
	plink = (PLINK)LocalLock( hlink );
	plink->HandleToThisInstance = hlink;
#else	
	plink = (PLINK)malloc( sizeof( LINK ) );
	if ( plink == NULL ) return( NULL );
#endif

	plink->LinkInitialised = FALSE;
	plink->PortNumber = PortNumber;
	plink->UploadState = UPLOAD_IDLE;

#ifdef _WINDOWS
	if ( !VVLCPIA_VXD_API_Init() ) {
		goto CreateFailed;
	}
	else {
		CPIA_RESOURCE_INFO pp_info;
		if ( !VVLCPIA_VXD_API_GetResources( plink->PortNumber, &pp_info ) ) {
			goto CreateFailed;
		}
		if ( pp_info.lptNo != plink->PortNumber ) {
			goto CreateFailed;
		}
		else {
			if ( !PP_CreatePPort( &plink->PPort, pp_info.IOaddr, pp_info.ECP, 
											  pp_info.irqNo, pp_info.dmaCh ) ) { 
				goto CreateFailed;
			}
			strncpy( plink->PortDescription, pp_info.deviceDesc, MAX_PORT_DESCRIPTION );

			if ( !PP_GetHWInfo( &plink->PPort, NULL, &plink->Type, NULL, NULL ) ) 
				goto CreateFailed;
	     	
	     	if ( plink->Type & DMA_PORT_TYPE ) {
				if ( SearchForFastDMAChannel( plink ) ) {
					plink->Type |= FAST_DMA_PORT_TYPE;						  	
					/* store the fact we've got Fast DMA in PPORT's port type variable	*/
					/* NOTE - PPORT does not use this flag								*/
					PP_SetHWInfo( &(plink->PPort), 0, plink->Type, 0, 0 ); 
				}  
			}
		}
	}
#else
	if ( !PP_DetectPortType( plink->PortNumber, &plink->PPort ) ) {
		goto CreateFailed;
	}
	if ( !PP_GetHWInfo( &plink->PPort, NULL, &plink->Type, NULL, NULL ) ) 
		goto CreateFailed;
#endif

	return( plink );

CreateFailed:
	PPLINK_DestroyLink( plink );
	return( NULL );
}
 

/*--------------------------------------------------------------------------*/
/*	PPLINK_DestroyLink 														*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI PPLINK_DestroyLink( PLINK pLink ) {

	RETURN_ERROR_IF_NULL_LINK;

	if ( pLink->LinkInitialised ) {
		PPLINK_DeinitialiseLink( pLink );
 	}

#ifdef _WINDOWS
	LocalUnlock( pLink->HandleToThisInstance );
	LocalFree( pLink->HandleToThisInstance );
#else
	free( pLink );
#endif

	return( TRUE );
}

 

/*--------------------------------------------------------------------------*/
/*	PPLINK_InitialiseLink													*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI PPLINK_InitialiseLink( PLINK pLink ) {

	RETURN_ERROR_IF_NULL_LINK;

	if ( !PP_InitialisePPort( &pLink->PPort ) ) goto InitialiseError;

	pLink->ChunkBufferSize = MAX_CHUNK_SIZE;

	/* allocate chunk buffer */
#ifdef _WINDOWS
   	if ( pLink->Type & DMA_PORT_TYPE ) {
		if ( !VVLCPIA_VXD_API_AllocDmaBuffer( (pLink->ChunkBufferSize-1), &pLink->ChunkBuffer, &pLink->ChunkPhysicalAddr ) )
			goto InitialiseError;

		if ( !PP_SetDMABuffer( &pLink->PPort, pLink->ChunkBuffer, pLink->ChunkPhysicalAddr, (pLink->ChunkBufferSize-1) ) )
			goto InitialiseError;

		if ( pLink->Type & FAST_DMA_PORT_TYPE ) {
			/* see if we can enable Type F DMA	*/
			if ( !SearchForFastDMAChannel( pLink ) || !EnableFastDMAChannel( pLink ) ) {
				/* couldn't get a Fast DMA channel so clear down the FAST_DMA_PORT_TYPE bit */
				(pLink->PPort).PortType &= ~FAST_DMA_PORT_TYPE;						  	
				pLink->Type &= ~FAST_DMA_PORT_TYPE;
			}
   			/* store whether we got, or not, the Fast DMA channel in PPORT's local variable	*/
			/* NOTE - PPORT doesn't actually not use this flag 								*/
			PP_SetHWInfo( &(pLink->PPort), 0, pLink->Type, 0, 0 ); 
 		}
	}
	else {
		/* not doing DMA so just allocate buffer as normal */
		pLink->ChunkHandle = GlobalAlloc( LPTR, pLink->ChunkBufferSize );
		if ( pLink->ChunkHandle == 0 ) goto InitialiseError;
    	pLink->ChunkBuffer = GlobalLock( pLink->ChunkHandle ) ;
    	pLink->ChunkPhysicalAddr = 0;	/* not used */
	}
#else
	pLink->ChunkBuffer = malloc( pLink->ChunkBufferSize );
	if ( pLink->ChunkBuffer == NULL ) goto InitialiseError;
   	pLink->ChunkPhysicalAddr = 0;	/* not used */
#endif

	pLink->UploadState = UPLOAD_IDLE;
	pLink->WaitForStreamReadyINT = FALSE;
 	pLink->DoBackgroundDMA = FALSE;

	pLink->StreamTimeoutPeriod = DEFAULT_STREAM_TIMEOUT;
	pLink->StreamTimeoutID = 0;
	pLink->ChunkTimeoutPeriod = DEFAULT_CHUNK_TIMEOUT;
	pLink->ChunkTimeoutID = 0;

	if ( pLink->Type & INT_PORT_TYPE ) {
		if ( !PP_SetPPCallback( &pLink->PPort, CallbackFromPP, PLINK_TO_DWORD( pLink ) ) )	{
	        printf("PPLINK_SelectLink: PP_SetPPCallback failed\n" );
			goto InitialiseError;
		}
		if ( !PP_EnablePPInterrupt( &pLink->PPort, TRUE ) ) {
        	printf("PPLINK_SelectLink: PP_EnablePPInterrupt failed\n" );
			goto InitialiseError;
		}
	 	if ( pLink->Type & DMA_PORT_TYPE ) {
			if ( !PP_SetDMACallback( &pLink->PPort, CallbackFromDMA, PLINK_TO_DWORD( pLink ) ) ) {
		        printf("PPLINK_SelectLink: PP_SetDMACallback failed\n" );
				goto InitialiseError;
			}
		}
#ifdef _WINDOWS
		/* initialise MMEDIA Timer Callback stuff 	*/
		{
			TIMECAPS time_caps;
			timeGetDevCaps( &time_caps, sizeof( time_caps ) );
			if ( time_caps.wPeriodMin > DEFAULT_TIMER_MIN_PERIOD ) 
				pLink->TimerMinPeriod = time_caps.wPeriodMin;
			else				
				pLink->TimerMinPeriod = DEFAULT_TIMER_MIN_PERIOD;
			timeBeginPeriod( pLink->TimerMinPeriod );
		}	
#endif
	}

	pLink->LinkInitialised = TRUE;

	return( TRUE );

InitialiseError:
	return( FALSE );
}


/*--------------------------------------------------------------------------*/
/*	PPLINK_DeinitialiseLink													*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI PPLINK_DeinitialiseLink( PLINK pLink ) {

	RETURN_ERROR_IF_NULL_LINK;

	if ( pLink->Type & FAST_DMA_PORT_TYPE ) {
		DisableFastDMAChannel( pLink );
	}

 	if ( pLink->Type & INT_PORT_TYPE ) {
#ifdef _WINDOWS
		/* de-initialise MMSYSTEM Timer Callback stuff 	*/
		if ( pLink->StreamTimeoutID ) {
			timeKillEvent( pLink->StreamTimeoutID );
			pLink->StreamTimeoutID = 0;
		}
		if ( pLink->ChunkTimeoutID ) {
			timeKillEvent( pLink->ChunkTimeoutID );
			pLink->ChunkTimeoutID = 0;
		}
		timeEndPeriod( pLink->TimerMinPeriod );
#endif
		PP_EnablePPInterrupt( &pLink->PPort, FALSE );
		PP_SetPPCallback( &pLink->PPort, NULL, 0 );
	}

	PP_ClosePPort( &pLink->PPort );

#ifdef _WINDOWS
   	if ( pLink->Type & DMA_PORT_TYPE ) {
		VVLCPIA_VXD_API_FreeDmaBuffer( pLink->ChunkBuffer );
    }
	else {
		if ( pLink->ChunkHandle ) {
		   GlobalUnlock( pLink->ChunkHandle ) ;
	       GlobalFree( pLink->ChunkHandle ) ;
		   pLink->ChunkHandle = 0;
		}
	}
#else
	if ( pLink->ChunkBuffer )
		free( pLink->ChunkBuffer );
	pLink->ChunkBuffer = NULL;
#endif

	pLink->LinkInitialised = FALSE;

	return( TRUE );
}
 

/*--------------------------------------------------------------------------*/
/*	PPLINK_SendWakeup 														*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI PPLINK_SendWakeup( PLINK pLink ) {
	BYTE VPP[8] = { VPP_PREAMBLE0, 
					VPP_PREAMBLE1,
					VPP_PREAMBLE2,
					VPP_PREAMBLE3,
					VPP_PREAMBLE4,
					VPP_PREAMBLE5,
					VPP_MANUAL_CMD,
					VPP_TERMINATOR };

	RETURN_ERROR_IF_NULL_LINK;

	RETURN_ERROR_IF_DOING_UPLOAD;

	PP_SendVPP( &pLink->PPort, 8, &VPP[0] );
	return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	PPLINK_Get1284DeviceID													*/
/*						   - use small MaxLength to check if camera alive	*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI PPLINK_Get1284DeviceID( PLINK pLink, char *DeviceID, WORD MaxLength ) { 	

	RETURN_ERROR_IF_NULL_LINK;

	RETURN_ERROR_IF_DOING_UPLOAD;

 	if ( !Get1284DeviceID( pLink, DeviceID, MaxLength ) ) 
 		return( FALSE );
	else 
		return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	PPLINK_ForceWatchdogReset												*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI PPLINK_ForceWatchdogReset( PLINK pLink ) {
	DWORD start;
	BYTE buff[4];

	RETURN_ERROR_IF_NULL_LINK;

	RETURN_ERROR_IF_DOING_UPLOAD;

	if ( !PP_SetTransferMode( &pLink->PPort, (REQUEST_DEVICE_ID | NIBBLE_TRANSFER) ) )
		 return( FALSE );

	if ( !PP_NibbleReadBuffer( &pLink->PPort, (BYTE __far *)buff, 4 ) ) 
		return( FALSE );

	/* now wait long enough to guarantee the CPIA Watchdog Timer will 	*/
	/* have timed out and reset the board. 							 	*/
    start = GetTime();                                        
	while( (((GetTime() - start)*1000)/(DWORD)CLOCKS_PER_SEC) < 1500); /* ie wait 1500mS */

	PP_EndTransferMode( &pLink->PPort );

	return( TRUE );
}
 
 
/*--------------------------------------------------------------------------*/
/*	PPLINK_GetPortInfo 														*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI PPLINK_GetPortInfo( PLINK pLink, WORD *PortAddr, BYTE *PortType, WORD *Timeout_mS ) {
	
	RETURN_ERROR_IF_NULL_LINK;

	if ( !PP_GetTimeout( &pLink->PPort, Timeout_mS ) ) return( FALSE );

	if ( !PP_GetHWInfo( &pLink->PPort, PortAddr, PortType, NULL, NULL ) ) 
		return( FALSE );
	else 
		return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	PPLINK_SetPortInfo 														*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI PPLINK_SetPortInfo( PLINK pLink, BYTE PortType, WORD Timeout_mS ) {

	RETURN_ERROR_IF_NULL_LINK;

	RETURN_ERROR_IF_DOING_UPLOAD;

	if ( pLink->LinkInitialised ) return( FALSE );	

	if ( !PP_SetTimeout( &pLink->PPort, Timeout_mS ) ) return( FALSE );

	if ( !PP_SetHWInfo( &pLink->PPort, 0, PortType, 0, 0 ) ) 
		return( FALSE );
	else {
	 	pLink->Type = PortType;
		return( TRUE );
	}
}


/*--------------------------------------------------------------------------*/
/*	PPLINK_GetHWInfo 														*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI PPLINK_GetHWInfo( PLINK pLink, BYTE *IntLine, 
										BYTE *DMAChannel, BYTE *AdditionalData ) {	
	RETURN_ERROR_IF_NULL_LINK;

	if ( AdditionalData ) *AdditionalData = 0;

	if ( !PP_GetHWInfo( &pLink->PPort, NULL, NULL, IntLine, DMAChannel ) ) 
		return( FALSE );
	else 
		return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	PPLINK_SetHWInfo 														*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI PPLINK_SetHWInfo( PLINK pLink, BYTE IntLine, 
										BYTE DMAChannel, BYTE AdditionalData ) {
	RETURN_ERROR_IF_NULL_LINK;

	RETURN_ERROR_IF_DOING_UPLOAD;

	if ( pLink->LinkInitialised ) return( FALSE );	

	if ( !PP_SetHWInfo( &pLink->PPort, 0, 0, IntLine, DMAChannel ) ) 
		return( FALSE );
	else 
		return( TRUE );
}

/*--------------------------------------------------------------------------*/
/*	CLINK_SetUploadMode														*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI CLINK_SetUploadMode( PLINK pLink, BOOL DoBackgroundUpload ) {

	RETURN_ERROR_IF_NULL_LINK;

	pLink->DoBackgroundDMA = DoBackgroundUpload;
	return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	CLINK_TransferMsg 														*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI CLINK_TransferMsg( PLINK pLink, BYTE Module, BYTE Proc, CMD_DATA *CmdData,
						BOOL DataReadFlag, WORD DataBytes, DATA_BUFFER *DataBuffer ) {

	WORD gDataPacketsPending;
	BYTE  Cmd[PACKET_LEN];
 	BYTE  __huge *DataPtr;

	RETURN_ERROR_IF_NULL_LINK;

	RETURN_ERROR_IF_DOING_UPLOAD;

 	Cmd[ CMD_TYPE ] = CAMERA_REQUEST;
	if ( DataReadFlag ) Cmd[ CMD_TYPE ] |= READ_DATA_REQUEST;
	
	Cmd[ CMD_DEST_ADDR ] = (BYTE)((Module & MODULE_ADDR_BITS) | (Proc & PROC_ADDR_BITS));

	if ( CmdData != NULL ) {
		Cmd[ CMD_D0 ] = CmdData->Raw[0];
		Cmd[ CMD_D1 ] = CmdData->Raw[1];
		Cmd[ CMD_D2 ] = CmdData->Raw[2];
		Cmd[ CMD_D3 ] = CmdData->Raw[3];
	}
	else {
		Cmd[ CMD_D0 ] = 0; Cmd[ CMD_D1 ] = 0;
		Cmd[ CMD_D2 ] = 0; Cmd[ CMD_D3 ] = 0;
	}

	Cmd[ CMD_DATA_BYTES_LO ] = (BYTE)(DataBytes&0xFF);
	Cmd[ CMD_DATA_BYTES_HI ] = (BYTE)(DataBytes>>8);

	DataPtr = (BYTE  __huge *)DataBuffer;

//	printf( "TM0: gPPIntCount = %d\n", gPPIntCount );
//	pLink->PPIntCount = 0;

	if ( !PP_SetTransferMode( &pLink->PPort, ECP_TRANSFER ) ) {
		printf( "TransferMsg: SetTransferMode failed\n" );
		goto TransferError;
	}
//	printf( "TM1: gPPIntCount = %d\n", gPPIntCount );
	if ( pLink->Type & ECP_PORT_TYPE ) {
		if ( !PP_ECPWriteBuffer( &pLink->PPort, (BYTE  __far *)Cmd, (DWORD)PACKET_LEN ) ) {
			printf( "TransferMsg: ECPWriteBuffer failed\n" );
			PP_EndTransferMode( &pLink->PPort );
			goto TransferError;
		}
	}
	else {
		if ( !PP_SimECPWriteBuffer( &pLink->PPort, (BYTE  __far *)Cmd, (DWORD)PACKET_LEN ) ) {
			printf( "TransferMsg: SimECPWriteBuffer failed\n" );
			PP_EndTransferMode( &pLink->PPort );
			goto TransferError;
		}
	}
	if( !PP_EndTransferMode( &pLink->PPort ) ) {
		printf( "TransferMsg: EndTransferMode failed\n" );
		goto TransferError;
	}

//	printf( "TM2: gPPIntCount = %d\n", gPPIntCount );

	if ( DataBytes !=0 ) {
		gDataPacketsPending = DataBytes / PACKET_LEN;

		while( gDataPacketsPending ) {

			if ( !DataReadFlag ) {
				/* write data packet to camera */	    
				if ( !PP_SetTransferMode( &pLink->PPort, ECP_TRANSFER ) ) {
					printf( "TransferMsg: SetTransferMode failed\n" );
					printf( "TransferMsg: gDataPacketsPending = %u\n", gDataPacketsPending );
					goto TransferError;
				}
				if ( pLink->Type & ECP_PORT_TYPE ) {
					if ( !PP_ECPWriteBuffer( &pLink->PPort, DataPtr, (DWORD)PACKET_LEN ) ) {
						printf( "TransferMsg: ECPWriteBuffer failed\n" );
						goto TransferError;
					}
				}
				else {
					if ( !PP_SimECPWriteBuffer( &pLink->PPort, DataPtr, (DWORD)PACKET_LEN ) ) {
						printf( "TransferMsg: SimECPWriteBuffer failed\n" );
						goto TransferError;
					}
				}
				if( !PP_EndTransferMode( &pLink->PPort ) ) {
					printf( "TransferMsg: EndTransferMode failed\n" );
					goto TransferError;
				}
			}
			else {
				/* read data packet from camera */	 
				if ( pLink->Type & ECP_PORT_TYPE ) {
					if ( !PP_SetTransferMode( &pLink->PPort, ECP_TRANSFER ) ) {
 						printf( "TransferMsg: SetTransferMode failed\n" );
						goto TransferError;
					}
					if ( !PP_Forward2Reverse( &pLink->PPort ) ) {
						PP_EndTransferMode( &pLink->PPort );
						goto TransferError;
					}
					if ( !PP_ECPReadBuffer( &pLink->PPort, DataPtr, (DWORD)PACKET_LEN ) ) {
						printf( "TransferMsg: ECPReadBuffer failed\n" );
						goto TransferError;
					}
					if ( !PP_Reverse2Forward( &pLink->PPort ) ) {
						PP_EndTransferMode( &pLink->PPort );
						goto TransferError;
					}
					if( !PP_EndTransferMode( &pLink->PPort ) ) {
						printf( "TransferMsg: EndTransferMode failed\n" );
						goto TransferError;
					}
				}
				else {
					if ( !PP_SetTransferMode( &pLink->PPort, NIBBLE_TRANSFER ) ) {
						printf( "TransferMsg: SetTransferMode failed\n" );
						goto TransferError;
					}
					if ( !PP_NibbleReadBuffer( &pLink->PPort, DataPtr, (DWORD)PACKET_LEN ) ) {
						printf( "TransferMsg: NibbleReadBuffer failed\n" );
						goto TransferError;
					}
					if( !PP_EndTransferMode( &pLink->PPort ) ) {
						printf( "TransferMsg: EndTransferMode failed\n" );
						goto TransferError;
					}
				}
			}
			gDataPacketsPending--;
			DataPtr += PACKET_LEN; 
		}
	}
	
	return( TRUE );

TransferError:
	return( FALSE );
}



/*--------------------------------------------------------------------------*/
/*	CLINK_UploadStreamData													*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI CLINK_UploadStreamData( PLINK pLink, BOOL EndOnFF, DWORD DataBytes,
												BYTE __huge *DataBuffer, DWORD *ActualBytes, DWORD StreamChunkSize ) {
	BOOL rtn;
	DWORD i, max_chunks;
	BYTE status;
	DWORD uploaded_bytes;
	BOOL old_do_background_DMA;

	RETURN_ERROR_IF_NULL_LINK;

	if ( pLink->UploadState != UPLOAD_IDLE ) return( FALSE );

	/* this routine is never to do real background DMA transfers */
	old_do_background_DMA = pLink->DoBackgroundDMA;
	pLink->DoBackgroundDMA = FALSE;

	rtn = CLINK_StartUpload( pLink, FALSE, FALSE, DataBytes, DataBuffer, (WORD)StreamChunkSize, EndOnFF ); 
	if ( ! rtn ) goto UploadError;
						
//	max_chunks =  (WORD)(DataBytes/StreamChunkSize)*2 + 2;
	max_chunks =  1000000;

	for( i = 0; i < max_chunks; i++ ) {
		if ( !CLINK_GetUploadStatus( pLink, &status, &uploaded_bytes ) ) goto UploadError;

		switch( status ) {
		case UPLOAD_IDLE: printf( "\nIDLE" ); break;
		case UPLOAD_STARTING: printf( " S" ); break;
		case UPLOAD_CHUNK_START: printf( " CS" ); break;
		case UPLOAD_CHUNK_END: printf( " CE" ); break;
		case UPLOAD_FINISHING: printf( "\nFINISHING" ); break;
		case UPLOAD_FAILED: printf( "\nFAILED" ); break;
		}

		if ( status == UPLOAD_IDLE ) break; 
		if ( status == UPLOAD_FAILED ) goto UploadError;
	}
	if ( !(status == UPLOAD_IDLE) ) goto UploadError; /* something went wrong ... */

	pLink->DoBackgroundDMA = old_do_background_DMA;

	/* we've succcessfully finished the transfer */
	*ActualBytes = uploaded_bytes;

	return( TRUE );

UploadError:
	CLINK_AbortUpload( pLink ); 
	pLink->DoBackgroundDMA = old_do_background_DMA;
	return( FALSE );
}



/*--------------------------------------------------------------------------*/
/* 	CLINK_StartUpload														*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI CLINK_StartUpload( PLINK pLink, BOOL UseStreamReadyINT, BOOL DoBackgroundUpload, 
										 DWORD MaxDataBytes, BYTE __huge *DataBuffer, DWORD StreamChunkSize,
										 BOOL EndOnFF ) {
 	BYTE status;
	DWORD actual_bytes;

	RETURN_ERROR_IF_NULL_LINK;

 	if ( pLink->UploadState != UPLOAD_IDLE ) return( FALSE );

	if ( (StreamChunkSize < 256) 
		  || (StreamChunkSize > MaxDataBytes) ) return( FALSE );
	
	if ( StreamChunkSize > (DWORD)pLink->ChunkBufferSize )
		StreamChunkSize = (DWORD)pLink->ChunkBufferSize;

	pLink->TargetBufferSize = MaxDataBytes;
	pLink->TargetBuffer = DataBuffer;
	pLink->StreamChunkSize = (WORD)StreamChunkSize;
	pLink->EndOnFF = EndOnFF;

	pLink->CurrTransferSize = 0;

	pLink->WaitForStreamReadyINT = FALSE;
	pLink->DoBackgroundDMA = DoBackgroundUpload;

	if ( !UseStreamReadyINT ) {
		pLink->StreamReadyDetected = TRUE;
 		pLink->UploadState = UPLOAD_STARTING;
		/* subsequent calls to CLINK_GetUploadStatus will initiate, control */
		/* and complete the transfer.									  	*/
  	}
   	else { 
   		/* we are to use the StreamReady INT to initiate the upload			*/
		/*																	*/
		/* first check to see if a StreamReady INT has already occured		*/ 
   		/* disable interrupts 												*/
		if ( pLink->StreamReadyDetected ) {
			pLink->UploadState = UPLOAD_STARTING;
			/* re-enable interrupts 		*/
			/* manually start the upload 	*/
			CLINK_GetUploadStatus( pLink, &status, &actual_bytes );
			if ( status != UPLOAD_CHUNK_START ) return( FALSE );
		}
		else {
			pLink->WaitForStreamReadyINT = TRUE;
			pLink->StreamTimedOut = FALSE;
#ifdef _WINDOWS
			pLink->StreamTimeoutID = timeSetEvent( pLink->StreamTimeoutPeriod, pLink->TimerMinPeriod,
										  StreamTimedOutCallback, PLINK_TO_DWORD( pLink ),
										  TIME_ONESHOT );
#endif		  
			pLink->UploadState = UPLOAD_STARTING;
			/* re-enable interrupts 		*/
			status = 0;			
		}
   	}	 

	return( TRUE );
}



/*--------------------------------------------------------------------------*/
/*	CLINK_GetUploadStatus													*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI CLINK_GetUploadStatus( PLINK pLink, BYTE *Status, DWORD *ActualBytes ) {
	BOOL rtn;

	RETURN_ERROR_IF_NULL_LINK;

	switch( pLink->UploadState ) {

	case UPLOAD_IDLE:
		/* do nothing */
 		break;

	case UPLOAD_STARTING:
		if ( !pLink->StreamReadyDetected ) {
			if ( pLink->StreamTimedOut ) {
				/* we didn't receive the StreamReady INT and have timed out	*/
				if ( pLink->StreamTimeoutID ) {
					timeKillEvent( pLink->StreamTimeoutID );
					pLink->StreamTimeoutID = 0;
				}
				pLink->UploadState = UPLOAD_FAILED;
			}
			else {
				/* things are ok - do nothing */
			} 
		}
		else {
			pLink->StreamReadyDetected = FALSE;
			if ( pLink->StreamTimeoutID ) {
				timeKillEvent( pLink->StreamTimeoutID );
				pLink->StreamTimeoutID = 0;
			}
 			if ( !OpenUploadStreamAndReadFirstByte( pLink, pLink->ChunkBuffer ) ) {
				pLink->UploadState = UPLOAD_FAILED;
			}
			else {
				*pLink->TargetBuffer = *(pLink->ChunkBuffer);
				pLink->CurrTransferSize = 1;
				pLink->UploadState = UPLOAD_CHUNK_START;
			}
 		}
		break;

	case UPLOAD_CHUNK_START: 			
		if ( (pLink->CurrTransferSize + pLink->StreamChunkSize) <= pLink->TargetBufferSize )
			pLink->CurrentChunkSize = pLink->StreamChunkSize;
		else 
			pLink->CurrentChunkSize = (WORD)(pLink->TargetBufferSize - pLink->CurrTransferSize);

		pLink->ChunkFinishedDetected = FALSE;
		pLink->ChunkTimedOut = FALSE;

		/* set up the timeout timer 	*/
		pLink->ChunkTimeoutID = timeSetEvent( pLink->ChunkTimeoutPeriod, pLink->TimerMinPeriod,
									  		   ChunkTimedOutCallback, PLINK_TO_DWORD( pLink ),
									  		   TIME_ONESHOT );
 		if ( !pLink->ChunkTimeoutID ) { 
				/* couldn't get a timer !! - bomb out ??? */
		}

 		if ( pLink->DoBackgroundDMA )
			rtn = ReadStreamChunk( pLink, FALSE ); 
		else 
			rtn = ReadStreamChunk( pLink, TRUE ); 

		if ( !rtn ) 
			pLink->UploadState = UPLOAD_FAILED;
		else
			pLink->UploadState = UPLOAD_CHUNK_END;

		break;

	case UPLOAD_CHUNK_END:
		if ( !pLink->ChunkFinishedDetected ) {

			if ( pLink->ChunkTimedOut ) {
 				if ( pLink->ChunkTimeoutID ) {
 					timeKillEvent( pLink->ChunkTimeoutID );
					pLink->ChunkTimeoutID = 0;
				}
				pLink->UploadState = UPLOAD_FAILED;
			}
			else {
				/* things are OK - leave UploadState as it is and return */
			} 
		}
		else {
			pLink->ChunkFinishedDetected = FALSE;

			/* kill the timeout timer */
			if ( pLink->ChunkTimeoutID ) {
				timeKillEvent( pLink->ChunkTimeoutID );
				pLink->ChunkTimeoutID = 0;
			}

			/* copy from the ChunkBuffer into the TargetBuffer */			
			memcpy( &pLink->TargetBuffer[ pLink->CurrTransferSize ],
					   pLink->ChunkBuffer, pLink->CurrentChunkSize );
		
			pLink->CurrTransferSize += (DWORD)pLink->CurrentChunkSize;

			if ( pLink->CurrTransferSize >= pLink->TargetBufferSize ) {
			   	pLink->UploadState = UPLOAD_FINISHING;
			}
			else {
 	   			if ( pLink->EndOnFF
 	   				 && (pLink->TargetBuffer[pLink->CurrTransferSize-4] == 0xFF)
					 && (pLink->TargetBuffer[pLink->CurrTransferSize-3] == 0xFF) 
					 && (pLink->TargetBuffer[pLink->CurrTransferSize-2] == 0xFF)
					 && (pLink->TargetBuffer[pLink->CurrTransferSize-1] == 0xFF) ) {
					pLink->UploadState = UPLOAD_FINISHING;
				}
			   	else {
			   		pLink->UploadState = UPLOAD_CHUNK_START;
				}
 			}	
		}

		break;

	case UPLOAD_FINISHING:
		pLink->StreamReadyDetected = FALSE;
		if ( !CloseUploadStream( pLink ) ) pLink->UploadState = UPLOAD_FAILED;
		else pLink->UploadState = UPLOAD_IDLE;
		break;
			
	case UPLOAD_FAILED:
		/* do nothing 		*/
		break;
	}
	
	if ( Status ) *Status = pLink->UploadState;
	if ( ActualBytes ) *ActualBytes = pLink->CurrTransferSize;

	return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	CLINK_AbortUpload 	aborts upload, sets state to UPLOAD_IDLE 			*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI CLINK_AbortUpload( PLINK pLink ) {

	RETURN_ERROR_IF_NULL_LINK;

	if ( pLink->ChunkTimedOut ) {
		PP_AbortDMAUnload( &pLink->PPort );
	}

	CloseUploadStream( pLink );

#ifdef _WINDOWS
	if ( pLink->StreamTimeoutID ) {
		timeKillEvent( pLink->StreamTimeoutID );
		pLink->StreamTimeoutID = 0;
	}
	if ( pLink->ChunkTimeoutID ) {
		timeKillEvent( pLink->ChunkTimeoutID );
		pLink->ChunkTimeoutID = 0;
	}
#endif

	pLink->UploadState = UPLOAD_IDLE;

	return( TRUE );
}


/* function to setup the UploadStatus callback function 		*/
/*  - call with NULL ptr to disable callback					*/
// BOOL CLINK_InstallUploadStatusCallback( CALLBACK_FUNC *Func() );

/* below called whenever there is a change in status of the upload stream */
// CALLBACK_FUNC UploadStatusCallback( UPLOAD_STATUS Status, DWORD ActualBytes );


/* functions to cater for the PP StreamReady Interrupt	*/
// BOOL CLINK_InstallStreamReadyCallback( CALLBACK_FUNC *Func() );

// CALLBACK_FUNC StreamReadyCallback( void );


/* functions to set/get timeout used in case DMA TC interrupt fails  	*/
/*--------------------------------------------------------------------------*/
/*	CLINK_SetDMATimeout														*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI CLINK_SetDMATimeout( PLINK pLink, WORD Timeout_ms );

/*--------------------------------------------------------------------------*/
/*	CLINK_GetDMATimeout														*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI CLINK_GetDMATimeout( PLINK pLink, WORD *Timeout_ms );


/* functions to set/get timeout used in case PP StreamReady Interrupt fails */ 
/*--------------------------------------------------------------------------*/
/*	CLINK_SetStreamReadyTimeout												*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI CLINK_SetStreamReadyTimeout( PLINK pLink, WORD Timeout_ms );


/*--------------------------------------------------------------------------*/
/*	CLINK_GetStreamReadyTimeout												*/
/*--------------------------------------------------------------------------*/
BOOL  __export WINAPI CLINK_GetStreamReadyTimeout( PLINK pLink, WORD *Timeout_ms );





/****************************************************************************/
/*																			*/
/*	Private functions of CPIALINK Module below here   						*/
/*																			*/
/****************************************************************************/

int Get1284DeviceID( PLINK pLink, char __huge *DestString, WORD MaxLen ) {
	WORD tmp;
	BYTE  length[20];
	
	DestString[0] = '\0';

	if ( !PP_SetTransferMode( &pLink->PPort, (REQUEST_DEVICE_ID | NIBBLE_TRANSFER) ) )
		 return( FALSE );

	if ( !PP_NibbleReadBuffer( &pLink->PPort, (BYTE  __far *)length, 2 ) ) {
//		printf("Get1284DeviceID:  NibbleReadBuffer returned an error\n" );
		PP_EndTransferMode( &pLink->PPort );
		DestString[0] = '\0';
		return( FALSE );
	}

	tmp = length[0]*256 + length[1];

	if ( tmp < 3 ) {
		PP_EndTransferMode( &pLink->PPort );
		return( FALSE );
	}
	tmp = tmp - 2;			/* reduce count to remove length BYTEs */
	if ( tmp >= MaxLen ) tmp = MaxLen - 1;

	if ( !PP_NibbleReadBuffer( &pLink->PPort, (BYTE  __far *)DestString, tmp ) ) {
		DestString[0] = '\0';
		PP_EndTransferMode( &pLink->PPort );
		return( FALSE );
	}
	DestString[ tmp ] = '\0';
	
	if ( !PP_EndTransferMode( &pLink->PPort ) ) return( FALSE );

 	return( TRUE );
}



BOOL OpenUploadStreamAndReadFirstByte( PLINK pLink, BYTE __far *Buffer ) {

	if ( pLink->Type & ECP_PORT_TYPE )	{
		if ( !PP_SetUploadMode( &pLink->PPort, ECP_UPLOAD, NORMAL_DATA_FLAG ) ) {
			goto OpenError;
		}
		if ( !PP_StartECPUpload( &pLink->PPort, Buffer ) ) goto OpenError;
	}
	else {
		if ( !PP_SetUploadMode( &pLink->PPort, NIBBLE_UPLOAD, NORMAL_DATA_FLAG ) ) {
			goto OpenError;
		}
		if ( !PP_NibbleUpload( &pLink->PPort, Buffer, 1 ) ) goto OpenError;
	}
	return( TRUE );

OpenError:
	printf( "OpenUploadStream: SetUploadMode failed\n" );
	return( FALSE );
}


BOOL CloseUploadStream( PLINK pLink ) {

 	if ( pLink->Type & ECP_PORT_TYPE )	{
		if ( !PP_FinishECPUpload( &pLink->PPort ) ) return( FALSE );
 	}

	if( !PP_EndUploadMode( &pLink->PPort ) ) {
		printf( "CloseUploadStream: EndUploadMode failed\n" );
		return( FALSE );
	}
	return( TRUE );
}


BOOL ReadStreamChunk( PLINK pLink, BOOL WaitForDMAToComplete ) {
	BOOL rtn;
	DWORD whileout;

	if ( pLink->Type & ECP_PORT_TYPE ) 
		{
		if ( !(pLink->Type & DMA_PORT_TYPE) ) 
			{
			if ( !PP_UnloadFIFO( &pLink->PPort, pLink->ChunkBuffer, pLink->CurrentChunkSize ) ) 
				{
				printf( "ReadUploadStream: UnloadFIFO failed\n" );
				PP_EndUploadMode( &pLink->PPort );
				goto ReadError;
				}
			pLink->ChunkFinishedDetected = TRUE;
			}
		else 
			{
		  	if ( !PP_StartDMAUnloadFIFO( &pLink->PPort, pLink->CurrentChunkSize ) )
			  	{
				goto DMAUnloadError;
				}
			else 
				{
				if ( WaitForDMAToComplete ) 
					while( !pLink->ChunkFinishedDetected ) 
						if ( pLink->ChunkTimedOut )	goto DMAUnloadError;
					
				}
			}
		}
	else 
		{
		if ( !PP_NibbleUpload( &pLink->PPort,  pLink->ChunkBuffer, pLink->CurrentChunkSize ) ) 
			{
			printf( "ReadUploadStream: NibbleUpload failed\n" );
			PP_EndUploadMode( &pLink->PPort );
			goto ReadError;
			}
		pLink->ChunkFinishedDetected = TRUE;
		}
	return( TRUE );

ReadError:
	return( FALSE );

DMAUnloadError:
	PP_AbortDMAUnload( &pLink->PPort );
	return( FALSE );
}







void __far CallbackFromPP( DWORD Context ) {
	PLINK plink;
	BYTE status;
	DWORD actual_bytes;

	if ( !Context ) return;

	plink = DWORD_TO_PLINK( Context );

	plink->StreamReadyDetected = TRUE;

	if ( plink->WaitForStreamReadyINT && (plink->UploadState == UPLOAD_STARTING) ) {
		CLINK_GetUploadStatus( plink, &status, &actual_bytes );
		if ( status != UPLOAD_CHUNK_START ) return;

		if ( plink->DoBackgroundDMA ) {
			CLINK_GetUploadStatus( plink, &status, &actual_bytes );
			if ( status != UPLOAD_CHUNK_END ) return;
		}
	}
 
	return;
}





void __far CallbackFromDMA( DWORD Context ) {
	PLINK plink;
	BYTE status;
	DWORD actual_bytes;

	if ( !Context ) return;

	plink = DWORD_TO_PLINK( Context );

	plink->ChunkFinishedDetected = TRUE;

 	if ( plink->DoBackgroundDMA && (plink->UploadState == UPLOAD_CHUNK_END) ) {

		/* finish the last transfer here */
		CLINK_GetUploadStatus( plink, &status, &actual_bytes );
		if ( status == UPLOAD_FINISHING ) {
			/* call GetUploadStatus once more to do a 1284 termination 	*/
			CLINK_GetUploadStatus( plink, &status, &actual_bytes );
			if ( status != UPLOAD_IDLE ) {
				/* flag an error */
			}
		}
		else {
			if ( status == UPLOAD_CHUNK_START ) {
				/* kick off the next chunk transfer						*/
				CLINK_GetUploadStatus( plink, &status, &actual_bytes );
				if ( status != UPLOAD_CHUNK_END ) {
					/* flag an error */
				}
			}
		}
	}

	return;
}



void __far PASCAL StreamTimedOutCallback( UINT idTimer, UINT Msg, DWORD dwUser, DWORD dw1, DWORD dw2 ) {
	PLINK plink;

	if ( !dwUser ) return;

	plink = DWORD_TO_PLINK( dwUser );
	if ( !plink->StreamReadyDetected ) plink->StreamTimedOut = TRUE;

	return;	
}


void __far PASCAL ChunkTimedOutCallback( UINT idTimer, UINT Msg, DWORD dwUser, DWORD dw1, DWORD dw2 ) {
	PLINK plink;

	if ( !dwUser ) return;

	plink = DWORD_TO_PLINK( dwUser );
	if ( !plink->ChunkFinishedDetected ) plink->ChunkTimedOut = TRUE;

	return;	
}


BOOL SearchForFastDMAChannel( PLINK pLink )		/* stores information about the channel directly into pLink */
{
	BOOL res;
	BYTE tmp;
	int device, channel;
	WORD IntelID[3] = { INTEL_823721FB_ID,
					    INTEL_823721SB_ID,
					    INTEL_823721AB_ID };

	res = FALSE;

	pLink->PCIBridgeVendor = 0;

	/* 
		look for an Intel Chipset that supports Fast DMA
	*/
	for( device = 0; device < 3; device++ ) 
	{
		if ( VVLCPIA_VXD_API_GetPCIcfgByte( INTEL_VENDOR_ID, IntelID[ device ],
											VENDOR_ID_ADDR_HI, &tmp ) )
    		break;
 	}
	if ( device != 3 ) 
	{	/* 
			we've found an Intel Chipset
			 - clear down Intel specific fields 
		*/
		pLink->PCI.Intel.DeviceID = 0;
		pLink->PCI.Intel.MBDMAx_register = 0;
		pLink->PCI.Intel.OldMBDMA0Value = 0;
		pLink->PCI.Intel.NewMBDMA0Value = 0;

		for( channel = 0; channel < 2; channel++ ) 	/* only two available channels			*/
		{
			if ( VVLCPIA_VXD_API_GetPCIcfgByte( INTEL_VENDOR_ID, IntelID[ device ],
												(BYTE)MBDMA0_ADDR+channel, &tmp ) ) 
			{ 									  
                if (    !(tmp & MBDMA_TYPE_F_ENABLE) 
					  || ((tmp&MBDMA_CHANNEL_MASK) == MBDMA_CHANNEL_DISABLE) 
					  || (tmp == (MBDMA_TYPE_F_ENABLE | MBDMA_DMC | ((pLink->PPort).DMAChannel))) ) 
				{
						/* OK, no one is using the Type F DMA mode 			*/
						/* or it is already set to the correct channel ! 	*/ 
					break;
				}
			}
		}
		if ( channel != 2 ) 
		{
			pLink->PCIBridgeVendor = INTEL_VENDOR_ID;
			pLink->PCI.Intel.DeviceID = IntelID[ device ];
			pLink->PCI.Intel.MBDMAx_register = MBDMA0_ADDR + channel;
			res = TRUE;
		}

		return( res );
	}
	/* 
		look for some other chipsets here 
	*/

	return( res );
}







BOOL EnableFastDMAChannel( PLINK pLink )
{	
	BOOL res = FALSE;
	BYTE tmp;

	if ( pLink->PCIBridgeVendor == INTEL_VENDOR_ID )
	{
		if ( pLink->PCI.Intel.DeviceID ) 
		{
			int channel;
			for( channel = 0; channel < 2; channel++ ) 	/* only two available channels			*/
			{
				if ( VVLCPIA_VXD_API_GetPCIcfgByte( pLink->PCIBridgeVendor, pLink->PCI.Intel.DeviceID,
															 (BYTE)MBDMA0_ADDR+channel, &tmp ) ) 
				{ 									  
	
                    if (    !(tmp & MBDMA_TYPE_F_ENABLE) 
						  || ((tmp&MBDMA_CHANNEL_MASK) == MBDMA_CHANNEL_DISABLE) 
						  || (tmp == (MBDMA_TYPE_F_ENABLE | MBDMA_DMC | ((pLink->PPort).DMAChannel))) ) 
					{
							/* OK, no one is using the Type F DMA mode 			*/
							/* or it is already set to the correct channel ! 	*/ 
						break;
					}
				}
			}
			if ( channel != 2 ) 
			{
			  	pLink->PCI.Intel.OldMBDMA0Value = tmp;	/* store it so it can be restored when we exit 		*/
				pLink->PCI.Intel.MBDMAx_register = MBDMA0_ADDR + channel;
				pLink->PCI.Intel.NewMBDMA0Value = (BYTE)(MBDMA_TYPE_F_ENABLE | MBDMA_DMC | ((pLink->PPort).DMAChannel) );

				if ( VVLCPIA_VXD_API_SetPCIcfgByte( pLink->PCIBridgeVendor, pLink->PCI.Intel.DeviceID,
																			pLink->PCI.Intel.MBDMAx_register, 
																			pLink->PCI.Intel.NewMBDMA0Value ) ) 
				{
					res = TRUE;
				}	
				else
				{
					pLink->PCI.Intel.DeviceID = 0;
					pLink->PCI.Intel.MBDMAx_register = 0;
					pLink->PCI.Intel.OldMBDMA0Value = 0;
					pLink->PCI.Intel.NewMBDMA0Value = 0;
				}
			}
		}
	}

	return( res );
}



BOOL DisableFastDMAChannel( PLINK pLink ) {
	BOOL res = FALSE;
	BYTE tmp;

	if ( pLink->PCIBridgeVendor == INTEL_VENDOR_ID )
	{
		if ( pLink->PCI.Intel.DeviceID && pLink->PCI.Intel.MBDMAx_register ) 
		{	/*
				read register before restoring the old state just in 
				case someone has stolen it from us !!
			*/
			if ( VVLCPIA_VXD_API_GetPCIcfgByte( pLink->PCIBridgeVendor, pLink->PCI.Intel.DeviceID,
																		pLink->PCI.Intel.MBDMAx_register,
																		&tmp ) ) 
			{
				if ( tmp == pLink->PCI.Intel.NewMBDMA0Value ) 
				{
					if ( VVLCPIA_VXD_API_SetPCIcfgByte( pLink->PCIBridgeVendor, pLink->PCI.Intel.DeviceID,
																				pLink->PCI.Intel.MBDMAx_register, 
																				pLink->PCI.Intel.OldMBDMA0Value ) ) 
					{
						res = TRUE;
					}
				}	
			}
		}
		pLink->PCI.Intel.DeviceID = 0;
		pLink->PCI.Intel.MBDMAx_register = 0;
		pLink->PCI.Intel.OldMBDMA0Value = 0;
		pLink->PCI.Intel.NewMBDMA0Value = 0;
	}

	return( res );
}



