/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */

#include <conio.h>

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
                 

#ifdef _WINDOWS
#include <windows.h>
#include <mmsystem.h>
#endif


                 
#include "vvl_defs.h"
#include "pport.h"
 
 
#include "cmd.h"
#include "pplink.h"
#include "cpialink.h"
#include "cpia_cmd.h"

#include "cpiautil.h"


#ifdef _WINDOWS
#define printf OutputDebugString        
#endif

        
        
  
BYTE gFW_Ver, gFW_Rev, gVC_Ver, gVC_Rev;


BOOL CheckForCPIA( PLINK *ppLink, BYTE PortNumber ) {
	BOOL rtn;
	WORD base;
	BYTE type;
	char deviceID[ 256 ];
    PLINK pLink;

	pLink = PPLINK_CreateLink( PortNumber );
	if ( pLink == NULL ) {
		printf( "cpiautil: PPLINK_CreateLink() failed - exiting\n" );
		return( FALSE );
	}

 	if ( PPLINK_GetPortInfo( pLink, &base, &type, NULL ) ) {
		printf( "\nLPT%d at 0x%x -- ", PortNumber, base );
		if ( type & ECP_PORT_TYPE ) printf( "ECP / " );
		if ( type & COMPAT_PORT_TYPE ) printf( "4 Bit\n" );
		printf( "\n" );
	}

	if ( !PPLINK_SetPortInfo( pLink, type, 800 ) ) {
		printf( "cpiautil: SetPortInfo() failed\n" );
		goto CheckForCPIAFailed;
	}

	if ( !PPLINK_InitialiseLink( pLink ) ) { 
		printf( "\ncpiautil: LPT%d does not exist\n", PortNumber );
		PPLINK_DeinitialiseLink( pLink );
		goto CheckForCPIAFailed;
	}

	if ( !PPLINK_SendWakeup( pLink ) ) {
		printf( "cpiautil: SendWakeup() failed\n" );
		goto CheckForCPIAFailed;
	}

	if ( !PPLINK_Get1284DeviceID( pLink, deviceID, 256 ) ) {
		printf( "cpiautil: Get1284DeviceID() failed\n" );
		goto CheckForCPIAFailed;
	}
	printf( "DeviceID = %s\n", deviceID );	

	if ( strstr( deviceID, "VLSI Vision Ltd" ) == NULL ) {
		printf( "cpiautil: DeviceID did not contain 'VLSI Vision Ltd'\n" );
		goto CheckForCPIAFailed;
	}

	rtn = CPIA_GetCameraStatus( pLink, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL );

    *ppLink = pLink;

	return( rtn );

CheckForCPIAFailed:
	PPLINK_DestroyLink( pLink );
    *ppLink = NULL;
	return( FALSE );
}


BOOL DoSystemTest( PLINK pLink, BYTE __huge *Buffer ) {
	BOOL rtn, failed;
	BYTE sys, vp, stream, fatal, cmd, vp_stat;
	BYTE cam_setup0, cam_status;
	WORD base, timeout;
	BYTE port_type;
	BYTE devicehi, devicelo;
	BOOL I2CReadCmdAvailable;
		
	failed = FALSE;

	printf( "Getting Firmware / VC version info   --- " );
	rtn = CPIA_GetVersion( pLink, &gFW_Ver, &gFW_Rev, &gVC_Ver, &gVC_Rev );
 	if ( rtn ) {
		printf( "OK\n" );
		printf( "  - Firmware = v%d-%02d    ", gFW_Ver, gFW_Rev );	
		printf( "  VC = v%d-%02d\n", gVC_Ver, gVC_Rev );	
		if ( ((gFW_Ver==1)&&(gFW_Rev > 8)) || (gFW_Ver > 1) ) 
			I2CReadCmdAvailable = TRUE;
		else
			I2CReadCmdAvailable = FALSE;
	}
	else {
		printf( "FAILED\n" );
		printf( "    Aborting System Test\n" );
		goto SystemTestFailed;
	}

	printf( "Clearing Fatal and Cmd status        --- " );
	rtn = CPIA_ModifyCameraStatus( pLink, 3, 0, 0 ); 
	rtn &= CPIA_ModifyCameraStatus( pLink, 4, 0, 0 ); 
	if ( !rtn ) goto SendFailed;
	else printf( "OK\n" );

	printf( "Switching to Hi-Power mode           --- " );

	rtn = CPIA_GotoHiPower( pLink );
	rtn &= CPIA_GetCameraStatus( pLink, &sys, &vp, &stream, &fatal, &cmd, NULL, &vp_stat, NULL ); 
	if ( rtn && (!cmd) && (!fatal) ) printf( "OK\n" );
	else {
		printf( "FAILED\n" );
		if ( fatal & VP_CTRL_FLAG ) printf( "    VP_CTRL FatalError\n" );
		if ( fatal & CAPTURE_FLAG ) printf( "    CAPTURE FatalError\n" );
		if ( cmd & VP_CTRL_FLAG ) printf( "    VP_CTRL CmdError\n" );
		if ( cmd & CAPTURE_FLAG ) printf( "    CAPTURE CmdError\n" );
		printf( "    Aborting System Test\n" );
		goto SystemTestFailed;					
	}

	CPIA_WriteVCReg( pLink, 0, 0x7F, 0x80 );

  	/* test dummy data transfer in 4 bit mode */
	rtn = PPLINK_GetPortInfo( pLink, &base, &port_type, &timeout );

	rtn = PPLINK_DeinitialiseLink( pLink );
	rtn = PPLINK_SetPortInfo( pLink, (BYTE)(port_type & ~(ECP_PORT_TYPE|BIDIR_PORT_TYPE)), timeout );
	rtn = PPLINK_InitialiseLink( pLink );

	printf( "Uploading test data in Nibble mode   --- " );
	rtn = UploadTestData( pLink, 256, Buffer );
	if ( rtn ) printf( "OK\n" );
	else printf( "FAILED\n" );

  	/* test dummy data transfer in ECP mode if possible*/
	if ( port_type & ECP_PORT_TYPE ) {
		rtn = PPLINK_DeinitialiseLink( pLink );
		rtn = PPLINK_SetPortInfo( pLink, port_type, timeout );
		rtn = PPLINK_InitialiseLink( pLink );
		printf( "Uploading test data in ECP mode      --- " );
		rtn = UploadTestData( pLink, 256, Buffer );
		if ( rtn ) printf( "OK\n" );
		else printf( "FAILED\n" );
	}
	else {
		printf( "Unable to upload test data in ECP mode\n" );
	}

	/* disable AEC, ACB and wait */
	printf( "Checking for camera ...\n" );
	rtn = CPIA_SetColourBalance( pLink, 3, 0, 0, 0 );
	rtn = CPIA_SetExposure( pLink, 0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
	Pause( 400 );
	CPIA_ReadVPRegs( pLink, 56, 57, 0, 0, &cam_status, &cam_setup0, NULL, NULL );
	if ( !(cam_status & 0x20) ) { 
		printf( "  - Camera not detected by VP        --- FAILED\n" );
		failed = TRUE;
	}

	/* check VP_Busy --> no camera OR unprogrammed EEPROM*/
	if ( cam_status & 0x40 ) {
		printf( "  - VPBusy=HI, no camera/bad eeprom  --- FAILED\n" );
		failed = TRUE;
	}
  	
	/* write camera register */	
	printf( "  - I2C write to sensor              --- " );
	rtn = CPIA_ModifyCameraStatus( pLink, 4, 0, 0 );	/* clear the CmdError field */
	rtn &= CPIA_SerialStart( pLink );
	rtn &= CPIA_SerialWrite( pLink, 0x20 );
	rtn &= CPIA_SerialWrite( pLink, 0x80 );	/* MSB set for auto-inc */
	rtn &= CPIA_SerialStop( pLink );
	rtn &= CPIA_GetCameraStatus( pLink, NULL, NULL, NULL, NULL, &cmd, NULL, NULL, NULL );
	if ( !rtn || (cmd&SYSTEM_FLAG) ) {
		printf( "FAILED\n" );
		failed = TRUE;
	}
	else {
		printf( "OK\n" );
	}

	if ( I2CReadCmdAvailable ) {
		printf( "  - I2C read from sensor             --- " );
		rtn = CPIA_SerialStart( pLink );
		rtn &= CPIA_SerialWrite( pLink, 0x21 );
		rtn &= CPIA_SerialRead( pLink, TRUE, &devicehi );	/* dummy read to get rid of index */
		rtn &= CPIA_SerialRead( pLink, TRUE, &devicehi );
		rtn &= CPIA_SerialRead( pLink, FALSE, &devicelo );
		rtn &= CPIA_SerialStop( pLink );
		if ( !rtn || (cmd&SYSTEM_FLAG) ) {
			printf( "FAILED\n" );
			failed = TRUE;
		}
		else {
			WORD id;
			printf( "OK\n" );
			id = (devicehi<<4) + (devicelo>>4);
			printf( "  - Sensor ID = %d  Mask Rev = %d\n", id, (devicelo&0x0F) );
		}
	}
	else {
		printf( "  - Firmware does not support I2C read\n" );
	} 

	/* read EEPROM and verify checksum */
	printf( "Testing I2C to EEPROM ...\n" );
	printf( "  - I2C write to EEPROM              --- " );
	rtn = CPIA_ModifyCameraStatus( pLink, 4, 0, 0 );	/* clear the CmdError field */
	rtn &= CPIA_SerialStart( pLink );
	rtn &= CPIA_SerialWrite( pLink, 0xA0 );
	rtn &= CPIA_SerialWrite( pLink, 0x00 );
	rtn &= CPIA_SerialStop( pLink );
	rtn &= CPIA_GetCameraStatus( pLink, NULL, NULL, NULL, NULL, &cmd, NULL, NULL, NULL );
	if ( !rtn || (cmd&SYSTEM_FLAG) ) {
		printf( "FAILED\n" );
		failed = TRUE;
	}
	else {
	 	printf( "OK\n" );
	}		

	if ( I2CReadCmdAvailable ) {
		printf( "  - Verifying EEPROM checksum        --- " );
		rtn = TestEEPROM( pLink );
		if ( rtn ) printf( "OK\n" );
		else {
			printf( "FAILED\n" );
			failed = TRUE;
		}
	}
	else {
		printf( "  - Firmware does not support I2C read\n" );
	} 

	CPIA_GotoLoPower( pLink );

	if ( !failed ) return( TRUE );
	else return( FALSE );

SystemTestFailed:
	CPIA_GotoLoPower( pLink );
	return( FALSE );

SendFailed:
	printf( "\nUnable to send command --- aborting test\n\n" );
	return( FALSE );
}



BOOL UploadTestData( PLINK pLink, WORD DummyBlocks, BYTE __huge *Buffer ) {

	FILE *file;
	int tmp;
	char filename[] = "dummy.txt";
	DWORD pos;
	BYTE val;
	int count;
	BOOL upload_ok = TRUE;

	for( pos = 0; pos < ((DWORD)DummyBlocks * 256); pos++ ) Buffer[ pos ] = 0xFE;

	if ( !CPIA_MACRO_TestStream( pLink, 0, DummyBlocks, Buffer ) ) {
		printf( "UploadTestData: CPIA_TestStream() failed\n" );
	}
/*
	file = fopen( filename, "wb" );
	if ( file == NULL ) {
		printf( "UploadBlock: Unable to open %s file\n", filename );
		return;
	}
	for( pos = 0; pos < (DWORD)256*DummyBlocks; pos++ ) {
		fprintf( file, " %x", (BYTE )Buffer[pos] );
		if ( !(pos%1024) ) printf( "." );
	}
	fclose( file );
*/
	val = 0; count = 0;
	for( pos = 0; pos < ((DWORD)DummyBlocks * 256); pos++ ) {
		if ( Buffer[ pos ] != val ) {
			printf( "UploadTestData: Buffer at %lu   %d != %d\n", pos, Buffer[ pos ], val );
			count++;
			if ( count > 4 ) break;
			upload_ok = FALSE;
		}
		val++;
	}

	return( upload_ok );
}



#define EEPROM_SIZE		128

BOOL TestEEPROM( PLINK pLink ) {
	BOOL rtn, cmd;
	BYTE eeprom[ EEPROM_SIZE ];
	BYTE old_checksum;
	BYTE calc_checksum;
	int i;

	rtn = CPIA_ModifyCameraStatus( pLink, 4, 0, 0 );	/* clear the CmdError field */
	rtn &= CPIA_SerialStart( pLink );
	rtn &= CPIA_SerialWrite( pLink, 0xA0 );
	rtn &= CPIA_SerialWrite( pLink, 0x00 );
	rtn &= CPIA_SerialStop( pLink );
	rtn &= CPIA_GetCameraStatus( pLink, NULL, NULL, NULL, NULL, &cmd, NULL, NULL, NULL );
	if ( !rtn || (cmd&SYSTEM_FLAG) ) {
		return( FALSE );
	}

	rtn = CPIA_ModifyCameraStatus( pLink, 4, 0, 0 );	/* clear the CmdError field */

	for( i = 0; i < EEPROM_SIZE-1; i++ ) eeprom[ i ] = 0x80;

	rtn &= CPIA_SerialStart( pLink );
	rtn &= CPIA_SerialWrite( pLink, 0xA1 );
	for( i = 0; i < EEPROM_SIZE-1; i++ ) {
		rtn &= CPIA_SerialRead( pLink, TRUE, &eeprom[i] );
	}
	rtn &= CPIA_SerialRead( pLink, FALSE, &eeprom[i] );
	rtn &= CPIA_SerialStop( pLink );
	rtn &= CPIA_GetCameraStatus( pLink, NULL, NULL, NULL, NULL, &cmd, NULL, NULL, NULL );
	if ( !rtn || (cmd&SYSTEM_FLAG) ) return( FALSE );

	/* now check that the checksum is correct 	*/
	old_checksum = (int)eeprom[ 0x1F ];
	calc_checksum = 0;
	eeprom[ 0x1F ] = 0;
	for( i = 0; i < EEPROM_SIZE-1; i++ ) {
		calc_checksum += eeprom[i];
	}
//	printf( "\nold_checksum = %d    calc_checksum = %d\n", (int)old_checksum, (int)calc_checksum );

	if ( old_checksum != calc_checksum ) 
		return( FALSE );
	else
		return( TRUE );
}


 		
void Pause( WORD mSecs ) {
	DWORD start, stop;

  	start = GetTime();                                        
    do {
    	stop = GetTime();                                        
	} while( (WORD)(((stop - start) * 1000)/(DWORD)CLOCKS_PER_SEC) < mSecs );

	return;
}
