/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
//
//	File		dma_ctrl.c
//	Purpose		provides dma services to fill a buffer
//
//	Created by 	S. Gillies
//	Copyright	(c) 1997 VLSI Vision Ltd, all rights reserved.
//
//	Revision history	
//		0-01	16/12/97	SG	Created
//		0-02	24/12/97	SG	Working with DOS and WINDOWS
//		0-03	19-01-98	SG	A physical address aquired from a VxD can now be used
//								bufferType parameter added to DMA_Init
//								Physical parameter added to DMA_StartTransfer
//		0-06	10-02-98	SG	Support for VDS buffer added back in
//								DMA_FreeBuffer()
//								New parameter for DMA_SetBuffer
//		


//
//	System Includes
//
#include <conio.h>
#include <dos.h>
#include <stddef.h>


//
//	Macro controls
//

//	Comment #define line to prevent printf()s when running a debug build
#ifdef _DEBUG
//#define _DEBUG_PRINTF
#endif //_DEBUG
#ifdef _DEBUG_PRINTF
#include <stdio.h>
#endif //_DEBUG_PRINTF


//
//	Local Includes
//
#include "dma_ctrl.h"
#ifdef _WINDOWS
#include "vds.h"
#endif //_WINDOWS


//
//	Local Definitions
//
#define DMA_MAX_TRANSFER_SIZE		0x10000		// 64Kb
#define SEL(ch)   ((ch <  4) ? ch : (ch - 4))
#define CHANNEL_MASK_ENABLE		0x00
#define CHANNEL_MASK_DISABLE	0x04

#ifdef __STDC__
#define outp(a,b) _outp(a,b)
#define inp(a,b) _inp(a,b)
#endif // __STDC__

//
//	Local variables
//
#ifdef _WINDOWS
static VDS_DDS dds;
static LPVDS_DDS lpdds = (LPVDS_DDS)&dds;
#endif //_WINDOWS

//	Dma controller data
static WORD m_dma_reg_command [] =		{ 0x08, 0x08, 0x08, 0x08, 0xd0, 0xd0, 0xd0, 0xd0 } ;
static WORD m_dma_reg_status [] =		{ 0x08, 0x08, 0x08, 0x08, 0xd0, 0xd0, 0xd0, 0xd0 } ;
static WORD m_dma_reg_page [] =			{ 0x87, 0x83, 0x81, 0x82, 0x8f, 0x8b, 0x89, 0x8a } ;
static WORD m_dma_reg_addr [] =			{ 0x00, 0x02, 0x04, 0x06, 0xc0, 0xc2, 0xc4, 0xc6 } ;
static WORD m_dma_reg_count [] =		{ 0x01, 0x03, 0x05, 0x07, 0xc1, 0xc3, 0xc5, 0xc7 } ;
static WORD m_dma_reg_mode [] =			{ 0x0b, 0x0b, 0x0b, 0x0b, 0xd6, 0xd6, 0xd6, 0xd6 } ;
static WORD m_dma_reg_request [] =		{ 0x09, 0x09, 0x09, 0x09, 0xd2, 0xd2, 0xd2, 0xd2 } ;
static WORD m_dma_reg_channel_mask [] =	{ 0x0a, 0x0a, 0x0a, 0x0a, 0xd4, 0xd4, 0xd4, 0xd4 } ;
static WORD m_dma_reg_mask [] =			{ 0x0f, 0x0f, 0x0f, 0x0f, 0xde, 0xde, 0xde, 0xde } ;

static WORD m_dma_cmd_reset_flip_flop [] =	{ 0x0c, 0x0c, 0x0c, 0x0c, 0xd8, 0xd8, 0xd8, 0xd8 } ;
static WORD m_dma_cmd_master_clear [] =		{ 0x0d, 0x0d, 0x0d, 0x0d, 0xda, 0xda, 0xda, 0xda } ;
static WORD m_dma_cmd_clear_mask_reg [] =	{ 0x0e, 0x0e, 0x0e, 0x0e, 0xdc, 0xdc, 0xdc, 0xdc } ;

static WORD m_dma_status_tc [] =		{ 0x00, 0x02, 0x04, 0x08, 0x00, 0x02, 0x04, 0x08 } ;
static WORD m_dma_status_req [] =		{ 0x00, 0x20, 0x40, 0x80, 0x00, 0x20, 0x40, 0x80 } ;

//	Channel specific data
static BYTE mDmaMode;
static T_DMA_BUFFER mDmaBufferType;
static LPBYTE mpUserBuffer;
static WORD mBufferSize_1;
static BYTE mPhyP, mPhyH, mPhyL;


//
//	Public function definitions
//

BOOL DMA_Init( T_DMA_BUFFER BufferType )
{
	mDmaMode = DMA_MODE_DEMAND | DMA_MODE_WRITE | DMA_MODE_NOAUTOINIT | DMA_MODE_ADDRINC;
	mDmaBufferType = BufferType;
	mPhyP = 0;
	mPhyH = 0;
	mPhyL = 0;
	mBufferSize_1 = 0;
	switch (mDmaBufferType)
	{
#ifdef _WINDOWS
		case DMA_BUFFER_VDS:
		case DMA_BUFFER_PM_APP:
		case DMA_BUFFER_PHYS:
			if( !vdsIsInstalled () )
				goto Err;
			break;
#else
		case DMA_BUFFER_DOS_APP:
			break;
#endif
		default:
			goto Err;
	}

	return TRUE;

Err:
	return FALSE;
	
}


void DMA_Terminate( BYTE Channel )
{
	mDmaBufferType = DMA_BUFFER_NONE;
}


BOOL DMA_SetBuffer( BYTE Channel, LPBYTE Buffer, DWORD Physical, WORD Size_1 )
{

#ifndef _WINDOWS
DWORD segment, offset, physical;
#endif

	switch (mDmaBufferType)
	{

#ifdef _WINDOWS
		case DMA_BUFFER_VDS:

			//	Check parameters are valid and enter DMA parameters into DMA Descriptor
			if( (Size_1 <= DMA_MAX_TRANSFER_SIZE) &&
			    (Buffer != NULL) )
			{
				dds.Region_Size = (unsigned long)Size_1 + 1;
				dds.Offset = LOWORD (Buffer) ;
				dds.Seg_or_Select = HIWORD (Buffer) ;
				dds.Buffer_ID = 0 ;
				dds.Physical_Address = 0L ;
			} else {
				goto Error;
			}

			if(vdsRequestBuffer (lpdds, VDSUBR_NOBUFCOPY) != VDS_ERR_OK)
				mpUserBuffer = Buffer;
			else {
				goto Error;
			}

			//	Check physical address is no larger than 24 bit
			if( HIBYTE(HIWORD(dds.Physical_Address)) ) {			// 31:24 should all be 0
				vdsReleaseBuffer (lpdds, VDSUBR_NOBUFCOPY);
				goto Error;
			}

			mPhyP = LOBYTE(HIWORD(dds.Physical_Address));
			mPhyH = HIBYTE(LOWORD(dds.Physical_Address));
			mPhyL = LOBYTE(LOWORD(dds.Physical_Address));

			//	Disable DMA translation because we have a physical address
			vdsDisableTranslation( Channel );
			break;

		case DMA_BUFFER_PM_APP:

			//	Check parameters are valid and enter DMA parameters into DMA Descriptor
			if( (Size_1 <= DMA_MAX_TRANSFER_SIZE) &&
			    (Buffer != NULL) )
			{
				dds.Region_Size = (unsigned long)Size_1 + 1;
				dds.Offset = LOWORD (Buffer) ;
				dds.Seg_or_Select = HIWORD (Buffer) ;
				dds.Buffer_ID = 0 ;
				dds.Physical_Address = 0L ;
			} else {
				goto Error;
			}
		 
			//	Lock buffer and get physical address (of VDS buffer if caller's buffer is
			//	no use)
			if(vdsLockDmaRegion (lpdds, VDSLBR_64KLIMIT) != VDS_ERR_OK) {
				goto Error;
			}

			//	Check physical address is no larger than 24 bit
			if( HIBYTE(HIWORD(dds.Physical_Address)) ) {			// 31:24 should all be 0
  				vdsUnlockDmaRegion (lpdds, VDSUBR_NOBUFCOPY) ;
				goto Error;
			}

			mPhyP = LOBYTE(HIWORD(dds.Physical_Address));
			mPhyH = HIBYTE(LOWORD(dds.Physical_Address));
			mPhyL = LOBYTE(LOWORD(dds.Physical_Address));

			//	Disable DMA translation because we have a physical address
			vdsDisableTranslation( Channel );
			break;

		case DMA_BUFFER_PHYS:

			//	Disable DMA translation because we have a physical address
			vdsDisableTranslation( Channel );

			//	Check physical address is no larger than 24 bit
			if( HIBYTE(HIWORD(Physical)) )			// 31:24 should all be 0
				goto Error;
			mPhyP = LOBYTE(HIWORD(Physical));
			mPhyH = HIBYTE(LOWORD(Physical));
			mPhyL = LOBYTE(LOWORD(Physical));
			break;
#else

		case DMA_BUFFER_DOS_APP:

			segment = HIWORD(Buffer);
			offset = LOWORD(Buffer);
			physical = ( segment << 4 ) + offset;

			//	Check physical address is no larger than 24 bit
			if( HIBYTE(HIWORD(physical)) )			// 31:24 should all be 0
				goto Error;
			mPhyP = LOBYTE(HIWORD(physical));
			mPhyH = HIBYTE(LOWORD(physical));
			mPhyL = LOBYTE(LOWORD(physical));
			break;
#endif


		default:
			goto Error;
	}

	// Set member variable for buffer size
	mBufferSize_1 = Size_1;

	// Physical address
	#ifdef _DEBUG_PRINTF
		printf("mPhyP:%x\n",(WORD)mPhyP);
		printf("mPhyH:%x\n",(WORD)mPhyH);
		printf("mPhyL:%x\n",(WORD)mPhyL);
	#endif //_DEBUG_PRINTF

	return TRUE;

Error:
	return FALSE;
}


void DMA_FreeBuffer( BYTE Channel )
{

	//	Reset physical buffer address
	mPhyP = 0;
	mPhyH = 0;
	mPhyL = 0;

	mBufferSize_1 = 0;

	//	Re-enable DMA translation
	vdsEnableTranslation( Channel );
}


BOOL DMA_StartTransfer( BYTE Channel, WORD Bytes_1 )
{

	//	Check that a buffer has been assigned
	if ( (mPhyP == 0 &&  mPhyP == 0 &&  mPhyP == 0) || (mBufferSize_1 == 0) )
	{
		return FALSE;
	}

	// Check that transfer size is not greater than DMA buffer available
	if ( Bytes_1 > mBufferSize_1 )
	{
		return FALSE;
	}

	//
	//	Program the DMA controller
	//

	//	Disable DMA channel
	outp( m_dma_reg_channel_mask[Channel] , CHANNEL_MASK_DISABLE + SEL(Channel) ) ;

	// Set up the DMA buffer address and transfer count
	outp( m_dma_cmd_reset_flip_flop[Channel] , 0 );

	// Program physical address
	outp( m_dma_reg_page[Channel] , mPhyP);		// Buffer address 23:16
	outp( m_dma_reg_addr[Channel] , mPhyL);		// Buffer address 7:0
	outp( m_dma_reg_addr[Channel] , mPhyH);		// Buffer address 15:8

	// Set transfer size
	outp( m_dma_reg_count[Channel] , LOBYTE(Bytes_1) );		// Buffer size 7:0
	outp( m_dma_reg_count[Channel] , HIBYTE(Bytes_1) );		// Buffer size 15:8

	// Set transfer mode
	outp( m_dma_reg_mode[Channel] , mDmaMode + SEL(Channel) );

	// Re-enable channel
	outp( m_dma_reg_channel_mask[Channel] , CHANNEL_MASK_ENABLE + SEL(Channel) );

	return TRUE;
}


void DMA_EndTransfer( BYTE Channel )
{

	//	Disable DMA channel
	outp( m_dma_reg_channel_mask[Channel] , CHANNEL_MASK_DISABLE + SEL(Channel) ) ;

	switch ( mDmaBufferType )
	{
#ifdef _WINDOWS
		case DMA_BUFFER_VDS:

			//	Re-enable DMA translation
			vdsEnableTranslation( Channel );

			//	Copy from VDS buffer to user's buffer and release VDS buffer
			vdsCopyOutOfBuffer (lpdds, (char __far *)mpUserBuffer);
			vdsReleaseBuffer (lpdds, VDSUBR_NOBUFCOPY);
			break;

		case DMA_BUFFER_PM_APP:

			//	Unlock buffer (and copy to caller's buffer if VDS buffer was used)
  			vdsUnlockDmaRegion (lpdds, VDSUBR_BUFCOPY) ;
			break;

		case DMA_BUFFER_PHYS:
			break;

#else
		case DMA_BUFFER_DOS_APP:
			break;
#endif

		default:
			break;
	}

	return;
}


void DMA_AbortTransfer( BYTE Channel )
{

	//	Disable DMA channel
	outp( m_dma_reg_channel_mask[Channel] , CHANNEL_MASK_DISABLE + SEL(Channel) ) ;

	switch (mDmaBufferType)
	{
#ifdef _WINDOWS
		case DMA_BUFFER_VDS:
			//	Re-enable DMA translation
			vdsEnableTranslation( Channel );

			//	Release VDS buffer doing no copy
			vdsReleaseBuffer (lpdds, VDSUBR_NOBUFCOPY);
			break;

		case DMA_BUFFER_PM_APP:
			//	Unlock buffer doing no copy
	  		vdsUnlockDmaRegion (lpdds, VDSUBR_NOBUFCOPY) ;
			break;

		case DMA_BUFFER_PHYS:
			break;
#else
		case DMA_BUFFER_DOS_APP:
			break;
#endif

		default:
			break;
	}

	return;
}


BOOL DMA_CheckTCBit( BYTE Channel )
{
	//	Check tc bit in status register
	return ( inp(m_dma_reg_status[Channel]) & m_dma_status_tc[Channel] );
}


void DMA_SetMode( BYTE Mode )
{
	mDmaMode = Mode;
}
