/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */

#ifndef PPLINK
#define PPLINK
#endif


#ifndef _WINDOWS
#define __export
#define PASCAL 
#define WINAPI
#endif

#define MAX_LINK				3		
	
#define MAX_1284_DEVICE_ID		256

#define MAX_PORT_DESCRIPTION	40		/* MUST match definition used in CPIA_API.H */

#define MAX_CHUNK_SIZE 			((WORD)1024*16)


/* constants for use by the MMEDIA timer callback functions used to 		*/
/* implement timeout functionality											*/
#define DEFAULT_TIMER_MIN_PERIOD 		10
#define DEFAULT_STREAM_TIMEOUT			500	/* 2 * 1/7.5fps					*/
#define DEFAULT_CHUNK_TIMEOUT			133 /* 1/7.5 fps					*/


/* following defines are associated with the PCI/ISA Bridge chip that is	*/
/* tweeked to enable Type F dma.											*/
#define PCI_BUS					0x00
#define ISA_BRIDGE_DEVICE		0x07
#define ISA_BRIDGE_FUNCTION		0x00

#define INTEL_VENDOR_ID			0x8086
#define INTEL_823721FB_ID  		0x122E
#define INTEL_823721SB_ID  		0x7000
#define INTEL_823721AB_ID  		0x7110


/* standard PCI configuration regsister offsets	 */
#define VENDOR_ID_ADDR_LO		0x00
#define VENDOR_ID_ADDR_HI		0x01
#define DEVICE_ID_ADDR_LO		0x02
#define DEVICE_ID_ADDR_HI		0x03

/* 823721 specific configuration register offsets */
#define MBDMA0_ADDR				0x76			  
#define MBDMA1_ADDR				0x77			  

/* bits within the MBDMA0/1 registers				*/
#define MBDMA_TYPE_F_ENABLE		0x80
#define MBDMA_DMC				0x08
#define MBDMA_CHANNEL_MASK		0x07

#define MBDMA_CHANNEL_DISABLE	0x04





#pragma warning ( disable : 4121 )

typedef struct {
#ifdef _WINDOWS
	HLOCAL HandleToThisInstance;
#endif
	PPORT PPort;			/* private data used by PPORT module 	*/
	char PortDescription[40];

	BOOL LinkInitialised;
	BYTE PortNumber;
	BYTE Type;				/* uses XXXX_PORT_TYPE from pport.h 	*/

	/* these two control how the CPIALINK module attempts to transfer stream data */
	BOOL DoBackgroundDMA;
	BOOL WaitForStreamReadyINT; 

	BYTE UploadState;
	DWORD CurrTransferSize;

   	BOOL StreamReadyDetected;	
	BOOL ChunkFinishedDetected;

	BOOL StreamTimedOut;	/* set by the Timeout callback functions */
	BOOL ChunkTimedOut;

	/* parameters which define the upload requested */
	DWORD TargetBufferSize;
	BYTE __huge *TargetBuffer;
    WORD CurrentChunkSize;	/* size of current chunk being uploaded */	
    WORD StreamChunkSize;	/* max size of chunk we are to use 		*/
   	BOOL EndOnFF;

	/* info about our temporary DMA buffer 			*/
    WORD ChunkBufferSize;
	BYTE __far *ChunkBuffer;
	DWORD ChunkPhysicalAddr;	/* only used if we are doing DMA transfers	*/

#ifdef _WINDOWS
	HGLOBAL ChunkHandle;
#endif

	WORD TimerMinPeriod;		/* min period we will request when setting events */

	WORD StreamTimeoutPeriod;	
	UINT StreamTimeoutID;

	WORD ChunkTimeoutPeriod;
	UINT ChunkTimeoutID;

	/* following used to record state of old ISA bridge chip register that is	*/
	/* modified to enable Type F dma.											*/
	WORD PCIBridgeVendor;
	union {
		/* NOTE - the ISABridgeVendor field above is used to select which		*/
		/* union is valid														*/
		struct {
			WORD DeviceID;
			BYTE MBDMAx_register;
			BYTE OldMBDMA0Value;
			BYTE NewMBDMA0Value;
		} Intel;

		struct {
			BYTE tmp0;
		} AcreLabs;

	} PCI;

} LINK;

typedef LINK *PLINK;


#define PLINK_TO_DWORD(x)	((DWORD)(x))
#define DWORD_TO_PLINK(x)	((PLINK)(x))


#ifdef __cplusplus
extern "C"
{
#endif

PLINK  __export WINAPI PPLINK_CreateLink( BYTE PortNumber );
BOOL  __export WINAPI PPLINK_DestroyLink( PLINK pLink );

BOOL  __export WINAPI PPLINK_InitialiseLink( PLINK pLink );
BOOL  __export WINAPI PPLINK_DeinitialiseLink( PLINK pLink );

BOOL  __export WINAPI PPLINK_GetPortInfo( PLINK pLink, WORD *BaseAddr, BYTE *PortType, WORD *Timeout_mS );
BOOL  __export WINAPI PPLINK_SetPortInfo( PLINK pLink, BYTE PortType, WORD Timeout_mS );

BOOL  __export WINAPI PPLINK_GetHWInfo( PLINK pLink, BYTE *IntLine, BYTE *DMAChannel, BYTE *AdditionalData );
BOOL  __export WINAPI PPLINK_SetHWInfo( PLINK pLink, BYTE IntLine, BYTE DMAChannel, BYTE AdditionalData );


BOOL  __export WINAPI PPLINK_SendWakeup( PLINK pLink );
BOOL  __export WINAPI PPLINK_Get1284DeviceID( PLINK pLink, char *DeviceID, WORD MaxLength );
										 /* use small MaxLength to check if camera alive  	*/

BOOL  __export WINAPI PPLINK_ForceWatchdogReset( PLINK pLink );



/* private functions */
int Get1284DeviceID( PLINK pLink, char __huge *DestString, WORD MaxLen );

BOOL OpenUploadStreamAndReadFirstByte( PLINK pLink, BYTE __far *Buffer );
BOOL CloseUploadStream( PLINK pLink );
BOOL ReadStreamChunk( PLINK pLink, BOOL WaitForDMAToComplete );

void __far CallbackFromPP( DWORD Context );
void __far CallbackFromDMA( DWORD Context );	

void __far PASCAL StreamTimedOutCallback( UINT idTimer, UINT Msg, DWORD dwUser, DWORD dw1, DWORD dw2 );
void __far PASCAL ChunkTimedOutCallback( UINT idTimer, UINT Msg, DWORD dwUser, DWORD dw1, DWORD dw2 );


BOOL SearchForFastDMAChannel( PLINK pLink );	/* stores information about the channel directly into pLink */
BOOL EnableFastDMAChannel( PLINK pLink );	
BOOL DisableFastDMAChannel( PLINK pLink );	


#ifdef __cplusplus
}
#endif


