/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/* VDS.C
   19-05-97
   routines for accessing the virtual device specification interface */
                
#include <dos.h>
                
#include "vds.h"


/*-----------------------------------------------------------------*/
/* offset 7bh in the bios data area contains information to determine
   if vds is installed 
   
   return
     TRUE  - vds is installed
     FALSE - vds not installed
*/     
    
vdsIsInstalled (void)

{
 unsigned char far *lpbios = MK_FP (0x40, VDS_BIOSOFFSET) ;
 
 return (*lpbios & VDS_PRESENT) == VDS_PRESENT ;
}

/*-----------------------------------------------------------------*/
/* return version information */

vdsGetVersion (LPVDS_VERSION lpver)

{
 union REGS r ;
 int res = VDS_ERR_OK ;
 
 r. x. ax = 0x8102 ;
 r. x. dx = 0 ;
 
 int86 (VDS_INT, &r, &r) ;
 if (r. x. cflag == 0)      /* carry not set, function ok */
  {          
   lpver -> SpecVer       = r. x. ax ;
   lpver -> ProductNo     = r. x. bx ;
   lpver -> ProductRevNo  = r. x. cx ;
   lpver -> MaxDMABuf     = r. x. di ;
   lpver -> MaxDMABuf    += ((unsigned long) (r. x. si)) << 16 ;
   lpver -> DMA1MegOnly   = r. x. dx & 1 ;
   lpver -> RemapIn1Meg   = (r. x. dx & 2) == 2 ;
   lpver -> AutoRemap     = (r. x. dx & 4) == 4 ;
   lpver -> MemPhysContig = (r. x. dx & 8) == 8 ;
  }       
 else
  res = r. h. al ;
  
 return res ; 
}   

/*-----------------------------------------------------------------*/
/* lock a dma buffer region
   
    lpdds -> Region_Size   - number of bytes to lock
    lpdds -> Offset        - linear address offset
    lpdds -> Seg_or_Select - segment/selector part of address
    
    flags - combination of flags indicating what is required of the region
    
   return
    VDS_ERR_OK  - region locked
    VDS_??????  - error code (1, 2, 3, 5, 6, 7 10)
*/
    
vdsLockDmaRegion (LPVDS_DDS lpdds, unsigned short flags)

{
 union REGS r ; 
 struct SREGS s ;
 int res = VDS_ERR_OK ;
 
 r. x. ax = 0x8103 ;
 r. x. dx = flags ;
 segread (&s) ;
 
 s. es = FP_SEG (lpdds) ;
 r. x. di = FP_OFF (lpdds) ;
 
 int86x (VDS_INT, &r, &r, &s) ;
 
 if (r. x. cflag != 0)
  res = r. h. al ;
  
 return res ;
}

/*-----------------------------------------------------------------*/
/* unlock a region previously locked by vdsLockDmaRegion */

vdsUnlockDmaRegion (LPVDS_DDS lpdds, int copy_from_buffer)

{
 union REGS r ; 
 struct SREGS s ;
 int res = VDS_ERR_OK ;
 
 r. x. ax = 0x8104 ;
 r. x. dx = copy_from_buffer ;
 segread (&s) ;
 
 s. es = FP_SEG (lpdds) ;
 r. x. di = FP_OFF (lpdds) ;
 
 int86x (VDS_INT, &r, &r, &s) ;
 
 if (r. x. cflag != 0)
  res = r. h. al ;
  
 return res ;
}

/*-----------------------------------------------------------------*/

vdsRequestBuffer (LPVDS_DDS lpdds, int copy_to_buffer)

{
 union REGS r ; 
 struct SREGS s ;
 int res = VDS_ERR_OK ;
 
 r. x. ax = 0x8107 ;
 r. x. dx = copy_to_buffer ;
 segread (&s) ;
 
 s. es = FP_SEG (lpdds) ;
 r. x. di = FP_OFF (lpdds) ;
 
 int86x (VDS_INT, &r, &r, &s) ;
 
 if (r. x. cflag != 0)
  res = r. h. al ;
  
 return res ;
}

/*-----------------------------------------------------------------*/

vdsReleaseBuffer (LPVDS_DDS lpdds, int copy_from_buffer)

{
 union REGS r ; 
 struct SREGS s ;
 int res = VDS_ERR_OK ;
 
 r. x. ax = 0x8108 ;
 r. x. dx = copy_from_buffer ;
 segread (&s) ;
 
 s. es = FP_SEG (lpdds) ;
 r. x. di = FP_OFF (lpdds) ;
 
 int86x (VDS_INT, &r, &r, &s) ;
 
 if (r. x. cflag != 0)
  res = r. h. al ;
  
 return res ;
}

/*-----------------------------------------------------------------*/

vdsCopyIntoBuffer (LPVDS_DDS lpdds, char far *lpdma_offset)

{
 union REGS r ; 
 struct SREGS s ;
 int res = VDS_ERR_OK ;
 
 r. x. ax = 0x8109 ;
 r. x. bx = FP_SEG (lpdma_offset) ;
 r. x. cx = FP_OFF (lpdma_offset) ;
 r. x. dx = 0 ;
 segread (&s) ;
 
 s. es = FP_SEG (lpdds) ;
 r. x. di = FP_OFF (lpdds) ;
 
 int86x (VDS_INT, &r, &r, &s) ;
 
 if (r. x. cflag != 0)
  res = r. h. al ;
  
 return res ;
}

/*-----------------------------------------------------------------*/

vdsCopyOutOfBuffer (LPVDS_DDS lpdds, char far *lpdest)

{
 union REGS r ; 
 struct SREGS s ;
 int res = VDS_ERR_OK ;
 
 r. x. ax = 0x810a ;
 r. x. dx = 0 ;
 r. x. bx = FP_SEG (lpdest) ;
 r. x. cx = FP_OFF (lpdest) ;
 
 segread (&s) ;
 
 s. es = FP_SEG (lpdds) ;
 r. x. di = FP_OFF (lpdds) ;
 
 int86x (VDS_INT, &r, &r, &s) ;
 
 if (r. x. cflag != 0)
  res = r. h. al ;
  
 return res ;
}

/*-----------------------------------------------------------------*/

vdsDisableTranslation (int dma_channel)

{
 union REGS r ; 
 int res = VDS_ERR_OK ;
 
 r. x. ax = 0x810b ;
 r. x. dx = 0 ;
 r. x. bx = dma_channel ;
 
 int86 (VDS_INT, &r, &r) ;
 
 if (r. x. cflag != 0)
  res = r. h. al ;
  
 return res ;
}

/*-----------------------------------------------------------------*/

vdsEnableTranslation (int dma_channel)

{
 union REGS r ; 
 int res = VDS_ERR_OK ;
 
 r. x. ax = 0x810c ;
 r. x. dx = 0 ;
 r. x. bx = dma_channel ;
 
 int86 (VDS_INT, &r, &r) ;
 
 if (r. x. cflag != 0)
  res = r. h. al ;
  
 return res ;
}
