/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/* VDS.H
   19-05-97	JW
   header file for vds implementation */
   
#define VDS_BIOSOFFSET 0x7b    /* offset within Bios Data Area */
#define VDS_PRESENT    0x20    /* bit 5 at above offset in bda */
#define VDS_INT        0x4b    /* interrupt to generate */

/* flags for use with vdsLockDmaRegion */
#define VDSLBR_BUFCOPY      0x02  /* true if data to be copied to buffer */
#define VDSLBR_NOBUFCOPY    0x00  /* true if data to be copied to buffer */
#define VDSLBR_NOALLOC      0x04  /* disable allocation if alignment problems */
#define VDSLBR_NOREMAP      0x08  /* disable auto remap of buffer */
#define VDSLBR_64KLIMIT     0x10  /* region must not cross 64k boundary */
#define VDSLBR_128KLIMIT    0x20  /* region must not cross 128k boundary */

#define VDSUBR_BUFCOPY      0x02  /* copy data from buffer */
#define VDSUBR_NOBUFCOPY    0x00  /* copy data from buffer */

/* dma descriptor data structure */
typedef struct tag_vds_dds {
          unsigned long  Region_Size ;
          unsigned long  Offset ;
          unsigned short Seg_or_Select ;
          unsigned short Buffer_ID ;
          unsigned long  Physical_Address ;
         } VDS_DDS, *PVDS_DDS, far *LPVDS_DDS ;
         
typedef struct tag_vds_ver
          {
           unsigned short SpecVer ;         /* specification number */
           unsigned char  ProductNo ;       /* product number */
           unsigned char  ProductRevNo ;    /* product revision */
           unsigned long  MaxDMABuf ;       /* max dma buf supported (in bytes) */
           unsigned char  DMA1MegOnly ;     /* XT bux (dma in 1 meg only) */
           unsigned char  RemapIn1Meg ;     /* phys buffer in 1 meg */
           unsigned char  AutoRemap ;       /* auto remap supported */
           unsigned char  MemPhysContig ;   /* all memory physically contiguous */
          } VDS_VERSION, *PVDS_VERSION, far *LPVDS_VERSION ;
          
enum { VDS_ERR_OK,                      /* no error */
       VDS_ERR_REGION_NOT_CONTIG,       /* region not in contiguous memory */
       VDS_ERR_REGION_BOUNDARY,         /* region crossed a phys align bound */
       VDS_ERR_CANT_LOCK_PAGES,         /* unable to lock pages */
       VDS_ERR_NO_BUFFER_AVAIL,         /* no buffer available */
       VDS_ERR_REGION_TOO_LARGE,        /* region too large for buffer */
       VDS_ERR_BUFFER_IN_USE,           /* buffer currently in use */
       VDS_ERR_INVALID_MEMORY,          /* invalid memory region */
       VDS_ERR_REGION_NOT_LOCKED,       /* region was not locked */
       VDS_ERR_PHY_PAGE_GT_TABLE,       /* numb of phys pages > table length */
       VDS_ERR_INVALID_BUFFER_ID,       /* invalid VDS_DSS -> Buffer_ID */
       VDS_ERR_BUFFER_BOUNDARY,         /* buffer boundary violated */
       VDS_ERR_INVALID_DMA_CHAN,        /* invalid dma channel number */
       VDS_ERR_DISABLE_COUNT_OVERFLOW,  /* disable count overflow */
       VDS_ERR_DISABLE_COUNT_UNDERFLOW, /* disable count underflow */
       VDS_ERR_FUNC_NOT_SUPPORTED,      /* function not supported */
       VDS_ERR_INVALID_FLAGS            /* reserved flag bit set in dx */
     } ;
     
vdsIsInstalled (void) ;
vdsGetVersion (LPVDS_VERSION lpver) ;
vdsLockDmaRegion (LPVDS_DDS lpdds, unsigned short flags) ;
vdsUnlockDmaRegion (LPVDS_DDS lpdds, int copy_from_buffer) ;
vdsRequestBuffer (LPVDS_DDS lpdds, int copy_to_buffer) ;
vdsReleaseBuffer (LPVDS_DDS lpdds, int copy_from_buffer) ;
vdsCopyIntoBuffer (LPVDS_DDS lpdds, char far *lpdma_offset) ;
vdsCopyOutOfBuffer (LPVDS_DDS lpdds, char far *lpdest) ;
vdsDisableTranslation (int dma_channel) ;
vdsEnableTranslation (int dma_channel) ;
     
