/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/* CZOOM.C
   25/8/97
   class to support selecting roi within a cif image, using fake
   pan/tilt
*/

#pragma warning (disable : 4001 4100 4505 4759)

#include <windows.h>
#include <stdarg.h>
#include <memory.h>

#include "czoom.h"

static LONG cz_create (HWND hwnd, LPCZOOM lpz) ;
static LONG cz_destroy (HWND hwnd, LPCZOOM lpz) ;
static LONG cz_paint (HWND hwnd, LPCZOOM lpz) ;
static void draw_crect (HWND hwnd, HDC hdc, LPCZOOM lpz, HBRUSH hbr) ;
static LONG cz_buttondown (HWND hwnd, LPCZOOM lpz, WPARAM wparam, LPARAM lparam) ;
static LONG cz_buttonup (HWND hwnd, LPCZOOM lpz, WPARAM wparam, LPARAM lparam) ;
static LONG cz_mousemove (HWND hwnd, LPCZOOM lpz, WPARAM wparam, LPARAM lparam) ;
static BOOL rect_valid (LPRECT lpr, int *xofs, int *yofs) ;
static LONG cz_setposition (HWND hwnd, LPCZOOM lpz, LPARAM lparam) ;
static BOOL notify_parent (HWND hwnd, UINT msg) ;
static void clip_mouse (HWND hwnd, LPCZOOM lpz) ;
static LONG cz_setsize (HWND hwnd, LPCZOOM lpz, LPARAM lparam) ;
static long cz_keydown (HWND hwnd, LPCZOOM lpz, WPARAM wparam, LPARAM lparam) ;
static long cz_keyup (HWND hwnd, LPCZOOM lpz, WPARAM wparam, LPARAM lparam) ;
static void DbgPrint (LPSTR str, ...) ;

#ifdef _DEBUG
    #ifndef KdPrint
        #define KdPrint(x)  DbgPrint x
    #endif
#else
    #ifndef KdPrint
        #define KdPrint(x)
    #endif
#endif

/*-----------------------------------------------------------------*/
/* register the class */

BOOL czInit (HINSTANCE hinst)

{       
WNDCLASS w ;
 
    memset (&w, '\0', sizeof (w)) ;      
 
    w.lpfnWndProc = (WNDPROC) czWndProc ;      
    w.cbWndExtra  = sizeof (LPCZOOM) ;
    w.hInstance   = hinst ;
    w.hbrBackground = (HBRUSH) (COLOR_BTNFACE + 1);
    w.lpszClassName = STR_CZOOM ;
    w.hCursor       = LoadCursor (NULL, IDC_ARROW);
 
    return RegisterClass (&w) ;
}                       

/*-----------------------------------------------------------------*/

LRESULT CALLBACK _loadds czWndProc (HWND hwnd, UINT msg, WPARAM wparam, 
                                    LPARAM lparam)

{ 
LONG res = 0 ;       
LPCZOOM lpz ;
int parx,
    pary ;
#ifndef _WIN32
    RECT rect ;
    HBRUSH hbr ;
#endif
HBRUSH htmp ;
                              
    switch (msg)
    {
    case WM_CREATE :
        lpz = (LPCZOOM) ((PCZOOM) LocalAlloc (LPTR, sizeof (CZOOM))) ;
        if (lpz == NULL)
        {
            res = -1 ;
        }
        else
        {
            SetWindowLong (hwnd, 0, (LONG) lpz) ;
            res = cz_create (hwnd, lpz) ;
        }
        break ;
                            
        /* it is possible to press ESCAPE while the mouse is tracking. this
           causes the mouse to stay within its previous clip region. If this
           is likely to happen (mouse down), it's released
        */
    case WM_DESTROY :
        lpz = CZ_PTR (hwnd) ;         
        if (lpz->state != ST_NONE)
        {
            ReleaseCapture () ;
            ClipCursor (NULL) ;
            ShowCursor (TRUE) ;
        }
        res = cz_destroy (hwnd, lpz) ;
        break ;
        
    case WM_LBUTTONDOWN :
        res = cz_buttondown (hwnd, CZ_PTR (hwnd), wparam, lparam) ;
        break ;
        
    case WM_LBUTTONUP :     
        res = cz_buttonup (hwnd, CZ_PTR (hwnd), wparam, lparam) ;
        break ;
        
    case WM_MOUSEMOVE :
        res = cz_mousemove (hwnd, CZ_PTR (hwnd), wparam, lparam) ;
        break ;
        
    case WM_ENABLE :
        InvalidateRect (hwnd, NULL, TRUE) ;
        UpdateWindow (hwnd) ;
        res = 0 ;
        break ;
                  
    case WM_PAINT :
        lpz = CZ_PTR (hwnd) ;
        res = cz_paint (hwnd, lpz) ;
        break ;                

#ifndef _WIN32

    case WM_ERASEBKGND:
        GetClientRect (hwnd, &rect);  
        hbr = CreateSolidBrush(GetSysColor(COLOR_BTNFACE)) ;
        FillRect ((HDC)wparam, &rect, hbr);
        DeleteObject (hbr) ;
        res = 1 ;
        break;                    

    case WM_CTLCOLOR:
        lpz = CZ_PTR (hwnd) ;         
        SetBkColor ((HDC) wparam, GetSysColor (COLOR_BTNFACE)) ;
        res = (LRESULT) lpz->hctlcolour ;
        break ;

#endif

    case WM_KEYDOWN :
        res = cz_keydown (hwnd, CZ_PTR (hwnd), wparam, lparam) ;
        break ;

    case WM_KEYUP :
        res = cz_keyup (hwnd, CZ_PTR (hwnd), wparam, lparam) ;
        break ;

    case WM_KILLFOCUS :
    case WM_SETFOCUS :
        if (IsWindowEnabled (hwnd))
        {
            /* the inner rectangle is drawn in red if the window has the focus.
               All this does is swap focus and non-focus brushes (assuming the
               window is enabled in the first place)
            */
            lpz = CZ_PTR (hwnd) ;
            htmp = lpz->hbr ;
            lpz->hbr = lpz->hfocus ;
            lpz->hfocus = htmp ; 
            InvalidateRect (hwnd, NULL, TRUE) ;
            UpdateWindow (hwnd) ;
        }
        break ;

        /* tell the dialog manager to pass on arrow keys. This allows for
           keyboard control of the pan/tilt rectangle
        */
    case WM_GETDLGCODE :
        res = DLGC_WANTARROWS ;
        break ;
                        
        /* set the top left of the rectangle to a particular location */
    case CZ_SETPOSITION :
        res = cz_setposition (hwnd, CZ_PTR (hwnd), lparam) ;
        break ; 
        
        /* change the size of the inner rectangle */
    case CZ_SETSIZE :
        res = cz_setsize (hwnd, CZ_PTR (hwnd), lparam) ;
        break ;
        
        /* return the top left of the rectangle in the range 0..qcif */
    case CZ_GETPOSITION :
        lpz = CZ_PTR (hwnd) ;
        parx = (lpz->r.left - 1) * 2 ;
        pary = (lpz->r.top - 1) * 2 ;
        res = MAKELONG (parx, pary) ;
        /* tracemsg (TRC_MSG, "cz_getpos : parx = %d pary = %d", parx, pary) ;*/
        break ;
        
    default :
        res = DefWindowProc (hwnd, msg, wparam, lparam) ;
        break ;
    }          

    return res ; 
}

/*-----------------------------------------------------------------*/
/* calculate the rectangle starting positions, and create the brushes.
   The default start size is defined in the header, and works out at
   qcif / 2
*/

static LONG cz_create (HWND hwnd, LPCZOOM lpz)

{
RECT crect ;
 
    lpz->state = ST_NONE ;
 
    crect.top = 0 ;
    crect.left = 0 ;
    crect.right = QCIF_WIDTH ;
    crect.bottom = QCIF_HEIGHT ;
 
    cz_setsize (hwnd, lpz, (LPARAM) &crect) ;
 
    /* work out parent rectangle offset within window */
    GetClientRect (hwnd, &crect) ;
    lpz->parofs.x = (crect.right - PAR_WIDTH) / 2 ;
    lpz->parofs.y = (crect.bottom - PAR_HEIGHT) / 2 ;
                            
    /* create brushes */
    lpz->hbr = CreateSolidBrush (RGB (0, 0, 0)) ;                /* non-focus */
    /* COLOR_WINDOW changed to btnface to cater for the forced grey look 28-10-97 */
    lpz->hback = CreateSolidBrush (GetSysColor (COLOR_BTNFACE)) ; /* erase */
    lpz->hfocus = CreateSolidBrush (RGB (255, 0, 0)) ;            /* focus */
    lpz->hctlcolour = CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
 
    return 0 ;
}

/*-----------------------------------------------------------------*/
/* free any allocated resources */ 

static LONG cz_destroy (HWND hwnd, LPCZOOM lpz)

{
    DeleteObject (lpz->hbr) ;         
    DeleteObject (lpz->hback) ;
    DeleteObject (lpz->hfocus) ;
    DeleteObject (lpz->hctlcolour) ;
 
    LocalFree ((HANDLE) (PCZOOM) lpz) ;
 
    return 0 ;
}

/*-----------------------------------------------------------------*/
/* draw a PAR_xx sized rectangle centered within hwnd.
   draw lpz->rect at its proper position */
   
static LONG cz_paint (HWND hwnd, LPCZOOM lpz)

{  
HDC hdc ;
PAINTSTRUCT ps ;
RECT crect,
     rpar ;
int wid,
    ht ;
      
    hdc = BeginPaint (hwnd, &ps) ;
           
    GetClientRect (hwnd, &crect) ;
    wid = crect.right ;
    ht = crect.bottom ;
    SetRect (&rpar, 
             (wid - PAR_WIDTH) / 2,
             (ht - PAR_HEIGHT) / 2,
             ((wid - PAR_WIDTH) / 2) + PAR_WIDTH,
             ((ht - PAR_HEIGHT) / 2) + PAR_HEIGHT) ;
    FrameRect (hdc, &rpar, GetStockObject (BLACK_BRUSH)) ;
 
    draw_crect (hwnd, hdc, lpz, lpz->hbr) ;
           
    EndPaint (hwnd, &ps) ;
 
    return 0 ;
}

/*-----------------------------------------------------------------*/
/* draw the inner rectangle. This involves taking the rectangle's position
   relative to its enclosing rectangle, and adding that rectangle's offset
   from the window origin
   If the window is disabled, the rectangle isn't drawn
*/
   
static void draw_crect (HWND hwnd, HDC hdc, LPCZOOM lpz, HBRUSH hbr)

{
RECT rloc ;
 
    SetRect (&rloc, 
             lpz->r.left   + lpz->parofs.x,
             lpz->r.top    + lpz->parofs.y,
             lpz->r.right  + lpz->parofs.x,
             lpz->r.bottom + lpz->parofs.y) ;
          
    FrameRect (hdc, &rloc, IsWindowEnabled (hwnd) ? hbr : lpz->hback) ;
}

/*-----------------------------------------------------------------*/
/* left button pressed. Capture the focus, and move to ST_MDOWN
   check if the point pressed is within the rectangle.
   lparam contains client relative co-ordinates */

static LONG cz_buttondown (HWND hwnd, LPCZOOM lpz, WPARAM wparam, LPARAM lparam)

{
RECT r ;
#ifdef _WIN32
    POINTS points ;
#endif
POINT pt ;
   
    if (lpz->state == ST_NONE)
    {
        /* create a rectangle of the inner rectangle relative to the client
           window */ 
        SetRect (&r,  
                 lpz->r.left   + lpz->parofs.x,
                 lpz->r.top    + lpz->parofs.y,
                 lpz->r.right  + lpz->parofs.x,
                 lpz->r.bottom + lpz->parofs.y) ;                

#ifdef _WIN32
        points = MAKEPOINTS (lparam) ;
        POINTSTOPOINT (pt, points) ;
#else
        pt = MAKEPOINT (lparam) ;
#endif

        if (PtInRect (&r, pt))
        {                                       
            /* capture the mouse. This captures all mouse and keyboard clicks */
            if (wparam != -1)
            {
                //lpz->hcur = SetCursor (LoadCursor (NULL, IDC_CROSS)) ;
                SetCapture (hwnd) ;
                ShowCursor (FALSE) ;
                clip_mouse (hwnd, lpz) ;
            }
            lpz->state = ST_MDOWN ;
            lpz->mouseofs = pt ;
            notify_parent (hwnd, CZN_BUTTONDOWN) ;
        } 
  
        SetFocus (hwnd) ;
    }
  
    return 0 ;
}

/*-----------------------------------------------------------------*/
/* there may be stray button up messages, although I can't think of any! */

static LONG cz_buttonup (HWND hwnd, LPCZOOM lpz, WPARAM wparam, LPARAM lparam)

{
POINT pt ;

    if (lpz->state == ST_MDOWN)
    {
        if (wparam != -1)
        {
            //SetCursor (lpz->hcur) ;
            ReleaseCapture () ;
            ClipCursor (NULL) ;

            pt = lpz->mouseofs ;
            ClientToScreen (hwnd, &pt) ;
            SetCursorPos (pt.x, pt.y) ;

            ShowCursor (TRUE) ;
        }
        lpz->state = ST_NONE ;
        notify_parent (hwnd, CZN_BUTTONUP) ;
    }
 
    return 0 ;
}

/*-----------------------------------------------------------------*/
/* mouse has been moved. If the new location is still within the parent
   rectangle, erase the old frame and draw the new one. The parent window
   is also informed that movement has ocurred.
   This caters for situations where movement is possible in one plane, but
   not the other (eg at right hand side - can still move up/down, but not
   to the right.
   In this situation the mouse is repositioned to a valid point. This isn't
   perfect, as the mouse can be far away before the message is received, so
   it can be a bit jerky.
*/

static LONG cz_mousemove (HWND hwnd, LPCZOOM lpz, WPARAM wparam, LPARAM lparam)

{          
POINT pt ;
int difx,
    dify ;
HDC hdc ;
BOOL move_mouse = FALSE ;
#ifdef _WIN32
    POINTS points ;
#endif
                
    if (lpz->state == ST_MDOWN)
    {
#ifdef _WIN32
        points = MAKEPOINTS (lparam) ;
        POINTSTOPOINT (pt, points) ;
#else
        pt = MAKEPOINT (lparam) ;
#endif

        difx = pt.x - lpz->mouseofs.x ;    /* x movement */
        dify = pt.y - lpz->mouseofs.y ;
   
        if (difx < 0)    /* moved to the left */
        {
            difx = -difx ;    /* change to +ve number */
            if (lpz->r.left - difx < 1)
            {
                /* too far left. Constrain rectangle to 1, and keep mouse
                   in same relative position
                */
                difx = lpz->r.left - 1 ;   /* x distance of mouse */
                move_mouse = TRUE ;
            }
            difx = -difx ;   /* restore to a -ve number */
        }
        else
        {
            if (lpz->r.right + difx >= PAR_WIDTH)
            {
                /* too far right */
                difx = (PAR_WIDTH - 1) - lpz->r.right ;
                move_mouse = TRUE ;
            }
        }

        if (dify < 0)     /* moved up */
        {
            dify = -dify ;
            if (lpz->r.top - dify < 1)
            {
                dify = lpz->r.top - 1 ;
                move_mouse = TRUE ;
            }
            dify = -dify ;
        }
        else
        {
            if (lpz->r.bottom + dify >= PAR_HEIGHT)
            {
                dify = (PAR_HEIGHT - 1) - lpz->r.bottom ;
                move_mouse = TRUE ;
            }
        }

        lpz->mouseofs.x += difx ;
        lpz->mouseofs.y += dify ;
/*        if (move_mouse && wparam != -1)    CLIP
        {
            pt = lpz->mouseofs ;
            ClientToScreen (hwnd, &pt) ;
            SetCursorPos (pt.x, pt.y) ;
        }
*/
        /* erase old rectangle */
        hdc = GetDC (hwnd) ;
        draw_crect (hwnd, hdc, lpz, lpz->hback) ;

        /* offset and draw new one */
        OffsetRect (&lpz->r, difx, dify) ;
        draw_crect (hwnd, hdc, lpz, lpz->hbr) ;
     
        ReleaseDC (hwnd, hdc) ;
     
        if (wparam != -1)
        {
            clip_mouse (hwnd, lpz) ;
        }
     
        /* Notify the parent that something has happened. It will be up
           to the parent to send CZ_GETPOSITION to get the new info */
        if (difx || dify)
        {
            /*tracemsg (TRC_MSG, "sending czn_moved") ;*/
            notify_parent (hwnd, CZN_MOVED) ;
        }
    }
  
 return 0 ;
}
 
/*-----------------------------------------------------------------*/
/* see if the change in offsets would result in the rectangle going past
   the top left, or bottom right */
   
static BOOL rect_valid (LPRECT lpr, int *xofs, int *yofs)

{  
BOOL wcheck = TRUE,
     hcheck = TRUE ;
 
    if (lpr->left + *xofs <= 0 || lpr->right + *xofs >= PAR_WIDTH)
    {
        wcheck = FALSE ;
        *xofs = 0 ;
    }

    if (lpr->top + *yofs <= 0 || lpr->bottom + *yofs >= PAR_HEIGHT)
    {
        hcheck = FALSE ;
        *yofs = 0 ;
    }
 
    /* if (!wcheck && !hcheck)
    tracemsg (TRC_MSG, "invalid rect") ;*/
                        
    /* fail the call only if both checks are wrong */                                           
    return (wcheck == FALSE) && (hcheck == FALSE) ? FALSE : TRUE ;
}

/*-----------------------------------------------------------------*/
/* application requested position for the inner rectangle.
   The framing rectangle is QCIF sized (+ 2 for a border), and the inner
   rectangle is half this, with top left of (1,1) - just makes some of
   the drawing easier.
   The supplied co-ords are scaled down, then offset to match the above
   requirements
   
   The supplied rectangle is checked for validity before being accepted
*/

static LONG cz_setposition (HWND hwnd, LPCZOOM lpz, LPARAM lparam)

{
LPPOINT pt = (LPPOINT) lparam ;
RECT tmp ;
int x = 0,
    y = 0 ;
LONG res = 0 ;
 
    tmp.left   = (pt->x / 2) + 1 ;
    tmp.top    = (pt->y / 2) + 1 ;
    tmp.right  = tmp.left + IRECTWID (lpz->r) ;
    tmp.bottom = tmp.top + IRECTHT (lpz->r) ;
 
    if (rect_valid (&tmp, &x, &y))
    {
        CopyRect (&lpz->r, &tmp) ;
        InvalidateRect (hwnd, NULL, TRUE) ;
        UpdateWindow (hwnd) ;     
        res = 1 ;
    }

    return res ;
}
 
/*-----------------------------------------------------------------*/
/* send a notification message to the parent window */

static BOOL notify_parent (HWND hwnd, UINT msg)

{
#ifdef _WIN32
    return PostMessage (GetParent (hwnd), 
                        WM_COMMAND, 
                        MAKELONG ((WORD) GetWindowLong (hwnd, GWL_ID), msg),
                        (LPARAM) hwnd) ;
#else
    return PostMessage (GetParent (hwnd), 
                        WM_COMMAND, 
                        GetWindowWord (hwnd, GWW_ID),
                        MAKELONG (hwnd, msg)) ;
#endif
} 

/*-----------------------------------------------------------------*/
/* this clips the mouse to a rectangle 1 pixel bigger than the inner
   rectangle
*/   

static void clip_mouse (HWND hwnd, LPCZOOM lpz)

{
RECT r ;
POINT pt ;
int clip_margin = 4 ;

    pt.x = lpz->r.left + lpz->parofs.x ;
    pt.y = lpz->r.top  + lpz->parofs.y ;
 
    ClientToScreen (hwnd, &pt) ;
 
    SetRect (&r, 
             pt.x - clip_margin, 
             pt.y - clip_margin, 
             pt.x + IRECTWID (lpz->r) + clip_margin,
             pt.y + IRECTHT (lpz->r) + clip_margin) ;
          
    ClipCursor (&r) ;
}

/*-----------------------------------------------------------------*/
/* set the size of the inner rectangle. This must be a roi <= qcif sized.
   co-ordinates are supplied unscaled, ie in pixels.
   the rectangle is centered in the parent rectangle */
   
static LONG cz_setsize (HWND hwnd, LPCZOOM lpz, LPARAM lparam)

{
LPRECT lpr = (LPRECT) lparam ;
RECT tmp ;
               
    CopyRect (&tmp, lpr) ;
 
    /* scale the roi to fit */               
    tmp.left   = (tmp.left / 2) + 1 ;
    tmp.right  = (tmp.right / 2) + 1 ;
    tmp.top    = (tmp.top / 2) + 1 ;
    tmp.bottom = (tmp.bottom / 2) + 1 ;
 
    /* topleft of child rectangle within parent */
    lpz->topleft.x = (PAR_WIDTH - IRECTWID (tmp)) / 2 ;
    lpz->topleft.y = (PAR_HEIGHT - IRECTHT (tmp)) / 2 ;
 
    /* fit the rectangle within the client window */
    SetRect (&lpz->r, 
             lpz->topleft.x,
             lpz->topleft.y,
             lpz->topleft.x + IRECTWID (tmp), 
             lpz->topleft.y + IRECTHT (tmp)) ;
          
    return 0 ;
}

/*-----------------------------------------------------------------*/
/* fake a button down and mouse move message */

static long cz_keydown (HWND hwnd, LPCZOOM lpz, WPARAM wparam, LPARAM lparam)

{
short repcount,
      xmove = 0,
      ymove = 0,
      lt,
      tp ;

    if (wparam == VK_LEFT || wparam == VK_RIGHT || wparam == VK_UP ||
        wparam == VK_DOWN)
    {
        lt = (short)(((lpz->r.right - lpz->r.left) / 2) + lpz->parofs.x + lpz->r.left) ;
        tp = (short)(((lpz->r.bottom - lpz->r.top) / 2) + lpz->parofs.y + lpz->r.top) ;

        /* send a mousedown message */
        SendMessage (hwnd, WM_LBUTTONDOWN, (WPARAM) -1, MAKELPARAM (lt, tp)) ;

        repcount = LOWORD (lparam) ;
        switch (wparam)
        {
        case VK_LEFT :
            xmove = (short)(lpz->mouseofs.x - (repcount * 4)) ;
            ymove = (short)lpz->mouseofs.y ;
            break ;

        case VK_RIGHT :
            xmove = (short)(lpz->mouseofs.x + (repcount * 4)) ;
            ymove = (short)lpz->mouseofs.y ;
            break ;

        case VK_UP :
            ymove = (short)(lpz->mouseofs.y - (repcount * 2)) ;
            xmove = (short)lpz->mouseofs.x ;
            break ;

        case VK_DOWN :
            ymove = (short)(lpz->mouseofs.y + (repcount * 2)) ;
            xmove = (short)lpz->mouseofs.x ;
            break ;
        }

        SendMessage (hwnd, WM_MOUSEMOVE, (WPARAM) -1, MAKELPARAM (xmove, ymove)) ;
    }

    return 0 ;
}

/*-----------------------------------------------------------------*/

static long cz_keyup (HWND hwnd, LPCZOOM lpz, WPARAM wparam, LPARAM lparam)

{
    SendMessage (hwnd, WM_LBUTTONUP, (WPARAM) -1, 0) ;

    return 0 ;
}

/*-----------------------------------------------------------------*/

static void DbgPrint (LPSTR str, ...)

{  
#ifdef _DEBUG
                   
static char buf [256] ;
va_list argptr ;
                        
    /* set up the variable list, and create the message */
    va_start (argptr, str) ;
    wvsprintf (buf, str, argptr) ;                       
    va_end (argptr) ;

    OutputDebugString ("**** ") ;                
    OutputDebugString (buf) ;

#endif
}

