/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/* DEBUGLVL.H
   6-05-97
   macros and #defines for controlling the debuglevel variable */
  
/* #defines for debuglevel masks */
#define DBG_NOCAMERA     0x0001  /* camera not detected on startup */
#define DBG_INITPPC      0x0004  /* display errors during ppc initialisation */

#define DBG_DISABLE_DMA  0x0008  /* disable ECP DMA mode  */
#define DBG_DISABLE_ECP  0x0010  /* disable ECP ( force it to use 4 bit ) */
#define DBG_TRACEMSG     0x0020  /* message tracing. (tracemsg function) - kindof working */
#define DBG_DISABLE_FDMA 0x0040  /* disable Type F DMA transfers		 */
#define DBG_DISABLE_STREAM_INT 0x0080   /* disable the use of the StreamReadyInt by GetFrame */

/* if the debug bit is set, a messagebox is displayed showing the supplied
   message */
#define DEBUG_MSG(a, b)            \
  { extern WORD gDebugLevel ;      \
    if ((gDebugLevel & a) == a)    \
     MessageBox (NULL, b, #a, MB_OK | MB_ICONEXCLAMATION) ; \
  }

/* if the flag is set, perform the operation. If it isn't set, don't perform
   the operation */                        
#define IF_DEBUG_DO(a, b)       \
  { extern WORD gDebugLevel ;   \
    if ((gDebugLevel & a) == a) \
     b ;                        \
  }

/* if the flag is set, don't perform the operation. If it isn't set, perform
   the operation */                          
#define IF_DEBUG_DONT(a, b)     \
  { extern WORD gDebugLevel ;   \
    if ((gDebugLevel & a) == 0) \
     b ;                        \
  }

/*-----------------------------------------------------------------*/                              
/* test if one or more debug flags are set */

_inline static BOOL dbg_is_bit_set (WORD debug_bit)

{
 extern WORD gDebugLevel ;
 
 return (gDebugLevel & debug_bit) == debug_bit ;
}

/*-----------------------------------------------------------------*/
/* set one or more flags */

_inline static void dbg_set_bit (WORD debug_bit)

{
 extern WORD gDebugLevel ;
                               
 gDebugLevel |= debug_bit ;
}

/*-----------------------------------------------------------------*/
/* clear one or more flags */

_inline static void dbg_clear_bit (WORD debug_bit)

{
 extern WORD gDebugLevel ;
                               
 gDebugLevel &= ~debug_bit ;
}

/* function prototypes */                              
_inline static BOOL dbg_is_bit_set (WORD debug_bit) ;
_inline static void dbg_set_bit (WORD debug_bit) ;
_inline static void dbg_clear_bit (WORD debug_bit) ;
