/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/****************************************************************************
 *
 *   drvproc.c
 * 
 *   Driver Entry Point 
 *
 *   Microsoft Video for Windows Sample Capture Driver
 *   
 *
 ***************************************************************************/
/**************************************************************************
 *
 *  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 *  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
 *  PURPOSE.
 *
 *  Copyright (c) 1992, 1993  Microsoft Corporation.  All Rights Reserved.
 * 
 **************************************************************************/

#include "common.h"

#include "vvlcap.h"
#include "dvm_msg.h"
#include "resource.h"
#include "vfwdlg.h"

extern HINSTANCE ghModule;   /* dll instance handle. Declared in dvm_msg.c */

//
//  we use this driverID to determine when we where opened as a video
//  device or from the control panel, etc....
//
#define BOGUS_DRIVER_ID     1

/*
DriverProc  The entry point for an installable driver.
                
DWORD dwDriverId  For most messages, dwDriverId is the DWORD
      value that the driver returns in response to a DRV_OPEN message.
      Each time that the driver is opened, through the DrvOpen API,
      the driver receives a DRV_OPEN message and can return an
      arbitrary, non-zero value. The installable driver interface
      saves this value and returns a unique driver handle to the 
      application. Whenever the application sends a message to the
      driver using the driver handle, the interface routes the message
*      to this entry point and passes the corresponding dwDriverId.
      This mechanism allows the driver to use the same or different
      identifiers for multiple opens but ensures that driver handles
      are unique at the application interface layer.
 
      The following messages are not related to a particular open
      instance of the driver. For these messages, the dwDriverId
      will always be zero.
 
          DRV_LOAD, DRV_FREE, DRV_ENABLE, DRV_DISABLE, DRV_OPEN
 
HDRVR  hDriver  This is the handle returned to the
      application by the driver interface.
           
UINT  uiMessage  The requested action to be performed. Message
      values below DRV_RESERVED are used for globally defined messages.
      Message values from DRV_RESERVED to DRV_USER are used for
      defined driver protocols. Messages above DRV_USER are used
      for driver specific messages.
 
LPARAM  lParam1  Data for this message.  Defined separately for
      each message
 
LPARAM  lParam2  Data for this message.  Defined separately for
      each message
 
Returns: Defined separately for each message. 
*/

BOOL gfInDriver = FALSE;
 
 
LRESULT FAR PASCAL _loadds DriverProc( DWORD dwDriverID, HDRVR hDriver, 
                                       UINT uiMessage, LPARAM lParam1, 
                                       LPARAM lParam2 )
{
    int port;
    char message[80],
            str[256],     
            desc [255],
            language[80], 
            title[80];     
    HWND hparent;
    LRESULT res;

/*
//    
//  The driver must allow DVM_FRAME messages to be processed while the 
//  dialog boxes are being displayed to allow AVICAP to display live 
//  video during this time
//
    if ( gfInDriver ) 
    {
        if ( TMSET() )
            D1( "VVLCPIA: reentrant call" );
            tracemsg( TRC_MSG, "VVLCPIA: reentrant call Message = 0x%04x", uiMessage );
            return( DRV_CANCEL );
    }
*/

    gfInDriver = TRUE;
 
    res = DRV_CANCEL;
    
    switch( uiMessage )
    {
    case DRV_LOAD:
    /*
    Sent to the driver when it is loaded. Always the first
    message received by a driver.
    
      dwDriverID is 0L. 
      lParam1 is 0L.
      lParam2 is 0L.
      
        Return 0L to fail the load.
        
          DefDriverProc will return NON-ZERO so we don't have to
          handle DRV_LOAD
        */
        if ( TMSET() )
            tracemsg( TRC_MSG, "DRV_LOAD - ok" );
        res = DRV_OK; 
        break;
        
    case DRV_FREE:
    /*
    Sent to the driver when it is about to be discarded. This
    will always be the last message received by a driver before
    it is freed. 
    
      dwDriverID is 0L. 
      lParam1 is 0L.
      lParam2 is 0L.
      
        Return value is ignored.
        */
        
        if ( TMSET() )
        {
            tracemsg( TRC_MSG, "DRV_FREE - ok" );
            tracemsg( TRC_DESTROY );
        }
        res = DRV_OK;
        break;
        
    case DRV_OPEN:
        D1( "DRV_OPEN" );
        
        /*
            Sent to the driver when it is opened. 
        
            dwDriverID is 0L.
          
            lParam1 is a far pointer to a zero-terminated string
            containing the name used to open the driver.
            
            lParam2 is passed through from the drvOpen call. It is
            NULL if this open is from the Drivers Applet in control.exe
            It is LPVIDEO_OPEN_PARMS otherwise.
              
            Return 0L to fail the open.
               
            DefDriverProc will return ZERO so we do have to
            handle the DRV_OPEN message.
        */
        
        //
        //  if we were opened without a open structure then just
        //  return a phony (non zero) id so the OpenDriver() will
        //  work.
        //
        if ( lParam2 == NULL )
        {
            if ( TMSET() )
                tracemsg( TRC_MSG, "DRV_OPEN - bogus driver id" );
            res = BOGUS_DRIVER_ID;
            break;
        }
        
        //  Verify this open is for us, and not for an installable
        //  codec.
        if ( (((LPVIDEO_OPEN_PARMS)lParam2)->fccType) != OPEN_TYPE_VCAP )
        {
            if ( TMSET() )
                tracemsg( TRC_MSG, "DRV_OPEN - not TYPE_VCAP" );
            res = 0L;
            break;
        }
        
        res = (DWORD)VideoOpen( (char FAR *)lParam1, (LPVIDEO_OPEN_PARMS)lParam2 );
        if ( TMSET () )
            tracemsg( TRC_MSG, "    DRV_OPEN - handle = 0x%08lx", res );
        break;

    case DRV_CLOSE:
        if ( TMSET() )
            tracemsg( TRC_MSG, "DRV_CLOSE - 0x%08lx", dwDriverID );
        
        /*
            Sent to the driver when it is closed. Drivers are unloaded
            when the close count reaches zero. 
            
            dwDriverID is the driver identifier returned from the
            corresponding DRV_OPEN.
              
            lParam1 is passed through from the drvClose call.
                
            lParam2 is passed through from the drvClose call.
                  
            Return 0L to fail the close.
                    
            DefDriverProc will return ZERO so we do have to
            handle the DRV_CLOSE message.
        */
        
        if ( dwDriverID == BOGUS_DRIVER_ID || dwDriverID == 0 )
        {
            res = DRV_OK;
            break;
        }

        res = (VideoClose((HCHANNEL)dwDriverID) == DV_ERR_OK) ? 1L : 0;
        
        if ( TMSET() )
            tracemsg( TRC_MSG, "    DRV_CLOSE - res = %s", res?(LPSTR)"ok":(LPSTR)"error" );
    
        break;
        
    case DRV_ENABLE:
        if ( TMSET() )
            tracemsg( TRC_MSG, "DRV_ENABLE - ok" );
        
        /*
            Sent to the driver when the driver is loaded or reloaded
            and whenever Windows is enabled. Drivers should only
            hook interrupts or expect ANY part of the driver to be in
            memory between enable and disable messages
        
            dwDriverID is 0L. 
            lParam1 is 0L.
            lParam2 is 0L.
          
            Return value is ignored.
        */
        res = VideoEnable();
        break;
        
    case DRV_DISABLE:
        if ( TMSET() )
            tracemsg( TRC_MSG, "DRV_DISABLE - ok" );
        
        /*
            Sent to the driver before the driver is freed.
            and whenever Windows is disabled
        
            dwDriverID is 0L. 
            lParam1 is 0L.
            lParam2 is 0L.
          
            Return value is ignored.
        */
        res = VideoDisable();
        break;
        
    case DRV_QUERYCONFIGURE:
        if ( TMSET() )
            tracemsg( TRC_MSG, "DRV_QUERYCONFIGURE - ok");
        
        /*
            Sent to the driver so that applications can
            determine whether the driver supports custom
            configuration. The driver should return a
            non-zero value to indicate that configuration
            is supported.
            
            dwDriverID is the value returned from the DRV_OPEN
            call that must have succeeded before this message
            was sent.
              
            lParam1 is passed from the app and is undefined.
            lParam2 is passed from the app and is undefined.
                
            Return 0L to indicate configuration NOT supported.
        */
        
        res = DRV_OK;
        break;
        
    case DRV_CONFIGURE:
        if ( TMSET() )
            tracemsg( TRC_MSG, "DRV_CONFIGURE - ok");
        
        /*
            Sent to the driver so that it can display a custom
            configuration dialog box.
            
            lParam1 is passed from the app. and should contain
            the parent window handle in the loWORD.
            lParam2 is passed from the app and is undefined.
              
            Return value is undefined.
                
            Drivers should create their own section in system.ini.
            The section name should be the driver name.
        */
        
        hparent = (HWND)LOWORD( lParam1 );
        DialogBox( ghModule, MAKEINTRESOURCE(DlgAbout), hparent, DlgAboutProc );
        
        res = DRV_OK;
        break;
        
        
    case DRV_INSTALL:
        if ( TMSET() )
            tracemsg( TRC_MSG, "DRV_INSTALL - ok" );
        /*
            Windows has installed the driver, and since we
            are loaded on demand, there is no need to 
            restart Windows.
        */
        res = DRV_OK;
        break;
        
    case DRV_REMOVE:
        if ( TMSET() )
            tracemsg( TRC_MSG, "DRV_REMOVE - ok" );
        /* ConfigRemove(); */
        res = DRV_OK;
        break;
        
    default:
        if (dwDriverID == BOGUS_DRIVER_ID || dwDriverID == 0)
        {
            if ( TMSET() )
                tracemsg( TRC_MSG, "DRV_??? - unsupported msg - 0x%04x", uiMessage );
            res = DefDriverProc(dwDriverID, hDriver, uiMessage,lParam1, lParam2 );
        }
        else
        {
            res = VideoProcessMessage((HCHANNEL)dwDriverID, uiMessage, lParam1, lParam2 );
        }
    }

    gfInDriver = FALSE;
    return( res );
}

