/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */



#include <windows.h>
#include <mmsystem.h>
#include <msvideo.h>

#include <string.h>


#include "vvl_type.h"
#include "camtypes.h"

#include "cmd.h"
#include "cpia_vid.h"

#include "pport.h"
#include "pplink.h"
#include "cpialink.h"
#include "cpia_cmd.h"

#include "cpia_cam.h"
#include "capture.h"

#include "info_hdr.h"

#include "debuglvl.h"
#include "tracemsg.h"
#include "vfwdlg.h"

#include "decode.h"
#include "decode32.h"
#include "resize.h"

#include "getframe.h"


#define GRAB_TRIES	3




/*--------------------------------------------------------------------------*/
/*	GF_Init			 														*/
/*--------------------------------------------------------------------------*/
BOOL GF_Init( void ) {
    InitDecode();    
    InitDecode32();

	return( TRUE );
}

 
/*--------------------------------------------------------------------------*/
/*	GF_Fini			 														*/
/*--------------------------------------------------------------------------*/
BOOL GF_Fini( void ) {
	FiniDecode();     
    FiniDecode32();

	return( TRUE );
}

    
/*--------------------------------------------------------------------------*/
/*	GF_CreateGFrame 														*/
/*--------------------------------------------------------------------------*/
PGFRAME GF_CreateGFrame( void ) {
	HLOCAL hGFrame;
	PGFRAME pGFrame;

  	hGFrame = LocalAlloc( LPTR, sizeof( GFRAME ) );
	if ( hGFrame == 0 ) return( NULL );
   	pGFrame = (PGFRAME)LocalLock( hGFrame );
	pGFrame->HandleToThisInstance = hGFrame;

	return( pGFrame );
}


/*--------------------------------------------------------------------------*/
/*	GF_DestroyGFrame 														*/
/*--------------------------------------------------------------------------*/
BOOL GF_DestroyGFrame( PGFRAME pGFrame ) {
 	HLOCAL hGFrame;

	if ( !pGFrame )	return( FALSE );

	hGFrame = pGFrame->HandleToThisInstance;
	LocalUnlock( hGFrame);
	LocalFree( hGFrame );

	return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	GF_InitialiseGFrame														*/
/*--------------------------------------------------------------------------*/
BOOL GF_InitialiseGFrame( PGFRAME pGFrame, PLINK pLink, PCPIA_CAM pCamera, 
														PCAPTURE pCap ) {
 	BOOL rtn;
 	BYTE port_type;

	/* check our parameters are valid 	*/	
	if ( !pGFrame ) return( FALSE );
	if ( !pLink || !pCamera || !pCap ) return( FALSE );

	pGFrame->pLink = pLink;
	pGFrame->pCamera = pCamera;
	pGFrame->pCap = pCap;
	     
	if ( !PPLINK_GetPortInfo( pGFrame->pLink, NULL, &port_type, NULL ) ) return( FALSE );
 
	if ( (port_type & DMA_PORT_TYPE) && (port_type & INT_PORT_TYPE) ) 
		pGFrame->AllowGrabAhead = TRUE;		
	else
		pGFrame->AllowGrabAhead = FALSE;		


	/* alloc RawBuffers[2] and DecompressedImage */
	if ( !AllocBuffers( pGFrame ) ) return( FALSE );

	pGFrame->RawBufferState[0] = BUFFER_EMPTY;
	pGFrame->RawBufferState[1] = BUFFER_EMPTY;
	pGFrame->BytesUploaded[0] = 0;
	pGFrame->BytesUploaded[1] = 0;
	pGFrame->UploadChunkSize = DEFAULT_CHUNK_SIZE;

	pGFrame->FillBuffer = 0;
	pGFrame->DecodeBuffer = 0;

	/* initialise GetFrame control variables			*/
	pGFrame->StreamMode = FALSE;
	pGFrame->UseStreamReadyINT = FALSE;
	pGFrame->DoGrabAhead = pGFrame->AllowGrabAhead;
	pGFrame->Use32BitDecode = TRUE;
	pGFrame->NeedUncompressedFrame = TRUE;
	pGFrame->UseOwnBuffer = TRUE;

	pGFrame->StreamStartLine = 0;

	pGFrame->LastAppRequestTime = timeGetTime();
	pGFrame->PreviousFrameProcessingTime = 200;

	rtn = CPIA_SetGrabMode( pGFrame->pLink, TRUE );

	return( rtn );
}


/*--------------------------------------------------------------------------*/
/*	GF_DeinitialiseGFrame													*/
/*--------------------------------------------------------------------------*/
BOOL GF_DeinitialiseGFrame( PGFRAME pGFrame ) {
	BYTE status;
	DWORD actual_bytes;

	if ( !pGFrame )	return( FALSE );

	do { 
		if ( !CLINK_GetUploadStatus( pGFrame->pLink, &status, &actual_bytes )
			 || ( status == UPLOAD_FAILED ) ) break;
   	} while( status != UPLOAD_IDLE );
	
	if ( status == UPLOAD_FAILED ) CLINK_AbortUpload( pGFrame->pLink );

	CPIA_SetGrabMode( pGFrame->pLink, FALSE );

/*	
	do {
		GF_GetFrameState( pGFrame, &state );
	} while( state != GF_FRAME_IDLE );
*/	

	DeallocBuffers( pGFrame );

	return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	GF_GetFrame		 														*/
/*--------------------------------------------------------------------------*/
BOOL GF_GetFrame( PGFRAME pGFrame, BYTE _huge *pImageBuffer, DWORD MaxBytes, DWORD *CapTime ) {
	BOOL rtn, cmds_sent, is_compressed;
	WORD grab_tries;
	BYTE status, stream_state;
	DWORD actual_bytes, tmp;
	DWORD t0, t1, t2 , t3, t4;

	if ( !pGFrame )	return( FALSE );

	t0 = timeGetTime();
	pGFrame->AppFrameRequestTime = (WORD)(t0 - pGFrame->LastAppRequestTime);
	
	if ( pGFrame->DoGrabAhead && (pGFrame->AppFrameRequestTime > 1000) )
	{
		pGFrame->NeedUncompressedFrame = TRUE;
	}
	
	pGFrame->LastAppRequestTime = t0;

	grab_tries = 0;
	do {

		if ( pGFrame->RawBufferState[ pGFrame->FillBuffer ] == BUFFER_EMPTY ) {
			
			/* 
				======== start a new upload if buffer has not been filling ==========
			*/
			cmds_sent = SendFrameTimeCmds( pGFrame ); 
			cmds_sent |= SendCaptureParams( pGFrame, pGFrame->NeedUncompressedFrame );

			pGFrame->UseStreamReadyINT = FALSE;
			if ( !pGFrame->StreamMode ) 
		        if ( !CPIA_GrabFrame( pGFrame->pLink, pGFrame->StreamStartLine ) ) goto GetFrameError;

            do {
                /* poll camera here to check if STREAM_READY */
                if ( GET_STREAM_STATE( pGFrame->pLink, &stream_state ) )
                {
                    if ( stream_state == STREAM_PAUSED )
                    {
                        /* wow, this isn't good */
                        ClearStall( pGFrame );
                        goto GetFrameError;
                    }
                }
                else
                {
                    goto GetFrameError;
                }
            } while( stream_state != STREAM_READY );

			rtn = CLINK_StartUpload( pGFrame->pLink, 
									 pGFrame->UseStreamReadyINT,
									 FALSE, 
									 pGFrame->RawBufferSize[ pGFrame->FillBuffer ],
									 pGFrame->pRawBuffer[ pGFrame->FillBuffer ],
									 pGFrame->UploadChunkSize,
									 TRUE );

	    	if ( rtn ) pGFrame->RawBufferState[ pGFrame->FillBuffer ] = BUFFER_FILLING;
		}		   									

		/* 
			======== wait for upload to finish ==========
		*/
		do { 
			if ( !CLINK_GetUploadStatus( pGFrame->pLink, &status, &actual_bytes ) ) goto GetFrameError;	

			if ( status == UPLOAD_FAILED ) 
            {
				if ( pGFrame->UseStreamReadyINT ) 
                {
   	             	// if the upload failed it may be due to the StreamReadyINT not
    	            // working and us not detecting the StreamReadyINT
        	        // -- justdisbale it for now and hope it works....
            	    //
                	dbg_set_bit( DBG_DISABLE_STREAM_INT );
          	    	if ( TMSET() ) tracemsg( TRC_MSG, "GF_GetFrame: DBG_DISABLE_STREAM_INT" );
				}
                goto GetFrameError;
            }
		} while( status != UPLOAD_IDLE );
	   	pGFrame->RawBufferState[ pGFrame->FillBuffer ] = BUFFER_FULL;

		t1 = timeGetTime();


		/* swop Fill and Decode buffers */
		pGFrame->DecodeBuffer = pGFrame->FillBuffer;
		pGFrame->FillBuffer = !pGFrame->FillBuffer;


		/* calc new chunksize for next upload */
 	    tmp = DEFAULT_CHUNK_SIZE;
 	    if ( actual_bytes ) tmp = actual_bytes / 12;

	  	if ( tmp < MIN_CHUNK_SIZE ) tmp = MIN_CHUNK_SIZE;
		else if ( tmp > pGFrame->RawBufferSize[ pGFrame->FillBuffer ] )
			 tmp = pGFrame->RawBufferSize[ pGFrame->FillBuffer ];
 		
 		pGFrame->UploadChunkSize = tmp;


		/* 
			======== if doing GrabAhead then kick off new grab and upload ========
		*/		
		if ( pGFrame->DoGrabAhead && !pGFrame->NeedUncompressedFrame ) {

            cmds_sent = SendFrameTimeCmds( pGFrame ); 
 			cmds_sent |= SendCaptureParams( pGFrame, pGFrame->NeedUncompressedFrame );

//
// Horrible use of global here to prevent the use of the StreamReadyINT
//
            if ( dbg_is_bit_set( DBG_DISABLE_STREAM_INT ) )
            {
                /*
                    pretend that commands where sent to prevent
                    the use of the StreamReadyINT
                */
                cmds_sent = TRUE;
            }

            if ( !pGFrame->StreamMode ) 
            {
                if ( !CPIA_GrabFrame( pGFrame->pLink, pGFrame->StreamStartLine ) ) goto GetFrameError;
            }

			if ( !cmds_sent ) 
			{  /* 
					no commands where sent to the camera, so we can trust the 
					StreamReady INT.
				*/
				pGFrame->UseStreamReadyINT = TRUE;
			}
			else
			{	/* 
			    	commands where sent to the camera so we can not trust 
					the StreamReady INT for the next upload. thus we must poll
					for STREAM_READY before starting th upload 
				*/			
				pGFrame->UseStreamReadyINT = FALSE;

                do { 
                    /* poll camera here to check if STREAM_READY */
                    if ( GET_STREAM_STATE( pGFrame->pLink, &stream_state ) )
                    {
                        if ( stream_state == STREAM_PAUSED )
                        {
                            /* wow, this isn't good */
                            ClearStall( pGFrame );
                            goto GetFrameError;
                        }
                    }
                    else
                    {
                        goto GetFrameError;
                    }
                } while( stream_state != STREAM_READY );
			}

			rtn = CLINK_StartUpload( pGFrame->pLink, 
										 pGFrame->UseStreamReadyINT,
										 TRUE, 
										 pGFrame->RawBufferSize[ pGFrame->FillBuffer ],
										 pGFrame->pRawBuffer[ pGFrame->FillBuffer ],
										 pGFrame->UploadChunkSize,
										 TRUE );

            if ( !rtn && pGFrame->UseStreamReadyINT )
            {
                //
                // if the StartUpload failed it may be due to us having falsely 
                // detected the StreamReadyINT, as happened on some ASUS motherboards
                // -- so try disabling use of StreamReadyINT  
                //
                dbg_set_bit( DBG_DISABLE_STREAM_INT );
          	    if ( TMSET() ) tracemsg( TRC_MSG, "GF_GetFrame: DBG_DISABLE_STREAM_INT" );
                goto GetFrameError;
            }

			if ( !pGFrame->UseStreamReadyINT ) {
				/* 
					if we're not using StreamReady INT we need to call GetUploadStatus
					enough times to get the DMA going by its self.
				*/
				CLINK_GetUploadStatus( pGFrame->pLink, &status, &actual_bytes );
				CLINK_GetUploadStatus( pGFrame->pLink, &status, &actual_bytes );
			}
	    	if ( rtn ) pGFrame->RawBufferState[ pGFrame->FillBuffer ] = BUFFER_FILLING;
	   	}

		t3 = timeGetTime();

		/* 
			======== decode RawBuffer into the target image ======== 
		*/
		if ( pGFrame->RawBufferState[ pGFrame->DecodeBuffer ] == BUFFER_FULL ) {
		
			if ( !ImageVerify( pGFrame, pGFrame->pRawBuffer[pGFrame->DecodeBuffer],
										MaxBytes, &is_compressed, &(pGFrame->FrameHeader[0]) ) ) {
 				/* the buffer is bad so force a new grab/upload sequence */
				pGFrame->NeedUncompressedFrame = TRUE;
				if ( TMSET() )
					tracemsg( TRC_MSG, "GF_GetFrame: ImageVerify failed" );
 			}
			else {
 				/* the buffer is good so decode it */
//				if ( pGFrame->NeedUncompressedFrame && is_compressed )
//					tracemsg( TRC_MSG, "GF_GetFrame: Need uncompressed, got compressed\n" );

 				if ( !pGFrame->NeedUncompressedFrame 
	 					 || (pGFrame->NeedUncompressedFrame && !is_compressed) ) {


					rtn = ImageDecode( pGFrame, pGFrame->pRawBuffer[pGFrame->DecodeBuffer], 
												pImageBuffer, MaxBytes );

				    if ( CapTime )
						  *CapTime = timeGetTime();   

 					pGFrame->NeedUncompressedFrame = FALSE;
				}
			}
	   		pGFrame->RawBufferState[ pGFrame->DecodeBuffer ] = BUFFER_EMPTY;
		}
		else {
			/* something must have wrong as we don't have a buffer to decode */	
			goto GetFrameError;
		}

	} while( (pGFrame->NeedUncompressedFrame)&& (grab_tries++ < GRAB_TRIES) );

	t4 = timeGetTime();

	if ( grab_tries < GRAB_TRIES ) 
	{
		t0 = timeGetTime();
		pGFrame->PreviousFrameProcessingTime = (WORD)(t0 - pGFrame->LastAppRequestTime);

		return( TRUE );
	}
	else 
		goto GetFrameError;	
	

GetFrameError:

	if ( TMSET() ) tracemsg( TRC_MSG, "GF_GetFrame: Error" );

	GF_ResetGetFrame( pGFrame );
	
	return( FALSE );
}


/*--------------------------------------------------------------------------*/
/*	GF_EnableGrabAhead 														*/
/*--------------------------------------------------------------------------*/
BOOL GF_EnableGrabAhead( PGFRAME pGFrame, BOOL EnableGrabAhead ) {
	BYTE state;

	if ( !pGFrame )	return( FALSE );

	if ( EnableGrabAhead ) 
	{
		pGFrame->DoGrabAhead = pGFrame->AllowGrabAhead;
//		tracemsg( TRC_MSG, "Enabling Grab Ahead\n" );
	}
	else 
	{
		pGFrame->DoGrabAhead = FALSE;
//		tracemsg( TRC_MSG, "Disabling Grab Ahead\n" );

		/* wait for current upload to finish before returning */
		do {
			GF_GetFrameState( pGFrame, &state );
			if ( state == GF_FRAME_ERROR ) break;
		} while( state != GF_FRAME_IDLE );
		
		if ( state == GF_FRAME_ERROR ) GF_ResetGetFrame( pGFrame );	
	}

	return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	GF_GetFrameState 														*/
/*--------------------------------------------------------------------------*/
BOOL GF_IsStreamReady( PGFRAME pGFrame, BOOL *Ready ) {
	BYTE stream_state, upload_status; 
	BOOL ready, rtn;
	static int count = 0;

	if ( !pGFrame )	return( FALSE );

	ready = TRUE;

	if ( pGFrame->StreamMode )
	{
		if ( pGFrame->RawBufferState[ pGFrame->FillBuffer ] == BUFFER_FILLING ) {
			CLINK_GetUploadStatus( pGFrame->pLink, &upload_status, NULL );
	
			if ( upload_status == UPLOAD_FAILED ) 
				ready = TRUE;
	
				/* a buffer was being marked as filling ... see how its doing */
			if ( upload_status != UPLOAD_IDLE &&  upload_status != UPLOAD_FAILED ) {
				ready = FALSE;
				count++;
			}			
		}

		if ( ready ) count = 0;
//		tracemsg( TRC_MSG, "GF_IsStreamReady: UseINT = %d   ready = %d   count = %d", 
//							pGFrame->UseStreamReadyINT, ready, count );
	}	  

	if ( Ready ) *Ready = ready;

	return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	GF_GetFrameState 														*/
/*--------------------------------------------------------------------------*/
BOOL GF_GetFrameState( PGFRAME pGFrame, BYTE *State ) {
	BYTE upload_status, stream_state; 
	BYTE state;
    BOOL rtn;
    
	if ( !pGFrame )	return( FALSE );

	state = GF_FRAME_IDLE;

	if ( !pGFrame->RawBufferState[ pGFrame->FillBuffer ] == BUFFER_FILLING ) {
		/* not doing anything at present */		
		state = GF_FRAME_IDLE;
	}
	else 
	{
		/* 
            a buffer was being marked as filling ... see how its doing
        */
		
        CLINK_GetUploadStatus( pGFrame->pLink, &upload_status, NULL );

		switch ( upload_status ) {
		case UPLOAD_IDLE:
				state = GF_FRAME_IDLE;
                break;
		case UPLOAD_STARTING:
				state = GF_FRAME_WAITING_FOR_STREAM;
				break;
		case UPLOAD_CHUNK_START:
		case UPLOAD_CHUNK_END:
		case UPLOAD_FINISHING:
				state = GF_FRAME_UPLOADING;
				break;
		case UPLOAD_FAILED:
				state = GF_FRAME_ERROR;
		}			
	}
	
	if ( state == GF_FRAME_ERROR ) {
		CLINK_AbortUpload( pGFrame->pLink ); 
	}

	if ( State ) *State = state;

	return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	GF_NeedReferenceFrame													*/
/*--------------------------------------------------------------------------*/
BOOL GF_NeedReferenceFrame( PGFRAME pGFrame ) {
	if ( !pGFrame )	return( FALSE );
	pGFrame->NeedUncompressedFrame = TRUE;
//	tracemsg( TRC_MSG, "GF_NeedRefenceFrame:\n" );
	return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	GF_UseOwnBuffer															*/
/*--------------------------------------------------------------------------*/
BOOL GF_UseOwnBuffer( PGFRAME pGFrame, BOOL UseOwnBuffer ) {
	if ( !pGFrame )	return( FALSE );
	pGFrame->UseOwnBuffer = UseOwnBuffer;
	return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	GF_ResetGetFrame 														*/
/*--------------------------------------------------------------------------*/
BOOL GF_ResetGetFrame( PGFRAME pGFrame ) {
	BYTE status;
	DWORD actual_bytes;
 
	if ( !pGFrame )	return( FALSE );

 	if ( pGFrame->RawBufferState[ pGFrame->FillBuffer ] == BUFFER_FILLING ) {
		CLINK_GetUploadStatus( pGFrame->pLink, &status, &actual_bytes );
		
		if ( TMSET() ) {
//			tracemsg( TRC_MSG, "GF_ResetGetFrame: upload_status = %d", status );
			tracemsg( TRC_MSG, "GF_ResetGetFrame: pLink - StreamTimedOut=%d  ChunkTimedOut=%d", 
								(pGFrame->pLink)->StreamTimedOut, (pGFrame->pLink)->ChunkTimedOut );
		}

		if ( status != UPLOAD_IDLE ) {
			CLINK_AbortUpload( pGFrame->pLink );
		}
		pGFrame->RawBufferState[ pGFrame->FillBuffer ] = BUFFER_EMPTY;
	}
	else {
//		if ( TMSET() ) 
//			tracemsg( TRC_MSG, "GF_ResetGetFrame: no buffer filling" );
	}

	return( TRUE );
}



/*--------------------------------------------------------------------------*/
/*	GF_SetFormats	 														*/
/*--------------------------------------------------------------------------*/
BOOL GF_SetFormats( PGFRAME pGFrame, LPBITMAPINFOHEADER SrcBmp, LPBITMAPINFOHEADER DestBmp ) {
 	BOOL res;
 	WORD bits;
  	DWORD compression;
    COLOUR_FORMAT cf;
       
	if ( !pGFrame )	return( FALSE );

 	pGFrame->StretchBy2 = FALSE;

 	BIH_GetType( DestBmp, &bits, &compression );

 	/* check that the DECODE module can handle the FOURCC being requested */
 	cf = FormatToSupplyFourCC32( compression, bits );
  	cf = FormatToSupplyFourCC( compression, bits );
	if ( cf == NOT_SUPPORTED ||	cf == NO_FORMAT ) 
 		return( FALSE );
	else
		pGFrame->ImageFormat = cf;

	res = FALSE;
 	/* check that the Src / Dest image sizes are sensible */
	if ( (SrcBmp->biWidth == DestBmp->biWidth) 
    		 && (SrcBmp->biHeight == DestBmp->biHeight) )
  	{
	 	if ( DestBmp->biCompression == BI_RGB )
 		{
      		res = TRUE ;
    	}
		else
	  	{	/* it;s a YUV format														*/
	   		/* BI_IYUV and BI_I420 are the same format. If IYUV requested, substitute I420 */    
	   		if ( DestBmp->biCompression == BI_IYUV ) DestBmp->biCompression = BI_I420 ;

			if ( DestBmp->biCompression == BI_YVU9 || 
	        	 DestBmp->biCompression == BI_YUY2 ||
	        	 DestBmp->biCompression == BI_UYVY ||
	        	 DestBmp->biCompression == BI_YV12 ||
	        	 DestBmp->biCompression == BI_I420)
	    		res = TRUE ;
	  	}
	}
	else
   	{
		/* Src and Dest image sizes where not the same so see if we can		*/
		/* resize the src to provide the the dest.							*/
	 	if ( (DestBmp->biCompression == BI_RGB)
			 && (DestBmp->biBitCount == 24)
			 &&	((SrcBmp->biWidth * 2) == DestBmp->biWidth) 
    		 && ((SrcBmp->biHeight * 2) == DestBmp->biHeight) )  
  		{
			 	pGFrame->StretchBy2 = TRUE;
  				res = TRUE ;
		}
	}

	if ( res )
	{
   		/* OK we can handle the requested format */
   		memcpy( &(pGFrame->SrcFormat), SrcBmp, sizeof( BITMAPINFOHEADER ) );
   		memcpy( &(pGFrame->DestFormat), DestBmp, sizeof( BITMAPINFOHEADER ) );
  	}  
   
 	return( res );
}


BOOL GF_GetDecodeErrorCount( PGFRAME pGFrame, DWORD *Count ) {
	if ( !pGFrame )	return( FALSE );
	return( FALSE );
}


BOOL GF_ClearDecodeErrorCount( PGFRAME pGFrame ) {
	if ( !pGFrame )	return( FALSE );
	return( FALSE );
}


/*--------------------------------------------------------------------------*/
/*	GF_InitFrameStream														*/
/*--------------------------------------------------------------------------*/
BOOL GF_InitFrameStream( PGFRAME pGFrame, WORD SkipFrames, WORD Flags ) {
	BOOL rtn;
	WORD col_start, col_end, row_start, row_end;

	if ( !pGFrame )	return( FALSE );

	if ( pGFrame->StreamMode ) return( FALSE );

	pGFrame->OldGrabAheadSetting = pGFrame->DoGrabAhead;

	if ( !GF_EnableGrabAhead( pGFrame, FALSE ) ) return( FALSE );

	pGFrame->Use32BitDecode = FALSE;

	rtn = CPIA_SetGrabMode( pGFrame->pLink, FALSE );

    if ( rtn )
    {                         
		capGetROI( pGFrame->pCap , &col_start, &col_end, &row_start, &row_end );

 		if ( pGFrame->AllowGrabAhead ) 
			pGFrame->StreamStartLine = ((row_end - row_start)/4); /* 25% through image 	*/
		else
			pGFrame->StreamStartLine = ((row_end - row_start)/2); /* 50% through image	*/

//		tracemsg( TRC_MSG, "GF_InitFrameStream: skip frames = %d\n", SkipFrames );
       	rtn &= CPIA_InitStreamCap( pGFrame->pLink, 
       								(BYTE)SkipFrames, 
       								pGFrame->StreamStartLine );
    }

	if ( rtn ) pGFrame->StreamMode = TRUE;

	return( rtn );
}


/*--------------------------------------------------------------------------*/
/*	GF_FiniFrameStream														*/
/*--------------------------------------------------------------------------*/
BOOL GF_FiniFrameStream( PGFRAME pGFrame ) {
   	BOOL rtn;

	if ( !pGFrame )	return( FALSE );

	if ( !pGFrame->StreamMode ) return( FALSE );

	pGFrame->Use32BitDecode = TRUE;

    rtn = CPIA_FiniStreamCap(pGFrame->pLink);
    if ( rtn )
    {
     	rtn = CPIA_SetGrabMode( pGFrame->pLink, TRUE );
     	if ( rtn )
        {
         	pGFrame->StreamMode = FALSE;
		 	rtn = GF_NeedReferenceFrame( pGFrame );
        }
    }

	if ( !GF_EnableGrabAhead( pGFrame, pGFrame->OldGrabAheadSetting ) ) return( FALSE );

	return( rtn );
}


/*--------------------------------------------------------------------------*/
/*	GF_StartFrameStream														*/
/*--------------------------------------------------------------------------*/
BOOL GF_StartFrameStream( PGFRAME pGFrame ) {
	BOOL rtn;

	if ( !pGFrame )	return( FALSE );

	if ( !pGFrame->StreamMode ) return( FALSE );
    
	if ( !GF_EnableGrabAhead( pGFrame, TRUE ) ) return( FALSE );

    rtn = CPIA_StartStreamCap( pGFrame->pLink );

	return( rtn );
}


/*--------------------------------------------------------------------------*/
/*	GF_StopFrameStream														*/
/*--------------------------------------------------------------------------*/
BOOL GF_StopFrameStream( PGFRAME pGFrame ) {
	BOOL rtn;

	if ( !pGFrame )	return( FALSE );

	if ( !pGFrame->StreamMode ) return( FALSE );

	if ( !GF_EnableGrabAhead( pGFrame, FALSE ) ) return( FALSE );

    rtn = CPIA_StopStreamCap( pGFrame->pLink );

	if ( !rtn ) tracemsg( TRC_MSG, "CPIA_StopStreamCap returned FALSE\n" );

	return( rtn );
}





/*##########################################################################*/
/*#						Private Functions								   #*/
/*##########################################################################*/

  
/*--------------------------------------------------------------------------*/
/*	AllocBuffers															*/
/*--------------------------------------------------------------------------*/
BOOL AllocBuffers( PGFRAME pGFrame ) {
	int i;
 	
	for ( i = 0; i < RAW_BUFFERS; i++ ) {
		pGFrame->hRawBuffer[i] = (HGLOBAL)GlobalAlloc( LPTR, RAW_BUFFER_SIZE );
 		if ( pGFrame->hRawBuffer[i] != NULL ) {                                                  
			pGFrame->pRawBuffer[i] = (BYTE __huge *)GlobalLock( pGFrame->hRawBuffer[i] );
			GlobalPageLock( pGFrame->hRawBuffer[i] );
 			pGFrame->RawBufferSize[i] = RAW_BUFFER_SIZE ;
	   	}
	   	else 
	   		goto AllocFailed;
	}

	pGFrame->hDecompressedImage = GlobalAlloc( LPTR, DECOMPRESSED_IMAGE_SIZE );
	if ( pGFrame->hDecompressedImage != NULL ) {
		pGFrame->pDecompressedImage = (BYTE __huge *)GlobalLock( pGFrame->hDecompressedImage );
		pGFrame->DecompressedImageSize =  DECOMPRESSED_IMAGE_SIZE;
	}
	else 
		goto AllocFailed;

	return( TRUE );


AllocFailed:
	DeallocBuffers( pGFrame );
	return( FALSE );
}


/*--------------------------------------------------------------------------*/
/*	DeallocBuffers															*/
/*--------------------------------------------------------------------------*/
BOOL DeallocBuffers( PGFRAME pGFrame ) {
	int i;
 	
	for ( i = 0; i < RAW_BUFFERS; i++ ) {
 		if ( pGFrame->hRawBuffer[i] != NULL ) {                                                  
			GlobalPageUnlock( pGFrame->hRawBuffer[i] );
    		GlobalUnlock( pGFrame->hRawBuffer[i] );
     		GlobalFree( pGFrame->hRawBuffer[i] );
			pGFrame->RawBufferSize[i] = 0;
		}
	}

	if ( pGFrame->hDecompressedImage != NULL ) {
		GlobalPageUnlock( pGFrame->hRawBuffer[i] );
   		GlobalUnlock( pGFrame->hRawBuffer[i] );
   		GlobalFree( pGFrame->hRawBuffer[i] );
		pGFrame->DecompressedImageSize = 0;
	}

	return( TRUE );
}


/*--------------------------------------------------------------------------*/
/*	SendFrameTimeCmds														*/
/*--------------------------------------------------------------------------*/
BOOL SendFrameTimeCmds( PGFRAME pGFrame ) 
{
	BOOL cmd_sent = FALSE;

	if ( pGFrame->ValidFrameHeader ) 
	{
 		cmd_sent = camDoFrameTimeProcessing( pGFrame->pCamera, pGFrame->FrameHeader );

		if ( !pGFrame->StreamMode )
 			cmd_sent |= capDoFrameTimeProcessing( pGFrame->pCap, pGFrame->FrameHeader,
 												  pGFrame->AppFrameRequestTime, 
 												  pGFrame->PreviousFrameProcessingTime );

	}

	return( cmd_sent );		
}


/*--------------------------------------------------------------------------*/
/*	SendCaptureParamsAndGrab												*/
/*--------------------------------------------------------------------------*/
BOOL SendCaptureParams( PGFRAME pGFrame, BOOL DisableCompression ) 
{
	BOOL cmd_sent = FALSE;
	BOOL rtn;

	if ( !DisableCompression )
	{	/* we don't need an uncompressed frame 						*/

		cmd_sent = capSendCaptureParams( pGFrame->pCap );
	}
	else
	{	/* we need an uncompressed frame 							*/
 		rtn &= capDisableCompression( pGFrame->pCap );
		cmd_sent = TRUE;
	}

	return( cmd_sent );
}


/*--------------------------------------------------------------------------*/
/*	ImageVerify																*/
/*--------------------------------------------------------------------------*/
BOOL ImageVerify( PGFRAME pGFrame, BYTE __huge *pSrc, DWORD MaxBytes,
									BOOL *IsACompressedFrame, BYTE *pFrameHeader )
{
	DECODE_ERROR vres;
	DECODE_HEADER hd;
	DECODE_ERROR_DETAILS ded;    
	WORD ht, wd;

	pGFrame->ValidFrameHeader = FALSE;

	if ( !pGFrame->Use32BitDecode )
	    vres = VerifyStream( pSrc, &hd, &ded );
	else
       	vres = VerifyStream32( pSrc, &hd, &ded );

	if ( vres == DECODE_OK || vres == OFFSET_HEADER ) {
		/* if we have not got an offset header then copy out the frameheader info */
 		if ( vres != OFFSET_HEADER ) 
 		{
 			memcpy( pFrameHeader, pSrc, CV_FRAME_HEADER_LEN );
 			pGFrame->ValidFrameHeader = TRUE;
		}

		/* check that the streamed data is for an image of the expected size */
 		wd = (WORD)pGFrame->SrcFormat.biWidth;
 		wd = (WORD)pGFrame->SrcFormat.biWidth;
		ht = (WORD)pGFrame->SrcFormat.biHeight;
	  	if ( ((hd.Col_End - hd.Col_Start) == (wd*2)) 	/* *2 ? - don't know why !! */
	  		  && ((hd.Row_End - hd.Row_Start) == ht) ) {
			/* OK the frame is good */
			*IsACompressedFrame = hd.Comp_Enable;
			return( TRUE );
		}
		else 
		{
//			if ( TMSET() )
//				tracemsg( TRC_MSG, "ImageVerify: wrong image size\n" );
			return( FALSE );
		}
	}
	else {
//		if ( TMSET() )
//			tracemsg( TRC_MSG, "ImageVerify: VerifyStream() failed\n" );
		return( FALSE );
	}
}


/*--------------------------------------------------------------------------*/
/*	ImageDecode																*/
/*--------------------------------------------------------------------------*/
BOOL ImageDecode( PGFRAME pGFrame, BYTE __huge *pSrc, BYTE __huge *pDest, DWORD MaxBytes )
{
	DECODE_ERROR res;
	DECODE_ERROR_DETAILS ded;    
	BYTE __huge *tmp_dest;
	WORD ht, wd;
	DWORD buflen;

	if ( pGFrame->UseOwnBuffer )
		tmp_dest =  pGFrame->pDecompressedImage;
	else
		tmp_dest = pDest;

	wd = (WORD)pGFrame->SrcFormat.biWidth;
	ht = (WORD)pGFrame->SrcFormat.biHeight;

	if ( !pGFrame->Use32BitDecode )
		res = FormatStream( pGFrame->ImageFormat, pSrc, tmp_dest, wd, ht, &(pGFrame->DState) );
	else
   		res = FormatStream32( pGFrame->ImageFormat, pSrc, tmp_dest, wd, ht, &(pGFrame->DState) );

	if ( (res == DECODE_OK) ) {

        /* copy image to final target buffer if required 							*/
        if ( pGFrame->UseOwnBuffer ) {
	    	/*  - max number of bytes to copy is the shorter of the 2 buffers		*/
        	/*    involved. Applications may pass larger buffers than strictly		*/
            /*    required, so make sure don't overrun internal buffer if they do	*/
	        buflen = min( pGFrame->DecompressedImageSize, 
                      pGFrame->StretchBy2 ? (MaxBytes / 4) : MaxBytes ) ;
          	copy_image( pDest, tmp_dest, buflen );
		}

		/* stretch image within target buffer if required	*/
	        if ( pGFrame->StretchBy2 ) {
    		if ( !pGFrame->Use32BitDecode )
 	      		Resize_by_2_PolyPhase( pDest, ht, wd );
        	else
            	Resize_by_2_PolyPhase32( pDest,  ht, wd );
        }
        
		/* all done OK - return TRUE */
		return( TRUE );
	}

	/* if it gets here then there's been a problem */
  	return( FALSE );
}

/*--------------------------------------------------------------------------*/
/*	ClearStall																*/
/*--------------------------------------------------------------------------*/
BOOL ClearStall( PGFRAME pGFrame ) {
	BOOL rtn;
	BYTE status, stream_state;
	DWORD actual_bytes;

	GET_STREAM_STATE( pGFrame->pLink, &stream_state );
	if ( stream_state != STREAM_PAUSED ) return( FALSE );
	
	/* pretend we're not really filling buffer */
   	pGFrame->RawBufferState[ pGFrame->FillBuffer ] = BUFFER_EMPTY;

	rtn = CLINK_StartUpload( pGFrame->pLink, 
									 FALSE, FALSE, 
									 pGFrame->RawBufferSize[ pGFrame->FillBuffer ],
									 pGFrame->pRawBuffer[ pGFrame->FillBuffer ],
									 pGFrame->UploadChunkSize,
									 TRUE );

   	if ( !rtn ) goto ClearStallFailed;

	/* wait for upload to finish */
	do { 
		if ( !CLINK_GetUploadStatus( pGFrame->pLink, &status, &actual_bytes ) ) goto ClearStallFailed;	
		if ( status == UPLOAD_FAILED ) goto ClearStallFailed;
 	} while( status != UPLOAD_IDLE );

   	pGFrame->RawBufferState[ pGFrame->FillBuffer ] = BUFFER_EMPTY;

	return( TRUE );

ClearStallFailed:
	CLINK_GetUploadStatus( pGFrame->pLink, &status, &actual_bytes );

	if ( status != UPLOAD_IDLE ) {
		CLINK_AbortUpload( pGFrame->pLink );
	}
	return( FALSE );
}



/*--------------------------------------------------------------------------*/
/*	copy_image																*/
/*--------------------------------------------------------------------------*/
void copy_image (BYTE _huge *dest, BYTE _huge *src, DWORD numb)

{
 BYTE _huge *lpdata,
      _huge *lpsrc ;
 WORD chunksize = 32768,
      i,
      left,
      chunks ;

 chunks = numb / chunksize ;
 for (i = 0; i < chunks; i++)
  {
   _fmemcpy (dest, src, chunksize) ;
   dest += chunksize ;
   src += chunksize ;
  }

 if ((left = (numb % chunksize)) != 0)
  _fmemcpy (dest, src, left) ;
}

