/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */



/* private types and functions */
#define RAW_BUFFERS				2 	/* don't increase as code depends on doing !Buffer to switch */
#define A_BUFFER				0
#define B_BUFFER				1
#define NO_BUFFER				0x69

#define RAW_BUFFER_SIZE  (DWORD) ((DWORD) STREAMEDRLBROWS * (DWORD) STREAMEDRLBCOLS) /* max size required for a SIF image */



#define BUFFER_EMPTY		0
#define BUFFER_FILLING		1
#define BUFFER_FULL			2


#define DECOMPRESSED_IMAGE_SIZE	 ((DWORD)352*288*3)


#define DEFAULT_CHUNK_SIZE	4096

#define MIN_CHUNK_SIZE		2048



typedef struct {
	HGLOBAL HandleToThisInstance;
	
	PLINK pLink; /* pointer to Link object used to communicate with camera */

	PCPIA_CAM pCamera;	/* pointer to the CPIA_CAM object		*/
	PCAPTURE pCap;		/* pointer to the CAPTURE object		*/

	/* flags to control grab/upload/decode process */
	BOOL StreamMode;
	BOOL UseStreamReadyINT;
	BOOL AllowGrabAhead;
	BOOL DoGrabAhead;
	BOOL Use32BitDecode;
	BOOL UseOwnBuffer;
	BOOL NeedUncompressedFrame;
	BOOL OldGrabAheadSetting;	/* when we go into stream mode we store old setting here */ 	

	BYTE StreamStartLine;
	DWORD LastAppRequestTime; 
	WORD AppFrameRequestTime; 
	WORD PreviousFrameProcessingTime;

	BYTE FrameState;

	BYTE FillBuffer;
	BYTE DecodeBuffer;

	BYTE RawBufferState[ RAW_BUFFERS ];
    HGLOBAL hRawBuffer[ RAW_BUFFERS ];      
	BYTE __huge *pRawBuffer[ RAW_BUFFERS ];
    DWORD RawBufferSize[ RAW_BUFFERS ];
    DWORD BytesUploaded[ RAW_BUFFERS ];
	
	DWORD UploadChunkSize;

    HGLOBAL hDecompressedImage;      /* used when compression on */
    BYTE _huge *pDecompressedImage;
    DWORD DecompressedImageSize;


    BITMAPINFOHEADER SrcFormat;
    BITMAPINFOHEADER DestFormat;

    COLOUR_FORMAT ImageFormat;  /* used by DECODE module to store format 	*/
    unsigned short DState;		/* private data used by DECODE module 		*/

    BOOL StretchBy2;     
    
    BOOL ValidFrameHeader;
    BYTE FrameHeader[ CV_FRAME_HEADER_LEN ];  

} GFRAME;
				 

typedef GFRAME *PGFRAME;




#define GF_FRAME_IDLE		   		0
#define GF_FRAME_WAITING_FOR_STREAM	1
#define GF_FRAME_UPLOADING			2
#define GF_FRAME_DECODING			3
#define GF_FRAME_ERROR				4

BOOL GF_Init( void );
BOOL GF_Fini( void );

					  
PGFRAME GF_CreateGFrame( void );
BOOL GF_DestroyGFrame( PGFRAME pGFrame );


BOOL GF_InitialiseGFrame( PGFRAME pGFrame, PLINK pLink, PCPIA_CAM pCamera, PCAPTURE pCap );

BOOL GF_DeinitialiseGFrame( PGFRAME pGFrame );


BOOL GF_SetFormats( PGFRAME pGFrame, LPBITMAPINFOHEADER SrcBmp, LPBITMAPINFOHEADER DestBmp );

BOOL GF_GetFrame( PGFRAME pGFrame, BYTE _huge *pImageBuffer, DWORD MaxBytes, DWORD *CapTime );

BOOL GF_IsStreamReady( PGFRAME pGFrame, BOOL *Ready );

BOOL GF_GetFrameState( PGFRAME pGFrame, BYTE *GetFrameState );


BOOL GF_EnableGrabAhead( PGFRAME pGFrame, BOOL EnableGrabAhead );    /* FALSE by default     */

BOOL GF_NeedReferenceFrame( PGFRAME pGFrame );

BOOL GF_UseOwnBuffer( PGFRAME pGFrame, BOOL UseOwnBuffer );

BOOL GF_ResetGetFrame( PGFRAME pGFrame );




BOOL GF_GetDecodeErrorCount( PGFRAME pGFrame, DWORD *Count );

BOOL GF_ClearDecodeErrorCount( PGFRAME pGFrame );



BOOL GF_InitFrameStream( PGFRAME pGFrame, WORD SkipFrames, WORD Flags );	 /* flags may be used to control how the request is honoured */

BOOL GF_FiniFrameStream( PGFRAME pGFrame );

BOOL GF_StartFrameStream( PGFRAME pGFrame );

BOOL GF_StopFrameStream( PGFRAME pGFrame );



/* 
	private functions
*/

BOOL AllocBuffers( PGFRAME pGFrame );
BOOL DeallocBuffers( PGFRAME pGFrame );

BOOL SendFrameTimeCmds( PGFRAME pGFrame );

BOOL SendCaptureParams( PGFRAME pGFrame, BOOL DisableCompression );


BOOL ImageVerify( PGFRAME pGFrame, BYTE __huge *pSrc, DWORD MaxBytes,
								   BOOL *IsACompressedFrame, BYTE *pFrameHeader );
BOOL ImageDecode( PGFRAME pGFrame, BYTE __huge *pSrc, BYTE __huge *pDest, DWORD MaxBytes );

BOOL ClearStall( PGFRAME pGFrame );


void copy_image (BYTE _huge *dest, BYTE _huge *src, DWORD numb);



