/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/* PPC.C
   18-07-97
   c:\john\vfw\cpia\0-01\ppc.c
*/   

#include "common.h"

#include "cpialink.h"
#include "cpia_cmd.h"

#include "resize.h"
#include "vfwdlg.h"

#include "ppc.h"


extern WORD gDebugLevel;

/* PPC private functions */
static PLINK FindCamera( PPPC pPPC, int *PortNumber, 
							BYTE PortTypeMask, BYTE *DetectedPortType );
static PLINK DetectCamera( PPPC pPPC, int PortNumber, 
							BYTE PortTypeMask, BYTE *DetectedPortType );
static BOOL BackOffPortType( BYTE DetectedPortType, BYTE *PortTypeMask);
static BOOL AdjustColBal(WORD curexp, WORD oldexp);
static PPPC LockPPCMem( HPPC hPPC );
static BOOL UnlockPPCMem( HPPC hPPC );


#define MB(m) MessageBox(NULL, m, "ppc.c", MB_OK);


HPPC ghPPC;
PPPC gpPPC;


/*-----------------------------------------------------------------*/
BOOL OpenPPC( void )
{
 	BOOL res = FALSE;
 
 	ghPPC = 0;
 	gpPPC = NULL;

	CLINK_Init();

 	CPIA_CmdInit();               
 	
 	return res;
}


/*-----------------------------------------------------------------*/
void ClosePPC( void )
{
 	CPIA_CmdFini();               /* close down the camera */
 
 	CLINK_Fini();
}


/*-----------------------------------------------------------------*/
/* initialise the camera.

   char *InitFile    - ini file for settings
   int *PortNumber   - on input contains the port the camera is believed to 
                       be on, or 0.On output contains the port the camera is
                       on, or 0 if not found
   PPC_INIT_ERR *Errcode - error code describing the result of the init operation 
   
   return
     camera found  - HPPC pointing to a valid PPC struct
     !camera found - NULL
*/
HPPC InitPPCDevice( char *InitFile, int *PortNumber, PPC_INIT_ERR *ErrCode )
{
	HPPC hPPC = 0;
	PPPC pPPC = NULL;
	BYTE port_type_mask, detected_port_type;
	BOOL done_backoff, try_again, camera_initialised;

	*ErrCode = ERR_OK;

	if ( (hPPC =(HPPC)LocalAlloc(LPTR, sizeof(PPC))) == 0 )
 	{
  		*ErrCode = ERR_MEMORY;
  		return NULL;
 	}

	pPPC =(PPPC)LocalLock( hPPC );

 	done_backoff = FALSE;
	try_again = TRUE;
 	port_type_mask = 0xFF;

	do {
	   	if ( gDebugLevel & DBG_DISABLE_FDMA ) port_type_mask &= (COMPAT_PORT_TYPE|BIDIR_PORT_TYPE|ECP_PORT_TYPE|INT_PORT_TYPE|DMA_PORT_TYPE);
   		if ( gDebugLevel & DBG_DISABLE_DMA ) port_type_mask &= (COMPAT_PORT_TYPE|BIDIR_PORT_TYPE|ECP_PORT_TYPE);
	   	if ( gDebugLevel & DBG_DISABLE_ECP ) port_type_mask &= COMPAT_PORT_TYPE;
	
	 	pPPC->pLink = FindCamera( pPPC, PortNumber, port_type_mask, &detected_port_type );
	 	if ( pPPC->pLink != NULL ) 
		{
	 		pPPC->PortNumber = *PortNumber;
			pPPC->PortType = detected_port_type & port_type_mask;

			/* create and initialise the CPIA_CAM object 		*/
	 		pPPC->pCamera = camCreateCPIA_CAM( 0, 0x20, pPPC->pLink );   /* init camera pointers */

            camera_initialised = FALSE;
			if ( pPPC->pCamera )
			{
                if ( !camInitialiseCPIA_CAM( pPPC->pCamera, pPPC->pLink ) ) 
                {
					DEBUG_MSG(DBG_INITPPC, "Failed camInitialiseCPIA_CAM\n");
				}
	 			else
	 			{
		 			if ( !camSetPower( pPPC->pCamera, PWR_HI) == OK )
					{
					 	DEBUG_MSG(DBG_INITPPC, "Failed camInitialiseCPIA_CAM\n");
		   			}
					else
					{
						if ( !camDoHiPowerCommsTest( pPPC->pCamera ) )
                        {
                            DEBUG_MSG(DBG_INITPPC, "Failed DoHiPowerCommsTest");
                        }
                        else
                        {
                            camera_initialised = TRUE;
                        }
                    }
                }
            }
            
            if ( !camera_initialised ) 
            {
			    BYTE effective_type;
 
                camDeinitialiseCPIA_CAM( pPPC->pCamera );
			    camDestroyCPIA_CAM( pPPC->pCamera );

			    effective_type = detected_port_type & port_type_mask;
			    try_again = ((effective_type & ECP_PORT_TYPE) != 0);

			    if ( try_again ) 
			    {
				    if ( effective_type & FAST_DMA_PORT_TYPE )
					    gDebugLevel |= DBG_DISABLE_FDMA;

				    else if ( effective_type & (DMA_PORT_TYPE | INT_PORT_TYPE) )
					    gDebugLevel |= DBG_DISABLE_DMA;		

				    else if ( effective_type & ECP_PORT_TYPE )
					    gDebugLevel |= DBG_DISABLE_ECP;
			    }

    		    done_backoff = TRUE;
                pPPC->pCamera = NULL;		
			}
		}
	} while( pPPC->pLink && !pPPC->pCamera && try_again );   			


	if ( !pPPC->pCamera )
	{
	 	/* unable to detect cpia camera */
   		DEBUG_MSG(DBG_INITPPC, "Unable to detect a camera");    
   		*ErrCode = ERR_CAMNOTFOUND;

	}
	else
	{
		// we've managed to speak OK to the camera
		/*	Do this no more
		if ( done_backoff )
		{	 
			//	save the gDebugLevel settings to INI file 
			
			char tmp[32];

	   		if ( gDebugLevel & DBG_DISABLE_ECP ) 
			   WritePrivateProfileString( "VVLCPIA", INI_DISABLE_ECP, "1", InitFile );

   			if ( gDebugLevel & DBG_DISABLE_DMA ) 
			   WritePrivateProfileString( "VVLCPIA", INI_DISABLE_DMA, "1", InitFile );

    		if ( gDebugLevel & DBG_DISABLE_FDMA )
			   WritePrivateProfileString( "VVLCPIA", INI_DISABLE_FDMA, "1", InitFile );
		}
		*/

		if ( pPPC->pLink && pPPC->pCamera )
	   	{		
	   		pPPC->FwVersion = camGetFwVersion( pPPC->pCamera );

			if ( camInitVP( pPPC->pCamera ) ) 
			{	/*
					CPIA_CAM is initialised and camera is in HI power state
					now create and initialise the CAPTURE object 
				*/
				pPPC->pCapture =  capCreateCAPTURE(); 
				if ( pPPC->pCapture && capInitialiseCAPTURE( pPPC->pCapture, pPPC->pLink ) ) 
				{
				 	GF_Init();
 
		  			pPPC->pGFrame = GF_CreateGFrame();
					if ((pPPC->pGFrame != NULL) && GF_InitialiseGFrame( pPPC->pGFrame, pPPC->pLink, 
																			  pPPC->pCamera, pPPC->pCapture ) )
	 				{
	      				/* initialise the rest of the PPC object 	*/
						if (!SelectPPCDevice(hPPC)) 
  						{
   							DEBUG_MSG(DBG_INITPPC, "SelectPPCDevice failed");
   							*ErrCode = ERR_PPCSELECT;
  						}
                        else if ( !SetPPCDefaults() ) 
	    				{
	     					DEBUG_MSG(DBG_INITPPC, "SetPPCDefaults failed");
	     					*ErrCode = ERR_PPCDEFAULTS;
	    				}
                        else if ( !ReadPPCInit( pPPC, InitFile ) ) 
	    	 			{
	      					DEBUG_MSG(DBG_INITPPC, "ReadPPCInit failed");
	      					*ErrCode = ERR_PPCINIT;
	     				}
		   			}
		   			else
					{ 
						/* GF_InitialiseGetFrame() failed */
						DEBUG_MSG(DBG_INITPPC, "Failed to initialise GetFrame");
						*ErrCode = ERR_CAMPOWER;
					}
		        }
		        else             
			    {
			   		/* cam set power */
		          	DEBUG_MSG(DBG_INITPPC, "Failed to switch camera to high power");
		          	*ErrCode = ERR_CAMPOWER;
				}
			}
		}
	 }	


	/*
		Either everything has worked, which would be simply splendid, or ErrCode
	 	has been set(bad).If there has been an error, close anything that was open and
   		ditch any allocated memory. 
   	*/

	if ( *ErrCode != ERR_OK || !hPPC )
	{
  		if ( pPPC != NULL )
		{
    		/* call pPPC->Camera Destructor function here */
		    if ( pPPC->pGFrame != NULL ) 
	    	{
				GF_DeinitialiseGFrame( pPPC->pGFrame );
				GF_DestroyGFrame( pPPC->pGFrame );
				GF_Fini();	
			}

			if ( pPPC->pCapture != NULL )
			{
				capDeinitialiseCAPTURE( pPPC->pCapture );
     			capDestroyCAPTURE( pPPC->pCapture );
			}

			if ( pPPC->pCamera != NULL )
			{
				camDeinitialiseCPIA_CAM( pPPC->pCamera );
     			camDestroyCPIA_CAM( pPPC->pCamera );
			}

	    	if (pPPC->pLink != NULL)
			{
				PPLINK_DestroyLink(pPPC->pLink);
		    	pPPC->pLink = NULL;
		    }
      
     		LocalUnlock(hPPC);
			LocalFree(hPPC);
   			hPPC = NULL;
		}
		return NULL;
	}
	else
	{
	 	return hPPC;
	}
}



/*-----------------------------------------------------------------*/
/* tidy up all the resources associated with a PPC device */

void FiniPPCDevice( HPPC hPPC, char *InitFile )
{
 	PPPC pPPC;
    
    if ( hPPC )
    {
 	    if ( ghPPC != hPPC )
  	    {
   		    /* it is not the current device */
   		    if ((pPPC = LockPPCMem( hPPC )) == NULL)
    		    return;
	    }
	    else 
	    {
		    pPPC = gpPPC;
	    }


        if ( pPPC ) 
        {
    	    WritePPCInit( pPPC, InitFile );

	        /* free pPPC->Camera here */
	        if ( pPPC->pGFrame )
	        {
		        GF_DeinitialiseGFrame( pPPC->pGFrame );
		        GF_DestroyGFrame( pPPC->pGFrame );
		        GF_Fini();	
	        }
	        
	        if ( pPPC->pCapture )
	        {
		        capDeinitialiseCAPTURE( pPPC->pCapture );
		        capDestroyCAPTURE( pPPC->pCapture );
	        }

	        if ( pPPC->pCamera )
		        {
		        camDeinitialiseCPIA_CAM( pPPC->pCamera );
		        camDestroyCPIA_CAM( pPPC->pCamera );
	        }

 	        PPLINK_DestroyLink( pPPC->pLink );   /* close down CPIALINK */
        }

	    LocalUnlock( hPPC );
	    LocalFree( hPPC );
    }
}

/*-----------------------------------------------------------------*/
/* lock a ppc device, and any other structures required for it.
   Also sets global variables here, until I weed them all out
*/
PPPC SelectPPCDevice(HPPC hPPC)
{
 PPPC res = NULL;
 
 if (hPPC == ghPPC)
  return gpPPC;
     
 /* if a PPCDevice is already selected, deselect it */ 
 if (ghPPC != 0)
  {
   if (!UnlockPPCMem(ghPPC))
    return NULL;  /* failed to unlock current device */
    
   ghPPC = 0;
   gpPPC = NULL;
  }

 if ((gpPPC = LockPPCMem( hPPC )) != NULL)
  {
     ghPPC = hPPC;
     res = gpPPC;
  }
  
 if (res == NULL)
  {
   /* failed to select device.Unlock current one and zero the global vars */
   UnlockPPCMem( ghPPC );
 
   ghPPC = 0;
   gpPPC = NULL;
  }

 return res;
}

/*-----------------------------------------------------------------*/
/* return the currently selected ppc device */
    
HPPC GetPPCDevice( void )

{
 return ghPPC;
}


/*-----------------------------------------------------------------*/
/* Stop/Start whether GetFrame is allows to do background uploads */

BOOL DisableBackgroundUpload( BOOL DisableIt ) 
{
	BOOL rtn;
	rtn = GF_EnableGrabAhead( gpPPC->pGFrame, !DisableIt );  
	return( rtn );
}





/*-----------------------------------------------------------------*/
/* 
  compmode CMP_OFF - switch camera to uncompressed mode
           CMP_MANUAL - not used at 0-11, ignored
           CMP_AUTOMATIC - if videoconf set, confval used to prime
                           setcompressiontarget, otherwise capval
                           used.
*/     
#define R100_TO_R10(x) (10 -((x-1)/10))

BOOL SetPPCOptions(COMP_MODE compmode, BOOL videoconf, WORD confval, 
                    WORD capval, WORD manualval)

{                                     
 WORD cmode;
 BOOL res = FALSE,
      read_ref = FALSE;
 
 cmode = compmode == CMP_OFF    ? CAP_COMP_OFF :
         compmode == CMP_MANUAL ? CAP_COMP_MANUAL : 
                                  CAP_COMP_AUTO;
 
// tracemsg( TRC_MSG, "SetPPCOptions() - CompMode %d  VideoConf %d   VidCapVal %d   VidConfVal %d\n",
// 				compmode, videoconf, capval, confval );

 /* if using compression, need to use own image buffer.Compressed data is
    uploaded into pRawBuffer, and patched into the existing image in decode.
    Also, if no longer using compression, need to ensure that frames are
    fully uncompressed, otherwise compressed data may be returned, which
    corrupts the image
 */  
 if ( cmode == CAP_COMP_OFF )
	GF_UseOwnBuffer( gpPPC->pGFrame, FALSE );
 else
 	GF_UseOwnBuffer( gpPPC->pGFrame, TRUE );

 /* If compression mode is changing, take a reference frame.This is to
      ensure that the correct type of frames are being uploaded before
      continuing.There have been cases where running in CIF caused a 
      compressed frame to be uploaded, even after the compression had been
      switched off.Reading a ref frame gets round this */
 if (gpPPC->cmp.compression != compmode /*&& cmode != CAP_COMP_OFF*/)
  {
   GF_NeedReferenceFrame( gpPPC->pGFrame );
  }

   
 if (capSetCompression(gpPPC->pCapture, cmode, CAP_NULL,
                        CAP_NULL, CAP_NULL))
  {
   gpPPC->cmp.compression = compmode;
   gpPPC->cmp.vidconf = videoconf;
   gpPPC->cmp.vidconfval = confval;
   gpPPC->cmp.vidcapval = capval;
   gpPPC->cmp.manualval = manualval;
   res = TRUE;
  }
  
 if ( compmode == CMP_MANUAL )
  res = SetPPCManQuality( manualval );
 else
  if ( compmode == CMP_AUTOMATIC )
   res = capSetCompressionTarget( gpPPC->pCapture, videoconf, 
   								  R100_TO_R10(confval), R100_TO_R10(capval),
   								  gpPPC->AppFrameRate );

 return res;
}

 
/*-----------------------------------------------------------------*/
/* set the desired manual compression quality.This changes the Ythreshold
   and UVthreshold values 
*/
           
BOOL SetPPCManQuality(int ImageQuality)

{
 int tmp;

 if (ImageQuality > 100) 
  ImageQuality = 100;
 else
  if (ImageQuality < 0) 
   ImageQuality = 0;

 gpPPC->cmp.manualval = gpPPC->ImageQuality = ImageQuality;

 tmp = MIN_DIFF_THRESHOLD +
      ((MAX_DIFF_THRESHOLD-MIN_DIFF_THRESHOLD) *(100 - ImageQuality)) / 100;
 capSetYUVThresholds(gpPPC->pCapture, tmp, tmp / 2 );          
  
 return TRUE;
}

/*-----------------------------------------------------------------*/

void GetPPCOptions(COMP_MODE *compression, BOOL *vidconf, WORD *vidconfval,
                    WORD *vidcapval, WORD *manualval)

{
 *compression = gpPPC->cmp.compression;
 *vidconf     = gpPPC->cmp.vidconf;
 *vidconfval  = gpPPC->cmp.vidconfval;
 *vidcapval   = gpPPC->cmp.vidcapval;
 *manualval   = gpPPC->cmp.manualval;
}

/*-----------------------------------------------------------------*/

void GetPPCParams( int *ImageQuality )

{
 *ImageQuality = gpPPC->cmp.manualval;    /* gpPPC->ImageQuality; */
}


/*-----------------------------------------------------------------*/
/* decide what camera capture format to use, and any roi required.
   If an image > CIF is requested, a smaller image will be captured and
   a 2 x 2 stretch performed
   
   LPBITMAPINFOHEADER RequestedBmp - details of required format
   
   return
    TRUE  - image details acceptable, camera ready
    FALSE - invalid size requested
*/
BOOL SetPPCImageType( const LPBITMAPINFOHEADER RequestedBmp )
{
 	LONG tmp, 
 		 x,	y,
      	 src_x,src_y;
 	WORD bits,
      	 xs, ys, xe, ye,
         xstart, xend, ystart, yend;
 	DWORD compression;
 	BOOL res = TRUE,
         stretch = FALSE;
 	COLOUR_FORMAT cf;
 	FRAMERATE fr;
 
 	BIH_GetType( RequestedBmp, &bits, &compression );
 	BIH_GetImageSize( RequestedBmp, &x, &y );

 	/* 
 		if the image size isn't divisable by the correct amounts, fail it.
    	- it will be up to the format dialogue to adjust any user settings.
    	ignore 0 size images, and images that are too big, or aren't divisable
    	by 8 horizontally and 4 vertically.
 	*/
 	if ((x == 0) ||(y == 0) || x > CIF_COLS * 2 || y > CIF_ROWS * 2 
 	   || !check_image_size(RequestedBmp, NULL, NULL) ) 
   	{
   		return FALSE;
 	}
  
 	/* 
 		image zoom stays on across an image colour format change
 	*/ 
 	if (!(x == QCIF_COLS && y == QCIF_ROWS))
  		gpPPC->ZoomVideo = FALSE;       /* reset when changing image size */

 	/* 
 		if image > CIF in either direction, set StretchBy2 = TRUE, and divide the
    	image size by 2 
 	*/
 	if (x > CIF_COLS || y > CIF_ROWS)
  	{
   		stretch = TRUE;
   		x /= 2;
   		y /= 2;
  	}

 	memcpy( &(gpPPC->DestBmp), RequestedBmp, sizeof(BITMAPINFOHEADER) );
 
 	/* 
 		now work out which capture mode we should use to honour request
 	*/
 	gpPPC->InvalidFormat = FALSE;

 	/* 
 		calculate the image to set, and the region of interest to request.
    	If ZoomVideo is still set, then there is no need to update the
    	capture size or ROI 
 	*/
 	if (!gpPPC->ZoomVideo)
  	{
   		if (x <= QCIF_COLS && y <= QCIF_ROWS)
    	{
     		capSetSize(gpPPC->pCapture, CAP_FMT_QCIF);
	     	xstart =(QCIF_COLS - x) / 2;
    	 	ystart =(QCIF_ROWS - y) / 2;
    	}
   		else
	    {
    	 	/* image too big to fit in qcif, so work out roi in cif instead */
     		capSetSize(gpPPC->pCapture, CAP_FMT_CIF);
     		xstart =(CIF_COLS - x) / 2;
     		ystart =(CIF_ROWS - y) / 2;
    	}

   		/* work out the ends of the ROI and set it */
   		xend = xstart + x;
   		yend = ystart + y;
   		capSetROI(gpPPC->pCapture, xstart, xend, ystart, yend);
  	}
  
	if ( gpPPC->SrcBmp.biWidth <= QCIF_COLS
       		|| gpPPC->SrcBmp.biHeight <= QCIF_ROWS )
    		/* QCIF or smaller */
    		ppcSetAppFramerate(15);
   		else
    		ppcSetAppFramerate(8);
	
	   
   

 	/* capture details stored.Set the details in the source and dest bitmaps */
 	BIH_Init( &(gpPPC->SrcBmp) );
 	BIH_SetType( &(gpPPC->SrcBmp), 16, BI_PYUV);
 	BIH_SetImageSize( &(gpPPC->SrcBmp), x, y);

 	/* now inform GetFrame of the new Src and Dest formats */       

 	if ((res = GF_SetFormats( gpPPC->pGFrame, &gpPPC->SrcBmp, &gpPPC->DestBmp)) == TRUE)
  	{                 
   		gpPPC->InvalidFormat = FALSE;
    
		res = SetFourCCInfo( &gpPPC->DestBmp, gpPPC );

  		/* 
   			changing image sizes.If currently using a compressed buffer, need to
      		snapshot a new reference frame 
      	*/
	   if ( res ) GF_NeedReferenceFrame( gpPPC->pGFrame );
  	}
   
 	return res;
}

/*-----------------------------------------------------------------*/

BOOL GetPPCImageType( LPBITMAPINFOHEADER ReturnBmp )

{
 if (!gpPPC->InvalidFormat)
  memcpy( ReturnBmp, &(gpPPC->DestBmp), sizeof( BITMAPINFOHEADER ) );

 return !gpPPC->InvalidFormat;
}

/*-----------------------------------------------------------------*/
/* upload a frame from the camera */

NextStatus NextFrame(BYTE _huge *ImageBuffer, DWORD MaxBytes, 
                      DWORD *CaptureTime, BOOL StreamMode )
{   
	BOOL rtn;
	BOOL ready;
	                
	PPPC pc = gpPPC;
	
	if ( !StreamMode ) 
	{
		rtn = GF_GetFrame( pc->pGFrame, ImageBuffer, MaxBytes, CaptureTime );
	}
	else 
	{
		rtn = GF_IsStreamReady( pc->pGFrame, &ready );
		if ( rtn ) 
		{
			if ( ready )
				rtn &= GF_GetFrame( pc->pGFrame, ImageBuffer, MaxBytes, CaptureTime );
			else
				return( NextNotReady );
		}			
	}

	if ( rtn ) return( NextOK );
	else return( NextError );
}
    

/*-----------------------------------------------------------------*/

PPPC LockPPCMem( HPPC hPPC )

{
 PPPC pPPC;
    
 if ((pPPC =(PPPC)LocalLock( hPPC )) == NULL)
    {
     LocalUnlock(hPPC);
     pPPC = NULL;
    }
    
 return pPPC;    
}

/*-----------------------------------------------------------------*/

BOOL UnlockPPCMem( HPPC hPPC )

{
 PPPC pPPC;
    
 pPPC =(PPPC)LocalLock( hPPC );

 return pPPC != NULL;
}


/*-----------------------------------------------------------------*/
BOOL SetPPCDefaults( void )
{
 	BITMAPINFOHEADER DefaultBMP;
 	BOOL res = FALSE;

 	gpPPC->InvalidFormat = FALSE;
 	gpPPC->FlickerCtrl = FALSE;
 	gpPPC->AppFrameRate = DEFAULT_QCIF_FRAMERATE;
 
   	BIH_Init( &DefaultBMP );

   	BIH_SetType( &DefaultBMP,  DEFAULT_IMAGE_BITS, DEFAULT_IMAGE_TYPE );                   
   	/* BIH_SetType( &DefaultBMP, 16, BI_RGB ); */
   
   	/* BIH_SetImageSize( &DefaultBMP, SIF_COLS, SIF_ROWS ); */
   	BIH_SetImageSize( &DefaultBMP, DEFAULT_IMAGE_COLS, DEFAULT_IMAGE_ROWS ); 

   	/* NOTE - SetImageType will set up gpPPC->SrcBmp to any appropriate format */
   	if (SetPPCImageType(&DefaultBMP)) 
    	res = TRUE;

 	return res;
}


/*-----------------------------------------------------------------*/
/* set the capture format and yuv byte order depending on the type
   of image the application requires */
   
BOOL SetFourCCInfo( LPBITMAPINFOHEADER DestBmp, PPPC pPPC)
{            
 BOOL res = FALSE;

 switch( DestBmp->biCompression )
  {
   case BI_RGB :
		if ( !GetPrivateProfileInt( "VVLCPIA", INI_DISABLE_RGB24, 0, "vvlcpia.ini" ) )
		{
			/* always use 4:2:0 for RGB modes */
//			capSetSampleAndYUV(pPPC->pCapture, CAP_FMT_YUV420, CAP_FMT_YUYV);
 			capSetSampleAndYUV(pPPC->pCapture, CAP_FMT_YUV422, CAP_FMT_YUYV);
			res = TRUE;
		}
        break;

   case BI_YUY2 :
		if ( !GetPrivateProfileInt( "VVLCPIA", INI_DISABLE_YUY2, 0, "vvlcpia.ini" ) )
		{
			capSetSampleAndYUV(pPPC->pCapture, CAP_FMT_YUV422, CAP_FMT_YUYV);
			res = TRUE;
		}
        break;

   case BI_UYVY :
		if ( !GetPrivateProfileInt( "VVLCPIA", INI_DISABLE_UYVY, 0, "vvlcpia.ini" ) )
		{
			capSetSampleAndYUV(pPPC->pCapture, CAP_FMT_YUV422, CAP_FMT_YUYV);
			res = TRUE;
		}
        break;

   case BI_YV12 :   
		if ( !GetPrivateProfileInt( "VVLCPIA", INI_DISABLE_YV12, 0, "vvlcpia.ini" ) )
		{
//	        capSetSampleAndYUV(pPPC->pCapture, CAP_FMT_YUV420, CAP_FMT_YUYV);
			capSetSampleAndYUV(pPPC->pCapture, CAP_FMT_YUV422, CAP_FMT_YUYV);
			res = TRUE;
		}
        break;

   case BI_I420 :
		if ( !GetPrivateProfileInt( "VVLCPIA", INI_DISABLE_I420, 0, "vvlcpia.ini" ) )
		{
//	        capSetSampleAndYUV(pPPC->pCapture, CAP_FMT_YUV420, CAP_FMT_YUYV);
			capSetSampleAndYUV(pPPC->pCapture, CAP_FMT_YUV422, CAP_FMT_YUYV);
			res = TRUE;
		}
        break;
 }
  
 return res;
}



/*-----------------------------------------------------------------*/
/* set the auto gain feature on the camera */

BOOL SetCameraAGC(BOOL bstate)

{
/* return camSetAutoGain(gpPPC->Camera, bstate) == OK;*/
 return camSetAutoExposure(gpPPC->pCamera, bstate) == OK;

}

/*-----------------------------------------------------------------*/
/* set the auto exposure feature on the camera */

BOOL SetCameraAEC(BOOL bstate)

{   
 return camSetAutoExposure(gpPPC->pCamera, bstate) == OK;
}


BOOL GetCameraAEC( void )
{   
	return camGetAutoExposure( gpPPC->pCamera );
}

/*-----------------------------------------------------------------*/
/* set the auto colour balance on the camera */
BOOL SetCameraACB( COL_BALANCE cbMode, BYTE RedGain, BYTE GreenGain, BYTE BlueGain )
{
	return ( camSetColBal( gpPPC->pCamera, cbMode, RedGain, GreenGain, BlueGain ) == OK );
}

/* get the current auto colour balance mode and values from the camera */
BOOL GetCameraACB( COL_BALANCE *pCbMode, BYTE *pRedGain, BYTE *pGreenGain, BYTE *pBlueGain )
{
	if ( camGetColBal( gpPPC->pCamera,
			pCbMode,
			pRedGain,
			pGreenGain,
			pBlueGain ) == ERR )
	{
		return FALSE;
	}
	else
	{
		return TRUE;
	}
}

/*-----------------------------------------------------------------*/
/* set the exposure value on the camera to a specific value */

BOOL SetCameraExposure(WORD coarse, WORD fine, BYTE gain)

{
	if ( ppcGetFlicker() )
	{
	 	return ( camSetFlicker(gpPPC->pCamera, TRUE, coarse * (1<<gain)) == OK );
	}
	else
	{
		return ( camSetExposure(gpPPC->pCamera, coarse, fine, gain) == OK );
	}
}               

/*-----------------------------------------------------------------*/
/* return the current exposure setting on the camera */

BOOL GetCameraExposure(WORD *coarse, WORD *fine, BYTE *gain)

{
	if ( ppcGetFlicker() )
	{
	 	return ( camGetExpFromFlicker(gpPPC->pCamera, coarse, fine, gain) == OK	);
	}
	else
	{
		return ( camGetExposure(gpPPC->pCamera, coarse, fine, gain) == OK );
	}
}


/*-----------------------------------------------------------------*/
/* returns the type of parallel port attached to the machine
   returns one of
     NULL_PORT_TYPE
     COMPAT_PORT_TYPE
     BIDIR_PORT_TYPE
     ECP_PORT_TYPE
     EPP_PORT_TYPE
*/   
   
int GetPortMode( void )
{
 	BYTE port_type;
	WORD port_addr, timeout;
	int res = NULL_PORT_TYPE;

 	if ( PPLINK_GetPortInfo( gpPPC->pLink, &port_addr, &port_type, &timeout) )
	{
		if ( port_type & FAST_DMA_PORT_TYPE )
	 		res = FAST_DMA_PORT_TYPE;
	 	else if ( port_type & DMA_PORT_TYPE ) 
	  		res = DMA_PORT_TYPE;
	  	else if ( port_type & ECP_PORT_TYPE )
	   		res = ECP_PORT_TYPE;
	  	else if ( port_type & BIDIR_PORT_TYPE )
	    	res = BIDIR_PORT_TYPE;
	   	else
	    	res = COMPAT_PORT_TYPE;
   	}
 	return res;
}


/*-----------------------------------------------------------------*/
/* check that the image size is supported for the compression type
   requested.
   
   The image size is valid if the calculated size is the same as that 
   supplied when the function was called.
*/
   
BOOL check_image_size(LPBITMAPINFOHEADER bih, int *nwid, int *nht)

{
 int wid,
     ht,
     maxwid,
     maxht;
                 
 /* work out max image size */
 if (bih->biCompression == BI_RGB) 
  {
   switch(bih->biBitCount)
    {
     case 24 :
          maxwid = CIF_COLS * 2;
          maxht = CIF_ROWS * 2;
          break;
         
     case 16 :
     case 8 :
          maxwid = CIF_COLS;
          maxht = CIF_ROWS;
          break;
         
     default :
          /* unknown rgb format(possibly 32 bit).Return FALSE */
          if (nwid)
           *nwid = bih->biWidth;
          if (nht)
           *nht = bih->biHeight;
           
          return FALSE;
    }
  }                      
 else
  {
   /* YUV format */
   maxwid = CIF_COLS;
   maxht = CIF_ROWS;
  }
                   
 /* if too large, trim image */     
 wid = min((int) bih->biWidth, maxwid);
 ht = min((int) bih->biHeight, maxht);
 
 /* min width = 8, min height = 4 */
 wid = max(wid, 8);
 ht = max(ht, 4);
 
 /* if the supplied value is >= 1/2 the difference, move up to the next
    valid value, else move back to the previous valid value */                              
 if (wid > CIF_COLS || ht > CIF_ROWS)
  {
   wid =((wid / 16) +((wid % 16) >= 8)) * 16;
   ht =((ht / 8) +((wid % 8) >= 4)) * 8;
   }
 else
  {
   wid =((wid / 8) +((wid % 8) >= 4)) * 8;
   ht =((ht / 4) +((wid % 4) >= 2)) * 4;
  }
             
 /* copy back new values if asked */             
 if (nwid)
  *nwid = wid;
  
 if (nht)
  *nht = ht;
  
 return(wid == bih->biWidth) &&(ht == bih->biHeight); 
}



BOOL ppcGetSensorID( WORD *SensorID, BYTE *SensorRevision ) 
{
	return( camGetSensorID( gpPPC->pCamera, SensorID, SensorRevision ) );
}


/*-----------------------------------------------------------------*/
/* returns the roi start x and y co-ords in pixels.
   returns TRUE if the camera is showing a less than cif image 
   at present 
*/
   
BOOL ppcGetPanTiltInfo(LPRECT roi, BOOL *zoom)

{
 WORD xs,
      xe,
      ys,
      ye;
 BOOL res;
      
 capGetROI(gpPPC->pCapture, &xs, &xe, &ys, &ye);    
 *zoom = gpPPC->ZoomVideo;
 
 roi->left = xs;
 roi->top = ys;
 roi->bottom = ye;
 roi->right = xe;
 
 return(xe - xs <= QCIF_COLS) &&(ye - ys <= QCIF_ROWS);
}                                         

/*-----------------------------------------------------------------*/
/* alter the roi on the camera.This moves a qcif sized image within a
   cif capture, giving the illusion of pan and tilt.
   The first time it's called will be jerky, as there will also be a 
   transition from QCIF to an ROI within CIF 
*/
   
BOOL ppcSetPanTiltInfo(BOOL zoom, LPRECT roi)

{
 BOOL res = FALSE;
 RECT tmp;
 int wid,
     ht,
     xstart,
     ystart,
	 imwidth,
	 imheight;
     
 CopyRect(&tmp, roi);

 if (zoom && !gpPPC->ZoomVideo)
  {
	gpPPC->ImageSize = capGetSize( gpPPC->pCapture );
	capSetSize(gpPPC->pCapture, CAP_FMT_CIF);
  }
 else
  if (!zoom && gpPPC->ZoomVideo)
   {                                         
    /* moving back to roi within qcif.reposition the roi to be valid within
	  a qcif sized image */
	  capSetSize( gpPPC->pCapture, gpPPC->ImageSize );
	  if (gpPPC->ImageSize == CAP_FMT_QCIF)
	  {
		  imwidth = QCIF_COLS ;
		  imheight = QCIF_ROWS ;
	  }
	  else
	  {
		  imwidth = CIF_COLS ;
		  imheight = CIF_ROWS ;
	  }
    wid = tmp.right - tmp.left;
    ht = tmp.bottom - tmp.top;
    xstart =(QCIF_COLS - wid) / 2;
    ystart =(QCIF_ROWS - ht) / 2;
    xstart = (imwidth - wid) / 2;
    ystart = (imheight - ht) / 2;
    SetRect(&tmp, xstart, ystart, xstart + wid, ystart + ht);
   }
   
 gpPPC->ZoomVideo = zoom;
 
 if ((res = capSetROI(gpPPC->pCapture, 
                           tmp.left, tmp.right,
                           tmp.top, tmp.bottom)) == TRUE)
  {
   /* changing image perspective.If currently using a compressed buffer, 
      need to snapshot a new reference frame */
	GF_NeedReferenceFrame( gpPPC->pGFrame );
  }

 return res;              
}


/*-----------------------------------------------------------------*/
/* return the current camera framerate */

BOOL ppcGetSensorFramerate( FRAMERATE *Framerate, BOOL *AutoFramerate, WORD *Period )
{ 
	if ( camGetSensorFramerate( gpPPC->pCamera, Framerate, AutoFramerate ) != OK ) 
	{
		/* couldn't get the sensor framerate from the CPIA_CAM module */
		return FALSE;
	}
	else
	{
		if ( Period )
		{
			switch( *Framerate ) {
			case FPS_75:
				*Period = 133;
				break;
			case FPS_125:
				*Period = 80;
				break;
			case FPS_15:
				*Period = 66;
				break;
			case FPS_25:
				*Period = 40;
				break;
			case FPS_30:
			default:
				*Period = 33;
				break;
			}
		}
	}
	return TRUE;
 }

 

/*-----------------------------------------------------------------	*/
/* set a new camera framerate 										*/
/*																	*/
BOOL ppcSetSensorFramerate( FRAMERATE Framerate, BOOL AutoFramerate )
{
 	int res;

	res = camSetSensorFramerate( gpPPC->pCamera, Framerate, AutoFramerate );
		 	
 	return( res == OK );  
}

/*-----------------------------------------------------------------	*/
/* return the current state of flicker control						*/
/*																	*/
int ppcGetFlicker(void)
{
 	return gpPPC->FlickerCtrl;
}

/*-----------------------------------------------------------------	*/
/* switch flicker control on or off 								*/
/*																	*/
BOOL ppcSetFlicker( BOOL flickon )
{
 	gpPPC->FlickerCtrl = flickon;
 	return camSetFlicker( gpPPC->pCamera, flickon, 
                       		gpPPC->LastExposure) == OK;
}

/*-----------------------------------------------------------------*/
/* return the current state of vertical decimation */

int ppcGetDecimation(void)

{
 return capGetDecimation(gpPPC->pCapture);
}

/*-----------------------------------------------------------------*/
/* switch decimation on or off */

BOOL ppcSetDecimation(BOOL dec_on)

{
 return capSetDecimation(gpPPC->pCapture, dec_on);
}


/*-----------------------------------------------------------------*/
/* favour the centre of the image when setting exposure(weighton).
   Treat all parts of the image equally(!weighton)
*/
BOOL ppcSetCentreWeight( BOOL weighton )
{
 	return camSetCentreWeight( gpPPC->pCamera, weighton ) == OK;
}


/*-----------------------------------------------------------------*/
BOOL ppcGetCentreWeight( void )
{
 	return camGetCentreWeight( gpPPC->pCamera );
}


/*-----------------------------------------------------------------*/
/* sets the camera rate.This isn't a true indication of the speed of the
   camera, but is simply used to give a reasonable value to 
   capSetCompressionTarget.
   It's updated by vstream when an app requests a specific frame rate, 
   otherwise it sits at the default(15fps)
*/
BOOL ppcSetAppFramerate( int newrate )
{
 	BOOL res = TRUE,
      	 vconf;
 	COMP_MODE comp;
 	WORD vconfval, vcapval, manualval;

//	tracemsg( TRC_MSG, "ppcSetAppFramerate - New rate %d\n", newrate );
 
 	gpPPC->AppFrameRate = newrate;
 
 	/* if the camera is running in auto compression, need to update its
    	framerate vs quality parameters */
 	GetPPCOptions( &comp, &vconf, &vconfval, &vcapval, &manualval );
 	if ( comp == CMP_AUTOMATIC )
   		res = SetPPCOptions(comp, vconf, vconfval, vcapval, manualval);

 	return TRUE;
}


/*-----------------------------------------------------------------*/
/* return the app's belief of its target frame rate */
int ppcGetAppFramerate(void)
{
 	return gpPPC->AppFrameRate;
}

/*-----------------------------------------------------------------*/
/* called if flicker control is running, and there is a jump in exposure.
   adjust colour balances in ways that will become clear
*/
   
static BOOL AdjustColBal(WORD curexp, WORD oldexp)

{
 BYTE red,
      green,
      blue,
      mingain;
 BOOL res = FALSE;

 return TRUE;
 
 /* VERSION_NOTE ??? */
 /* function not used */
      
 if (camGetColBal(gpPPC->pCamera, NULL, &red, &green, &blue) == OK)
  {
   /*tracemsg(TRC_MSG, "adjustcb r = %d, g = %d, b = %d", red, green, blue);*/
   mingain = min(red, min(green, blue));
   red = NEWGAIN(mingain, red, curexp, oldexp);
   green = NEWGAIN(mingain, green, curexp, oldexp);
   blue = NEWGAIN(mingain, blue, curexp, oldexp);
   
   tracemsg(TRC_MSG, "         newex = %d oldex = %d", curexp, oldexp);
   tracemsg(TRC_MSG, "         r = %d, g = %d, b = %d", red, green, blue);

   res = camSetColBal(gpPPC->pCamera, CB_MANUAL, red, green, blue );
  }
  
 return res;
      
}


/*-----------------------------------------------------------------*/
/* set the brightness, contrast and colour values on the camera */

void ppcSetColourSettings( int Brightness, int Contrast, int Colour )

{    /* 0->100 range       */

 /* these are just stored in the colour space until I weed out
    all the references to them */
       
 gpPPC->Brightness = Brightness;
 gpPPC->Contrast = Contrast;
 gpPPC->Colour = Colour;
       
 /* write the values to the camera */   
 camSetColPar(gpPPC->pCamera,(BYTE) Brightness,(BYTE) Contrast, 
              (BYTE) Colour);
}

/*-----------------------------------------------------------------*/
/* read the current colour settings directly from the camera */

void ppcGetColourSettings( int *Brightness, int *Contrast, int *Colour ) 

{
 /* 0->100 range       */
 
 *Brightness = *Contrast = *Colour = 0;
 camGetColPar(gpPPC->pCamera,(BYTE *) Brightness,(BYTE *) Contrast, 
              (BYTE *) Colour);
 
}

 /*-----------------------------------------------------------------*/
/* prime the flicker values for a specific frequency and frame rate
   combination 
*/
   
BOOL ppcSetFlickerParams( MAINS_FREQ mf )
{
 	return camSetFlickerParams( gpPPC->pCamera, mf ) == OK;
}
      
/*-----------------------------------------------------------------*/
/* read the current mains frequency and flicker framerate from the
   camera */
   
BOOL ppcGetFlickerParams( MAINS_FREQ *mf )
{
 	return camGetFlickerParams( gpPPC->pCamera, mf ) == OK;
}
      
/*-----------------------------------------------------------------*/

WORD ppcGetFwVersion( void)
{
 	return camGetFwVersion( gpPPC->pCamera);
}


BOOL ppcSetSensorMatrix( BYTE Matrix[9] )
{
	return camSetSensorMatrix( gpPPC->pCamera, Matrix );
}


// Not operational yet (and may never work anyway )
BOOL ppcGetSensorMatrix( BYTE Matrix[9] )
{
	return camGetSensorMatrix( gpPPC->pCamera, Matrix );
}



/*-----------------------------------------------------------------*/
/* look for a cpia camera on each parallel port, starting with that supplied
   in *PortNumber
   
   BYTE SerialAddr - not used
   int *PortNumber - on input contains a port number to look for the camera.
                     if found, fine, else all ports are scanned
*/
static PLINK FindCamera( PPPC pPPC, int *PortNumber, 
							BYTE PortTypeMask, BYTE *DetectedPortType )
{          
 	int port_number;                      
 	PLINK ptmp;
    
    port_number = *PortNumber;
    ptmp = NULL;
 
   	if ( (port_number >= LPT1) && (port_number <= LPT3) )
    {
     	ptmp = DetectCamera( pPPC, port_number, PortTypeMask, DetectedPortType );
    } 
 	
   	/* no portnumber supplied, or camera not there */
   	if ( ptmp == NULL )
    {
     	for( port_number = LPT1; port_number <= LPT3; port_number++ )
      	{
      		ptmp = DetectCamera( pPPC, port_number, PortTypeMask, DetectedPortType );
      		if ( ptmp != NULL ) break;
		}
    }
	
	if ( ptmp ) *PortNumber = port_number;
 	return ptmp;
}


/*-----------------------------------------------------------------	*/
/* check to see if a camera exists on the requested port 			*/
/*																	*/
static PLINK DetectCamera( PPPC pPPC, int PortNumber, 
								BYTE PortTypeMask, BYTE *DetectedPortType )
{ 
 	BOOL res = FALSE;
 	WORD base, timeout;
 	BYTE type;
 	char deviceID [257];
 	PLINK ptmp;

 	/* see if a parallel port exists */
 	if ( (ptmp = PPLINK_CreateLink(PortNumber)) != NULL )
	{
   		PPLINK_GetPortInfo( ptmp, &base, &type, &timeout );

		if ( TMSET() )
	   		tracemsg( TRC_MSG, "Detected LPT%d : Base=0x%x    Type=0x%x\n", PortNumber, base, type );

		if ( DetectedPortType ) *DetectedPortType = type;
		type &= PortTypeMask;
   		PPLINK_SetPortInfo( ptmp, type, 80 );            /* set timeout to 50 ms */

   		if ( PPLINK_InitialiseLink( ptmp ) )
		{
    		if ( PPLINK_SendWakeup( ptmp ) )   /* send a wake up incase camera was in pass-through mode */
			{
				if ( PPLINK_Get1284DeviceID( ptmp, deviceID, 256 ) )
      			{
       				/* check the Device ID, if it contains our name, assume it's us */
       				if ( strstr( deviceID, "VLSI Vision Ltd") != NULL )
        			{
					 	if ( TMSET() )
	   					tracemsg( TRC_MSG, "Found camera on LPT%d\n", PortNumber );

       					res = TRUE;
      				}
      			}
  			}
		}
    }

 	if ( !res )
  	{    
		if ( ptmp ) PPLINK_DeinitialiseLink( ptmp );
		if ( ptmp ) PPLINK_DestroyLink( ptmp );
		ptmp = NULL;
  	}
     
 	return ptmp;
}














