/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/* PPC.H
   29-07-97
   header file for ppc.c 
*/   


/* The below represent the hardware capabilities of the parallel port. 
   These were lifted from pport.h */
#define NULL_PORT_TYPE          0x00
#define COMPAT_PORT_TYPE        0x01
#define BIDIR_PORT_TYPE         0x02
#define ECP_PORT_TYPE           0x04
#define EPP_PORT_TYPE           0x08

/* INI file settings */
#define DEFAULT_ZOOM_VIDEO         1
#define DEFAULT_COMPRESSION        1     /* camera starts video preview compression */
#define DEFAULT_AUTO_THRESH        0
#define DEFAULT_IMAGE_QUALITY     75     /* default manual compression */
#define DEFAULT_CAMERA_GAIN        0
#define DEFAULT_FAST_STREAM_MODE   1 
#define DEFAULT_VCONF_QUALITY      50     /* default slider position */
#define DEFAULT_VCAP_QUALITY       50
#define DEFAULT_VCONF              0     /* vid conf check box clear by default */
#define DEFAULT_SHARPNESS          1     /* sharpness on by default */
#define DEFAULT_FLICKER            1
#define DEFAULT_MAINS_FREQ         MAINS_60HZ
#define DEFAULT_AUTOFRAMERATE	   1
#define DEFAULT_BACKLIT			   0
#define DEFAULT_FLICK_MAX           250
#define DEFAULT_FLICK_EXP           146
#define DEFAULT_USE_INI_MATRIX		FALSE
#define DEFAULT_MATRIX				0xFFFF

#define DEFAULT_IMAGE_BITS			24	
#define DEFAULT_IMAGE_TYPE			BI_RGB	
#define DEFAULT_IMAGE_COLS			176	
#define DEFAULT_IMAGE_ROWS			144


/* used to be in colspace.h		*/
#define DEFAULT_BRIGHTNESS  50
#define DEFAULT_CONTRAST    50
#define DEFAULT_COLOUR      50



/* default colour balance gains */
#define DEFAULT_RED_GAIN          32
#define DEFAULT_GREEN_GAIN        6
#define DEFAULT_BLUE_GAIN         92  
#define MAX_CB_GAIN              212     /* max allowable colourbalance gain */

/* string defines for the ini file */
#define INI_COMPRESSION     "Compression"
#define INI_VIDEOCONF       "VideoConf"
#define INI_VIDEOCONFVAL    "VideoConfVal"
#define INI_VIDEOCAPVAL     "VideoCapVal"
#define INI_CAMERAGAIN      "CameraGain"
#define INI_REDGAIN         "RedGain"
#define INI_GREENGAIN       "GreenGain"
#define INI_BLUEGAIN        "BlueGain"
#define INI_AUTOFRAMERATE   "AutoFrameRate"
#define INI_FRAMERATE       "FrameRate"
#define INI_BANDING         "Banding"
#define INI_MAINSFREQ       "MainsFreq"
#define INI_SHARPNESS       "Sharpness"
#define INI_BACKLIT			"Backlit"
#define INI_IMAGE_COLS		"ImageCols"
#define INI_IMAGE_ROWS		"ImageRows"
#define INI_FLICK_MAX       "FlickMax"
#define INI_FLICK_EXP       "FlickExp"
#define INI_USE_INI_MATRIX	"UseIniMatrix"
#define	INI_MATRIX			"matrix_"
#define INI_LANGUAGE			"Language"
#define INI_DISABLE_ECP		"DisableECP"
#define INI_DISABLE_DMA		"DisableDMA"
#define INI_DISABLE_FDMA	"DisableFDMA"
#define INI_DISABLE_IRQ		"DisableIRQ"
#define INI_BRIGHTNESS		"Brightness"
#define INI_CONTRAST		"Contrast"
#define INI_COLOUR			"Colour"
#define INI_LPT_PORT		"LPTPort"
#define INI_DISABLE_RGB24	"DisableRGB24"
#define INI_DISABLE_RGB16	"DisableRGB16"
#define INI_DISABLE_I420	"DisableI420"
#define INI_DISABLE_YV12	"DisableYV12"
#define INI_DISABLE_UYVY	"DisableUYVY"
#define INI_DISABLE_YUY2	"DisableYUY2"

/* define the language options in the [VVLCPIA] language= setting of the INI file */
#define		ENGLISH_LANG	0
#define		FRENCH_LANG		1
#define		ITALIAN_LANG	2
#define		GERMAN_LANG		3
#define		SPANISH_LANG	4	
#define		PORTUGUESE_LANG	5


#define NEWGAIN(mingain, rgbgain, curexp, oldexp) \
 (BYTE)(((float) mingain +   \
          (float)(rgbgain - mingain) *((float) oldexp /(float) curexp)) + 0.5)


/* following defines control the temporal differencing control */
#define MIN_DIFF_THRESHOLD  0   /* ppc value 2  */
#define MAX_DIFF_THRESHOLD  32  /* ppc value 16 */
#define QCIF_FRIG_LINES     10      /* need 130 lines rather than 120 */


#define DEFAULT_QCIF_FRAMERATE   	30  /* used in compression target things */
#define DEFAULT_CIF_FRAMERATE   	15  /* used in compression target things */


/* max image buffer returned to the application.
   This holds the image before it's multiplied by 2(eg 640 * 480)
*/

#define RAW_RGB_BUFSIZE(DWORD)((DWORD) CIF_COLS *(DWORD) CIF_ROWS * 3)


typedef enum { NextError, NextOK, NextNotReady } NextStatus;


typedef enum {
      ERR_OK,
      ERR_MEMORY,
      ERR_MEMLOCK,
      ERR_MEMFREE,
      ERR_CAMNOTFOUND,
      ERR_CAMSELFTEST,
      ERR_CAMPOWER,
      ERR_CAMMODE,
      ERR_CAPINIT,
      ERR_CAPSELECT,
      ERR_COLINIT,
      ERR_COLSELECT,
      ERR_PPCINIT,
      ERR_PPCSELECT,
      ERR_PPCDEFAULTS,
      ERR_PORTINUSE         /* someone else using port */
} PPC_INIT_ERR;



typedef struct {
    int PortNumber;
	BYTE PortType;

    PLINK pLink;         		/* CPIALINK object 					*/                                  
    PCPIA_CAM pCamera;			/* CPIA_CAM object					*/
    PCAPTURE pCapture;			/* CAPTURE object					*/
    PGFRAME	pGFrame;			/* GETFRAME object		 			*/

    BOOL InvalidFormat;
    
    BOOL ZoomVideo;


    int ImageQuality;
    BITMAPINFOHEADER DestBmp;
    BITMAPINFOHEADER SrcBmp;

    
    BOOL FlickerCtrl;         /* TRUE if frames should be flicker corrected */
    WORD FwVersion;           /* firmware version */
    WORD LastExposure;        /* exposure value on last frame */
    WORD AppFrameRate;           /* framerate requested from app, or assumed
                                  by driver */
    struct {
            COMP_MODE compression;
            BOOL vidconf;
            WORD vidconfval;
            WORD vidcapval;
            WORD manualval;
           } cmp; 
	int Brightness, Contrast, Colour;       /* range 0 -> 100, 50 == normal */

	BYTE ImageSize;
       
} PPC, *PPPC;

typedef HLOCAL HPPC;


BOOL OpenPPC( void );
void ClosePPC( void );

HPPC InitPPCDevice(char *InitFile, int *PortNumber, PPC_INIT_ERR *ErrCode );
void FiniPPCDevice( HPPC PPCDevice, char *InitFile );

PPPC SelectPPCDevice( HPPC hPPC );
HPPC GetPPCDevice( void );


int GetPortMode(void);

BOOL DisableBackgroundUpload( BOOL DisableIt );

BOOL SetPPCOptions( COMP_MODE compression, BOOL vidconf, WORD vidconfval,
                    WORD vidcapval, WORD manualval );
BOOL SetPPCManQuality( int ImageQuality );

void GetPPCOptions( COMP_MODE *compression, BOOL *vidconf, WORD *vidconfval,
                    WORD *vidcapval, WORD *manualval );
void GetPPCParams( int *ImageQuality );

BOOL SetPPCImageType( const LPBITMAPINFOHEADER ImageHeader );
BOOL GetPPCImageType( LPBITMAPINFOHEADER ImageHeader );

NextStatus NextFrame( BYTE _huge *ImageBuffer, DWORD MaxBytes,
                      DWORD *captime, BOOL StreamMode );

BOOL ppcSetSensorMatrix( BYTE Matrix[9] );
// Not operational yet (and may never work anyway )
BOOL ppcGetSensorMatrix( BYTE Matrix[9] );

/************************************/
/* PPC Private functions below here */
/************************************/

BOOL SetPPCDefaults( void );

/* functions to Get / Set gain/exposure on sensor  */
BOOL SetCameraAGC( BOOL bstate );
BOOL SetCameraAEC( BOOL bstate );
BOOL GetCameraAEC( void );

BOOL SetCameraExposure(WORD coarse, WORD fine, BYTE gain);
BOOL GetCameraExposure(WORD *coarse, WORD *fine, BYTE *gain);

BOOL SetCameraACB( COL_BALANCE cbMode, BYTE RedGain, BYTE GreenGain, BYTE BlueGain );
BOOL GetCameraACB( COL_BALANCE *pCbMode, BYTE *pRedGain, BYTE *pGreenGain, BYTE *pBlueGain );

DWORD GetTime();
BOOL check_image_size( LPBITMAPINFOHEADER bih, int *nwid, int *nht );

BOOL ppcGetSensorID( WORD *SensorID, BYTE *SensorRevision );

BOOL ppcGetPanTiltInfo( LPRECT roi, BOOL *zoom );
BOOL ppcSetPanTiltInfo( BOOL zoom, LPRECT roi );

BOOL SetFourCCInfo( LPBITMAPINFOHEADER DestBmp, PPPC pPPC );

BOOL ppcGetSensorFramerate( FRAMERATE *Framerate, BOOL *AutoFramerate, WORD *Period );
BOOL ppcSetSensorFramerate( FRAMERATE Framerate, BOOL AutoFramerate );

int ppcGetFlicker( void );
BOOL ppcSetFlicker( BOOL flickon );

int ppcGetDecimation( void );
BOOL ppcSetDecimation( BOOL dec_on );

BOOL ppcSetCentreWeight( BOOL weighton );
BOOL ppcGetCentreWeight( void );

BOOL ppcSetAppFramerate( int newrate );
int ppcGetAppFramerate( void );

void ppcSetColourSettings( int Brightness, int Contrast, int Colour ); 
void ppcGetColourSettings( int *Brightness, int *Contrast, int *Colour );  

BOOL ppcSetFlickerParams( MAINS_FREQ mf );
BOOL ppcGetFlickerParams( MAINS_FREQ *mf );

WORD ppcGetFwVersion(void);



/* PPC1 Public Functions 	*/

BOOL ReadPPCInit( PPPC pPPC, char *InitFile );
BOOL WritePPCInit( PPPC pPPC, char *InitFile );


