/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/* PPC1.C
23/09/97
overflow of ppc module
*/

#include "common.h"

#include "vvl_type.h"                    
#include "resize.h"
#include "cpia_cmd.h"
#include "vfwdlg.h"

#include "ppc.h"

/* PPC1 Private Functions 	*/
static int iniGetVidConf(LPSTR InitFile);
static int iniGetVidConfVal(LPSTR InitFile);
static int iniGetVcapVal(LPSTR InitFile);
static int iniGetAutoFrameRate(LPSTR InitFile);
static int iniGetFrameRate(LPSTR InitFile);
static int iniGetColourBalance(LPSTR InitFile, BYTE *RedGain, BYTE *GreenGain, BYTE *BlueGain);
static MAINS_FREQ iniGetMainsFreq(LPSTR InitFile);
static BOOL iniGetSharpness(LPSTR InitFile);
static BOOL iniGetBanding(LPSTR InitFile);
static int iniGetCompression(LPSTR InitFile);
static BOOL iniGetCentreWeight(LPSTR InitFile);
static BOOL iniGetImageSize( LPSTR InitFile, long *ImageCols, long *ImageRows );
static BOOL iniGetFlickerThreshold( LPSTR InitFile, BYTE *FlickMax, BYTE *FlickExp );
static BOOL iniGetMatrix( LPSTR InitFile, BYTE *Matrix);


#define M404_00	0x60
#define M404_01	0xE0
#define M404_02	0xF8
#define M404_10	0xE8
#define M404_11	0x60
#define M404_12	0xF3
#define M404_20	0xEF
#define M404_21	0xE8
#define M404_22	0x60

#define M409_00	0x60
#define M409_01	0xE2
#define M409_02	0xFE
#define M409_10	0xF7
#define M409_11	0x59
#define M409_12	0xF0
#define M409_20	0xFC
#define M409_21	0xE4
#define M409_22	0x60

/*-----------------------------------------------------------------*/
/* restore all saved settings to the camera */

BOOL ReadPPCInit( PPPC pPPC, char *InitFile )
{
	BOOL rtn = TRUE,
		TempDiff, 
		AutoThresh, 
		FastStream,
		flicker,
		sharpness,
		autoframerate,
		centre_weight,
		useIniMatrix,
		gotID;
	int compression,
		vidconf,
		vconfval,
		vcapval,
		manualval=50;
	BYTE RedGain, GreenGain, BlueGain,
         flick_max, flick_exp;
	BYTE matrix[9];
	FRAMERATE framerate;
	MAINS_FREQ mf;
	BITMAPINFOHEADER DefaultBMP;
 	long rows, cols;
	WORD sensorID;
	BYTE sensorRevision;

	if( !pPPC || InitFile == NULL )
		rtn = FALSE;
	else
	{
		/* read and validate the ini file settings */
		compression = iniGetCompression( InitFile);
		vidconf = iniGetVidConf( InitFile);
		vconfval = iniGetVidConfVal( InitFile);
		vcapval = iniGetVcapVal( InitFile);
		autoframerate = iniGetAutoFrameRate( InitFile); 
		framerate = iniGetFrameRate( InitFile); 
		iniGetColourBalance( InitFile, &RedGain, &GreenGain, &BlueGain);
		mf = iniGetMainsFreq( InitFile);
		sharpness = iniGetSharpness( InitFile);
		flicker = iniGetBanding( InitFile);
		centre_weight = iniGetCentreWeight( InitFile );
		iniGetImageSize( InitFile, &cols, &rows );
        iniGetFlickerThreshold( InitFile, &flick_max, &flick_exp );
		useIniMatrix = iniGetMatrix( InitFile, matrix );

   		BIH_Init( &DefaultBMP );
   		BIH_SetType( &DefaultBMP,  DEFAULT_IMAGE_BITS, DEFAULT_IMAGE_TYPE );                   
	   	BIH_SetImageSize( &DefaultBMP, cols, rows ); 

	   	if ( !SetPPCImageType( &DefaultBMP ) ) 
    		rtn = FALSE;

		/* set the previous values, then switch auto balance on */                                 
		if( camSetColBal(pPPC->pCamera, CB_MANUAL, RedGain, GreenGain, BlueGain) != OK )
			rtn = FALSE;
		
		if( camSetColBal(pPPC->pCamera, CB_AUTOMATIC, 0, 0, 0) != OK )
			rtn = FALSE;
        
		if( camSetFlickerThreshold(pPPC->pCamera, flick_max, flick_exp) != OK )
			rtn = FALSE;
        
		if ( !ppcSetSensorFramerate( framerate, autoframerate ) )
			rtn = FALSE;
		
		/* restore the mains frequency */
		if ( !ppcSetFlickerParams( mf ) )
			rtn = FALSE;
		
		if( !sharpness)
			ppcSetDecimation( TRUE);  /* decimation off, sharpness on */
		
		if( !SetPPCOptions(compression, vidconf, vconfval, vcapval, manualval) ) 
			rtn = FALSE;
		
		/* if flicker control required, switch it on */
		if( flicker)
			ppcSetFlicker( TRUE);

		if ( useIniMatrix )
		{
			if ( !ppcSetSensorMatrix( matrix ) )
			{
				rtn = FALSE;
			}
		}
		else
		{
			gotID = ppcGetSensorID( &sensorID, &sensorRevision );
			// Sensor ID coded in sensorRevision
			if ( sensorRevision <= 3 )
			{
				sensorID = 404;
			}
			else if ( sensorRevision <= 4 )
			{
				sensorID = 407;
			}
			else if ( sensorRevision <= 14 )
			{
				sensorID = 409;
			}
			else
			{
				sensorID = 404;
			}
			
			if ( gotID && (sensorID == 409) )
			{
				matrix[0] = M409_00;
				matrix[1] = M409_01;
				matrix[2] = M409_02;
				matrix[3] = M409_10;
				matrix[4] = M409_11;
				matrix[5] = M409_12;
				matrix[6] = M409_20;
				matrix[7] = M409_21;
				matrix[8] = M409_22;
			}
			else					// Download VVL404 matrix
			{
				matrix[0] = M404_00;
				matrix[1] = M404_01;
				matrix[2] = M404_02;
				matrix[3] = M404_10;
				matrix[4] = M404_11;
				matrix[5] = M404_12;
				matrix[6] = M404_20;
				matrix[7] = M404_21;
				matrix[8] = M404_22;

			}
			if ( !ppcSetSensorMatrix( matrix ) )
			{
				rtn = FALSE;
			}

		}

		
		{
			int bright, contrast,colour;
			bright = GetPrivateProfileInt( "PPC", "Brightness", DEFAULT_BRIGHTNESS, InitFile );
			contrast = GetPrivateProfileInt( "PPC", "Contrast", DEFAULT_CONTRAST, InitFile );
			colour = GetPrivateProfileInt( "PPC", "Colour", DEFAULT_COLOUR, InitFile );
			ppcSetColourSettings( bright, contrast, colour );
		}
		
		if ( !ppcSetCentreWeight( centre_weight ) ) 
			rtn = FALSE;
	}
	
	return rtn;
}

/*-----------------------------------------------------------------*/

#define TMP_LEN 10

BOOL WritePPCInit( PPPC pPPC, char *InitFile )
{
	char tmp[TMP_LEN];
	BOOL dummy, 
		res = FALSE,
		sharpness,
		vidconf,
		flicker, 
		autoframerate,
		centre_weight;
	int ImageQuality, 
		compression;
	WORD vidconfval,
		vidcapval,
		manualval;
	FRAMERATE framerate;
	MAINS_FREQ mf;
	BYTE RedGain, GreenGain, BlueGain, 
        flick_max, flick_exp; 
	BITMAPINFOHEADER DefaultBMP;
	long cols, rows;
	COL_BALANCE CbMode;
	
	if( pPPC && InitFile != NULL )
	{
	   	GetPPCImageType( &DefaultBMP );
		BIH_GetImageSize( &DefaultBMP, &cols, &rows );

		GetPPCOptions( &compression, &vidconf, &vidconfval, &vidcapval, &manualval );
		camGetColBal( pPPC->pCamera, &CbMode, &RedGain, &GreenGain, &BlueGain );
		camGetSensorFramerate( pPPC->pCamera, &framerate, &autoframerate );
		camGetFlickerThreshold( pPPC->pCamera, &flick_max, &flick_exp );
        ppcGetFlickerParams( &mf );
		sharpness = !ppcGetDecimation( );
		flicker = ppcGetFlicker( );
		centre_weight = ppcGetCentreWeight();
		
		WritePrivateProfileString( "PPC", INI_IMAGE_COLS, 
			itoa( (int)cols, tmp, TMP_LEN), InitFile );
		WritePrivateProfileString( "PPC", INI_IMAGE_ROWS, 
			itoa( (int)rows, tmp, TMP_LEN), InitFile );

		WritePrivateProfileString( "PPC", INI_COMPRESSION, 
			itoa( compression, tmp, TMP_LEN), InitFile );
		
		WritePrivateProfileString( "PPC", INI_VIDEOCONF, 
			itoa( vidconf, tmp, TMP_LEN), InitFile);
		WritePrivateProfileString( "PPC", INI_VIDEOCONFVAL, 
			itoa( vidconfval, tmp, TMP_LEN), InitFile);
		WritePrivateProfileString( "PPC", INI_VIDEOCAPVAL, 
			itoa( vidcapval, tmp, TMP_LEN), InitFile);
		
		WritePrivateProfileString( "PPC", INI_REDGAIN, 
			itoa(RedGain, tmp, TMP_LEN), InitFile );
		
		WritePrivateProfileString( "PPC", INI_GREENGAIN, 
			itoa(GreenGain, tmp, TMP_LEN), InitFile );
		
		WritePrivateProfileString( "PPC", INI_BLUEGAIN, 
			itoa(BlueGain, tmp, TMP_LEN), InitFile );
		
		WritePrivateProfileString( "PPC", INI_AUTOFRAMERATE, 
			itoa(autoframerate, tmp, TMP_LEN), InitFile );
		
		WritePrivateProfileString( "PPC", INI_FRAMERATE, 
			itoa(framerate, tmp, TMP_LEN), InitFile );
		
		WritePrivateProfileString( "PPC", INI_BANDING, 
			itoa(flicker, tmp, TMP_LEN), InitFile );
		
		WritePrivateProfileString( "PPC", INI_FLICK_MAX, 
			itoa(flick_max, tmp, TMP_LEN), InitFile );

        WritePrivateProfileString( "PPC", INI_FLICK_EXP, 
			itoa(flick_exp, tmp, TMP_LEN), InitFile );

        WritePrivateProfileString( "PPC", INI_MAINSFREQ, 
			itoa(mf, tmp, TMP_LEN), InitFile );
		
		WritePrivateProfileString( "PPC", INI_SHARPNESS, 
			itoa(sharpness, tmp, TMP_LEN), InitFile );

		WritePrivateProfileString( "PPC", INI_BACKLIT, 
			itoa(centre_weight, tmp, TMP_LEN), InitFile );
		
		{
			int bright, contrast,colour;
			ppcGetColourSettings( &bright, &contrast, &colour );
			WritePrivateProfileString( "PPC", INI_BRIGHTNESS, itoa(bright, tmp, TMP_LEN), InitFile );
			WritePrivateProfileString( "PPC", INI_CONTRAST, itoa(contrast, tmp, TMP_LEN), InitFile );
			WritePrivateProfileString( "PPC", INI_COLOUR, itoa(colour, tmp, TMP_LEN), InitFile );
		}
		
		
		res = TRUE;
	}
	
	return res;
}


static int iniGetCompression( LPSTR InitFile) 
{
	int compression;
	
	compression = GetPrivateProfileInt( "PPC", INI_COMPRESSION, 
		DEFAULT_COMPRESSION, InitFile);
	if( compression < CMP_OFF || compression > CMP_MANUAL)
		compression = CMP_OFF;
    
	return compression;  
}

/*-----------------------------------------------------------------*/
/* FALSE if in video preview mode, TRUE if in video conference mode */

static int iniGetVidConf( LPSTR InitFile)
{
	int vidconf;
	
	vidconf = GetPrivateProfileInt( "PPC", INI_VIDEOCONF, 
		DEFAULT_VCONF, InitFile);
	if ( (vidconf < 0) || (vidconf > 1) )
		vidconf = 0;
	
	return vidconf;
}

/*-----------------------------------------------------------------*/
/* default video conference quality required */

static int iniGetVidConfVal( LPSTR InitFile)
{
	int vconfval;
	
	vconfval = GetPrivateProfileInt( "PPC", INI_VIDEOCONFVAL, 
		DEFAULT_VCONF_QUALITY, InitFile);
	/* VIDCONF_MAX from vfwdlg.h */                                    
	if( vconfval < 0 || vconfval > VIDCOMP_MAX)
		vconfval = DEFAULT_VCONF_QUALITY;
	
	return vconfval;
}                                           

/*-----------------------------------------------------------------*/
/* vcapval is the quality setting to use when video preview is being
used
*/

static int iniGetVcapVal( LPSTR InitFile)
{
	int vcapval;
	
	vcapval = GetPrivateProfileInt( "PPC", INI_VIDEOCAPVAL, 
		DEFAULT_VCAP_QUALITY, InitFile);
	if( vcapval < 0 || vcapval > VIDCOMP_MAX)
		vcapval = DEFAULT_VCAP_QUALITY;
	
	return vcapval;
}

/*-----------------------------------------------------------------*/
static int iniGetAutoFrameRate( LPSTR InitFile)
{
	int autoframerate;
	
	autoframerate = GetPrivateProfileInt( "PPC", INI_AUTOFRAMERATE, DEFAULT_AUTOFRAMERATE, InitFile);
	
	return autoframerate;
}

/*-----------------------------------------------------------------*/
/* driver only uses 7.5, 15 and 30fps rates at present. Restrict 
the acceptable values to one of these 
*/
static int iniGetFrameRate( LPSTR InitFile)
{
	int framerate;
	
	framerate = GetPrivateProfileInt( "PPC", INI_FRAMERATE, FPS_30, InitFile);
	
	/* deal with the framerate. If it isn't valid, set it to 30fps */
	if( framerate != FPS_75 
		&& framerate != FPS_15 
		&& framerate != FPS_30 )
	{
		framerate = FPS_30;
	}
	return framerate;
}


/*-----------------------------------------------------------------*/
/* read the colour balances stored when the camera last closed. These
are used to prime the colour balance algorithm when the camera is
initialised.
The max colour balance setting is 212. Some of the low level routines
return -1 if they fail to read colour balance on exit, so this traps
such values, and resets them to valid ones.
If all colour balances are set to 255( -1), then returned frames are
black...
*/

static BOOL iniGetColourBalance( LPSTR InitFile, BYTE *RedGain, BYTE *GreenGain, 
					BYTE *BlueGain ) 
{
	/* reload and set the colour balance defaults */
    
	*RedGain = GetPrivateProfileInt( "PPC", INI_REDGAIN, DEFAULT_RED_GAIN, 
		InitFile);
	*GreenGain = GetPrivateProfileInt( "PPC", INI_GREENGAIN, DEFAULT_GREEN_GAIN, 
		InitFile);
	*BlueGain = GetPrivateProfileInt( "PPC", INI_BLUEGAIN, DEFAULT_BLUE_GAIN, 
		InitFile );
	
	if( *RedGain < 1 || *RedGain > MAX_CB_GAIN)
		*RedGain = DEFAULT_RED_GAIN;
	
	if( *GreenGain < 1 || *GreenGain > MAX_CB_GAIN)
		*GreenGain = DEFAULT_GREEN_GAIN;
	
	if( *BlueGain < 1 || *BlueGain > MAX_CB_GAIN)
		*BlueGain = DEFAULT_BLUE_GAIN;
	
	return TRUE;
}

/*-----------------------------------------------------------------*/

static MAINS_FREQ iniGetMainsFreq( LPSTR InitFile )
{
	MAINS_FREQ mf;
	
	mf = GetPrivateProfileInt( "PPC", INI_MAINSFREQ, DEFAULT_MAINS_FREQ, InitFile);
	
	if( mf != MAINS_50HZ && mf != MAINS_60HZ)
		mf = MAINS_50HZ;
	
	return mf;
}

/*-----------------------------------------------------------------*/
/* whether to switch image decimation on */

static BOOL iniGetSharpness( LPSTR InitFile )
{
	int sharpness;
	
	sharpness = GetPrivateProfileInt( "PPC", INI_SHARPNESS, 
		DEFAULT_SHARPNESS, InitFile);
	if( sharpness < 0 || sharpness > 1)
		sharpness = 1;   /* default to sharpness on */
	
	return sharpness;
}

/*-----------------------------------------------------------------*/
/* whether to run with flicker enabled or disabled */

static BOOL iniGetBanding( LPSTR InitFile )
{
	int banding;
	
	banding = GetPrivateProfileInt( "PPC", INI_BANDING, 
		DEFAULT_FLICKER, InitFile);
	if( banding < 0 || banding > 1)
		banding = 0;   /* default to flicker support off */
	
	return banding;
}

/*-----------------------------------------------------------------*/
/* whether to enabled or disabled centre-weighed exposure 		*/

static BOOL iniGetCentreWeight( LPSTR InitFile )
{
	int backlit;
	
	backlit = GetPrivateProfileInt( "PPC", INI_BACKLIT, 
		DEFAULT_BACKLIT, InitFile);
	
	return( (BOOL)backlit );
}




static BOOL iniGetImageSize( LPSTR InitFile, long *ImageCols, long *ImageRows ) 
{
    
	*ImageCols = (long)GetPrivateProfileInt( "PPC", INI_IMAGE_COLS, DEFAULT_IMAGE_COLS, 
		InitFile);

	*ImageRows = (long)GetPrivateProfileInt( "PPC", INI_IMAGE_ROWS, DEFAULT_IMAGE_ROWS, 
		InitFile);

	/* 
		should sanity check values here 
	*/	
 	
	return TRUE;
}


static BOOL iniGetFlickerThreshold( LPSTR InitFile, BYTE *FlickMax, BYTE *FlickExp ) 
{
    
	*FlickMax = (long)GetPrivateProfileInt( "PPC", INI_FLICK_MAX, DEFAULT_FLICK_MAX, 
	                                    	InitFile);

	*FlickExp = (long)GetPrivateProfileInt( "PPC", INI_FLICK_EXP, DEFAULT_FLICK_EXP, 
                                    		InitFile);
 	
	return TRUE;
}

#define N_MATRIX				9
#define N_MAX_PROFILE_NAME		20

static BOOL iniGetMatrix( LPSTR InitFile, BYTE *Matrix)
{
	BOOL useIniMatrix;
	BYTE index;
	char strProfileName[N_MAX_PROFILE_NAME];
	char strIndex[2];
	int readValue;
	
	useIniMatrix = GetPrivateProfileInt( "PPC", INI_USE_INI_MATRIX, DEFAULT_USE_INI_MATRIX, InitFile);
	if ( useIniMatrix )
	{
		for( index = 0; index < N_MATRIX; index++ )
		{	                        
			strcpy( strProfileName, INI_MATRIX );
			strIndex[0] = (char)('0' + index);
			strIndex[1] = (char)'\0';
			strcat( strProfileName, strIndex );
			readValue = GetPrivateProfileInt( "PPC", strProfileName, DEFAULT_MATRIX, InitFile );
			if ( readValue == DEFAULT_MATRIX )
			{
				//MessageBox( NULL , "NoMatrix" , strProfileName, MB_OK | MB_ICONEXCLAMATION );
				useIniMatrix = FALSE;
				break;
			}
			else
			{
				Matrix[index] = (BYTE)readValue;
			}
		}
	}

	return useIniMatrix;
}
