/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/* TRACEMSG.C
   21-05-97
   message tracing. Controls a modeless dialog box and displays any supplied
   messages, as long as DBG_TRACEMSG flag set
*/

#include <windows.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
                                
#include "warning.h"
#include "resource.h"                                
#include "debuglvl.h"
#include "tracemsg.h"

tracemsg (int actn, ...)

{                     
 static HWND hdlg = NULL ;          /* handle to modeless dialog */
 LPSTR gbuf,
       ptr ;
 HGLOBAL hglob ;            
 char windir [_MAX_PATH],
      tmp [_MAX_PATH + 40] ;
 extern HINSTANCE ghModule ;
 va_list argptr ;
 int lbnumb,
     i ;
                        
 if (TMSET ())
  {               
   switch (actn)
    {             
     case TRC_INIT :
          if (hdlg == NULL)
           hdlg = CreateDialog (ghModule, MAKEINTRESOURCE (DLG_TRACE), 
                                GetDesktopWindow (), DlgTrace) ;
          break ;
          
     case TRC_MSG :
          /* if not on screen, create */
          if (hdlg && IsWindow (hdlg))
           {                      
            if ((hglob = GlobalAlloc (GPTR, TRC_BUFSIZE)) != NULL)
             {                                  
              if ((gbuf = (LPSTR) GlobalLock (hglob)) != NULL)
               {
                /* set up the variable list, and create the message */
                va_start (argptr, actn) ;
                ptr = va_arg (argptr, LPSTR) ;
                wvsprintf (gbuf, ptr, argptr) ;                       
                
                /* unlock the handle, and post it to the dialog box */
                GlobalUnlock (hglob) ;
                
                PostMessage (hdlg, TRC_DLGMSG, 0, (LPARAM) hglob) ;
               }                      
              else
               GlobalFree (hglob) ;
             }
           }
           
          break ;
        
     case TRC_DESTROY :
          if (hdlg)    
           {                    
            /* the messagebox allows all the postmessages to catch up and be 
               displayed. This wasn't happening if the driver was opened,
               queried and closed. */
            *windir = '\0' ;
            GetWindowsDirectory (windir, sizeof (windir)) ;
            if (windir [lstrlen (windir) - 1] != '\\')
             lstrcat (windir, "\\") ;
            lstrcat (windir, "vvltrace.log") ;
            wsprintf (tmp, "Save trace details to %s?", (LPSTR) windir) ;
            if (MessageBox (NULL, 
                            tmp,
                            "Trace ends",
                            MB_YESNO | MB_ICONQUESTION) == IDYES)
              SendMessage (hdlg, TRC_DLGSAVE, 0, (LPARAM) (LPSTR) windir) ;
            
            DestroyWindow (hdlg) ;
            hdlg = NULL ;
           }
          
          break ;   
          
    }
  }
  
 return TRUE ;
}

/*-----------------------------------------------------------------*/
/* decode a fourcc to a readable value. The static buffer is overwritten
   each call */
   
LPSTR trc_fourcc (DWORD comp)

{                            
 static char buf [5] ;          
 int i ;
 
 if (comp == 0)
  lstrcpy (buf, "RGB") ;
 else
  {                    
   for (i = 0; i <4; i++)
    {
     buf [i] = (comp & 0xff) ;
     comp >>= 8 ;
    }                    
   buf [4] = '\0' ;
  }
                     
 return (LPSTR) buf ;
}

/*-----------------------------------------------------------------*/

BOOL FAR PASCAL _loadds DlgTrace (HWND hdlg, UINT msg, WPARAM wparam,
                                  LPARAM lparam)
                                  
{             
 static BOOL show_items = TRUE ;
 static HMENU hmenu ;
 UINT state ;
 BOOL res = FALSE ;
 HWND hlb ;                                       
 int numb,
     i ;
 LPSTR lpbuf,
       fname ;
 HFILE hfile ;
 char buf [256] ;
 
  
 switch (msg)
  {
   case WM_INITDIALOG :           
        hmenu = GetSystemMenu (hdlg, FALSE) ;
        AppendMenu (hmenu, MF_MENUBREAK, 0, NULL) ;
        AppendMenu (hmenu, MF_STRING, IDM_PAUSE, "&Pause") ;
        EnableMenuItem (hmenu, SC_CLOSE, MF_BYCOMMAND | MF_GRAYED) ;
        DrawMenuBar (hdlg) ;
        ShowWindow (hdlg, SW_SHOW) ;
        break ;                     
               
   case WM_SYSCOMMAND :
        if (wparam == IDM_PAUSE)
         {    
          show_items = !show_items ;
          state = show_items ? MF_UNCHECKED : MF_CHECKED ;
          CheckMenuItem (hmenu, IDM_PAUSE, MF_BYCOMMAND | state) ;
          res = TRUE ;
         }
        break ;

        /* if window resized, resize the list box to fill the area */
   case WM_SIZE :
        MoveWindow (GetDlgItem (hdlg, IDD_TRACELB), 
                    0, 0, LOWORD (lparam), HIWORD (lparam), TRUE) ;
        res = TRUE ;
        break ;
        
   case TRC_DLGMSG :            
        if (show_items)
         {
          hlb = GetDlgItem (hdlg, IDD_TRACELB) ;
          
          /* lock the buffer, send the message and unlock the buffer */
          lpbuf = (LPSTR) GlobalLock ((HGLOBAL) lparam) ;
          SendMessage (hlb, LB_ADDSTRING, 0, (LPARAM) lpbuf) ;
          GlobalUnlock ((HGLOBAL) lparam) ;

          /* scroll the new string into view */          
          numb = SendMessage (hlb, LB_GETCOUNT, 0, 0L) ;
          SendMessage (hlb, LB_SETCURSEL, numb - 1, 0L) ;
          
          /* only display last 300 messages */
          if (SendMessage (hlb, LB_GETCOUNT, 0, 0L) > 300)
           SendMessage (hlb, LB_DELETESTRING, 0, 0L) ;
         }
        GlobalFree ((HGLOBAL) lparam) ;
        res = TRUE ;
        break ;        

        /* save the listbox entries to the filename supplied in lparam */        
   case TRC_DLGSAVE :  
        if (lparam)
         {
          fname = (LPSTR) lparam ;
        
          if ((hfile = _lcreat (fname, 0)) != HFILE_ERROR)
           {
            hlb = GetDlgItem (hdlg, IDD_TRACELB) ;
            numb = SendMessage (hlb, LB_GETCOUNT, 0, 0L) ;
            for (i = 0; i < numb; i++)
             {
              SendMessage (hlb, LB_GETTEXT, i, (LPARAM) (LPSTR) buf) ;
              lstrcat (buf, "\r\n") ;
              _lwrite (hfile, buf, lstrlen (buf)) ;
             }                                     
            _lclose (hfile) ;
           }
          else                       
           {
            wsprintf (buf, "Unable to create %s", (LPSTR) fname) ;
            MessageBox (NULL, "vvlcap trace", buf, MB_OK | MB_ICONEXCLAMATION) ;
           }
         }
        res = TRUE ;
        break ;
  }
 
 return res ;
}                                  
