/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/* VFWDLG.C
   18-07-97
   vfw dialog boxes
*/   

#include "common.h"

#include <ver.h>
#include <stdarg.h>

#include "resource.h"
#include "vfwdlg.h" 

#include "vvlcap.h"
#include "ppc.h"
#include "oem.h"
#include "dvm_msg.h"     
#include "advdlg.h"
#include "drv_ver.h"

                         

static void SetCaptureDetails(HWND hdlg, int sbitem, int editem, 
                               SliderParam *sb);
static void UKSpelling(HWND hdlg, int item);

static suggest_image_size(HWND hwnd, LPBITMAPINFOHEADER bih, int xcon, int ycon);
static void show_details(HWND hwnd, int port);
static void EnableWindowEx(HWND hwnd, int numb, BOOL state, ...);


/* Image Settings dialog box local params */  

int ImageX,
    ImageY ; 
    
SliderParam BrightnessSB,
            ContrastSB,
            ColourSB,
            VidCompSB;

/* Image Format dialog box local params */  

BITMAPINFOHEADER NewFormat,
                 OldFormat;


void InitDialogParams(void)
{
/*
    video capture initialisation
*/
    BrightnessSB.Min     = BRIGHTNESS_MIN;
    BrightnessSB.Max     = BRIGHTNESS_MAX;
    BrightnessSB.Current = BRIGHTNESS_INITIAL;
    BrightnessSB.Step    = BRIGHTNESS_STEP;
    BrightnessSB.Lstep   = BRIGHTNESS_LSTEP;

    ContrastSB.Min     = CONTRAST_MIN;
    ContrastSB.Max     = CONTRAST_MAX;
    ContrastSB.Current = CONTRAST_INITIAL;
    ContrastSB.Step    = CONTRAST_STEP;
    ContrastSB.Lstep   = CONTRAST_LSTEP;

    ColourSB.Min     = COLOUR_MIN;
    ColourSB.Max     = COLOUR_MAX;
    ColourSB.Current = COLOUR_INITIAL;
    ColourSB.Step    = COLOUR_STEP;
    ColourSB.Lstep   = COLOUR_LSTEP;

    VidCompSB.Min     = VIDCOMP_MIN;
    VidCompSB.Max     = VIDCOMP_MAX;
    VidCompSB.Current = VIDCOMP_INITIAL;
    VidCompSB.Step    = VIDCOMP_STEP;
    VidCompSB.Lstep   = VIDCOMP_LSTEP; 
}

/*-----------------------------------------------------------------*/

void CloseDialog()

{
}

/*-----------------------------------------------------------------*/
/*
   video source dialog box. This handles brightness/colour/saturation,
   image compression, and zooming and pan/tilt control
*/

BOOL FAR PASCAL _loadds VideoCaptureSettingsDialogProc(HWND hDlg,
                                                        UINT message,
                                                        WPARAM wParam,
                                                        LPARAM lParam)
{
 static RECT pos ;   /* previous position of dialog box */
 static BOOL initdlg, tmp_vconf;
 static COMP_MODE tmp_comp;
 static HBRUSH hctlcolour;
 static int OldBrightness,
            OldContrast,
            OldColour,
            OldCompMode,
            OldVidCapVal,
            OldVidConfVal,
            CurrCompMode,
            CurrVidCapVal,
            CurrVidConfVal,
            oldmanualval;   	 

 HWND hCtl;
 WORD wID;
 char buf [80] ;      
 LONG tres ;
 RECT tmp ;
 BOOL res = FALSE,
      err ;

 COMP_MODE comp ;
 RECT rect ;
 HBRUSH hbr ;
 
 switch(message)
 
  {
   case WM_INITDIALOG:
        initdlg = TRUE ;
        hctlcolour = CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
        GetPPCImageType( &OldFormat );
        ppcGetColourSettings( &OldBrightness, &OldContrast, &OldColour );

        BrightnessSB.Current = OldBrightness;
        ContrastSB.Current = OldContrast;
        ColourSB.Current = OldColour;

        /* set up the compression slider and radio buttons to the correct state */
        GetPPCOptions( &tmp_comp, &tmp_vconf, &OldVidConfVal, &OldVidCapVal, 
                       &oldmanualval );
		if ( !tmp_vconf )	
			OldCompMode = IDC_VIDCAP;
		else 
			OldCompMode = IDC_VIDCONF;

		CurrCompMode = OldCompMode;
	    CurrVidCapVal = OldVidCapVal;
	    CurrVidConfVal = OldVidConfVal;

		if ( CurrCompMode == IDC_VIDCAP )
		{
	        CheckRadioButton( hDlg, IDC_VIDCAP, IDC_VIDCONF, IDC_VIDCAP );
   		    VidCompSB.Current = CurrVidCapVal;
	 	}
		else 
		{
	        CheckRadioButton( hDlg, IDC_VIDCAP, IDC_VIDCONF, IDC_VIDCONF );
	        VidCompSB.Current =  CurrVidConfVal;
    	}                  
        SetCaptureDetails(hDlg, IDC_VIDCOMPSB, IDC_VIDCOMPSBVAL, &VidCompSB);          
 
        /* set the Brightness / Contrast / Colour sliders  */        
        SetCaptureDetails( hDlg, IDC_BRIGHTNESS, IDC_BRIGHTNESSVAL, &BrightnessSB );
        SetCaptureDetails( hDlg, IDC_CONTRAST, IDC_CONTRASTVAL, &ContrastSB );
        SetCaptureDetails( hDlg, IDC_COLOUR, IDC_COLOURVAL, &ColourSB );
                  
        UKSpelling(hDlg, IDC_COLOURTEXT);
        SetFocus(GetDlgItem(hDlg, IDOK));
        
        /* restore the dialog box to its previous location */
        if ( !IsRectEmpty(&pos) )
         	SetWindowPos(hDlg, HWND_TOP,
            	           pos. left, pos. top, 
                	       0, 0,
                    	   SWP_NOSIZE | SWP_NOZORDER);
        initdlg = FALSE ;
        break;

   case WM_HSCROLL:
        hCtl = HIWORD(lParam);
        wID = GetWindowWord(hCtl,GWW_ID);
        switch(wID)
         {
          case IDC_BRIGHTNESS:
               BrightnessSB.Current = ProcessScroll(hDlg, hCtl, wParam,
                                                    lParam,
                                                    IDC_BRIGHTNESSVAL,
                                                    &BrightnessSB);
               ppcSetColourSettings( BrightnessSB.Current, ContrastSB.Current, 
                                  ColourSB.Current ); 
               break;

          case IDC_CONTRAST:
               ContrastSB.Current = ProcessScroll(hDlg, hCtl, wParam, lParam,
                                                  IDC_CONTRASTVAL,
                                                  &ContrastSB);
               ppcSetColourSettings( BrightnessSB.Current, ContrastSB.Current, 
                                  ColourSB.Current ); 
               break;

          case IDC_COLOUR:
               ColourSB.Current = ProcessScroll(hDlg, hCtl, wParam, lParam,
                                                 IDC_COLOURVAL,
                                                 &ColourSB);
               ppcSetColourSettings( BrightnessSB.Current, ContrastSB.Current, 
                                  ColourSB.Current ); 
               break;             
                    
          case IDC_VIDCOMPSB:
               	VidCompSB.Current = ProcessScroll(hDlg, hCtl, wParam, lParam,
                                                 IDC_VIDCOMPSB, &VidCompSB);
    			if ( CurrCompMode == IDC_VIDCAP )
				{
        	        CurrVidCapVal = VidCompSB.Current;
	          		SetPPCOptions( CMP_AUTOMATIC, FALSE, CurrVidConfVal, CurrVidCapVal, oldmanualval );
				}
				else 
				{
        	        CurrVidConfVal = VidCompSB.Current;
	            	SetPPCOptions( CMP_AUTOMATIC, TRUE, CurrVidConfVal, CurrVidCapVal, oldmanualval );
				}
               oldmanualval = 100 ;
               break;                                         
         }

        break;

   case WM_COMMAND:
        switch(wParam)
         {
//          case IDC_NODIFF :         
          case IDC_VIDCAP :                                     
          case IDC_VIDCONF :
               if ( wParam != CurrCompMode )
               {
					CurrCompMode = wParam;
	    			if ( CurrCompMode == IDC_VIDCAP )
    	    	    {
    	    	        VidCompSB.Current = CurrVidCapVal;
	       	         	SetPPCOptions( CMP_AUTOMATIC, FALSE, CurrVidConfVal, CurrVidCapVal, oldmanualval );
					}
					else 
					{
    	    	        VidCompSB.Current = CurrVidConfVal;
	       	         	SetPPCOptions( CMP_AUTOMATIC, TRUE, CurrVidConfVal, CurrVidCapVal, oldmanualval );
					}
 					SetScrollPos( GetDlgItem( hDlg, IDC_VIDCOMPSB ), SB_CTL, VidCompSB.Current, TRUE );  
               }
               res = TRUE ;
               break;                          

          case IDC_VIDCOMPSBVAL :
               /* this is an edit box that controls the scroll bar if data 
                  typed into it - manual quality control */
               switch(HIWORD(lParam))
                {
                 case EN_CHANGE :
                      VidCompSB.Current = ProcessEditBox(hDlg, wParam, 
                                                           IDC_VIDCOMPSB, 
                                                           &VidCompSB);
                      if (!initdlg)
                      {
                        /* call to func that does the auto stuff. This will also 
                           require the state of the videoconf button */
		    			if ( CurrCompMode == IDC_VIDCAP )
	    	    	    {
	    	    	        CurrVidCapVal = VidCompSB.Current;
		       	         	SetPPCOptions( CMP_AUTOMATIC, FALSE, CurrVidConfVal, CurrVidCapVal, oldmanualval );
						}
						else 
						{
	    	    	        CurrVidConfVal = VidCompSB.Current;
		       	         	SetPPCOptions( CMP_AUTOMATIC, TRUE, CurrVidConfVal, CurrVidCapVal, oldmanualval );
						}
                      }
                      break ;
                }
               break ;   

          case IDC_BRIGHTNESSVAL :
               switch(HIWORD(lParam))
                {
                 case EN_CHANGE :
                      BrightnessSB. Current = ProcessEditBox(hDlg, wParam, 
                                                              IDC_BRIGHTNESS, 
                                                              &BrightnessSB);
                      if (!initdlg)
                       ppcSetColourSettings(BrightnessSB.Current, 
                                          ContrastSB.Current, 
                                          ColourSB.Current); 
                      break ;
                }
               break ;
               
          case IDC_CONTRASTVAL :
               switch(HIWORD(lParam))
                {
                 case EN_CHANGE :
                      ContrastSB. Current = ProcessEditBox(hDlg, wParam, 
                                                            IDC_CONTRAST, 
                                                            &ContrastSB);
                      if (!initdlg)                                         
                       ppcSetColourSettings(BrightnessSB.Current, 
                                          ContrastSB.Current, 
                                          ColourSB.Current); 
                      break ;
                }
               break ;
               
          case IDC_COLOURVAL :
               switch(HIWORD(lParam))
                {
                 case EN_CHANGE :
                      ColourSB. Current = ProcessEditBox(hDlg, wParam, 
                                                          IDC_COLOUR, 
                                                          &ColourSB);
                      if (!initdlg)
                       ppcSetColourSettings(BrightnessSB.Current, 
                                          ContrastSB.Current, 
                                          ColourSB.Current); 
                      break ;
                }
               break ;
               
          case IDC_RESETDEF :
               /* restore defaults for the camera */
               SetDlgItemInt(hDlg, IDC_BRIGHTNESSVAL, DEFAULT_BRIGHTNESS, FALSE);
               SetDlgItemInt(hDlg, IDC_CONTRASTVAL, DEFAULT_CONTRAST, FALSE);
               SetDlgItemInt(hDlg, IDC_COLOURVAL, DEFAULT_COLOUR, FALSE);
           
               /* vidcap quality and vidconf quality must be set explicitly, as
                  the edit controls are no longer there */

				CurrCompMode = IDC_VIDCAP;
			    CurrVidCapVal = 50;
			    CurrVidConfVal = 50;
				if ( CurrCompMode == IDC_VIDCAP )
				{
			        CheckRadioButton( hDlg, IDC_VIDCAP, IDC_VIDCONF, IDC_VIDCAP );
		   		    VidCompSB.Current = CurrVidCapVal;
			 	}
				else 
				{
			        CheckRadioButton( hDlg, IDC_VIDCAP, IDC_VIDCONF, IDC_VIDCONF );
			        VidCompSB.Current =  CurrVidConfVal;
		    	}                  
		        SetCaptureDetails(hDlg, IDC_VIDCOMPSB, IDC_VIDCOMPSBVAL, &VidCompSB);          
		                                
               	/* reset so that new value will always be taken */                                           
				CurrCompMode = IDC_VIDCONF;	/* temporarily change this so that the next line		*/
											/* will actually cause the new values to be applied !! */
               	SendMessage(hDlg, WM_COMMAND, IDC_VIDCAP, 
                            MAKELONG(GetDlgItem(hDlg, IDC_VIDCAP),
                                      BN_CLICKED));
               	break ;                              
               
               
          case IDC_ADVANCED :
               AdvancedCaptureOptions(hDlg);
               res = TRUE ;
               break ;

          case IDCANCEL:
               	ppcSetColourSettings( OldBrightness, OldContrast, OldColour);
    			if ( OldCompMode == IDC_VIDCAP )
					SetPPCOptions( CMP_AUTOMATIC, FALSE, OldVidConfVal, OldVidCapVal, oldmanualval );
				else
					SetPPCOptions( CMP_AUTOMATIC, TRUE, OldVidConfVal, OldVidCapVal, oldmanualval );
              	EndDialog(hDlg,TRUE);
				res = TRUE ;
  				break;

          case IDOK:
               DeleteObject(hctlcolour);
               GetWindowRect(hDlg, &pos);
               EndDialog(hDlg,TRUE);
               res = TRUE ;
               break ;
         }
        break;
        
   case WM_ERASEBKGND:
        GetClientRect(hDlg, &rect);  
        hbr = CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
        FillRect((HDC)wParam, &rect, hbr);
        DeleteObject(hbr);
        res = TRUE ;
        break;                    

   case WM_CTLCOLOR:
        SetBkColor((HDC) wParam, GetSysColor(COLOR_BTNFACE));
        return(int) hctlcolour ;
 }

 return res ;
}

/*-----------------------------------------------------------------*/
/* image dimensions and format */

BOOL FAR PASCAL _loadds VideoFormatDialogProc(HWND hDlg, UINT message, 
                                               WPARAM wParam, LPARAM lParam)          
                                               
{             
 static RECT pos ;
 static HBRUSH hctlcolour ;
 HWND hCtl;
 WORD wID;   
 int btnid,
     colbtn ;
 BOOL fFlag,
      res = FALSE ;
 char dummy[64];
 RECT rect ; 
 HBRUSH hbr ;

 switch(message)
    {
     case WM_INITDIALOG:
          hctlcolour = CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
          GetPPCImageType( &OldFormat );
          NewFormat = OldFormat;
          
          btnid = colbtn = 0 ;

          /* settings in the Image Format group box. */
          if (NewFormat.biCompression == BI_RGB)
           {
            if (NewFormat.biBitCount == 24)
             colbtn = IDC_FORM24 ;
            else
             if (NewFormat.biBitCount == 16)
              colbtn = IDC_FORM16 ;
             else
              if (NewFormat.biBitCount == 8)
               colbtn = IDC_FORM8 ;
           }       
          else
           if (NewFormat.biCompression == BI_I420)
            colbtn = IDC_FORM_I420 ;
          
          /* If an unknown compression mode is requested, no radio button is 
             set */
          if (colbtn)
           CheckRadioButton(hDlg, IDC_FORM24, IDC_FORM_I420, colbtn);

          ImageX = NewFormat. biWidth;
          ImageY = NewFormat. biHeight;
          SetDlgItemInt(hDlg, IDC_SIZEX, ImageX, TRUE);
          SetDlgItemInt(hDlg, IDC_SIZEY, ImageY, TRUE);
                     
          btnid = 0 ;
          
          /* image size. This covers both video size and still image size */
          if (ImageX == CIF_COLS * 2 && ImageY == CIF_ROWS * 2)
           btnid = IDC_SIZE704 ;
          else
           if ((ImageX == SIF_COLS * 2) &&(ImageY == SIF_ROWS * 2) )
            btnid = IDC_SIZE640 ;
           else                  
            if (ImageX == CIF_COLS && ImageY == CIF_ROWS)
             btnid = IDC_SIZE352 ;
            else            
             if ((ImageX == SIF_COLS) &&(ImageY == SIF_ROWS) ) 
              btnid = IDC_SIZE320 ;
             else
              if ((ImageX == QCIF_COLS) &&(ImageY == QCIF_ROWS) )
               btnid = IDC_SIZE176 ;
              else
               if ((ImageX == QSIF_COLS) &&(ImageY == QSIF_ROWS) )
                btnid = IDC_SIZE160 ;

            /* If an unknown image size is requested, no radio button is set */
            if (btnid)
             CheckRadioButton(hDlg, IDC_SIZE352, IDC_SIZE640, btnid);
                 
            CheckFormat(hDlg, colbtn);
                                               
            /* limit text input to 3 characters(999) max */
            SendDlgItemMessage(hDlg, IDC_SIZEX, EM_LIMITTEXT, 3, 0L);
            SendDlgItemMessage(hDlg, IDC_SIZEY, EM_LIMITTEXT, 3, 0L);

            SetFocus(GetDlgItem(hDlg, IDOK));

            if (!IsRectEmpty(&pos))
             SetWindowPos(hDlg, HWND_TOP,
                           pos. left, pos. top, 
                           0, 0,
                           SWP_NOSIZE | SWP_NOZORDER);
            break;
                
        case WM_COMMAND:
             switch(wParam)
              {
               case IDC_FORM24:
                    CheckFormat(hDlg, IDC_FORM24);
                    BIH_SetType( &NewFormat, 24, BI_RGB );
                    break;

               case IDC_FORM16:
                    CheckFormat(hDlg, IDC_FORM16);
	                BIH_SetType( &NewFormat, 16, BI_RGB );
                    break;

               case IDC_FORM8:
                    CheckFormat(hDlg, IDC_FORM8);
                    BIH_SetType( &NewFormat, 8, BI_RGB );
                    break;

               case IDC_FORM_I420:                  
                    CheckFormat(hDlg, IDC_FORM_I420);
                    BIH_SetType( &NewFormat, 8, BI_I420 );
                    break;
               
               case IDC_SIZE704 :
                    SetXYSize( hDlg, CIF_COLS * 2, CIF_ROWS * 2);
                    break;
                
               case IDC_SIZE640:
                    SetXYSize( hDlg, SIF_COLS * 2, SIF_ROWS * 2);
                    break;

               case IDC_SIZE352 :
                    SetXYSize( hDlg, CIF_COLS, CIF_ROWS);
                    break;
                
               case IDC_SIZE320:
                    SetXYSize( hDlg, SIF_COLS, SIF_ROWS );
                    break;

               case IDC_SIZE176:
                    SetXYSize( hDlg, QCIF_COLS, QCIF_ROWS );
                    break;

               case IDC_SIZE160:
                    SetXYSize( hDlg, QSIF_COLS, QSIF_ROWS );
                    break;

               case IDOK:  
                    GetWindowRect(hDlg, &pos);
                    BIH_SetImageSize(&NewFormat, 
                                    (LONG)GetDlgItemInt(hDlg, IDC_SIZEX, 
                                                          &fFlag, FALSE),
                                    (LONG)GetDlgItemInt(hDlg, IDC_SIZEY,
                                                          &fFlag, FALSE) );

                    if ( !SetPPCImageType( &NewFormat ) ) 
                     {
                      suggest_image_size(hDlg, &NewFormat, IDC_SIZEX, 
                                          IDC_SIZEY);
//                      SetPPCImageType( &OldFormat );
                      break ;           /* don't close dlg */
                     }

                    DeleteObject(hctlcolour);
                    EndDialog(hDlg,TRUE);
                    return(TRUE);

               case IDCANCEL:
                    GetWindowRect(hDlg, &pos);
                    DeleteObject(hctlcolour);
                    EndDialog(hDlg,TRUE);
                    return(TRUE);
              }
            break;
            
        case WM_ERASEBKGND:
             GetClientRect(hDlg, &rect);  
             hbr = CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
             FillRect((HDC)wParam, &rect, hbr);
             DeleteObject(hbr);
             res = TRUE ;
             break;                    

        case WM_CTLCOLOR:
             SetBkColor((HDC) wParam, GetSysColor(COLOR_BTNFACE));
             return(int) hctlcolour;

    }

    return res ;
}

/*-----------------------------------------------------------------*/

int ProcessScroll(HWND hDlg, HWND hCtl, WPARAM wParam, LPARAM lParam, 
                  WORD wIDEditBox, SliderParam* param)

{                                                     
 int iVal;
     iVal = param->Current;
 
 
     switch(wParam)
      {
        case SB_LINEDOWN:
           iVal += param->Step;
           break;

        case SB_LINEUP: 
           iVal -= param->Step;
           break;

        case SB_PAGEDOWN:
           iVal += param->Lstep;
           break;

        case SB_PAGEUP: 
           iVal -= param->Lstep;
           break;

        case SB_THUMBTRACK:
            iVal = LOWORD(lParam);
            break;

        default:
            break;
      }

     if (iVal > param->Max)
      iVal = param->Max;
     else
      if (iVal < param->Min)
       iVal = param->Min;

     SetScrollPos(hCtl, SB_CTL, iVal, TRUE);
     
     if (wIDEditBox != IDC_STATIC)
      SetDlgItemInt(hDlg, wIDEditBox, iVal, TRUE);

     return iVal;
}

/*-----------------------------------------------------------------*/
/* get the value typed into an edit box. If it's a valid integer, set
   the corresponding scrollbar to the appropriate position */
   
ProcessEditBox(HWND hwnd, int edctl, int sbctl, SliderParam *sb)

{
 int edval,
     cur ;
 BOOL edvalid ;
 
 edval = GetDlgItemInt(hwnd, edctl, &edvalid, FALSE);
 if (edvalid && edval >= sb->Min && edval <= sb->Max)
  SetScrollPos(GetDlgItem(hwnd, sbctl), SB_CTL, edval, TRUE);
 else
  edval = sb->Current ;
  
 return edval ;
}

/*-----------------------------------------------------------------*/

void SetXYSize(HWND hDlg, int X, int Y )
{
    SetDlgItemInt(hDlg, IDC_SIZEX, X, TRUE);
    SetDlgItemInt(hDlg, IDC_SIZEY, Y, TRUE);
} 

/*-----------------------------------------------------------------*/

static void SetCaptureDetails(HWND hdlg, int sbitem, int editem, 
                               SliderParam *sb)

{
 HWND hctl = GetDlgItem(hdlg, sbitem);

 SetScrollRange(hctl, SB_CTL, sb->Min, sb->Max, FALSE);
 SetScrollPos(hctl, SB_CTL, sb->Current, TRUE);  
 if (editem != IDC_STATIC)
  SetDlgItemInt(hdlg, editem, sb->Current, TRUE);
}

/*-----------------------------------------------------------------*/
/* if the current language is uk english, spell colour properly, else
   spell it incorrectly(it's spelt the other way by default in the .rc)
   Also spells optimise correctly
*/
   
static void UKSpelling(HWND hdlg, int item)

{                                                                               
 extern int gVideoCaptureSettings ;  /* declared in dvm_msg.c */
 char buf [40],
      deflt [4] ;

 lstrcpy(deflt, "enu"); 
 GetProfileString("Intl", "sLanguage", deflt, buf, sizeof(buf));
 
 if (lstrcmpi(buf, "eng") == 0 && gVideoCaptureSettings == VideoCaptureSettings)
  {
   SetDlgItemText(hdlg, IDC_COLOURTEXT, "Colou&r");
   SetDlgItemText(hdlg, IDC_VIDCONF, "Optimise for &Video Conference");
   SetDlgItemText(hdlg, IDC_VIDCAP, "Optimise for Video &Preview");
  }
}

/*-----------------------------------------------------------------*/

BOOL FAR PASCAL _loadds DlgAboutProc(HWND hDlg, UINT message, WPARAM wParam,
                                     LPARAM lParam)
                                     
{
	extern WORD gDebugLevel;
    static HBRUSH hctlcolour;
	static BOOL lang_changed, port_options_changed;
	int tmp_lang;
    BOOL res = FALSE ;
    char desc [255],
        buf [255] ;
    RECT rect ;
    HBRUSH hbr ;
    char language [80],
          verbuf [20];
    DWORD index;
    
    switch(message)                        
    {
    case WM_INITDIALOG :
            hctlcolour = CreateSolidBrush(GetSysColor(COLOR_BTNFACE));

            lang_changed = FALSE;
			port_options_changed = FALSE;

            SendDlgItemMessage( hDlg, IDC_PORT_DISABLE, CB_ADDSTRING, 0, (LPARAM)((LPSTR)"Automatic") );
            SendDlgItemMessage( hDlg, IDC_PORT_DISABLE, CB_ADDSTRING, 0, (LPARAM)((LPSTR)"Don't do Fast DMA") );
            SendDlgItemMessage( hDlg, IDC_PORT_DISABLE, CB_ADDSTRING, 0, (LPARAM)((LPSTR)"Don't do DMA") );
            SendDlgItemMessage( hDlg, IDC_PORT_DISABLE, CB_ADDSTRING, 0, (LPARAM)((LPSTR)"Don't do ECP") );
            SendDlgItemMessage( hDlg, IDC_USE_IRQ, BM_SETSTATE, TRUE, 0 );

            get_version( verbuf, sizeof(verbuf));
            SetDlgItemText( hDlg, IDC_VERSION, verbuf );  /* DRV_VERSION */
 

            SetDlgItemText( hDlg, IDC_DATEBUILD, __DATE__ ); /* build date from __DATE__ */


		    SendDlgItemMessage( hDlg, IDC_LANGUAGE, CB_ADDSTRING, 0, (LPARAM)((LPSTR)"English") );
            SendDlgItemMessage( hDlg, IDC_LANGUAGE, CB_ADDSTRING, 0, (LPARAM)((LPSTR)"French") );
            SendDlgItemMessage( hDlg, IDC_LANGUAGE, CB_ADDSTRING, 0, (LPARAM)((LPSTR)"Italian") );
            SendDlgItemMessage( hDlg, IDC_LANGUAGE, CB_ADDSTRING, 0, (LPARAM)((LPSTR)"German") );
            SendDlgItemMessage( hDlg, IDC_LANGUAGE, CB_ADDSTRING, 0, (LPARAM)((LPSTR)"Spanish") );
            SendDlgItemMessage( hDlg, IDC_LANGUAGE, CB_ADDSTRING, 0, (LPARAM)((LPSTR)"Portuguese") );
            
			tmp_lang = ReadLanguageOption( DEFAULT_INI_FILE );
			SendDlgItemMessage( hDlg, IDC_LANGUAGE, CB_SETCURSEL, (DWORD)tmp_lang, 0 );

			if (IsDeviceInitialised())
            {
            /* only need to display the "wait" message if the driver isn't
                already running */
                get_camera_info(hDlg);
                show_controls(hDlg, TRUE);
                SetFocus(GetDlgItem(hDlg, IDOK));
            }                            
            else
            {
                show_controls(hDlg, FALSE);
                PostMessage(hDlg, WM_USER + 42, 0, 0L);
            }
 			
			break ;
        
    case WM_USER + 42 :
            get_camera_info(hDlg);
            show_controls(hDlg, TRUE);
            SetFocus(GetDlgItem(hDlg, IDOK));
            res = TRUE ;
            break ;
        
    case WM_COMMAND :
            switch(wParam)
            {
            case IDC_LANGUAGE:
                {   
                    if ( HIWORD((DWORD)lParam) == CBN_SELCHANGE ) 
                    {
                        index = SendDlgItemMessage( hDlg, IDC_LANGUAGE, CB_GETCURSEL, 0, 0 );
						SetDialogLanguage( (int)index );
						lang_changed = TRUE;
					}
                }
                res = TRUE ;
                break;
            case IDC_PORT_DISABLE:
                {   
                    if ( HIWORD((DWORD)lParam) == CBN_SELCHANGE ) 
                    {
                        index = SendDlgItemMessage( hDlg, IDC_PORT_DISABLE, CB_GETCURSEL, 0, 0 );
						gDebugLevel = gDebugLevel & ~(DBG_DISABLE_ECP|DBG_DISABLE_DMA|DBG_DISABLE_FDMA);
						switch( (int)index ) 
						{
						case 0:
							break;
						case 1:
							gDebugLevel |= DBG_DISABLE_FDMA;
							break;
						case 2:
							gDebugLevel |= DBG_DISABLE_DMA;
							break;
						case 3:
							gDebugLevel |= DBG_DISABLE_ECP;
							break;
						}
						port_options_changed = TRUE;
					}
                }
                res = TRUE ;
                break;

            case IDC_USE_IRQ:
				index = IsDlgButtonChecked( hDlg, IDC_USE_IRQ );
				gDebugLevel &= ~DBG_DISABLE_STREAM_INT;
				if ( index == 0 ) gDebugLevel |= DBG_DISABLE_STREAM_INT;
				port_options_changed = TRUE;
                res = TRUE ;
                break;

            case IDOK:
                if ( lang_changed ) 
				{
					index = SendDlgItemMessage( hDlg, IDC_LANGUAGE, CB_GETCURSEL, 0, 0 );
 	 				WriteLanguageOption( DEFAULT_INI_FILE, (int)index );
				}
				if ( port_options_changed )
				{
					index = SendDlgItemMessage( hDlg, IDC_PORT_DISABLE, CB_GETCURSEL, 0, 0 );
 					WritePortOptions( DEFAULT_INI_FILE, gDebugLevel );
				}

				/* just go on and do the IDCANCEL clean up code below	*/				

            case IDCANCEL:
                DeleteObject(hctlcolour);
                EndDialog(hDlg, TRUE);
                res = TRUE ;
                break ;
            }
            break ;
        
    case WM_ERASEBKGND:
            GetClientRect(hDlg, &rect);  
            hbr = CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
            FillRect((HDC)wParam, &rect, hbr);
            DeleteObject(hbr);
            res = TRUE ;
            break;                    
            
    case WM_CTLCOLOR:
            SetBkColor((HDC) wParam, GetSysColor(COLOR_BTNFACE));
            return(int) hctlcolour ;
    }
    
    return res ;
}

/*-----------------------------------------------------------------*/
/* show or hide the controls in the dialog box. Either all controls but the
"please wait" string are visible, or only it is visible */

show_controls(HWND hwnd, BOOL show)

{                                       
    static int ctrl [] = { 
        IDC_CAMPORT,
        IDC_STR_CAMPORT,
        IDC_CAMERATYPE,
        IDC_STR_CAMERATYPE,
        IDC_FWARE,
        IDC_STR_FWARE,
        IDC_USE_IRQ,
		IDC_PORT_DISABLE };
    int i ;
    
    for(i = 0; i < sizeof(ctrl) / sizeof(int); i++)
        ShowWindow(GetDlgItem(hwnd, ctrl [i]), show ? SW_SHOW : SW_HIDE);
    
    ShowWindow(GetDlgItem(hwnd, IDC_RETRIEVE), show ? SW_HIDE : SW_SHOW);
     
    return TRUE ;
}

/*-----------------------------------------------------------------*/
/* retrieve the information from the various places and write to the
   dialog controls */
   
get_camera_info(HWND hwnd)

{
 extern char gInitFile [] ;
 extern WORD gDebugLevel;
 extern int gPortNumb ;
 int port, ptype, port_disable;
 char buf [60] ;
 HPPC tmpPPC ;
 WORD tsensor ;                   
 PPC_INIT_ERR errcode ;
              
 /* retrieving the port and camera type will be the time consuming part */
 
 if (IsDeviceInitialised())
 { 
	 /* driver already loaded. Just use the details it contains */
	 show_details(hwnd, GetCurrentPort());
 }
 else
 {
	 /* driver not running. Look for the camera */
	 port = GetPrivateProfileInt("VVLCPIA", INI_LPT_PORT, 0, DEFAULT_INI_FILE);
	 
	 /* load the DebugLevel ini setting. This section is executed when the
	 driver hasn't been loaded. As VideoOpen has not been called, debuglevel
	 won't have been set. 
	 NOTE - this uses the default ini file, not any command line over-ride
	 */
	 gDebugLevel = GetPrivateProfileInt( "VVLCPIA", "DebugLevel", 0, 
												DEFAULT_INI_FILE ); 
	 
	 /* read the port options from the INI file */
	 ReadPortOptions( DEFAULT_INI_FILE, &gDebugLevel );
 
	 /* figure out the type of camera */
	 if ((tmpPPC = InitPPCDevice(DEFAULT_INI_FILE, &port, &errcode)) != NULL)
	 {
		 if (SelectPPCDevice(tmpPPC))   /* make current */
		 {           
			 show_details(hwnd, port);
			 FiniPPCDevice(tmpPPC, DEFAULT_INI_FILE);
		 }
		 else
			 SetDlgItemText(hwnd, IDC_CAMERATYPE, "Unable to find camera");
	 }
	 else
		 SetDlgItemText(hwnd, IDC_CAMERATYPE, "Error initialising camera");
 }
 
                         
 return TRUE ;
}

/*-----------------------------------------------------------------*/
/* show the port number, type and sensor details */

static void show_details( HWND hwnd, int port )
{
	extern WORD gDebugLevel ;
	int ptype, port_disable;
	char buf [60];
	WORD firmware; 
	WORD tmp_level;
	WORD SensorID;
	BYTE SensorRevision;
	
	wsprintf( buf, "LPT%d", port );
	SetDlgItemText(hwnd, IDC_CAMPORT, buf); 
	
	ptype = GetPortMode();
	wsprintf(buf, "%s%s", ptype == FAST_DMA_PORT_TYPE ? (LPSTR)"Fast DMA" : 
				ptype == DMA_PORT_TYPE ? (LPSTR)"ECP DMA" : 
				ptype == ECP_PORT_TYPE ? (LPSTR)"ECP" : 
				ptype == COMPAT_PORT_TYPE ? (LPSTR)"4 bit" :
				(LPSTR)"Unknown",
					( (gDebugLevel & DBG_DISABLE_ECP)
					||(gDebugLevel & DBG_DISABLE_DMA)
					||(gDebugLevel & DBG_DISABLE_FDMA) ) ? (LPSTR)" *" : (LPSTR)"");
	SetDlgItemText(hwnd, IDC_TRANSFERMODE, buf);
	
	DisableBackgroundUpload( TRUE );
	ppcGetSensorID( &SensorID, &SensorRevision );
	DisableBackgroundUpload( FALSE );
	if ( SensorID == 404 ) 
	{
		if ( SensorRevision < 4 )
			SetDlgItemText(hwnd, IDC_CAMERATYPE, "404");
		else if ( SensorRevision < 8 )
			SetDlgItemText(hwnd, IDC_CAMERATYPE, "407");
		else if ( SensorRevision < 16 )
			SetDlgItemText(hwnd, IDC_CAMERATYPE, "409");
	}
	else
	{
		SetDlgItemText(hwnd, IDC_CAMERATYPE, "Unknown");
	}


	/* show the firmware version */
	firmware = ppcGetFwVersion();
	wsprintf(buf, "%d-%02d", FW_GETVERSION(firmware), FW_GETREVISION(firmware));
	SetDlgItemText(hwnd, IDC_FWARE, buf);
	
	ReadPortOptions( DEFAULT_INI_FILE, &tmp_level );
	port_disable = 0;
	if ( tmp_level & DBG_DISABLE_ECP )
		port_disable = 3;
	else if ( tmp_level & DBG_DISABLE_DMA )
		port_disable = 2;
	else if ( tmp_level & DBG_DISABLE_FDMA )
		port_disable = 1;
	SendDlgItemMessage( hwnd, IDC_PORT_DISABLE, CB_SETCURSEL, (DWORD)port_disable, 0 );

	if ( tmp_level & DBG_DISABLE_STREAM_INT )
		CheckDlgButton( hwnd, IDC_USE_IRQ, FALSE );
	else
		CheckDlgButton( hwnd, IDC_USE_IRQ, TRUE );

	return;
}


/*-----------------------------------------------------------------*/
/* Not all colours support all resolutions. If a colour depth is chosen, 
   un-supported sizes are greyed, and the highest supported colour selected
*/
   
CheckFormat(HWND hdlg, int button)

{
 switch(button)
  {
   case IDC_FORM24 :
        /* enable all sizes, only for 24 bit RGB mode and don't change any settings */
        EnableWindowEx(hdlg, 8, TRUE, 
                        IDC_SIZE352, IDC_SIZE320, IDC_SIZE640, IDC_SIZE704, 
                        IDC_LABEL_CIF, IDC_LABEL_SIF, IDC_LABEL_VGA, 
                        IDC_LABEL_4CIF);
/*      
        EnableWindow(GetDlgItem(hdlg, IDC_SIZE352), TRUE);
        EnableWindow(GetDlgItem(hdlg, IDC_SIZE320), TRUE);
        EnableWindow(GetDlgItem(hdlg, IDC_SIZE704), TRUE);
        EnableWindow(GetDlgItem(hdlg, IDC_SIZE640), TRUE);
        EnableWindow(GetDlgItem(hdlg, IDC_LABEL_CIF), TRUE);
        EnableWindow(GetDlgItem(hdlg, IDC_LABEL_SIF), TRUE);
        EnableWindow(GetDlgItem(hdlg, IDC_LABEL_VGA), TRUE);
        EnableWindow(GetDlgItem(hdlg, IDC_LABEL_4CIF), TRUE);
*/        
        break ;                                              
           
   default:   
        EnableWindowEx(hdlg, 4, TRUE, 
                        IDC_SIZE352, IDC_SIZE320, 
                        IDC_LABEL_CIF, IDC_LABEL_SIF);
/*      
        EnableWindow(GetDlgItem(hdlg, IDC_SIZE320), TRUE);
        EnableWindow(GetDlgItem(hdlg, IDC_SIZE352), TRUE);
        EnableWindow(GetDlgItem(hdlg, IDC_LABEL_CIF), TRUE);
        EnableWindow(GetDlgItem(hdlg, IDC_LABEL_SIF), TRUE);
*/        
        if (IsDlgButtonChecked(hdlg, IDC_SIZE640) ||
            IsDlgButtonChecked(hdlg, IDC_SIZE704))
         {           
          /* move the radio button to CIF control */                            
          CheckRadioButton(hdlg, IDC_SIZE352, IDC_SIZE640, IDC_SIZE352);
          SendMessage(hdlg, WM_COMMAND, IDC_SIZE352, 0);
         }
        /* disable the still image sizes */
        EnableWindowEx(hdlg, 4, FALSE, 
                        IDC_SIZE640, IDC_SIZE704, 
                        IDC_LABEL_VGA, IDC_LABEL_4CIF);
/*        EnableWindow(GetDlgItem(hdlg, IDC_SIZE640), FALSE);
        EnableWindow(GetDlgItem(hdlg, IDC_SIZE704), FALSE);
        EnableWindow(GetDlgItem(hdlg, IDC_LABEL_VGA), FALSE);
        EnableWindow(GetDlgItem(hdlg, IDC_LABEL_4CIF), FALSE);
*/        
        break ;

  }                                               
 
 return TRUE ;
}

/*-----------------------------------------------------------------*/

static void EnableWindowEx(HWND hwnd, int numb, BOOL state, ...)

{
 int i,
     child_id ;
 va_list argptr ;
 
 va_start(argptr, state);
 
 for(i = 0; i < numb; i++)
  {
   child_id = va_arg(argptr, int);
   EnableWindow(GetDlgItem(hwnd, child_id), state);
  }        

 va_end(argptr);
}

/*-----------------------------------------------------------------*/
/* an invalid image size has been suggested. Calculate the nearest valid
   image size and write it to the x and y controls */
   
static suggest_image_size(HWND hwnd, LPBITMAPINFOHEADER bih, int xcon, 
                           int ycon)

{
 int wid,
     ht ;
 
 /* get what the correct size should be */                
 check_image_size(bih, &wid, &ht);
                                   
 /* if the width and height are the same as they were at the start, something
    has gone wrong with SetPPCImageType. This could be due to an rgb bitdepth not being
    supported.
 */                                  
 if (wid == bih->biWidth && ht == bih->biHeight)
  {
   MessageBox(hwnd, "Unsupported image type requested.",
               "vvlcpia", MB_OK);
  }
 else
  {   
   MessageBox(hwnd, 
             "Invalid image size\nSize will be amended to nearest valid size",
             "vvlcpia", 
             MB_OK | MB_ICONEXCLAMATION);

   SetXYSize(hwnd, wid, ht);
   bih->biWidth = wid ;
   bih->biHeight = ht ;
  }
  
 return TRUE ;
}



