/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/****************************************************************************

  VSTREAM.C
  
	
**************************************************************************/


#include "common.h"

#include "vvlcap.h"

#include "ppc.h"
#include "dvm_msg.h"
#include "vstream.h"


#define DEFAULT_TIME_RES    2
#define QUICK_CALLBACK_TIME 4

#define GetTime timeGetTime

extern DWORD gStreamStartTime;

extern PPPC gpPPC;

#ifndef DriverCallback
BOOL WINAPI DriverCallback(DWORD dwCallback, UINT uFlags,
						   HANDLE hDevice, UINT uMessage, DWORD dwUser, DWORD dwParam1, DWORD dwParam2);
#endif


//
//  we use the reserved fields of a VIDEOHDR for linked list pointers                                                                                                                                                                     l
//
#define NEXT(p)     (LPVIDEOHDR)(p->dwReserved[0])





/* Local to this module */
BOOL        fVideoInited = FALSE;       /* also used in DVM_MSG.C */
BOOL        fVideoStarted = FALSE;      
LPVIDEOHDR  lpVHdrFirst;
LPVIDEOHDR  lpVHdrLast;
WORD        wSensorPeriod;
DWORD       dwTimeStart;
DWORD       dwTimeStream;
DWORD       dwNextFrameNum;
DWORD       dwLastVideoClock;
DWORD       dwFramesSkipped = 0;
WORD        wTimerID;
WORD        wTimerPeriod;
WORD        wTimerRes;
WORD        wOldFrameRate ;

VIDEO_STREAM_INIT_PARMS  CaptureStreamParms;



/*
*
* Initalize video driver for input.  
* This function implements the DVM_STREAM_INIT message.
*
*/
DWORD InStreamInit( LPVIDEO_STREAM_INIT_PARMS lpStreamInitParms )
{
    WORD requested_period, periods;
    TIMECAPS TimeCaps;
	FRAMERATE fps;		/* not actually used */
	
    if ( fVideoInited )
        return DV_ERR_NONSPECIFIC;
    
    /* store old frame rate so that it can be reset by InStreamFini() 	*/
    wOldFrameRate = ppcGetAppFramerate();          
	
	/* find out what rate the sensor is currently running at */
	ppcGetSensorFramerate( &fps, NULL, &wSensorPeriod );
	
	/* convert requested period into mS and check it is not less than 	*/
	/* the actual sensor period 										*/
	requested_period = (WORD)(lpStreamInitParms->dwMicroSecPerFrame / 1000UL);
	
	// 	if ( requested_period < 66 ) requested_period = 66;
	
	if ( requested_period > wSensorPeriod ) 
		periods = requested_period / wSensorPeriod;
	else
		periods = 1;	 
	
	wTimerPeriod = (WORD)wSensorPeriod * periods;
	
	/* tell ppc module how many fps the app has requested				*/
    /* - used to control the dynamic compression algorithm				*/ 
    ppcSetAppFramerate( 1000 / wTimerPeriod );
	
	/* setup the global variables used during streaming */    
    CaptureStreamParms = *lpStreamInitParms;
    
    lpVHdrFirst = NULL;
    lpVHdrLast = NULL;
	
    dwNextFrameNum = 0L;
    dwTimeStream = 0L;
    dwFramesSkipped = 0L;
	
	
    fVideoStarted = FALSE;
	
    fVideoInited = TRUE;
	
    /* setup stuff for mmedia timer callback functions */
	
    wTimerID = 0;
    wTimerRes = DEFAULT_TIME_RES;
	
    timeGetDevCaps( &TimeCaps, sizeof(TIMECAPS) );
    
    if ( TimeCaps.wPeriodMin > wTimerRes ) wTimerRes = TimeCaps.wPeriodMin;
    if ( TimeCaps.wPeriodMax < wTimerPeriod ) wTimerPeriod = TimeCaps.wPeriodMax;
	
    timeBeginPeriod( wTimerRes );
	
    if (TMSET ())
		tracemsg (TRC_MSG, "InStreamInit: Requested Period = %umS    Using = %umS",
		requested_period, wTimerPeriod );
	
	
    /* switch camera into stream capture mode */                                
    if ( !GF_InitFrameStream( gpPPC->pGFrame, (periods - 1), 0 ) )
	{
		if (TMSET ())
			tracemsg (TRC_MSG, "Error returned from GF_InitFrameStream()" );
		return DV_ERR_NONSPECIFIC ;
	}
	
	/* notify app we're open via callback								*/
	videoCallback( MM_DRVM_OPEN, 0L );  
	
    return DV_ERR_OK;
}

/*
*
* Fini video driver input
* This function implements the DVM_STREAM_FINI message.
*
*/
DWORD InStreamFini( void )
{
    if( !fVideoInited )
        return DV_ERR_NONSPECIFIC;
	
    if ( lpVHdrFirst )
        return DV_ERR_STILLPLAYING;
	
    fVideoInited = FALSE;
	
    timeEndPeriod( wTimerRes );
	
    lpVHdrFirst = NULL;
    lpVHdrLast = NULL; 
	
    videoCallback( MM_DRVM_CLOSE, 0L ); // Notify app we're closed via callback
	
    if ( !GF_FiniFrameStream( gpPPC->pGFrame ) )
	{
		if (TMSET ())
			tracemsg (TRC_MSG, "Error returned from GF_FiniFrameStream()" );
		return DV_ERR_NONSPECIFIC ;
	}
	
    /* restore camera's frame rate to what it was before the streaming */                                     
    ppcSetAppFramerate (wOldFrameRate) ;
	
    return DV_ERR_OK;
}



/*
*
* Add a buffer to the input queue
* This function implements the DVM_STREAM_ADDBUFFER message.
*
*/
DWORD InStreamAddBuffer( LPVIDEOHDR lpVHdr )
{
    DWORD tmp1, tmp2;
	
    if ( !fVideoInited )
        return DV_ERR_NONSPECIFIC;
	
    /* return error if no node passed */
    if ( !lpVHdr )
        return DV_ERR_NONSPECIFIC;
	
    /* return error if buffer has not been prepared */
    if ( !(lpVHdr->dwFlags & VHDR_PREPARED) )
        return DV_ERR_UNPREPARED;
	
    /* return error if buffer is already in the queue */
    if ( lpVHdr->dwFlags & VHDR_INQUEUE )
        return DV_ERR_NONSPECIFIC;
	
	//    if ( lpVHdr->dwBufferLength < biDest.biSizeImage )
	//        return DV_ERR_NONSPECIFIC;
	
    /* set back to basic flag */
    lpVHdr->dwFlags &= ~VHDR_DONE;
    lpVHdr->dwBytesUsed = 0;
	
    QueHeader( lpVHdr );
	
    return DV_ERR_OK;
}

/*
*
* Reset the buffers so that they may be unprepared and freed
* This function implements the DVM_STREAM_RESET message.
*
*/
DWORD InStreamReset( void )
{
    LPVIDEOHDR  lpVHdr;
	
    if( !fVideoInited )
        return DV_ERR_NONSPECIFIC;
	
    if ( fVideoStarted ) 
    {
        InStreamStop();
    }
 
    while( lpVHdr = DeQueHeader() ) 
    {
	   lpVHdr->dwFlags |= VHDR_DONE;
       videoCallback( MM_DRVM_DATA, (DWORD)lpVHdr);
    }

    lpVHdrFirst = NULL;
    lpVHdrLast = NULL;
    
    return DV_ERR_OK;
}


/*
*
* Start the recording - all buffers should be prepared
* This function implements the DVM_STREAM_START message.
*
*/
DWORD InStreamStart( void )
{                                         
/*
    Ver 1-33
    18-09-98    REM     Modified to accept multiple StreamStart's 
                        - required to pass WHQL tests.
*/

    /* fail if streaming hasn't been Inited*/
    if( !fVideoInited )
       return DV_ERR_NONSPECIFIC;
 
    dwNextFrameNum = 0L;
    dwTimeStart = timeGetTime();
    dwLastVideoClock = 0;
    gStreamStartTime = GetTime();

    if ( fVideoStarted )
        return DV_ERR_OK;
 
    if ( !GF_StartFrameStream( gpPPC->pGFrame ) )
	{
		if (TMSET ())
			tracemsg (TRC_MSG, "Error returned from GF_StartFrameStream()" );
		return DV_ERR_NONSPECIFIC ;
	}
	
    if ( !StartCaptureISR(QUICK_CALLBACK_TIME ) ) {
        D( "Unable to timeSetEvent" );
        return( DV_ERR_NONSPECIFIC );
    }
 
    fVideoStarted = TRUE;
    
    return DV_ERR_OK;
}

/*
*
* Stop the recording - Finish last buffer if needed 
* This function implements the DVM_STREAM_STOP message.
*
*/
DWORD InStreamStop( void )
{                                          
/*
    Ver 1-33
    18-09-98    REM     Modified to accept multiple StreamStop's 
                        - required to pass WHQL tests.
*/

    /* fail if stream not inited */
    if ( !fVideoInited )
        return DV_ERR_NONSPECIFIC;

    if ( !fVideoStarted )
        return DV_ERR_OK;
	
    KillCaptureISR();
    
    if ( !GF_StopFrameStream( gpPPC->pGFrame ) )
	{
		if (TMSET ())
			tracemsg (TRC_MSG, "Error returned from GF_StopFrameStream()" );
		return DV_ERR_NONSPECIFIC ;
	}
	
    fVideoStarted = FALSE;
	
    return DV_ERR_OK;
}


/* 
* Return the current stream time from the start of capture based
*      on the number of vsync interrupts.
* This function implements the DVM_STREAM_GETPOSITION message.
*/
DWORD InStreamGetPos( LPMMTIME lpMMTime, DWORD dwSize)
{
    if ( dwSize != sizeof(MMTIME) ) 
        return DV_ERR_NOTSUPPORTED;
    lpMMTime->wType = TIME_MS;
    lpMMTime->u.ms = dwTimeStream;
	
    return( DV_ERR_OK );
}


/* 
* Return the number of frames skipped to date.  This count will be
*      inaccurate if the frame rate requested is over 15fps.
* This function implements the DVM_STREAM_GETERROR message.
*/
DWORD InStreamError( LPDWORD lpdwErrorType, LPDWORD lpdwFramesSkipped )
{
/*        DWORD cap_rtn, rtn;
LPVIDEOHDR lpVHdr;
	*/
    if ( lpdwErrorType )
		*lpdwErrorType =  DV_ERR_OK;
	
    if ( lpdwFramesSkipped )
		*lpdwFramesSkipped = dwFramesSkipped;
	
    dwFramesSkipped = 0;
	
    return DV_ERR_OK ;
}



/****************************************************************************
*  videoCallback()  This calls DriverCallback for the input stream
*
*  msg      The message to send.
*
*  dw1     Message-dependent parameter.
*
* There is no return value.
***************************************************************************/

void  videoCallback( WORD msg, DWORD dw1 )
{
	// invoke the callback function, if it exists.  dwFlags contains driver-
	// specific flags in the LOWORD and generic driver flags in the HIWORD
	
	if (CaptureStreamParms.dwCallback )
		DriverCallback (CaptureStreamParms.dwCallback,      // client's callback DWORD
		HIWORD(CaptureStreamParms.dwFlags), // callback flags
		(HANDLE) CaptureStreamParms.hVideo, // handle to the device
		msg,                                // the message
		CaptureStreamParms.dwCallbackInst,  // client's instance data
		dw1,                                // first DWORD
		0) ;                                // second DWORD not used
}


/* adds a header to the end of the queue */
void QueHeader( LPVIDEOHDR lpVHdr )
{
    lpVHdr->dwFlags &= ~VHDR_DONE;
    lpVHdr->dwFlags |= VHDR_INQUEUE;    /* ie in queue for driver to fill */
	
    NEXT(lpVHdr) = NULL;
	
	_asm cli
		
		if ( lpVHdrLast )
			NEXT(lpVHdrLast) = lpVHdr;
		else
			lpVHdrFirst = lpVHdr;
		
		lpVHdrLast = lpVHdr;
		
		_asm sti
}

/* takes a header from start of the queue */
LPVIDEOHDR DeQueHeader()
{
    LPVIDEOHDR lpVHdr;
	
    if ( lpVHdr = lpVHdrFirst ) {
        lpVHdr->dwFlags &= ~VHDR_INQUEUE;
		
        _asm cli
			
			lpVHdrFirst = NEXT(lpVHdr);
		
        if ( lpVHdrFirst == NULL )
            lpVHdrLast = NULL;
		
		_asm sti
    }
	
    return lpVHdr;
}


_inline BOOL StartCaptureISR( WORD Delay ) 
{
    if ( wTimerID != 0 ) 
		return( FALSE );
	
    wTimerID = timeSetEvent( Delay, wTimerRes, AttemptCaptureISR, 0UL, TIME_ONESHOT );
    
    return wTimerID != 0 ;
}


_inline void KillCaptureISR()
{
	if ( wTimerID != 0 )
	{
		/* make sure we do this only once */
		timeKillEvent( wTimerID );
		wTimerID = 0;
	}
}


void FAR PASCAL _export AttemptCaptureISR( UINT TimerID, UINT Msg, DWORD dwUser, DWORD dw1, DWORD dw2 )
{      
	DWORD cap_rtn,
		start_upload, 
		end_upload, 
		upload_total;
	int next_period = wTimerPeriod ;
	LPVIDEOHDR lpVHdr;
	
	KillCaptureISR();
	
	lpVHdr = lpVHdrFirst;     /* next app supplied buffer to fill */
	
	if ( lpVHdr == NULL )
	{       // No buffers available
		dwFramesSkipped++;
		videoCallback( MM_DRVM_ERROR, dwFramesSkipped ); 
		next_period = QUICK_CALLBACK_TIME;
		goto ExitISR;
	}
	
	start_upload = GetTime();
	cap_rtn = CaptureFrame( lpVHdr, TRUE );
	end_upload = GetTime(); 
	
	switch( cap_rtn )
	{
	case DV_ERR_OK:
        if (lpVHdr -> dwBufferLength != 0)
		{
			DeQueHeader();   /* capture was done so remove from queue */
			videoCallback( MM_DRVM_DATA, (DWORD)lpVHdr ); // Notify app data is available
		}
        break;
		
	case DV_ERR_SYNC:
        /* kick off a short wait */
        next_period = QUICK_CALLBACK_TIME;
        // tracemsg (TRC_MSG, "dv_err_sync");
        goto ExitISR;   
		
	default:
        break;
	}
	
	if ( end_upload < start_upload )
	{
		/* timer has overflowed. do something safe    */
		next_period = wTimerPeriod;
		// tracemsg(TRC_MSG, "timer overflow");
		goto ExitISR;   
	}
	
	upload_total = end_upload - start_upload;
    
	// tracemsg(TRC_MSG, "upload time = %d", upload_total);
	
	if ( upload_total < wTimerPeriod )
    {
		next_period = max (QUICK_CALLBACK_TIME, 
			(int) wTimerPeriod - (int) upload_total) ;
		// tracemsg(TRC_MSG, "(%3ldms) upload < wtimerp, callback = %d", upload_total, next_period);
    }
   	else
    { 
	/* upload total > wTimerPeriod. If it's less than twice the sensor
	speed, try to resync, else schedule for half the requested period.
        This isn't great, but does give back the cpu */
		if ( upload_total < wSensorPeriod * 2 ) 
		{                          
			next_period = max (QUICK_CALLBACK_TIME, 
				(wSensorPeriod * 2) - upload_total);
			// tracemsg(TRC_MSG, "(%3ldms) upload < wsensorp * 2, callback = %d", upload_total, next_period);
		}
		else
		{                                
			next_period = max (wSensorPeriod, wSensorPeriod / 2) ;
			// tracemsg(TRC_MSG, "(%3ldms) upload >= wtimerp * 2, callback = %d", upload_total, next_period);
		}
   	}
	
ExitISR:
    if ( next_period < wTimerRes ) next_period = wTimerRes;
	
    StartCaptureISR( next_period );
}

/*
if (upload_total > wTimerPeriod - QUICK_CALLBACK_TIME)
{
int tmp = wTimerPeriod ;

  tmp = tmp - (upload_total % tmp) ; 
  next_period = max (QUICK_CALLBACK_TIME, tmp) ;
  tracemsg (TRC_MSG, "up (%lu) > tp : np = %u", upload_total, next_period) ;
  }
  else
  {                                              
  next_period = wTimerPeriod - (WORD) upload_total ;
  tracemsg (TRC_MSG, "up (%lu) <= tp : np = %u", upload_total, next_period) ;
  }           
*/
