/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */

#include "common.h"

#include <ver.h>
#include <stdarg.h>

#include "resource.h"
#include "vfwdlg.h" 

#include "vvlcap.h"
#include "ppc.h"
#include "oem.h"

#include "vvl_msg.h"     
#include "dvm_msg.h"     
#include "drv_ver.h"
                         
extern HINSTANCE ghModule ;   /* dll instance handle. Declared in dvm_msg.c */

extern WORD gDebugLevel;
extern PPPC gpPPC;


BOOL VVLMessage( T_VVL_MSG Msg, BOOL ReadFlag, T_VVL_DATA *Data ) {
	BOOL ret;

	ret = FALSE;

	switch( Msg ) 
	{
	case VVL_FW_VERSION :
		if ( ReadFlag ) 
		{
			WORD firmware;
			firmware = ppcGetFwVersion();
			Data->FwVersion.Version = FW_GETVERSION(firmware);
			Data->FwVersion.Revision = FW_GETREVISION(firmware);
			ret = TRUE;
		}
		else
		{
			/* don't accept write commands */
		}
		break;

	case VVL_DRV_VERSION :
		if ( ReadFlag )
		{
			char verbuf[20];
			BYTE index = 0;
			BYTE drv_version = 0;
			BYTE drv_revision = 0;
			ret = get_version(verbuf, sizeof(verbuf));
			while ( verbuf[index] != '-' && index < 18)
			{
				drv_version = drv_version * 10 + verbuf[index] - (BYTE)'0';
				index++;
			}
			index++;
			while ( verbuf[index] != '-' && index < 19 )
			{
				drv_revision = drv_revision * 10 + verbuf[index] - (BYTE)'0';
				index++;
			}
			Data->DrvVerParams.DrvVersion = drv_version;
			Data->DrvVerParams.DrvRevision = drv_revision;
		}
		else
		{
			// don't accept write commands
		}
		break;

	case VVL_PARALLEL_PORT_TYPE:
		if ( ReadFlag ) 
		{
			Data->ParallelPortType.PortNumber = (BYTE)gpPPC->PortNumber;
			Data->ParallelPortType.PortType = (BYTE)gpPPC->PortType;
			Data->ParallelPortType.DebugLevel = gDebugLevel;
			ret = TRUE;
		}
		else 
		{
			/* don't accept write commands */
		}
		break;

	case VVL_BACKGROUND_UPLOAD :
		if ( ReadFlag ) 
		{
			/* only accept write commands */
		}
		else
		{
			DisableBackgroundUpload( !Data->BackgroundUpload.Enable );
			ret = TRUE;
		}
		break;

	case VVL_COLOUR :
		if ( ReadFlag ) 
		{
			int a, b, c;
			ppcGetColourSettings( &a, &b, &c );
			Data->Colour.Brightness = a;
			Data->Colour.Contrast = b;
			Data->Colour.Colour = c;
			ret = TRUE;
		}
		else
		{
			ppcSetColourSettings( (int)Data->Colour.Brightness,
							   (int)Data->Colour.Contrast,
							   (int)Data->Colour.Colour		);
			ret = TRUE;

		}

		break;

	case VVL_COMPRESSION :
		if ( ReadFlag ) 
		{
			COMP_MODE m;
			BOOL vc;
			WORD a, b, c;
		
			GetPPCOptions( &m, &vc, &a, &b, &c );

			Data->Compression.Mode = m;
			Data->Compression.EnableVidConf = vc;
			Data->Compression.VidConfVal = (BYTE)a;
			Data->Compression.VidCapVal = (BYTE)b;
			Data->Compression.ManualVal = (BYTE)c;
		
			ret = TRUE;
		}
		else
		{
			ret = SetPPCOptions( (COMP_MODE)Data->Compression.Mode, 
									  (BOOL)Data->Compression.EnableVidConf,
									  (WORD)Data->Compression.VidConfVal,
									  (WORD)Data->Compression.VidCapVal,
									  (WORD)Data->Compression.ManualVal );
		}

		break;

	case VVL_FRAMERATE:
		if ( ReadFlag ) 
		{
			FRAMERATE f;
			BOOL a;
			ret = ppcGetSensorFramerate( &f, &a, NULL );
			Data->Framerate.Framerate = (BYTE)f;
			Data->Framerate.AutoFramerate = (BYTE)a;
		}
		else
		{
			ret = ppcSetSensorFramerate( (FRAMERATE)Data->Framerate.Framerate,
										      (BOOL)Data->Framerate.AutoFramerate );
		}
		break;

	case VVL_AEC_PARAMS:
		if ( ReadFlag ) 
		{
			BOOL cw, bf;
			MAINS_FREQ mf;
			EXPOSURE aec;
			cw = ppcGetCentreWeight();
			bf = ppcGetFlicker();
			aec = GetCameraAEC() ? EXP_AUTOMATIC : EXP_AUTODISABLE;
			ret = ppcGetFlickerParams( &mf );
			Data->AECParams.ExpMode = (BYTE)aec;
			Data->AECParams.CentreWeight = (BYTE)cw;
			Data->AECParams.BandingFilter = (BYTE)bf;
			Data->AECParams.MainsFreq = (BYTE)mf;
		}
		else
		{
			BOOL cw, bf;
			MAINS_FREQ mf;
			EXPOSURE aec;
			cw = ppcGetCentreWeight();
			bf = ppcGetFlicker();
			aec = GetCameraAEC() ? EXP_AUTOMATIC : EXP_AUTODISABLE;
			ret = ppcGetFlickerParams( &mf );

			if ( cw != Data->AECParams.CentreWeight )
			{
				ret &= ppcSetCentreWeight( Data->AECParams.CentreWeight );
			}
			if ( bf != Data->AECParams.BandingFilter )
			{
				ret &= ppcSetFlicker( Data->AECParams.BandingFilter );
			}
			if ( mf != Data->AECParams.MainsFreq )
			{
				ret &= ppcSetFlickerParams( Data->AECParams.MainsFreq );
				ret &= ppcSetFlicker( Data->AECParams.BandingFilter );
			}
			if ( aec != Data->AECParams.ExpMode )
			{
				ret &= SetCameraAEC( Data->AECParams.ExpMode == EXP_AUTOMATIC );
			}
		}
		break;

	case VVL_EXPOSURE:
		if ( ReadFlag ) 
		{
			WORD Cs;
			BYTE Fn, Gn;

			ret = GetCameraExposure( &Cs, &Fn, &Gn );
			Data->Exposure.Coarse = Cs;
			Data->Exposure.Fine = (BYTE)Fn;
			Data->Exposure.Gain = Gn;
			
		}
		else
		{
			ret = SetCameraExposure(
				Data->Exposure.Coarse,
				Data->Exposure.Fine,
				Data->Exposure.Gain
				);
		}

		break;

	case VVL_ACB_PARAMS:
		if ( ReadFlag ) 
		{
			ret = GetCameraACB(
				&Data->ACBParams.CbMode,
				&Data->ACBParams.RedGain,
				&Data->ACBParams.GreenGain,
				&Data->ACBParams.BlueGain
				);

		}
		else
		{
			ret = SetCameraACB(
				Data->ACBParams.CbMode,
				Data->ACBParams.RedGain,
				Data->ACBParams.GreenGain,
				Data->ACBParams.BlueGain
				);
		}
		break;

	case VVL_PAN_TILT:
		if ( ReadFlag ) 
		{	
			RECT roi;
			BOOL zoom_enabled;
			BOOL zoom_possible;
			
			zoom_possible = ppcGetPanTiltInfo( (LPRECT)&roi, &zoom_enabled );

			Data->PanTilt.ZoomPossible = zoom_possible;
			Data->PanTilt.ZoomEnable = zoom_enabled;

			Data->PanTilt.Top = roi.top/4;
			Data->PanTilt.Bottom = roi.bottom/4;
			Data->PanTilt.Left = roi.left/4;
			Data->PanTilt.Right = roi.right/4;

			ret = TRUE;
		}
		else
		{
			RECT roi;
			BOOL zoom_enable;

			zoom_enable = Data->PanTilt.ZoomEnable;
			roi.top		= Data->PanTilt.Top * 4;
			roi.bottom	= Data->PanTilt.Bottom * 4;
			roi.left	= Data->PanTilt.Left * 4;
			roi.right	= Data->PanTilt.Right * 4;
			
			ret = ppcSetPanTiltInfo( zoom_enable, (LPRECT)&roi );
		}
		break;

    case VVL_PUSH_BUTTON:
		if ( ReadFlag ) 
        {
            BOOL check, pushed;
            ret = camPushButtonGet( gpPPC->pCamera, &check, &pushed );
            Data->PushButton.Check = (BYTE)check;
	        Data->PushButton.Pushed = (BYTE)pushed;
        }
		else
		{
            ret = camPushButtonSet( gpPPC->pCamera, Data->PushButton.Check, Data->PushButton.Pushed );
        }        
        break;

	case VVL_LANGUAGE:
		if ( ReadFlag )
		{
			DWORD language;
			language = (DWORD)ReadLanguageOption( DEFAULT_INI_FILE );
			Data->Language = language;
			ret = TRUE;
		}
		else
		{
			/* don't accept write commands */
		}
		break;

	case VVL_SERIAL_START:
		if ( ReadFlag )
		{
			// Don't accept read commands
		}
		else
		{
			ret = camSerialStart( gpPPC->pCamera );
		}
		break;
	case VVL_SERIAL_STOP:
		if ( ReadFlag )
		{
			// Don't accept read commands
		}
		else
		{
			ret = camSerialStop( gpPPC->pCamera );
		}
		break;
	case VVL_SERIAL_DATA:
		if ( ReadFlag )
		{
			BYTE readData;
			ret = camSerialRead( gpPPC->pCamera, FALSE, &readData );
			Data->Serial.Value = readData;
		}
		else
		{
			ret = camSerialWrite( gpPPC->pCamera, Data->Serial.Value );
		}
		break;
	case VVL_SERIAL_READ_DATA_ACK:
		if ( ReadFlag )
		{
			BYTE readData;
			ret = camSerialRead( gpPPC->pCamera, TRUE, &readData );
			Data->Serial.Value = readData;
		}
		else
		{	
			// Don't accept write commands
		}
		break;

	default:
		break;
   
    }

	return( ret );
}


