/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */



#define		VVL_MSG_WRITE_MAGIC_NUMBER  0x23680000UL
#define		VVL_MSG_READ_MAGIC_NUMBER  	0x68320000UL
#define		VVL_MSG_RETURN_MAGIC_NUMBER	0x57750000UL

#define		VVL_MSG_MAGIC_BITS			0x7FFF0000UL
#define 	VVL_MSG_MESSAGE_BITS		0x0000FFFFUL

#define  	VVL_RETURN_DATA_MAGIC			0x0001




typedef enum {


	VVL_CPIA_CMD		= 0x0010,					


	/* general camera state and capabilities info		*/
	VVL_CAMERA_STATE	= 0x0020,				
	VVL_FW_VERSION,
	VVL_HW_INFO,	
	VVL_DEBUG_LEVEL,
	VVL_PARALLEL_PORT_TYPE,
	

	VVL_BACKGROUND_UPLOAD,


	/* below needed to implement Video Source Dlg		*/
	VVL_COLOUR			= 0x0040,	
	VVL_COMPRESSION,


	/* below needed to implement Advanced Settings Dlg	*/
	VVL_FRAMERATE			= 0x0050,
	VVL_AEC_PARAMS,
	VVL_EXPOSURE,
	VVL_PAN_TILT,
	VVL_ACB_PARAMS,

	/* below needed to implement manual control of AEC	*/
	VVL_AEC_MODE		= 0x0060,		
	VVL_AEC_VALUE,

	/* below needed to implement reading and reseting of switch circuit	*/
    VVL_PUSH_BUTTON     = 0x0070,

	VVL_DRV_VERSION		= 0x0080,
	VVL_LANGUAGE,

	VVL_SERIAL_START	= 0x0090,
	VVL_SERIAL_STOP,
	VVL_SERIAL_DATA,
	VVL_SERIAL_READ_DATA_ACK
    
} T_VVL_MSG;



#pragma pack( 1 )	//	ensure structure packing is compatible between 16 and 32 bit code

typedef union {
	DWORD Raw;

	struct {
		BYTE PortNumber;
		BYTE PortType;
		WORD DebugLevel;
	} ParallelPortType;

	struct {
		BYTE Enable;
	} BackgroundUpload;

	struct {
		BYTE Version;
		BYTE Revision;
	} FwVersion;

	struct {
		BYTE Brightness;
		BYTE Contrast;
		BYTE Colour;
		BYTE spare;
	} Colour;

	struct {
		BYTE Mode:4;		/* set to COMP_MODE value							*/
		BYTE EnableVidConf:4;	/* set to 1 to use video conference mode		*/
		BYTE VidConfVal;	/* compression value for video conferencing mode	*/
		BYTE VidCapVal;		/* compression value for video capture mode			*/
		BYTE ManualVal;		
	} Compression;


	struct {
		BYTE Framerate;
		BYTE AutoFramerate;
	} Framerate;


	struct {
		BYTE ExpMode;
		BYTE CentreWeight;
		BYTE BandingFilter;
		BYTE MainsFreq;
	} AECParams;

	struct {
		WORD Coarse;
		BYTE Fine;
		BYTE Gain;
	} Exposure;
	
	struct {
		BYTE CbMode;
		BYTE RedGain;
		BYTE GreenGain;
		BYTE BlueGain;
	} ACBParams;


	struct {
		BYTE ZoomPossible : 1;
		BYTE Top : 7;			/* Top, bottom, left and right all divided by 4 */
		BYTE ZoomEnable : 1;
		BYTE Bottom : 7;
		BYTE Left;
		BYTE Right;
	} PanTilt;

   	struct {
		BYTE Check;
		BYTE Pushed;			
	} PushButton;

   	struct {
		BYTE DrvVersion;
		BYTE DrvRevision;			
	} DrvVerParams;

	DWORD Language;

	struct {
		BYTE Value;
		BYTE Ack;
	} Serial;

} T_VVL_DATA;

#pragma pack( )		// set back to default


BOOL VVLMessage( T_VVL_MSG Msg, BOOL ReadFlag, T_VVL_DATA *Data );



    

    

