/* Copyright 2000 STMicroelectronics. Free for use under the Gnu Public License */
/****************************************************************************
*
*   dvm_msg.c
* 
*   Video Message Processing
* 
**************************************************************************/

#include <windowsx.h>

#include <ctype.h>


#include "common.h"

#include "vvlcap.h"

#include "resource.h"

#include "vvl_msg.h"


#include "vfwdlg.h"
#include "vstream.h"
#include "ppc.h"

#include "czoom.h"

#include "pal8bit.h"
#include "decode32.h"


#define DVM_MSG

#include "dvm_msg.h"

#define UNKNOWN_PP_PORT		0

/*
#define D1(sz)      (OutputDebugStr("\r\nvvlcap: "),OutputDebugStr(sz))
#define D(sz)       (OutputDebugStr(sz))
*/


#define TMP_STR_LEN         160

char      gTmpStr[TMP_STR_LEN];



extern BOOL fVideoInited;             /* declared in VSTREAM.C */
                                     

HINSTANCE   ghModule;                 // our module handle
WORD        gwDriverUsage;            // Capture+Display+VideoIn+VideoOut usage count
WORD        gwCaptureUsage;           // Usage count
WORD        gwVideoInUsage;           // Usage count
//WORD        gwDisplayUsage;           // Usage count
//WORD        gwVideoOutUsage;          // Usage count
BOOL        fDeviceInitialized = FALSE;


char        gInitFile[FILENAME_MAX];
char        gLanguage[80];
int         gPortNumb;
int         gVideoCaptureSettings, gAdvCapture, gVideoFormat;
HPPC        gCurrPPC;

BITMAPINFOHEADER gCurrImageType;

DWORD gStreamStartTime;

/* debug level. This is used in lots of places, and extern'd as required */
WORD gDebugLevel;


/* static functions */
static BOOL drvInitialise (LPSTR ParamString, int *pPort, PPC_INIT_ERR *errcode);

/*-----------------------------------------------------------------*/

int FAR PASCAL LibMain(HANDLE hModule, WORD wHeapSize, LPSTR lpCmdLine)

{
	/* save our module handle */
	ghModule = hModule;
	
	czInit (hModule);
	
	/* always succeed load! (if config error, we will not Enable...). */
	return ( 1 );
}

/*-----------------------------------------------------------------*/

DWORD VideoEnable( void )

{
	fDeviceInitialized = FALSE;
	
	gwDriverUsage = 0; 
    gwCaptureUsage = 0;
	gwVideoInUsage = 0;
//	gwDisplayUsage = 0;
//	gwVideoOutUsage = 0;
	
	return OpenPPC() ? DV_ERR_OK : DV_ERR_NONSPECIFIC;
}

/*-----------------------------------------------------------------*/

DWORD VideoDisable( void )

{
	ClosePPC();
	
	fDeviceInitialized = FALSE;
	
	return DV_ERR_OK;
}

/*-----------------------------------------------------------------*/

HCHANNEL PASCAL VideoOpen( char FAR *ParamString, 
						  LPVIDEO_OPEN_PARMS lpOpenParms)
						  
{
	LPDWORD lpdwError;
	DWORD dwFlags;
	HCHANNEL hChannel = NULL;
	PCHANNEL pChannel = NULL;  
	BOOL AcceptOpen;
    PPC_INIT_ERR errcode;
    
	lpdwError = &lpOpenParms->dwError;
	dwFlags = lpOpenParms->dwFlags;
	*lpdwError = DV_ERR_NOTDETECTED;

 	AcceptOpen = FALSE;

    hChannel = (HCHANNEL)LocalAlloc( LMEM_FIXED, sizeof(CHANNEL) );
    if ( !hChannel )
    {
		*lpdwError = DV_ERR_NOMEM;
        goto FailVideoOpen;
    }
    else
    {
        pChannel = (PCHANNEL)LocalLock( hChannel );
        if ( !pChannel )
        {
    		*lpdwError = DV_ERR_NOMEM;
            goto FailVideoOpen;
        }
    }  
	
    switch ( (int)(dwFlags) & (VIDEO_EXTERNALIN | VIDEO_EXTERNALOUT | VIDEO_IN | VIDEO_OUT) )
	{
	case VIDEO_EXTERNALIN:
		if ( gwCaptureUsage >= MAX_CAPTURE_CHANNELS )
        {
            *lpdwError = DV_ERR_TOOMANYCHANNELS;
        }
        else
        {
        	if ( !fDeviceInitialized )
            {
                if ( !drvInitialise( ParamString, &gPortNumb, &errcode ) )
                {
                    *lpdwError = DV_ERR_NOTDETECTED;
            		if ( TMSET () )
		            	tracemsg( TRC_MSG, "Video Open : VIDEO_EXTERNALIN - Failed to initialise camera" );
                    break;
                }
            }
			AcceptOpen = TRUE;
            gwCaptureUsage++;
            if ( TMSET () )
		        tracemsg( TRC_MSG, "Video Open : VIDEO_EXTERNALIN - OK" );
        }
        break;
		
	case VIDEO_EXTERNALOUT:
		if ( TMSET () )
			tracemsg( TRC_MSG, "Video Open : VIDEO_EXTERNALOUT - Not supported" );
        *lpdwError = DV_ERR_NOTSUPPORTED;
		break;
		
	case VIDEO_IN:
		if ( gwVideoInUsage >= MAX_IN_CHANNELS )
        {
            *lpdwError = DV_ERR_TOOMANYCHANNELS;
        }
        else
        {
        	if ( !fDeviceInitialized )
            {
                if ( !drvInitialise( ParamString, &gPortNumb, &errcode ) )
                {
                    *lpdwError = DV_ERR_NOTDETECTED;
            		if ( TMSET () )
		            	tracemsg( TRC_MSG, "Video Open : VIDEO_IN - Failed to initialise camera" );
                    break;
                }
            }
			AcceptOpen = TRUE;
            gwVideoInUsage++;
            if ( TMSET () )
		        tracemsg( TRC_MSG, "Video Open : VIDEO_EXTERNALIN - OK" );
        }
        break;

    case VIDEO_OUT:
		if ( TMSET () )
			tracemsg( TRC_MSG, "Video Open : VIDEO_OUT - Not supported" );
        *lpdwError = DV_ERR_NOTSUPPORTED;
		break;
		
	default:
		if ( TMSET () )
			tracemsg( TRC_MSG, "Video Open : error - unknown channel" );
		break;
	}

    if ( AcceptOpen ) 
    {
    	pChannel->fccType     = OPEN_TYPE_VCAP;
    	pChannel->dwOpenType  = (dwFlags & (VIDEO_EXTERNALIN|VIDEO_EXTERNALOUT|VIDEO_IN|VIDEO_OUT));
    	pChannel->dwOpenFlags = dwFlags;
    	pChannel->lpVHdr      = NULL;
    	pChannel->dwError     = 0L;
	    gwDriverUsage++;
    }
    else 
    {
         goto FailVideoOpen;
    }
 
    *lpdwError = DV_ERR_OK;
	D( " - VideoOpen OK" );
    return hChannel;


FailVideoOpen:
    if ( hChannel ) 
    {
        if ( pChannel ) 
            LocalUnlock( hChannel );
		LocalFree (hChannel);
    }        
	D( " - VideoOpen Failed" );
    return( NULL );
}


/*-----------------------------------------------------------------*/

DWORD PASCAL VideoClose(HCHANNEL hChannel)

{
	PCHANNEL pChannel;
    
    if ( fVideoInited ) 
    	return( DV_ERR_STILLPLAYING );

    pChannel = (PCHANNEL)LocalLock( hChannel );
    
	// Decrement the channel open counters
	
	switch (pChannel-> dwOpenType)
	{
	case VIDEO_EXTERNALIN:
        if ( gwCaptureUsage ) gwCaptureUsage--;
        break;
        
	case VIDEO_EXTERNALOUT:
//        if ( gwDisplayUsage ) gwDisplayUsage--;
        break;
        
	case VIDEO_IN:
        if ( gwVideoInUsage ) gwVideoInUsage--;
        break;
        
	case VIDEO_OUT:
//        if ( gwVideoOutUsage ) gwVideoOutUsage--;
        break;
        
	default:
        break;
	}
	
	gwDriverUsage--;  // Overall driver useage count
	
	if (gwDriverUsage < 1)
	{
		WritePrivateProfileString( "VVLCPIA", 
			INI_LPT_PORT, 
			itoa(gPortNumb, gTmpStr, TMP_STR_LEN), 
			gInitFile ); 
		FiniPPCDevice( gCurrPPC, gInitFile );
		fDeviceInitialized = FALSE;
		if (TMSET ())
		{
			tracemsg (TRC_MSG, "    Final close");
			/*     tracemsg (TRC_DESTROY);*/
		}
	}
	
	// Free the instance data
	LocalUnlock( hChannel );
	LocalFree( hChannel );         
    
	return DV_ERR_OK;
}

/****************************************************************************

  Main message handler
  
****************************************************************************/

DWORD PASCAL VideoProcessMessage(HCHANNEL hChannel, UINT msg, LONG lParam1, LONG lParam2)

{
    PCHANNEL    pChannel;
    DWORD       dwOpenType;
    DWORD       rtn;
    LPVIDEO_GETERRORTEXT_PARMS lperr;
    
    pChannel = (PCHANNEL)LocalLock( hChannel );
	
    dwOpenType = pChannel->dwOpenType;
    
    rtn = DV_ERR_NOTSUPPORTED;
    switch( msg ) {
		
	case DVM_DIALOG:
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_DIALOG");
        DisableBackgroundUpload( TRUE );
		rtn = VideoDialog( dwOpenType, (HWND)lParam1, (DWORD)lParam2 );
		DisableBackgroundUpload( FALSE );
		if (TMSET ())
			tracemsg (TRC_MSG, "    DVM_DIALOG - %s", rtn != DV_ERR_OK ? 
			(LPSTR) "error" : 
		(LPSTR) "ok");
		break;
		
	case DVM_FORMAT:      
		if (pChannel->dwOpenType != VIDEO_IN)
			rtn = DV_ERR_NOTSUPPORTED;
		else
			rtn = VideoFormatMessage( pChannel, msg, lParam1, lParam2 );
		
		if (TMSET ())
			tracemsg (TRC_MSG, "    DVM_FORMAT - %s", rtn != DV_ERR_OK ?
			(LPSTR) "error" :
		(LPSTR) "ok");
		break;
		
	case DVM_PALETTE:
		if (pChannel->dwOpenType != VIDEO_IN)
		{
			rtn = DV_ERR_NOTSUPPORTED;
		}
		else
		{
			if (TMSET ())
				tracemsg (TRC_MSG, "DVM_PALLETE");
			rtn = VideoPaletteMessage( pChannel, msg, lParam1, lParam2 );
		}
		
		if (TMSET ())
			tracemsg (TRC_MSG, "    DVM_PALLETE - %s", rtn != DV_ERR_OK ?
			(LPSTR) "error" : 
		(LPSTR) "ok");
		break;
		
	case DVM_FRAME:
		if (dwOpenType != VIDEO_IN)
		{
			rtn = DV_ERR_NOTSUPPORTED;
		}
		else
			rtn = CaptureFrame( (LPVIDEOHDR)lParam1, FALSE );
		
		if (TMSET ())
		{
			static DWORD fgood = 0,
				fbad = 0;
			
			/* only display a message every 50 frames */
			if (rtn == DV_ERR_OK)
				fgood++;
			else
				fbad++;
			
			if ((fgood + fbad) % 50 == 0)
				tracemsg (TRC_MSG, "DVM_FRAME - (good %ld bad %ld) cur = %s",
				fgood, fbad,
				rtn != DV_ERR_OK ? (LPSTR) "error" : (LPSTR) "ok");
		}
		
		break;
		
	case DVM_GETVIDEOAPIVER:    /* lParam1 = LPDWORD */
		D1("DVM_GETVIDEOAPIVER");
		if (lParam1)
		{
			tracemsg( TRC_MSG, "DVM GETVIDEOAPIVER: lParam1 = %ld", *(LPDWORD)lParam1 );

			*(LPDWORD) lParam1 = CAP_DRIVER_VERSION;
			rtn = DV_ERR_OK;
		}
		else
			rtn = DV_ERR_PARAM1;
		
		if (TMSET ())
			tracemsg( TRC_MSG, "DVM_GETVIDEOAPIVER - %s", rtn != DV_ERR_OK ?
			(LPSTR) "error" :
		(LPSTR)  "ok");
		break;
		
	case DVM_GETERRORTEXT:      /* lParam1 = LPVIDEO_GETERRORTEXT_PARMS */
        if ( TMSET () )
	 	    tracemsg( TRC_MSG, "DVM_GETERRORTEXT" );

        if ( (lperr = (LPVIDEO_GETERRORTEXT_PARMS)lParam1) != NULL )
		{
			if ( (lperr->dwLength > 0) && (lperr->lpText) )
            {
                *(lperr->lpText) = '\0';
                rtn = DV_ERR_OK;
            }
        }
        break;            
		
	case DVM_DST_RECT:  
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_DST_RECT - not supported");
		break;
		
	case DVM_PALETTERGB555:    
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_PALETTERGB555 - not supported");
		break;
		
	case DVM_GET_CHANNEL_CAPS: 
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_GET_CHANNEL_CAPS - not supported");
		break;
		
	case DVM_SRC_RECT:
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_SRC_RECT - not supported");
		break;
		
	case DVM_UPDATE:
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_UPDATE - not supported");
		break;
		
	case DVM_CONFIGURESTORAGE:
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_CONFIGURESTORAGE - not supported");
		break;         
		
	default:
	   	if ( msg >= DVM_STREAM_MSG_START && msg <= DVM_STREAM_MSG_END)
		{
		    rtn = VideoStreamMessage (pChannel, msg, lParam1, lParam2);
		}
		else
		{
			if (TMSET ())
				tracemsg (TRC_MSG, "Unknown message 0x%04x", msg);
			rtn = (DWORD)DV_ERR_NOTSUPPORTED;
		}
        break;
}
	
    LocalUnlock( hChannel ); 
	
    if ( (msg == DVM_STREAM_PREPAREHEADER) || (msg == DVM_STREAM_UNPREPAREHEADER) ) 
        return( rtn );
		/*
		if ( rtn != DV_ERR_OK ) {
        sprintf( gTmpStr, " Msg = %u  Param1 = %lu  Param2 = %lu", 
		msg, lParam1, lParam2 );
        D1( gTmpStr );
		}
	*/
    
    return( rtn );
}   

/*-----------------------------------------------------------------*/

DWORD PASCAL VideoDialog( DWORD dwOpenType, HWND hWndParent, DWORD dwFlags )
{
    int dlg_ret, ret;
    DWORD rtn = DV_ERR_NOTSUPPORTED;
	
    switch( dwOpenType )
	{
	case VIDEO_EXTERNALIN:
		/* video capture setting dialog box */
		if ( dwFlags & VIDEO_DLG_QUERY )
		{
			if (TMSET ())
				tracemsg (TRC_MSG, "    VIDEO_EXTERNALIN - query, ok");
			rtn = DV_ERR_OK;       /* Support the dialog */
		}
		else
		{        
			if (TMSET ())
				tracemsg (TRC_MSG, "    VIDEO_EXTERNALIN");
            dlg_ret = DialogBox( ghModule, MAKEINTRESOURCE(gVideoCaptureSettings),
			            	        hWndParent, VideoCaptureSettingsDialogProc );
			if ( dlg_ret && (dlg_ret != -1) )
			{
				rtn = DV_ERR_OK;
			}
            else
            {
                if (TMSET ())
				    tracemsg (TRC_MSG, "    - dialog box failed");
            }
        }
		break;

	case VIDEO_IN:
		/* video format dialog box */
		if ( dwFlags & VIDEO_DLG_QUERY )
		{
			if (TMSET ())
				tracemsg (TRC_MSG, "    VIDEO_IN - query, ok");
			rtn = DV_ERR_OK;       /* Support the dialog */
		}
		else
		{
			if (TMSET ())
				tracemsg (TRC_MSG, "    VIDEO_IN");
	        dlg_ret = DialogBox( ghModule, MAKEINTRESOURCE(gVideoFormat),
				                    hWndParent, VideoFormatDialogProc );
			if ( dlg_ret && (dlg_ret != -1) )
			{
				rtn = DV_ERR_OK;
			}
            else
            {
                if (TMSET ())
				    tracemsg (TRC_MSG, "    - dialog box failed");
            }
		}
		break;
		
	case VIDEO_OUT:
		if (TMSET ())
            tracemsg (TRC_MSG, "    VIDEO_OUT - fail");
		break;
		
	case VIDEO_EXTERNALOUT:
		if (TMSET ())
            tracemsg (TRC_MSG, "    VIDEO_EXTERNALOUT - fail");
		break;
		
	default :
		if (TMSET ())
            tracemsg (TRC_MSG, "    VideoDialog - invalid dwOpenType 0x%lx",
			dwOpenType);
		break;
    }
	
    return rtn;   /* on either cancel or OK  (returns not supported on cancel) */
}


/****************************************************************************

  handles Configure messages for video
  lParam1 is dwFlags
  lParam2 is LPVIDEOCONFIGPARMS
  
****************************************************************************/

DWORD PASCAL VideoFormatMessage( PCHANNEL pChannel, UINT msg, LONG lParam1, LONG lParam2 )

{
    DWORD rtn;
	
    LPVIDEOCONFIGPARMS lpcp;
    LPBITMAPINFOHEADER lpinf;
    LPDWORD lpdwReturn;     /* Return parameter from configure. */
    LPVOID  lpData1;        /* Pointer to data1. */
    DWORD   dwSize1;        /* size of data buffer1. */
    LPVOID  lpData2;        /* Pointer to data2. */
    DWORD   dwSize2;        /* size of data buffer2. */
    DWORD   dwFlags;

	DWORD magic;
	static BOOL ValidPrivateData = FALSE;
	static DWORD PrivateData;

    dwFlags = lParam1;

    lpcp = (LPVIDEOCONFIGPARMS)lParam2;
    lpdwReturn = lpcp->lpdwReturn;
    lpData1 = lpcp->lpData1;     
    dwSize1 = lpcp->dwSize1;     
    lpData2 = lpcp->lpData2;     
    dwSize2 = lpcp->dwSize2;     


	
    rtn = DV_ERR_NOTSUPPORTED;
	
    switch( dwFlags ) {
		
    case ( VIDEO_CONFIGURE_QUERY | VIDEO_CONFIGURE_GET ):    
        if (TMSET ())
			tracemsg (TRC_MSG, "VIDEO_CONFIGURE_QUERY : get - ok");
        rtn = DV_ERR_OK;
        break;
		
    case ( VIDEO_CONFIGURE_QUERYSIZE | VIDEO_CONFIGURE_GET ):
        if (TMSET ())
			tracemsg (TRC_MSG, "VIDEO_CONFIGURE_QUERYSIZE : ok");
        
        *lpdwReturn = sizeof(BITMAPINFOHEADER) + 10;
        rtn = DV_ERR_OK;
        break;
		
    case ( VIDEO_CONFIGURE_GET | VIDEO_CONFIGURE_CURRENT ):
        if (TMSET ())
			tracemsg (TRC_MSG, "VIDEO_CONFIGURE_GET : current");
        if ( dwSize1 >= sizeof(BITMAPINFOHEADER) )
		{

			if ( !GetPPCImageType( &gCurrImageType ) ) 
				D( "GetImageType returned FALSE " );
			*((LPBITMAPINFOHEADER)lpData1) = gCurrImageType;

			if ( ValidPrivateData == TRUE ) 
			{
				((LPBITMAPINFOHEADER)lpData1)->biXPelsPerMeter = VVL_MSG_RETURN_MAGIC_NUMBER;
				((LPBITMAPINFOHEADER)lpData1)->biYPelsPerMeter = PrivateData;
				ValidPrivateData = FALSE;
//				if TMSET() 
//					tracemsg( TRC_MSG, "Private Data %ld", PrivateData );
			}

			if (TMSET ())
				tracemsg (TRC_MSG, "    bitcnt = %u comp = %lx (%s) X size = %lu Y size = %lu",
				gCurrImageType. biBitCount, 
				gCurrImageType. biCompression,
				trc_fourcc (gCurrImageType. biCompression),
				gCurrImageType. biWidth, 
				gCurrImageType. biHeight);

			rtn = DV_ERR_OK;
		}
        else
		{
			if (TMSET ())
				tracemsg (TRC_MSG, "   DV_ERR_SIZEFIELD");
			rtn = DV_ERR_SIZEFIELD;
		}
        break;
		
    case VIDEO_CONFIGURE_SET :
		lpinf = (LPBITMAPINFOHEADER)lpData1;
		
		magic = (lpinf->biXPelsPerMeter & VVL_MSG_MAGIC_BITS);

 		if ( (magic == VVL_MSG_WRITE_MAGIC_NUMBER) || (magic == VVL_MSG_READ_MAGIC_NUMBER) ) 
		{
			/* its a special VVL Specific command */
			T_VVL_MSG msg;
			T_VVL_DATA data;
			BOOL read_flag;

			msg = (T_VVL_MSG)(lpinf->biXPelsPerMeter & VVL_MSG_MESSAGE_BITS);
			read_flag = (magic == VVL_MSG_READ_MAGIC_NUMBER);
			data.Raw = lpinf->biYPelsPerMeter;

			if ( !VVLMessage( msg, read_flag, &data ) )
			{
				if TMSET()
					tracemsg( TRC_MSG, "VVL_MSG : Failed Msg=0x%x", msg );
				break;
			}

			if ( read_flag )								
			{
				PrivateData = data.Raw;
				ValidPrivateData = TRUE;
				if TMSET()
					tracemsg( TRC_MSG, "VVL_MSG : Msg=0x%x  ReadData=0x%lx", msg, data.Raw );
			}
			else
			{
				if TMSET()
					tracemsg( TRC_MSG, "VVL_MSG : Msg=0x%x  WriteData=0x%lx", msg, data.Raw );
			}

	   		break; // this will return an error to the application */
    	}
    
    	    
		if (TMSET ())
		{
			tracemsg (TRC_MSG, "DVM_FORMAT : VIDEO_CONFIGURE_SET");
			tracemsg (TRC_MSG, "    bitcnt = %u comp = %lx (%s) X size = %lu Y size = %lu",
				lpinf -> biBitCount, 
				lpinf -> biCompression,
				trc_fourcc (lpinf -> biCompression),
				lpinf -> biWidth, 
				lpinf -> biHeight);
		}

        /* first check if we are already in this mode */     
        if ( !memcmp( &gCurrImageType, (LPBITMAPINFOHEADER)lpData1, sizeof( BITMAPINFOHEADER ) ) )
		{
			/* mode is identical to that currently set */
			if (TMSET ())
				tracemsg (TRC_MSG, "    Format already current");
			rtn = DV_ERR_OK;
			break;
		} 
        else
		{
			/* it's different from the currently set mode */
			if ( !SetPPCImageType( (LPBITMAPINFOHEADER)lpData1 ) )
			{
				LPBITMAPINFOHEADER InfoHeader;
				
				/* clear out old format */
				memset( &gCurrImageType, 0, sizeof(  BITMAPINFOHEADER ) );
				
				gCurrImageType = *((LPBITMAPINFOHEADER)lpData1);
				InfoHeader = (LPBITMAPINFOHEADER)lpData1;
				
				if (TMSET ())
					tracemsg (TRC_MSG, "    unsupported image type");
			}
			else
			{
				gCurrImageType = *((LPBITMAPINFOHEADER)lpData1);
				if (TMSET ())
					tracemsg (TRC_MSG, "    image type supported");
				rtn = DV_ERR_OK;
			}
		}
        break;
        
    default:
        sprintf( &gTmpStr[0], "\n\r - dwFlags = %lu", dwFlags );
        D( ((LPCSTR)&gTmpStr[0]) );
		
        break;
    } 
	
    return( rtn );
}

/*-----------------------------------------------------------------*/

DWORD PASCAL VideoPaletteMessage( PCHANNEL pChannel, UINT msg, LONG lParam1, LONG lParam2 )

{
	DWORD rtn,
		dwSize1,        // size of data buffer1.
		dwSize2,        // size of data buffer2.
		dwFlags;
	LPDWORD lpdwReturn;     // Return parameter from configure.
	LPVOID  lpData1,        // Pointer to data1.
		lpData2;        // Pointer to data2.
	LPVIDEOCONFIGPARMS lpcp;
	WORD wPalSize;
	
	dwFlags = lParam1;
	lpcp = (LPVIDEOCONFIGPARMS)lParam2;
	lpdwReturn = lpcp->lpdwReturn;
	lpData1 = lpcp->lpData1;     
	dwSize1 = lpcp->dwSize1;     
	lpData2 = lpcp->lpData2;     
	dwSize2 = lpcp->dwSize2;     
	
	rtn = DV_ERR_NOTSUPPORTED;
	
	switch( dwFlags )
	{
	case (VIDEO_CONFIGURE_QUERY | VIDEO_CONFIGURE_SET):
		if (TMSET ())
			tracemsg (TRC_MSG, "    VIDEO_CONFIGURE_QUERY : SET");
		rtn = DV_ERR_OK;
		break;
		
	case (VIDEO_CONFIGURE_QUERY | VIDEO_CONFIGURE_GET):
		if (TMSET ())
			tracemsg (TRC_MSG, "    VIDEO_CONFIGURE_QUERY : GET");
		rtn = DV_ERR_OK;
		break;
		
	case VIDEO_CONFIGURE_QUERYSIZE:
	case (VIDEO_CONFIGURE_QUERYSIZE | VIDEO_CONFIGURE_GET):
		if (TMSET ())
			tracemsg (TRC_MSG, "    VIDEO_CONFIGURE_QUERYSIZE%s", 
			dwFlags & VIDEO_CONFIGURE_QUERYSIZE ? (LPSTR) " : GET" :
		(LPSTR) "");
		Get8BitPalette( &wPalSize, NULL );
		*lpdwReturn = (DWORD)wPalSize;
		rtn = DV_ERR_OK;
		break;
		
	case VIDEO_CONFIGURE_SET:                     
	case (VIDEO_CONFIGURE_SET | VIDEO_CONFIGURE_CURRENT):
		if (TMSET ())
			tracemsg( TRC_MSG, "    VIDEO_CONFIGURE_SET%s", 
			            dwFlags & VIDEO_CONFIGURE_CURRENT ? (LPSTR) " : CURRENT" :
                        (LPSTR) "");
		
		if (!lpData1)
		{      // points to a LOGPALETTE structure.
			rtn = DV_ERR_PARAM1;
			break;
		}                                        
		
		/* need to set the palette in 16 and 32 bit decode modules */              
		Set8BitPalette( (LPLOGPALETTE)lpData1 );
		Set8BitPalette32( (LPLOGPALETTE)lpData1 );
		
		rtn = DV_ERR_OK;
		break;
		
	case VIDEO_CONFIGURE_GET:
	case (VIDEO_CONFIGURE_GET | VIDEO_CONFIGURE_CURRENT):
		if (TMSET ())
			tracemsg( TRC_MSG, "    VIDEO_CONFIGURE_GET%s", 
	            		dwFlags & VIDEO_CONFIGURE_CURRENT ? (LPSTR) " : CURRENT" :
    	            	(LPSTR) "");
		
        Get8BitPalette( &wPalSize , NULL );
        if ( wPalSize <= dwSize1 ) 
		{
            Get8BitPalette( &wPalSize , lpData1);
			rtn = DV_ERR_OK;
		}
        break;
		
	default:
        break;
	} // end of DVM_PALETTE switch
	
	return rtn;
}

/*-----------------------------------------------------------------*/

DWORD FAR PASCAL CaptureFrame( LPVIDEOHDR lpVHdr, BOOL StreamMode )

{                
	extern HWND hFrameRate;
	static DWORD periodstart,
		periodtime,
		framecount;
	NextStatus status;
	DWORD curtime,
		captime,
		rtn;
	char buf [30];
	
	rtn = DV_ERR_PARAM1;
	
	if (lpVHdr && 
		lpVHdr -> lpData != NULL &&
		lpVHdr -> dwBufferLength > 0)
	{
		/* curtime = timeGetTime ();*/
		status = NextFrame ((BYTE _huge *)(lpVHdr->lpData), lpVHdr->dwBufferLength,
			&captime, StreamMode);
		
		if ( status == NextOK ) 
			rtn = DV_ERR_OK;
		else if ( status == NextNotReady ) 
			rtn = DV_ERR_SYNC;
		else if ( status == NextError ) 
			rtn = DV_ERR_SYNC;
		 	
		if (rtn == DV_ERR_OK)
		{                        
			captime = timeGetTime ();
			lpVHdr->dwTimeCaptured = captime - gStreamStartTime;
			lpVHdr->dwBytesUsed = gCurrImageType.biSizeImage;
			lpVHdr->dwFlags |= (VHDR_DONE | VHDR_KEYFRAME);
			
			/* debug type code. Averages the frame rate every 4 frames. Isn't 
			particularly useful, but does help */
			/*   
			if (framecount == 0 || framecount == 4)
			{
			framecount = 1;
			periodstart = curtime;
			periodtime = captime - periodstart;
			}
			else
			{
			periodtime = captime - periodstart;
			framecount++;
			} 
			
			  if (hFrameRate)
			  { 
			  sprintf (buf, "%-02.02f fps", (double) (1000.0 / 
			  (float) (periodtime / framecount)));
			  SetWindowText (hFrameRate, buf);
			  }
			*/
		}
	}
	else
	{
		rtn = DV_ERR_PARAM1;
	}
	
	return( rtn );
}

/*-----------------------------------------------------------------*/
/*  handles Stream messages for video */

DWORD PASCAL VideoStreamMessage(PCHANNEL pChannel, UINT msg, LONG lParam1, LONG lParam2)

{
    DWORD dwOpenType = pChannel->dwOpenType;
	DWORD rtn;
	
    if ( dwOpenType == VIDEO_EXTERNALIN ) {       // Capture channel
        switch (msg) {
		case DVM_STREAM_INIT:
			break;
		case DVM_STREAM_FINI:
			break;
		default:
			return DV_ERR_NOTSUPPORTED;
        }
        return DV_ERR_OK;
    }

    if ( dwOpenType == VIDEO_EXTERNALOUT ) { // Overlay channel
        return DV_ERR_NOTSUPPORTED;
    }

    switch( msg ) {                         // Input channel
        //
        //  lParam1     -   LPVIDEO_STREAM_INIT_PARMS
        //
        //  lParam2     -   sizeof (LPVIDEO_STREAM_INIT_PARMS)
        //
	case DVM_STREAM_INIT:           
		D1( "DVM_STREAM_INIT" );
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_STREAM_INIT");
		rtn = InStreamInit( (LPVIDEO_STREAM_INIT_PARMS)lParam1 );
		if ( rtn != DV_ERR_OK ) 
			if (TMSET ())
				tracemsg (TRC_MSG, "- failed");
			return( rtn );
			
	case DVM_STREAM_FINI:
		D1( "DVM_STREAM_FINI" );
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_STREAM_FINI");
		return InStreamFini();
		
	case DVM_STREAM_ADDBUFFER:                 
		if (TMSET ())
		{
			static DWORD count = 0;
			
			if (count++ % 100 == 0)
                tracemsg (TRC_MSG, "DVM_STREAM_ADDBUFFER (100)");
		}
		//          D1( "DVM_STREAM_ADDBUFFER" );
		return InStreamAddBuffer( (LPVIDEOHDR)lParam1 );
		
	case DVM_STREAM_RESET:                          
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_STREAM_RESET");
		D1( "DVM_STREAM_RESET" );
		return InStreamReset();
		
	case DVM_STREAM_PREPAREHEADER:  // Handled by MSVideo
		if (TMSET ())
		{
			static DWORD count = 0;
			
			if (count++ % 100 == 0)
				tracemsg (TRC_MSG, "DVM_STREAM_PREPAREHEADER (100)");
		}
		return DV_ERR_NOTSUPPORTED;
		
	case DVM_STREAM_UNPREPAREHEADER: // Handled by MSVideo
		if (TMSET ())
		{
			static DWORD count = 0;
			
			if (count++ % 100 == 0)
                tracemsg (TRC_MSG, "DVM_STREAM_UNPREPAREHEADER (100)" );
		}
		return DV_ERR_NOTSUPPORTED;
		
	case DVM_STREAM_START:          
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_STREAM_START" );
		return InStreamStart();
		
	case DVM_STREAM_STOP:           
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_STREAM_STOP" );
		return InStreamStop();
		
	case DVM_STREAM_GETERROR:
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_STREAM_GETERROR" );
		return InStreamError( (LPDWORD)lParam1, (LPDWORD)lParam2 );
		
	case DVM_STREAM_GETPOSITION:
		if (TMSET ())
			tracemsg (TRC_MSG, "DVM_STREAM_GETPOSITION" );
		return InStreamGetPos( (LPMMTIME)lParam1, (DWORD)lParam2);
		
	default:               
		if (TMSET ())
			tracemsg (TRC_MSG, "stream msg, not supported (%u)", msg);
		return DV_ERR_NOTSUPPORTED;
		
    } // end switch on message type
}

/*-----------------------------------------------------------------*/


DWORD GetTime() {
    return( timeGetTime() );
}


/*-----------------------------------------------------------------*/

void Delay( WORD wMSecs )

{
	DWORD stop;
	stop = GetTime() + (wMSecs*CLOCKS_PER_SEC)/1000;
	while( GetTime() <= stop )
		;
}               

/*-----------------------------------------------------------------*/

void GetPortNumber( char *ParamString, int *PortNumber )

{
    int i;
    char tmp[FILENAME_MAX];
    char *ptr;
    char *dest;
    
    /* convert to uppercase, checking its null terminated */
    i = 0;
    ptr = ParamString;
    dest = tmp;
    while( *ptr != NULL ) {
        *dest++ = (char)toupper( *ptr );
        ptr++;
        i++;
        if ( i == FILENAME_MAX ) return;
    }
    *dest = '\0';
    
    ptr = strstr( tmp, "LPT" );
	
    if ( ptr == NULL ) return;
	
    /* ptr now points to either LPT or lpt */
    
    ptr +=3; 
    
    /* should now point to a number */
    
    i = *ptr -'0';
    
    if ( ( i > 0 ) && ( i < 4 ) ) *PortNumber = i;
    
    return;
}

/*-----------------------------------------------------------------*/

void GetInitFile( char *ParamString, char *InitFilename )

{
    int i, 
        end;
    char tmp [FILENAME_MAX],
		*ptr,
		*dest;
	
    /* convert to uppercase, checking its null terminated */
    i = 0;
    ptr = ParamString;
    dest = tmp;
    while( *ptr != NULL ) {
        *dest++ = toupper( *ptr );
        ptr++;
        i++;
        if ( i == FILENAME_MAX ) return;
    }
	
    ptr = strstr( tmp, ".INI" ); 
    if ( ptr == NULL ) return;
	
    /* backup one char to get onto the base filename        */    
    ptr--;
    i = ptr - &tmp[0];
	
    /* record the index of the last char of the filename    */
    end = i + 4;            /* ie end of basename +'.INI'   */
	
    /* now reverse back through string looking for a space  */
    /* if we hit the start of the string first then stop    */  
    while( i != 0 ) {
        if ( *(ptr-1) == ' ' ) break;
        ptr--;
        i--;
    }                
	
    /* ptr now points to the first char of the basename or  */
    /* we have reached the start of the string              */
	
    /* copy string to the requested location                */
    dest = InitFilename;
    ptr = &ParamString[i];  /* copy the non-uppercase original string */
	
    while( i <= end ) {
        *dest++ = *ptr++;
        i++;
    }
    
    *dest = '\0';
	
    return;
}

/*-----------------------------------------------------------------*/
/* returns the current state of the device. It will not be initialised
if there was a failure during init, or init has yet to be called */

BOOL IsDeviceInitialised (void)

{
	return fDeviceInitialized;
}

/*-----------------------------------------------------------------*/
/* return the camera's current port */

int GetCurrentPort (void)

{
	return gPortNumb;
}

/*-----------------------------------------------------------------*/
/* called to initialise the driver settings the first time the driver is       
opened */

static BOOL drvInitialise (LPSTR ParamString, int *pPort, PPC_INIT_ERR *pErrcode)
{            
	BOOL res = FALSE;
	int tmp_lang;

	strncpy (gInitFile, DEFAULT_INI_FILE, FILENAME_MAX);
	
	/* now check to see if an INI file was specified on the command line
    if no INI file was specified then this function will not touch the
    gInitFile string 
	*/
	GetInitFile (ParamString, gInitFile);     
	
	/* load the DebugLevel ini setting, so it's available to everything */
	gDebugLevel = GetPrivateProfileInt ("VVLCPIA", "DebugLevel", 0, gInitFile);

	/* create the debug output window if the DBG_TRACEMSG bit in gDebugLevel was set */
	if ( TMSET () )
		tracemsg (TRC_INIT);
	
	/* get which port the camera was attached to previously */
	*pPort = GetPrivateProfileInt( "VVLCPIA", INI_LPT_PORT, UNKNOWN_PP_PORT, gInitFile );

	/* read any port option settings into gDebugLevel							*/
	/* - note this will clear down any of the port option bits that are set !	*/
	ReadPortOptions( gInitFile, &gDebugLevel );
	
	/* Now set up dialog box "handles" depending on language */
	tmp_lang = ReadLanguageOption( gInitFile );
	SetDialogLanguage( tmp_lang );
		
	/* command line parameter overrides the INI file setting !!! */
	GetPortNumber( ParamString, &gPortNumb );
	
	/* set pPort to have the value of the last previously selected port. If this 
    isn't a valid port, it's changed in SelectPPCDevice */
    
	*pPort = gPortNumb;   
	if ((gCurrPPC = InitPPCDevice (gInitFile, pPort, pErrcode)) != 0)
	{
		if ( SelectPPCDevice (gCurrPPC) ) 
		{
			fDeviceInitialized = TRUE;
			gStreamStartTime = GetTime();
			InitDialogParams();
			res = TRUE;
		}
    }
 
	return res;
}



BOOL ReadPortOptions( char *InitFile, WORD *DebugFlags ) 
{
    WORD flags;

	/* clear down the flag options we're about to set */
 	flags = *DebugFlags & ~(DBG_DISABLE_ECP|DBG_DISABLE_DMA
							|DBG_DISABLE_FDMA|DBG_DISABLE_STREAM_INT);

	if ( GetPrivateProfileInt( "VVLCPIA", INI_DISABLE_ECP, 0, InitFile ) )
		flags |= DBG_DISABLE_ECP;
    else if ( GetPrivateProfileInt( "VVLCPIA", INI_DISABLE_DMA, 0, InitFile ) )
       		flags |= DBG_DISABLE_DMA;
	else if ( GetPrivateProfileInt( "VVLCPIA", INI_DISABLE_FDMA, 0, InitFile ) )
       		flags |= DBG_DISABLE_FDMA;

    if ( GetPrivateProfileInt( "VVLCPIA", INI_DISABLE_IRQ, 1, InitFile ) )
        flags |= DBG_DISABLE_STREAM_INT;
	
	if ( DebugFlags )
	{
		*DebugFlags = flags;
	}

    return( TRUE );
}


void WritePortOptions( char *InitFile, WORD DebugFlags ) 
{
	char buf[ 80 ];

	WritePrivateProfileString( "VVLCPIA", INI_DISABLE_ECP,	NULL, InitFile );
	WritePrivateProfileString( "VVLCPIA", INI_DISABLE_DMA,	NULL, InitFile );
	WritePrivateProfileString( "VVLCPIA", INI_DISABLE_FDMA, NULL, InitFile );

	if ( DebugFlags & DBG_DISABLE_ECP ) 
	{
		WritePrivateProfileString( "VVLCPIA", INI_DISABLE_ECP,	"1", InitFile );
	}
	else if ( DebugFlags & DBG_DISABLE_DMA ) 
	{
		WritePrivateProfileString( "VVLCPIA", INI_DISABLE_DMA,	"1", InitFile );
	}
	else if ( DebugFlags & DBG_DISABLE_FDMA ) 
	{
		WritePrivateProfileString( "VVLCPIA", INI_DISABLE_FDMA, "1", InitFile );
	}

	if ( DebugFlags & DBG_DISABLE_STREAM_INT ) 
	{
		WritePrivateProfileString( "VVLCPIA", INI_DISABLE_IRQ,	"1", InitFile );
	}
	else
	{
		WritePrivateProfileString( "VVLCPIA", INI_DISABLE_IRQ,	"0", InitFile );
	}

    return;
}


int ReadLanguageOption( char *InitFile )
{
	int tmp_lang;

	/* try getting setting as a number from the INI file */
	tmp_lang = GetPrivateProfileInt( "VVLCPIA", INI_LANGUAGE, 0, InitFile );

	/* ... also check for a textual description..... */
	GetPrivateProfileString( "VVLCPIA", "Language", "English", gLanguage, 80, InitFile );
	if ( (strcmp(gLanguage,"Frangais") == 0) || (strcmp(gLanguage,"French") == 0) )
		tmp_lang = FRENCH_LANG;

	if ( (strcmp(gLanguage,"Italiano") == 0)  || (strcmp(gLanguage,"Italian") == 0) )
		tmp_lang = ITALIAN_LANG;

	if ( (strcmp(gLanguage,"Deutsch") == 0) || (strcmp(gLanguage,"German") == 0) )
		tmp_lang = GERMAN_LANG;

	if ( (strcmp(gLanguage,"Espaqol") == 0) || (strcmp(gLanguage,"Spanish") == 0) )
		tmp_lang = SPANISH_LANG;
	
	if ( (strcmp(gLanguage,"Portugese") == 0) || (strcmp(gLanguage,"Portuguese") == 0) )
		tmp_lang = PORTUGUESE_LANG;
	
	return( tmp_lang );
}


void WriteLanguageOption( char *InitFile, int Lang )
{
	char buf[ 80 ];
	WritePrivateProfileString( "VVLCPIA", INI_LANGUAGE, 
								itoa( Lang, buf, 80), InitFile );
	return;
}


void SetDialogLanguage( int Lang )
{
	switch( Lang ) 
	{
	default:
	case ENGLISH_LANG:
			gVideoCaptureSettings = VideoCaptureSettings;
			gAdvCapture = AdvCapture;       
			gVideoFormat = VideoFormat;
			break;
	case FRENCH_LANG:
			gVideoCaptureSettings = VideoCaptureSettings_French;
			gAdvCapture = AdvCapture_French;       
			gVideoFormat = VideoFormat_French;      
			break;
	case ITALIAN_LANG:
			gVideoCaptureSettings = VideoCaptureSettings_Italian;
			gAdvCapture = AdvCapture_Italian;       
			gVideoFormat = VideoFormat_Italian;     
			break;
	case GERMAN_LANG:
			gVideoCaptureSettings = VideoCaptureSettings_German;
			gAdvCapture = AdvCapture_German;       
			gVideoFormat = VideoFormat_German;      
			break;
	case SPANISH_LANG:
			gVideoCaptureSettings = VideoCaptureSettings_Spanish;
			gAdvCapture = AdvCapture_Spanish;       
			gVideoFormat = VideoFormat_Spanish;     
			break;
	case PORTUGUESE_LANG:
			gVideoCaptureSettings = VideoCaptureSettings_Portuguese;
			gAdvCapture = AdvCapture_Portuguese;       
			gVideoFormat = VideoFormat_Portuguese;     
			break;
	}	
	return;
}
